@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -47,24 +47,25 @@
 
 #ifdef HAVE_LIBACL
 
-static int CheckPosixLinuxAccessACEs(EvalContext *ctx, Rlist *aces, AclMethod method, const char *file_path,
-                                     Attributes a, const Promise *pp, PromiseResult *result);
-static int CheckPosixLinuxDefaultACEs(EvalContext *ctx, Rlist *aces, AclMethod method, AclDefault acl_default,
-                                      const char *file_path, Attributes a, const Promise *pp, PromiseResult *result);
-static int CheckPosixLinuxACEs(EvalContext *ctx, Rlist *aces, AclMethod method, const char *file_path, acl_type_t acl_type, Attributes a,
-                               const Promise *pp, PromiseResult *result);
-static int CheckDefaultEqualsAccessACL(EvalContext *ctx, const char *file_path, Attributes a, const Promise *pp, PromiseResult *result);
-static int CheckDefaultClearACL(EvalContext *ctx, const char *file_path, Attributes a, const Promise *pp, PromiseResult *result);
-static int ParseEntityPosixLinux(char **str, acl_entry_t ace, int *is_mask);
-static int ParseModePosixLinux(char *mode, acl_permset_t old_perms);
+static bool CheckPosixLinuxAccessACEs(EvalContext *ctx, Rlist *aces, AclMethod method, const char *file_path,
+                                      const Attributes *a, const Promise *pp, PromiseResult *result);
+static bool CheckPosixLinuxDefaultACEs(EvalContext *ctx, Rlist *aces, AclMethod method, AclDefault acl_default,
+                                       const char *file_path, const Attributes *a, const Promise *pp, PromiseResult *result);
+static bool CheckPosixLinuxACEs(EvalContext *ctx, Rlist *aces, AclMethod method, const char *file_path, acl_type_t acl_type, const Attributes *a,
+                                const Promise *pp, PromiseResult *result);
+static bool CheckDefaultEqualsAccessACL(EvalContext *ctx, const char *file_path, const Attributes *a, const Promise *pp, PromiseResult *result);
+static bool CheckDefaultClearACL(EvalContext *ctx, const char *file_path, const Attributes *a, const Promise *pp, PromiseResult *result);
+static bool ParseEntityPosixLinux(char **str, acl_entry_t ace, int *is_mask);
+static bool ParseModePosixLinux(char *mode, acl_permset_t old_perms);
 static acl_entry_t FindACE(acl_t acl, acl_entry_t ace_find);
 static int ACLEquals(acl_t first, acl_t second);
 static int ACECount(acl_t acl);
 static int PermsetEquals(acl_permset_t first, acl_permset_t second);
 
 
-PromiseResult CheckPosixLinuxACL(EvalContext *ctx, const char *file_path, Acl acl, Attributes a, const Promise *pp)
+PromiseResult CheckPosixLinuxACL(EvalContext *ctx, const char *file_path, Acl acl, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     PromiseResult result = PROMISE_RESULT_NOOP;
 
     if (!CheckPosixLinuxAccessACEs(ctx, acl.acl_entries, acl.acl_method, file_path, a, pp, &result))
@@ -87,16 +88,18 @@ PromiseResult CheckPosixLinuxACL(EvalContext *ctx, const char *file_path, Acl ac
     return PROMISE_RESULT_NOOP;
 }
 
-static int CheckPosixLinuxAccessACEs(EvalContext *ctx, Rlist *aces, AclMethod method, const char *file_path,
-                                     Attributes a, const Promise *pp, PromiseResult *result)
+static bool CheckPosixLinuxAccessACEs(EvalContext *ctx, Rlist *aces, AclMethod method, const char *file_path,
+                                      const Attributes *a, const Promise *pp, PromiseResult *result)
 {
+    assert(a != NULL);
     return CheckPosixLinuxACEs(ctx, aces, method, file_path, ACL_TYPE_ACCESS, a, pp, result);
 }
 
-static int CheckPosixLinuxDefaultACEs(EvalContext *ctx, Rlist *aces, AclMethod method, AclDefault acl_default,
-                                      const char *file_path, Attributes a, const Promise *pp, PromiseResult *result)
+static bool CheckPosixLinuxDefaultACEs(EvalContext *ctx, Rlist *aces, AclMethod method, AclDefault acl_default,
+                                       const char *file_path, const Attributes *a, const Promise *pp, PromiseResult *result)
 {
-    int retval;
+    assert(a != NULL);
+    bool retval;
 
     switch (acl_default)
     {
@@ -135,9 +138,10 @@ static int CheckPosixLinuxDefaultACEs(EvalContext *ctx, Rlist *aces, AclMethod m
    set on the given file. If it doesn't, the ACL on the file is updated.
 */
 
-static int CheckPosixLinuxACEs(EvalContext *ctx, Rlist *aces, AclMethod method, const char *file_path, acl_type_t acl_type, Attributes a,
-                               const Promise *pp, PromiseResult *result)
+static bool CheckPosixLinuxACEs(EvalContext *ctx, Rlist *aces, AclMethod method, const char *file_path, acl_type_t acl_type, const Attributes *a,
+                                const Promise *pp, PromiseResult *result)
 {
+    assert(a != NULL);
     acl_t acl_existing;
     acl_t acl_new;
     acl_t acl_tmp;
@@ -329,7 +333,7 @@ static int CheckPosixLinuxACEs(EvalContext *ctx, Rlist *aces, AclMethod method,
     if (retv == 1)              // existing and new acl differ, update existing
     {
 
-        switch (a.transaction.action)
+        switch (a->transaction.action)
         {
         case cfa_warn:
 
@@ -402,12 +406,13 @@ static int CheckPosixLinuxACEs(EvalContext *ctx, Rlist *aces, AclMethod method,
   Returns 0 on success and -1 on failure.
  */
 
-static int CheckDefaultEqualsAccessACL(EvalContext *ctx, const char *file_path, Attributes a, const Promise *pp, PromiseResult *result)
+static bool CheckDefaultEqualsAccessACL(EvalContext *ctx, const char *file_path, const Attributes *a, const Promise *pp, PromiseResult *result)
 {
+    assert(a != NULL);
     acl_t acl_access;
     acl_t acl_default;
     int equals;
-    int retval = false;
+    bool retval = false;
 
     acl_access = NULL;
     acl_default = NULL;
@@ -438,7 +443,7 @@ static int CheckDefaultEqualsAccessACL(EvalContext *ctx, const char *file_path,
 
     case 1:                    // set access ACL as default ACL
 
-        switch (a.transaction.action)
+        switch (a->transaction.action)
         {
         case cfa_warn:
 
@@ -492,13 +497,13 @@ static int CheckDefaultEqualsAccessACL(EvalContext *ctx, const char *file_path,
   Checks if the default ACL is empty. If not, it is cleared.
 */
 
-int CheckDefaultClearACL(EvalContext *ctx, const char *file_path, Attributes a, const Promise *pp, PromiseResult *result)
+static bool CheckDefaultClearACL(EvalContext *ctx, const char *file_path, const Attributes *a, const Promise *pp, PromiseResult *result)
 {
     acl_t acl_existing;
     acl_t acl_empty;
     acl_entry_t ace_dummy;
     int retv;
-    int retval = false;
+    bool retval = false;
 
     acl_existing = NULL;
     acl_empty = NULL;
@@ -532,7 +537,7 @@ int CheckDefaultClearACL(EvalContext *ctx, const char *file_path, Attributes a,
             break;
         }
 
-        switch (a.transaction.action)
+        switch (a->transaction.action)
         {
         case cfa_warn:
 
@@ -822,7 +827,7 @@ static int PermsetEquals(acl_permset_t first, acl_permset_t second)
    (with unset permissions). Sets is_mask to true if this is a mask entry.
 */
 
-static int ParseEntityPosixLinux(char **str, acl_entry_t ace, int *is_mask)
+static bool ParseEntityPosixLinux(char **str, acl_entry_t ace, int *is_mask)
 {
     struct passwd *pwd;
     struct group *grp;
@@ -831,7 +836,7 @@ static int ParseEntityPosixLinux(char **str, acl_entry_t ace, int *is_mask)
     id_t id;
     char *ids;
     char *id_end;
-    int result = true;
+    bool result = true;
     int i;
 
     ids = NULL;
@@ -976,7 +981,7 @@ static int ParseEntityPosixLinux(char **str, acl_entry_t ace, int *is_mask)
   Returns true on success, false otherwise.
 */
 
-static int ParseModePosixLinux(char *mode, acl_permset_t perms)
+static bool ParseModePosixLinux(char *mode, acl_permset_t perms)
 {
     int retv;
     int more_entries;
@@ -1137,7 +1142,7 @@ static int ParseModePosixLinux(char *mode, acl_permset_t perms)
 
 #else /* !HAVE_LIBACL */
 
-PromiseResult CheckPosixLinuxACL(EvalContext *ctx, ARG_UNUSED const char *file_path, ARG_UNUSED Acl acl, Attributes a, const Promise *pp)
+PromiseResult CheckPosixLinuxACL(EvalContext *ctx, ARG_UNUSED const char *file_path, ARG_UNUSED Acl acl, const Attributes *a, const Promise *pp)
 {
     cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a,
          "Posix ACLs are not supported on this Linux system - install the Posix acl library");
