@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -28,7 +28,7 @@ ENTERPRISE_FUNC_8ARG_DEFINE_STUB(PromiseResult, LogFileChange,
                                  ARG_UNUSED EvalContext *, ctx,
                                  ARG_UNUSED const char *, file,
                                  ARG_UNUSED int, change,
-                                 ARG_UNUSED Attributes, a,
+                                 ARG_UNUSED const Attributes *, attr,
                                  ARG_UNUSED const Promise *, pp,
                                  ARG_UNUSED CopyRegularFileFunction, CopyRegularFilePtr,
                                  ARG_UNUSED const char *, destination,
@@ -42,8 +42,8 @@ ENTERPRISE_VOID_FUNC_1ARG_DEFINE_STUB(void, Nova_TrackExecution, ARG_UNUSED cons
 {
 }
 
-ENTERPRISE_VOID_FUNC_2ARG_DEFINE_STUB(void, GenerateReports, 
-                                      ARG_UNUSED const GenericAgentConfig *, config, 
+ENTERPRISE_VOID_FUNC_2ARG_DEFINE_STUB(void, GenerateReports,
+                                      ARG_UNUSED const GenericAgentConfig *, config,
                                       ARG_UNUSED const EvalContext *, ctx)
 {
 }
