@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -32,15 +32,14 @@
 #include <generic_agent.h>
 
 #if defined(__MINGW32__)
-PromiseResult VerifyRegistryPromise(EvalContext *ctx, Attributes a, const Promise *pp);
+PromiseResult VerifyRegistryPromise(EvalContext *ctx, const Attributes *a, const Promise *pp);
 #endif
 
 typedef bool (*CopyRegularFileFunction)(EvalContext *ctx,
                                        const char *source,
                                        const char *dest,
-                                       struct stat sstat,
-                                       struct stat dstat,
-                                       Attributes attr,
+                                       const struct stat *sstat,
+                                       const Attributes *attr,
                                        const Promise *pp,
                                        CompressedArray **inode_cache,
                                        AgentConnection *conn,
@@ -50,7 +49,7 @@ ENTERPRISE_FUNC_8ARG_DECLARE(PromiseResult, LogFileChange,
                              EvalContext *, ctx,
                              const char *, file,
                              int, change,
-                             Attributes, a,
+                             const Attributes *, attr,
                              const Promise *, pp,
                              CopyRegularFileFunction, CopyRegularFilePtr,
                              const char *, destination, DeleteCompressedArrayFunction, DeleteCompressedArrayPtr);
@@ -61,4 +60,3 @@ ENTERPRISE_VOID_FUNC_2ARG_DECLARE(void, GenerateReports, const GenericAgentConfi
 ENTERPRISE_VOID_FUNC_2ARG_DECLARE(void, Nova_NoteAgentExecutionPerformance, const char *,
                                   input_file, struct timespec, start);
 #endif
-
