@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -83,6 +83,10 @@
 #include <package_module.h>
 #include <string_lib.h>
 #include <cfnet.h>
+#include <repair.h>
+#include <dbm_api.h>                    /* CheckDBRepairFlagFile() */
+#include <sys/types.h>                  /* checking umask on writing setxid log */
+#include <sys/stat.h>                   /* checking umask on writing setxid log */
 
 #include <mod_common.h>
 
@@ -102,6 +106,7 @@ extern int PR_NOTKEPT;
 static bool ALLCLASSESREPORT = false; /* GLOBAL_P */
 static bool ALWAYS_VALIDATE = false; /* GLOBAL_P */
 static bool CFPARANOID = false; /* GLOBAL_P */
+static bool PERFORM_DB_CHECK = false;
 
 static const Rlist *ACCESSLIST = NULL; /* GLOBAL_P */
 
@@ -142,7 +147,7 @@ static void FreeFixedStringArray(int size, char **array);
 static void CheckAgentAccess(const Rlist *list, const Policy *policy);
 static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericAgentConfig *config);
 static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, void *param);
-static int NewTypeContext(TypeSequence type);
+static void NewTypeContext(TypeSequence type);
 static void DeleteTypeContext(EvalContext *ctx, TypeSequence type);
 static PromiseResult ParallelFindAndVerifyFilesPromises(EvalContext *ctx, const Promise *pp);
 static bool VerifyBootstrap(void);
@@ -194,6 +199,8 @@ static const struct option OPTIONS[] =
     {"log-modules", required_argument, 0, 0},
     {"show-evaluated-classes", optional_argument, 0, 0 },
     {"show-evaluated-vars", optional_argument, 0, 0 },
+    {"skip-bootstrap-policy-run", no_argument, 0, 0 },
+    {"skip-db-check", optional_argument, 0, 0 },
     {NULL, 0, 0, '\0'}
 };
 
@@ -222,6 +229,8 @@ static const char *const HINTS[] =
     "Enable even more detailed debug logging for specific areas of the implementation. Use together with '-d'. Use --log-modules=help for a list of available modules",
     "Show *final* evaluated classes, including those defined in common bundles in policy. Optionally can take a regular expression.",
     "Show *final* evaluated variables, including those defined without dependency to user-defined classes in policy. Optionally can take a regular expression.",
+    "Do not run policy as the last step of the bootstrap process",
+    "Do not run database integrity checks and repairs at startup",
     NULL
 };
 
@@ -236,6 +245,11 @@ int main(int argc, char *argv[])
     struct timespec start = BeginMeasure();
 
     GenericAgentConfig *config = CheckOpts(argc, argv);
+    bool force_repair = CheckDBRepairFlagFile();
+    if (force_repair || PERFORM_DB_CHECK)
+    {
+        repair_lmdb_default(force_repair);
+    }
     EvalContext *ctx = EvalContextNew();
 
     // Enable only for cf-agent eval context.
@@ -255,12 +269,22 @@ int main(int argc, char *argv[])
         DoCleanupAndExit(EXIT_FAILURE);
     }
 
+    int ret = 0;
+
     GenericAgentPostLoadInit(ctx);
     ThisAgentInit();
+    ConnCache_Init();
 
     BeginAudit();
     KeepPromises(ctx, policy, config);
 
+    if (EvalAborted(ctx))
+    {
+        ret = EC_EVAL_ABORTED;
+    }
+
+    ConnCache_Destroy();
+
     if (ALLCLASSESREPORT)
     {
         AllClassesReport(ctx);
@@ -289,7 +313,6 @@ int main(int argc, char *argv[])
     }
 
     PolicyDestroy(policy); /* Can we safely do this earlier ? */
-    int ret = 0;
     if (config->agent_specific.agent.bootstrap_argument && !VerifyBootstrap())
     {
         PolicyServerRemoveFile(GetWorkDir());
@@ -314,6 +337,74 @@ int main(int argc, char *argv[])
 /* Level 1                                                         */
 /*******************************************************************/
 
+static void ConfigureBootstrap(GenericAgentConfig *config, const char *argument)
+{
+    assert(config != NULL);
+    if (!BootstrapAllowed())
+    {
+        Log(LOG_LEVEL_ERR, "Not enough privileges to bootstrap CFEngine");
+        DoCleanupAndExit(EXIT_FAILURE);
+    }
+
+    if(strcmp(optarg, ":avahi") == 0)
+    {
+        if(!HasAvahiSupport())
+        {
+            Log(LOG_LEVEL_ERR, "Avahi support is not built in, please see options to the configure script and rebuild CFEngine");
+            DoCleanupAndExit(EXIT_FAILURE);
+        }
+
+        int err = AutomaticBootstrap(config);
+        if (err < 0)
+        {
+            Log(LOG_LEVEL_ERR, "Automatic bootstrap failed, error code '%d'", err);
+            DoCleanupAndExit(EXIT_FAILURE);
+        }
+        return;
+    }
+
+    if(IsLoopbackAddress(argument))
+    {
+        Log(LOG_LEVEL_ERR, "Cannot bootstrap to a loopback address");
+        DoCleanupAndExit(EXIT_FAILURE);
+    }
+
+    // temporary assure that network functions are working
+    OpenNetwork();
+
+    config->agent_specific.agent.bootstrap_argument = xstrdup(argument);
+
+    char *host, *port;
+    ParseHostPort(optarg, &host, &port);
+
+    char ipaddr[CF_MAX_IP_LEN] = "";
+    if (Hostname2IPString(ipaddr, host,sizeof(ipaddr)) == -1)
+    {
+        Log(LOG_LEVEL_ERR,
+            "Could not resolve hostname '%s', unable to bootstrap",
+            host);
+        DoCleanupAndExit(EXIT_FAILURE);
+    }
+
+    CloseNetwork();
+
+    MINUSF = true;
+    config->ignore_locks = true;
+    GenericAgentConfigSetInputFile(config, GetInputDir(), "promises.cf");
+
+    config->agent_specific.agent.bootstrap_ip = xstrdup(ipaddr);
+    config->agent_specific.agent.bootstrap_host = xstrdup(host);
+
+    if (port == NULL)
+    {
+        config->agent_specific.agent.bootstrap_port = NULL;
+    }
+    else
+    {
+        config->agent_specific.agent.bootstrap_port = xstrdup(port);
+    }
+}
+
 static GenericAgentConfig *CheckOpts(int argc, char **argv)
 {
     extern char *optarg;
@@ -345,7 +436,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
 
         case 'w':
             Log(LOG_LEVEL_INFO, "Setting workdir to '%s'", optarg);
-            putenv(StringConcatenate(2, "CFENGINE_TEST_OVERRIDE_WORKDIR=", optarg));
+            setenv_wrapper("CFENGINE_TEST_OVERRIDE_WORKDIR", optarg, 1);
             break;
 
         case 'f':
@@ -368,69 +459,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
 
         case 'B':
             {
-                if (!BootstrapAllowed())
-                {
-                    Log(LOG_LEVEL_ERR, "Not enough privileges to bootstrap CFEngine");
-                    DoCleanupAndExit(EXIT_FAILURE);
-                }
-
-                if(strcmp(optarg, ":avahi") == 0)
-                {
-                    if(!HasAvahiSupport())
-                    {
-                        Log(LOG_LEVEL_ERR, "Avahi support is not built in, please see options to the configure script and rebuild CFEngine");
-                        DoCleanupAndExit(EXIT_FAILURE);
-                    }
-
-                    int err = AutomaticBootstrap(config);
-                    if (err < 0)
-                    {
-                        Log(LOG_LEVEL_ERR, "Automatic bootstrap failed, error code '%d'", err);
-                        DoCleanupAndExit(EXIT_FAILURE);
-                    }
-                    break;
-                }
-
-                if(IsLoopbackAddress(optarg))
-                {
-                    Log(LOG_LEVEL_ERR, "Cannot bootstrap to a loopback address");
-                    DoCleanupAndExit(EXIT_FAILURE);
-                }
-
-                // temporary assure that network functions are working
-                OpenNetwork();
-
-                config->agent_specific.agent.bootstrap_argument = xstrdup(optarg);
-
-                char *host, *port;
-                ParseHostPort(optarg, &host, &port);
-
-                char ipaddr[CF_MAX_IP_LEN] = "";
-                if (Hostname2IPString(ipaddr, host,sizeof(ipaddr)) == -1)
-                {
-                    Log(LOG_LEVEL_ERR,
-                        "Could not resolve hostname '%s', unable to bootstrap",
-                        host);
-                    DoCleanupAndExit(EXIT_FAILURE);
-                }
-
-                CloseNetwork();
-
-                MINUSF = true;
-                config->ignore_locks = true;
-                GenericAgentConfigSetInputFile(config, GetInputDir(), "promises.cf");
-
-                config->agent_specific.agent.bootstrap_ip = xstrdup(ipaddr);
-                config->agent_specific.agent.bootstrap_host = xstrdup(host);
-
-                if (port == NULL)
-                {
-                    config->agent_specific.agent.bootstrap_port = NULL;
-                }
-                else
-                {
-                    config->agent_specific.agent.bootstrap_port = xstrdup(port);
-                }
+                ConfigureBootstrap(config, optarg);
             }
             break;
 
@@ -447,8 +476,8 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
                 else
                 {
-                    StringSetJoin(config->heap_soft, defined_classes);
-                    free(defined_classes);
+                    StringSetJoin(config->heap_soft, defined_classes, xstrdup);
+                    StringSetDestroy(defined_classes);
                 }
             }
             break;
@@ -462,8 +491,8 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
                 else
                 {
-                    StringSetJoin(config->heap_negated, negated_classes);
-                    free(negated_classes);
+                    StringSetJoin(config->heap_negated, negated_classes, xstrdup);
+                    StringSetDestroy(negated_classes);
                 }
             }
             break;
@@ -496,7 +525,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
         case 'h':
             {
                 Writer *w = FileWriter(stdout);
-                WriterWriteHelp(w, "cf-agent", OPTIONS, HINTS, true, NULL);
+                WriterWriteHelp(w, "cf-agent", OPTIONS, HINTS, NULL, false, true);
                 FileWriterDetach(w);
             }
             DoCleanupAndExit(EXIT_SUCCESS);
@@ -508,6 +537,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                              CF_AGENT_SHORT_DESCRIPTION,
                              CF_AGENT_MANPAGE_LONG_DESCRIPTION,
                              OPTIONS, HINTS,
+                             NULL, false,
                              true);
                 FileWriterDetach(out);
                 DoCleanupAndExit(EXIT_SUCCESS);
@@ -563,8 +593,9 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
 
         /* long options only */
         case 0:
-
-            if (strcmp(OPTIONS[longopt_idx].name, "log-modules") == 0)
+        {
+            const char *const option_name = OPTIONS[longopt_idx].name;
+            if (StringSafeEqual(option_name, "log-modules"))
             {
                 bool ret = LogEnableModulesFromString(optarg);
                 if (!ret)
@@ -572,7 +603,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                     DoCleanupAndExit(EXIT_FAILURE);
                 }
             }
-            else if (strcmp(OPTIONS[longopt_idx].name, "show-evaluated-classes") == 0)
+            else if (StringSafeEqual(option_name, "show-evaluated-classes"))
             {
                 if (optarg == NULL)
                 {
@@ -580,7 +611,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
                 config->agent_specific.agent.show_evaluated_classes = xstrdup(optarg);
             }
-            else if (strcmp(OPTIONS[longopt_idx].name, "show-evaluated-vars") == 0)
+            else if (StringSafeEqual(option_name, "show-evaluated-vars"))
             {
                 if (optarg == NULL)
                 {
@@ -588,12 +619,38 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
                 config->agent_specific.agent.show_evaluated_variables = xstrdup(optarg);
             }
-    break;
-
+            else if (StringSafeEqual(option_name, "skip-bootstrap-policy-run"))
+            {
+                config->agent_specific.agent.bootstrap_trigger_policy = false;
+            }
+            else if (StringSafeEqual(option_name, "skip-db-check"))
+            {
+                if (optarg == NULL)
+                {
+                    PERFORM_DB_CHECK = false; // Skip (no arg), check = false
+                }
+                else if (StringSafeEqual_IgnoreCase(optarg, "yes"))
+                {
+                    PERFORM_DB_CHECK = false; // Skip = yes, check = false
+                }
+                else if (StringSafeEqual_IgnoreCase(optarg, "no"))
+                {
+                    PERFORM_DB_CHECK = true; // Skip = no, check = true
+                }
+                else
+                {
+                    Log(LOG_LEVEL_ERR,
+                        "Invalid argument for --skip-db-check(yes/no): '%s'",
+                        optarg);
+                    DoCleanupAndExit(EXIT_FAILURE);
+                }
+            }
+            break;
+        }
         default:
             {
                 Writer *w = FileWriter(stdout);
-                WriterWriteHelp(w, "cf-agent", OPTIONS, HINTS, true, NULL);
+                WriterWriteHelp(w, "cf-agent", OPTIONS, HINTS, NULL, false, true);
                 FileWriterDetach(w);
             }
             DoCleanupAndExit(EXIT_FAILURE);
@@ -733,7 +790,6 @@ static void FreeFixedStringArray(int size, char **array)
 
 static void ThisAgentInit(void)
 {
-    FILE *fp;
     char filename[CF_BUFSIZE];
 
 #ifdef HAVE_SETSID
@@ -750,9 +806,14 @@ static void ThisAgentInit(void)
 */
 
     snprintf(filename, CF_BUFSIZE, "%s/cfagent.%s.log", GetLogDir(), VSYSNAME.nodename);
+    ToLowerStrInplace(filename);
     MapName(filename);
 
-    if ((fp = fopen(filename, "a")) != NULL)
+    const mode_t current_umask = umask(0777);  // Gets and changes umask
+    umask(current_umask); // Restores umask
+    Log(LOG_LEVEL_DEBUG, "Current umask is %o", current_umask);
+    FILE *fp = safe_fopen(filename, "a");
+    if (fp != NULL)
     {
         fclose(fp);
     }
@@ -763,6 +824,11 @@ static void ThisAgentInit(void)
 static void KeepPromises(EvalContext *ctx, const Policy *policy, GenericAgentConfig *config)
 {
     KeepControlPromises(ctx, policy, config);
+    /* Check if 'abortclasses' aborted evaluation or not. */
+    if (EvalAborted(ctx))
+    {
+        return;
+    }
     KeepPromiseBundles(ctx, policy, config);
 }
 
@@ -926,13 +992,8 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
 
             if (strcmp(cp->lval, CFA_CONTROLBODY[AGENT_CONTROL_CHILDLIBPATH].lval) == 0)
             {
-                char output[CF_BUFSIZE];
-
-                snprintf(output, CF_BUFSIZE, "Setting LD_LIBRARY_PATH to '%s'", (const char *)value);
-                if (putenv(xstrdup(output)) == 0)
-                {
-                    Log(LOG_LEVEL_VERBOSE, "Setting '%s'", output);
-                }
+                Log(LOG_LEVEL_VERBOSE, "Setting 'LD_LIBRARY_PATH=%s'", (const char *)value);
+                setenv_wrapper("LD_LIBRARY_PATH", value, 1);
                 continue;
             }
 
@@ -1093,7 +1154,7 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
                 for (const Rlist *rp = value; rp != NULL; rp = rp->next)
                 {
                     assert(strchr(RlistScalarValue(rp), '=')); /* Valid for putenv() */
-                    if (putenv(xstrdup(RlistScalarValue(rp))) != 0)
+                    if (putenv_wrapper(RlistScalarValue(rp)) != 0)
                     {
                         Log(LOG_LEVEL_ERR, "Failed to set environment variable '%s'. (putenv: %s)",
                             RlistScalarValue(rp), GetErrorStr());
@@ -1284,6 +1345,10 @@ static void KeepPromiseBundles(EvalContext *ctx, const Policy *policy, GenericAg
             ScheduleAgentOperations(ctx, bp);
             EvalContextStackPopFrame(ctx);
             EndBundleBanner(bp);
+            if (EvalAborted(ctx))
+            {
+                break;
+            }
         }
         else
         {
@@ -1306,8 +1371,8 @@ static void AllClassesReport(const EvalContext *ctx)
     char context_report_file[CF_BUFSIZE];
     snprintf(context_report_file, CF_BUFSIZE, "%s%callclasses.txt", GetStateDir(), FILE_SEPARATOR);
 
-    FILE *fp = NULL;
-    if ((fp = fopen(context_report_file, "w")) == NULL)
+    FILE *fp = safe_fopen(context_report_file, "w");
+    if (fp == NULL)
     {
         Log(LOG_LEVEL_INFO, "Could not open allclasses cache file '%s' (fopen: %s)", context_report_file, GetErrorStr());
     }
@@ -1353,10 +1418,7 @@ PromiseResult ScheduleAgentOperations(EvalContext *ctx, const Bundle *bp)
                 continue;
             }
 
-            if (!NewTypeContext(type))
-            {
-                continue;
-            }
+            NewTypeContext(type);
 
             SpecialTypeBanner(type, pass);
             EvalContextStackPushPromiseTypeFrame(ctx, sp);
@@ -1370,7 +1432,7 @@ PromiseResult ScheduleAgentOperations(EvalContext *ctx, const Bundle *bp)
                 PromiseResult promise_result = ExpandPromise(ctx, pp, KeepAgentPromise, NULL);
                 result = PromiseResultUpdate(result, promise_result);
 
-                if (Abort(ctx))
+                if (EvalAborted(ctx) || BundleAbort(ctx))
                 {
                     DeleteTypeContext(ctx, type);
                     EvalContextStackPopFrame(ctx);
@@ -1763,7 +1825,7 @@ static void BannerStatus(PromiseResult status, char *type, char *name)
 /* Type context                                                      */
 /*********************************************************************/
 
-static int NewTypeContext(TypeSequence type)
+static void NewTypeContext(TypeSequence type)
 {
 // get maxconnections
 
@@ -1774,7 +1836,6 @@ static int NewTypeContext(TypeSequence type)
         break;
 
     case TYPE_SEQUENCE_FILES:
-        ConnCache_Init();
         break;
 
     case TYPE_SEQUENCE_PROCESSES:
@@ -1794,7 +1855,7 @@ static int NewTypeContext(TypeSequence type)
         break;
     }
 
-    return true;
+    return;
 }
 
 /*********************************************************************/
@@ -1808,7 +1869,6 @@ static void DeleteTypeContext(EvalContext *ctx, TypeSequence type)
         break;
 
     case TYPE_SEQUENCE_FILES:
-        ConnCache_Destroy();
         break;
 
     case TYPE_SEQUENCE_PROCESSES:
