@@ -1,17 +1,16 @@
 /*
+  Copyright 2019 Northern.tech AS
 
-   Copyright 2018 Northern.tech AS
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
-
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -31,18 +30,17 @@
 typedef struct
 {
     int connected;
-    int result;
+    bool result;
     int row;
     unsigned int maxcolumns;
     unsigned int maxrows;
     int column;
     char **rowdata;
-    char *blank;
     DatabaseType type;
     void *data;                 /* Generic pointer to RDBMS-specific data */
 } CfdbConn;
 
-int CfConnectDB(CfdbConn *cfdb, DatabaseType dbtype, char *remotehost, char *dbuser, char *passwd, char *db);
+void CfConnectDB(CfdbConn *cfdb, DatabaseType dbtype, char *remotehost, char *dbuser, char *passwd, char *db);
 void CfCloseDB(CfdbConn *cfdb);
 void CfVoidQueryDB(CfdbConn *cfdb, char *query);
 void CfNewQueryDB(CfdbConn *cfdb, char *query);
