@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -32,7 +32,7 @@
 
 /*******************************************************************/
 
-int FixCompressedArrayValue(int i, char *value, CompressedArray **start)
+bool FixCompressedArrayValue(int i, char *value, CompressedArray **start)
 {
     CompressedArray *ap;
 
@@ -74,7 +74,7 @@ void DeleteCompressedArray(CompressedArray *start)
 
 /*******************************************************************/
 
-int CompressedArrayElementExists(CompressedArray *start, int key)
+bool CompressedArrayElementExists(CompressedArray *start, int key)
 {
     CompressedArray *ap;
 
