@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -32,7 +32,6 @@
 #include <promises.h>
 #include <actuator.h>
 #include <eval_context.h>
-#include <files_hashes.h>
 #include <known_dirs.h>
 
 /*
@@ -111,7 +110,7 @@ static void DeleteHashValue(ChecksumValue *chk_val)
     free(chk_val);
 }
 
-static int ReadHash(CF_DB *dbp, HashMethod type, const char *name, unsigned char digest[EVP_MAX_MD_SIZE + 1])
+static bool ReadHash(CF_DB *dbp, HashMethod type, const char *name, unsigned char digest[EVP_MAX_MD_SIZE + 1])
 {
     char *key;
     int size;
@@ -530,7 +529,7 @@ bool FileChangesCheckAndUpdateHash(EvalContext *ctx,
     assert(attr != NULL);
     bool ret = FileChangesCheckAndUpdateHash_impl(filename, digest, type, attr->change.update, pp, result);
     // TODO: Move cfPS even further up the call stack.
-    cfPS(ctx, LOG_LEVEL_DEBUG, *result, pp, *attr, "Updating promise status for files changes promise");
+    cfPS(ctx, LOG_LEVEL_DEBUG, *result, pp, attr, "Updating promise status for files changes promise");
     return ret;
 }
 
@@ -638,7 +637,7 @@ void FileChangesCheckAndUpdateDirectory(const char *name, const Seq *file_set, c
     CloseDB(db);
 }
 
-void FileChangesCheckAndUpdateStats(const char *file, struct stat *sb, bool update, const Promise *pp)
+void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, bool update, const Promise *pp)
 {
     struct stat cmpsb;
     CF_DB *dbp;
@@ -786,7 +785,6 @@ static char FileStateToChar(FileState status)
 
 void FileChangesLogChange(const char *file, FileState status, char *msg, const Promise *pp)
 {
-    FILE *fp;
     char fname[CF_BUFSIZE];
     time_t now = time(NULL);
 
@@ -806,7 +804,7 @@ void FileChangesLogChange(const char *file, FileState status, char *msg, const P
     }
 #endif /* !__MINGW32__ */
 
-    fp = safe_fopen(fname, "a");
+    FILE *fp = safe_fopen(fname, "a");
     if (fp == NULL)
     {
         Log(LOG_LEVEL_ERR, "Could not write to the hash change log. (fopen: %s)", GetErrorStr());
@@ -817,6 +815,4 @@ void FileChangesLogChange(const char *file, FileState status, char *msg, const P
 
     fprintf(fp, "%lld,%s,%s,%c,%s\n", (long long) now, handle, file, FileStateToChar(status), msg);
     fclose(fp);
-
-    safe_chmod(fname, 0600);
 }
