@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -36,7 +36,7 @@
 
 /*****************************************************************************/
 
-EditContext *NewEditContext(char *filename, Attributes a)
+EditContext *NewEditContext(char *filename, const Attributes *a)
 {
     EditContext *ec;
 
@@ -49,21 +49,22 @@ EditContext *NewEditContext(char *filename, Attributes a)
     ec = xcalloc(1, sizeof(EditContext));
 
     ec->filename = filename;
+
     ec->new_line_mode = FileNewLineMode(filename);
 
-    if (a.haveeditline)
+    if (a->haveeditline)
     {
-        if (!LoadFileAsItemList(&(ec->file_start), filename, a.edits))
+        if (!LoadFileAsItemList(&(ec->file_start), filename, a->edits))
         {
             free(ec);
             return NULL;
         }
     }
 
-    if (a.haveeditxml)
+    if (a->haveeditxml)
     {
 #ifdef HAVE_LIBXML2
-        if (!LoadFileAsXmlDoc(&(ec->xmldoc), filename, a.edits))
+        if (!LoadFileAsXmlDoc(&(ec->xmldoc), filename, a->edits))
         {
             free(ec);
             return NULL;
@@ -75,7 +76,7 @@ EditContext *NewEditContext(char *filename, Attributes a)
 #endif
     }
 
-    if (a.edits.empty_before_use)
+    if (a->edits.empty_before_use)
     {
         Log(LOG_LEVEL_VERBOSE, "Build file model from a blank slate (emptying)");
         DeleteItemList(ec->file_start);
@@ -87,7 +88,7 @@ EditContext *NewEditContext(char *filename, Attributes a)
 
 /*****************************************************************************/
 
-void FinishEditContext(EvalContext *ctx, EditContext *ec, Attributes a, const Promise *pp,
+void FinishEditContext(EvalContext *ctx, EditContext *ec, const Attributes *a, const Promise *pp,
                        PromiseResult *result)
 {
     if (*result == PROMISE_RESULT_NOOP || *result == PROMISE_RESULT_CHANGE)
@@ -102,7 +103,7 @@ void FinishEditContext(EvalContext *ctx, EditContext *ec, Attributes a, const Pr
         goto end;
     }
 
-    if (DONTDO || (a.transaction.action == cfa_warn))
+    if (DONTDO || (a->transaction.action == cfa_warn))
     {
         if (ec &&
             !CompareToFile(ctx, ec->file_start, ec->filename, a, pp, result) &&
@@ -120,9 +121,15 @@ void FinishEditContext(EvalContext *ctx, EditContext *ec, Attributes a, const Pr
             *result = PROMISE_RESULT_NOOP;
         }
     }
+    else if (ec && (ec->num_rewrites > 0))
+    {
+        cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_CHANGE, pp, a,
+             "Edit file '%s'", ec->filename);
+        *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
+    }
     else if (ec && (ec->num_edits > 0))
     {
-        if (a.haveeditline || a.edit_template || a.edit_template_string)
+        if (a->haveeditline || a->edit_template || a->edit_template_string)
         {
             if (CompareToFile(ctx, ec->file_start, ec->filename, a, pp, result))
             {
@@ -143,10 +150,10 @@ void FinishEditContext(EvalContext *ctx, EditContext *ec, Attributes a, const Pr
             }
         }
 
-        if (a.haveeditxml)
+        if (a->haveeditxml)
         {
 #ifdef HAVE_LIBXML2
-            if (XmlCompareToFile(ec->xmldoc, ec->filename, a.edits))
+            if (XmlCompareToFile(ec->xmldoc, ec->filename, a->edits))
             {
                 if (ec)
                 {
@@ -195,7 +202,7 @@ end:
 #ifdef HAVE_LIBXML2
 /***************************************************************************/
 
-int LoadFileAsXmlDoc(xmlDocPtr *doc, const char *file, EditDefaults edits)
+bool LoadFileAsXmlDoc(xmlDocPtr *doc, const char *file, EditDefaults edits)
 {
     struct stat statbuf;
 
@@ -254,7 +261,7 @@ static bool SaveXmlCallback(const char *dest_filename, void *param,
 
 /*********************************************************************/
 
-bool SaveXmlDocAsFile(xmlDocPtr doc, const char *file, Attributes a, NewLineMode new_line_mode)
+bool SaveXmlDocAsFile(xmlDocPtr doc, const char *file, const Attributes *a, NewLineMode new_line_mode)
 {
     return SaveAsFile(&SaveXmlCallback, doc, file, a, new_line_mode);
 }
