@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -46,6 +46,7 @@ typedef struct
     char *filename;
     Item *file_start;
     int num_edits;
+    int num_rewrites;
 #ifdef HAVE_LIBXML2
     xmlDocPtr xmldoc;
 #endif
@@ -53,15 +54,15 @@ typedef struct
 } EditContext;
 
 // filename must not be freed until FinishEditContext.
-EditContext *NewEditContext(char *filename, Attributes a);
+EditContext *NewEditContext(char *filename, const Attributes *a);
 void FinishEditContext(EvalContext *ctx, EditContext *ec,
-                       Attributes a, const Promise *pp,
+                       const Attributes *a, const Promise *pp,
                        PromiseResult *result);
 
 #ifdef HAVE_LIBXML2
-int LoadFileAsXmlDoc(xmlDocPtr *doc, const char *file, EditDefaults ed);
+bool LoadFileAsXmlDoc(xmlDocPtr *doc, const char *file, EditDefaults ed);
 bool SaveXmlDocAsFile(xmlDocPtr doc, const char *file,
-                      Attributes a, NewLineMode new_line_mode);
+                      const Attributes *a, NewLineMode new_line_mode);
 #endif
 
 #endif
