@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -38,7 +38,7 @@
 #define CF_MAXLINKLEVEL 4
 
 #if !defined(__MINGW32__)
-static bool MakeLink(EvalContext *ctx, const char *from, const char *to, Attributes attr, const Promise *pp, PromiseResult *result);
+static bool MakeLink(EvalContext *ctx, const char *from, const char *to, const Attributes *attr, const Promise *pp, PromiseResult *result);
 #endif
 static char *AbsLinkPath(const char *from, const char *relto);
 
@@ -51,7 +51,7 @@ static bool EnforcePromise(enum cfopaction action)
 
 #ifdef __MINGW32__
 
-PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source, Attributes attr, const Promise *pp)
+PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source, const Attributes *attr, const Promise *pp)
 {
     Log(LOG_LEVEL_VERBOSE, "Windows does not support symbolic links (at VerifyLink())");
     return PROMISE_RESULT_FAIL;
@@ -59,7 +59,7 @@ PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source
 
 #else
 
-PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source, Attributes attr, const Promise *pp)
+PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source, const Attributes *attr, const Promise *pp)
 {
     char to[CF_BUFSIZE], linkbuf[CF_BUFSIZE], absto[CF_BUFSIZE];
     struct stat sb;
@@ -94,14 +94,14 @@ PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source
         source_file_exists = false;
     }
 
-    if ((!source_file_exists) && (attr.link.when_no_file != cfa_force) && (attr.link.when_no_file != cfa_delete))
+    if ((!source_file_exists) && (attr->link.when_no_file != cfa_force) && (attr->link.when_no_file != cfa_delete))
     {
         Log(LOG_LEVEL_INFO, "Source '%s' for linking is absent", absto);
         cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, attr, "Unable to create link '%s' -> '%s', no source", destination, to);
         return PROMISE_RESULT_FAIL;
     }
 
-    if ((!source_file_exists) && (attr.link.when_no_file == cfa_delete))
+    if ((!source_file_exists) && (attr->link.when_no_file == cfa_delete))
     {
         PromiseResult result = PROMISE_RESULT_CHANGE;
         KillGhostLink(ctx, destination, attr, pp, &result);
@@ -112,16 +112,16 @@ PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source
 
     if (readlink(destination, linkbuf, CF_BUFSIZE - 1) == -1)
     {
-        if (!EnforcePromise(attr.transaction.action))
+        if (!EnforcePromise(attr->transaction.action))
         {
             Log(LOG_LEVEL_WARNING, "Link '%s' should be created", destination);
             return PROMISE_RESULT_WARN;
         }
 
-        if (!MakeParentDirectory2(destination, attr.move_obstructions, EnforcePromise(attr.transaction.action)))
+        if (!MakeParentDirectory2(destination, attr->move_obstructions, EnforcePromise(attr->transaction.action)))
         {
             cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, attr, "Unable to create parent directory of link '%s' -> '%s' (enforce %d)",
-                 destination, to, EnforcePromise(attr.transaction.action));
+                 destination, to, EnforcePromise(attr->transaction.action));
             return PROMISE_RESULT_FAIL;
         }
         else
@@ -147,7 +147,7 @@ PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source
     {
         int ok = false;
 
-        if ((attr.link.link_type == FILE_LINK_TYPE_SYMLINK) && (strcmp(linkbuf, to) != 0) && (strcmp(linkbuf, source) != 0))
+        if ((attr->link.link_type == FILE_LINK_TYPE_SYMLINK) && (strcmp(linkbuf, to) != 0) && (strcmp(linkbuf, source) != 0))
         {
             ok = true;
         }
@@ -158,9 +158,9 @@ PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source
 
         if (ok)
         {
-            if (attr.move_obstructions)
+            if (attr->move_obstructions)
             {
-                if (EnforcePromise(attr.transaction.action))
+                if (EnforcePromise(attr->transaction.action))
                 {
                     cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_CHANGE, pp, attr, "Overriding incorrect link '%s'", destination);
                     PromiseResult result = PROMISE_RESULT_CHANGE;
@@ -199,7 +199,7 @@ PromiseResult VerifyLink(EvalContext *ctx, char *destination, const char *source
 
 /*****************************************************************************/
 
-PromiseResult VerifyAbsoluteLink(EvalContext *ctx, char *destination, const char *source, Attributes attr, const Promise *pp)
+PromiseResult VerifyAbsoluteLink(EvalContext *ctx, char *destination, const char *source, const Attributes *attr, const Promise *pp)
 {
     char absto[CF_BUFSIZE];
     char expand[CF_BUFSIZE];
@@ -221,7 +221,7 @@ PromiseResult VerifyAbsoluteLink(EvalContext *ctx, char *destination, const char
 
     expand[0] = '\0';
 
-    if (attr.link.when_no_file == cfa_force)
+    if (attr->link.when_no_file == cfa_force)
     {
         if (!ExpandLinks(expand, absto, 0))     /* begin at level 1 and beam out at 15 */
         {
@@ -246,7 +246,7 @@ PromiseResult VerifyAbsoluteLink(EvalContext *ctx, char *destination, const char
 
 /*****************************************************************************/
 
-PromiseResult VerifyRelativeLink(EvalContext *ctx, char *destination, const char *source, Attributes attr, const Promise *pp)
+PromiseResult VerifyRelativeLink(EvalContext *ctx, char *destination, const char *source, const Attributes *attr, const Promise *pp)
 {
     char *sp, *commonto, *commonfrom;
     char buff[CF_BUFSIZE], linkto[CF_BUFSIZE];
@@ -328,7 +328,7 @@ PromiseResult VerifyRelativeLink(EvalContext *ctx, char *destination, const char
 
 /*****************************************************************************/
 
-PromiseResult VerifyHardLink(EvalContext *ctx, char *destination, const char *source, Attributes attr, const Promise *pp)
+PromiseResult VerifyHardLink(EvalContext *ctx, char *destination, const char *source, const Attributes *attr, const Promise *pp)
 {
     char to[CF_BUFSIZE], absto[CF_BUFSIZE];
     struct stat ssb, dsb;
@@ -402,7 +402,7 @@ PromiseResult VerifyHardLink(EvalContext *ctx, char *destination, const char *so
 
     Log(LOG_LEVEL_INFO, "'%s' does not appear to be a hard link to '%s'", destination, to);
 
-    if (!EnforcePromise(attr.transaction.action))
+    if (!EnforcePromise(attr->transaction.action))
     {
         Log(LOG_LEVEL_WARNING, "Hard link '%s' -> '%s' should be created", destination, to);
         return PROMISE_RESULT_WARN;
@@ -435,7 +435,7 @@ PromiseResult VerifyHardLink(EvalContext *ctx, char *destination, const char *so
 
 #ifdef __MINGW32__
 
-bool KillGhostLink(EvalContext *ctx, const char *name, Attributes attr, const Promise *pp, PromiseResult *result)
+bool KillGhostLink(EvalContext *ctx, const char *name, const Attributes *attr, const Promise *pp, PromiseResult *result)
 {
     Log(LOG_LEVEL_VERBOSE, "Windows does not support symbolic links (at KillGhostLink())");
     cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, attr, "Windows does not support killing link '%s'", name);
@@ -445,7 +445,7 @@ bool KillGhostLink(EvalContext *ctx, const char *name, Attributes attr, const Pr
 
 #else                           /* !__MINGW32__ */
 
-bool KillGhostLink(EvalContext *ctx, const char *name, Attributes attr, const Promise *pp,
+bool KillGhostLink(EvalContext *ctx, const char *name, const Attributes *attr, const Promise *pp,
                    PromiseResult *result)
 {
     char linkbuf[CF_BUFSIZE], tmp[CF_BUFSIZE];
@@ -476,7 +476,7 @@ bool KillGhostLink(EvalContext *ctx, const char *name, Attributes attr, const Pr
 
     if (stat(tmp, &statbuf) == -1)    /* link points nowhere */
     {
-        if ((attr.link.when_no_file == cfa_delete) || (attr.recursion.rmdeadlinks))
+        if ((attr->link.when_no_file == cfa_delete) || (attr->recursion.rmdeadlinks))
         {
             Log(LOG_LEVEL_VERBOSE, "'%s' is a link which points to '%s', but that file doesn't seem to exist", name,
                   linkbuf);
@@ -499,10 +499,10 @@ bool KillGhostLink(EvalContext *ctx, const char *name, Attributes attr, const Pr
 /*****************************************************************************/
 
 #if !defined(__MINGW32__)
-static bool MakeLink(EvalContext *ctx, const char *from, const char *to, Attributes attr, const Promise *pp,
+static bool MakeLink(EvalContext *ctx, const char *from, const char *to, const Attributes *attr, const Promise *pp,
                      PromiseResult *result)
 {
-    if (DONTDO || (attr.transaction.action == cfa_warn))
+    if (DONTDO || (attr->transaction.action == cfa_warn))
     {
         Log(LOG_LEVEL_WARNING, "Need to link files '%s' -> '%s'", from, to);
         return false;
@@ -530,7 +530,7 @@ static bool MakeLink(EvalContext *ctx, const char *from, const char *to, Attribu
 
 #ifdef __MINGW32__
 
-bool MakeHardLink(EvalContext *ctx, const char *from, const char *to, Attributes attr, const Promise *pp,
+bool MakeHardLink(EvalContext *ctx, const char *from, const char *to, const Attributes *attr, const Promise *pp,
                   PromiseResult *result)
 {                               // TODO: Implement ?
     Log(LOG_LEVEL_VERBOSE, "Hard links are not yet supported on Windows");
@@ -541,7 +541,7 @@ bool MakeHardLink(EvalContext *ctx, const char *from, const char *to, Attributes
 
 #else                           /* !__MINGW32__ */
 
-bool MakeHardLink(EvalContext *ctx, const char *from, const char *to, Attributes attr, const Promise *pp,
+bool MakeHardLink(EvalContext *ctx, const char *from, const char *to, const Attributes *attr, const Promise *pp,
                   PromiseResult *result)
 {
     if (DONTDO)
@@ -576,7 +576,7 @@ bool MakeHardLink(EvalContext *ctx, const char *from, const char *to, Attributes
 
 #ifdef __MINGW32__
 
-int ExpandLinks(char *dest, const char *from, int level)
+bool ExpandLinks(char *dest, const char *from, int level)
 {
     Log(LOG_LEVEL_ERR, "Windows does not support symbolic links (at ExpandLinks(%s,%s))", dest, from);
     return false;
@@ -584,7 +584,7 @@ int ExpandLinks(char *dest, const char *from, int level)
 
 #else                           /* !__MINGW32__ */
 
-int ExpandLinks(char *dest, const char *from, int level)
+bool ExpandLinks(char *dest, const char *from, int level)
 {
     char buff[CF_BUFSIZE];
     char node[CF_MAXLINKSIZE];
@@ -624,6 +624,7 @@ int ExpandLinks(char *dest, const char *from, int level)
 
         if (strcmp(node, "..") == 0)
         {
+            strcat(dest, "/..");
             continue;
         }
         else
