@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -30,6 +30,7 @@
 #include <files_copy.h>
 #include <item_lib.h>
 #include <mutex.h>
+#include <global_mutex.h>
 #include <policy.h>
 #include <string_lib.h>                                       /* PathAppend */
 
@@ -55,18 +56,18 @@ void SetRepositoryChar(char c)
 
 /*********************************************************************/
 
-bool GetRepositoryPath(ARG_UNUSED const char *file, Attributes attr, char *destination)
+bool GetRepositoryPath(ARG_UNUSED const char *file, const Attributes *attr, char *destination)
 {
-    if ((attr.repository == NULL) && (VREPOSITORY == NULL))
+    if ((attr->repository == NULL) && (VREPOSITORY == NULL))
     {
         return false;
     }
 
     size_t repopathlen;
 
-    if (attr.repository != NULL)
+    if (attr->repository != NULL)
     {
-        repopathlen = strlcpy(destination, attr.repository, CF_BUFSIZE);
+        repopathlen = strlcpy(destination, attr->repository, CF_BUFSIZE);
     }
     else
     {
@@ -84,7 +85,7 @@ bool GetRepositoryPath(ARG_UNUSED const char *file, Attributes attr, char *desti
 
 /*********************************************************************/
 
-int ArchiveToRepository(const char *file, Attributes attr)
+bool ArchiveToRepository(const char *file, const Attributes *attr)
  /* Returns true if the file was backup up and false if not */
 {
     char destination[CF_BUFSIZE];
@@ -95,7 +96,7 @@ int ArchiveToRepository(const char *file, Attributes attr)
         return false;
     }
 
-    if (attr.copy.backup == BACKUP_OPTION_NO_BACKUP)
+    if (attr->copy.backup == BACKUP_OPTION_NO_BACKUP)
     {
         return true;
     }
@@ -122,8 +123,13 @@ int ArchiveToRepository(const char *file, Attributes attr)
         return false;
     }
 
-    if (!MakeParentDirectory(destination, attr.move_obstructions))
+    if (!MakeParentDirectory(destination, attr->move_obstructions))
     {
+        // Could not create parent directory, assume this is okay,
+        // verbose logging in MakeParentDirectory()
+        Log(LOG_LEVEL_DEBUG,
+            "Could not create parent directory '%s'",
+            destination);
     }
 
     if (stat(file, &sb) == -1)
