@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -33,6 +33,7 @@
 #include <conversion.h>
 #include <match_scope.h>
 #include <string_lib.h>
+#include <string_sequence.h>
 #include <systype.h>
 #include <pipes.h>
 #include <nfs.h>
@@ -47,8 +48,10 @@
 static int FSTAB_EDITS = 0; /* GLOBAL_X */
 static Item *FSTABLIST = NULL; /* GLOBAL_X */
 
+static void GetHostAndSource(const char *buf, char *host, char *source);
+
 static void AugmentMountInfo(Seq *list, char *host, char *source, char *mounton, char *options);
-static int MatchFSInFstab(char *match);
+static bool MatchFSInFstab(char *match);
 static void DeleteThisItem(Item **liststart, Item *entry);
 
 static const char *const VMOUNTCOMM[] =
@@ -126,14 +129,61 @@ static const char *const VMOUNTOPTS[] =
     [PLATFORM_CONTEXT_ANDROID] = "defaults",         /* android */
 };
 
+static void GetHostAndSource(const char *buf, char *host, char *source)
+/* Extracts from buffer host & source of nfs, cifs and panfs network systems */
+{
+    assert(StringStartsWith(buf, "panfs://")
+           || StringStartsWith(buf, "//")
+           || strchr(buf, ':') != NULL);
+    int index = 0; // Index for buf and host
+    int source_index = 0;
+    size_t slashes = 0;
+
+    // panfs & cifs
+    if (StringStartsWith(buf, "panfs://") || StringStartsWith(buf, "//"))
+    {
+        // copy host
+        while (slashes != 3)
+        {
+            // TODO: Add example of what this string looks like
+            if (buf[index] == '/' || buf[index] == '\\')
+            {
+                slashes++;
+            }
+            host[index] = buf[index];
+            index++;
+        }
+        host[--index] = '\0';
+    }
+    // nfs
+    else
+    {
+        while (buf[index] != ':')
+        {
+            host[index] = buf[index];
+            index++;
+        }
+        host[index++] = '\0';
+    }
+
+    // copy source
+    while ((buf[index] != '\0') && (buf[index] != ' '))
+    {
+        source[source_index++] = buf[index++];
+    }
+    source[source_index] = '\0';
+}
+
 bool LoadMountInfo(Seq *list)
 /* This is, in fact, the most portable way to read the mount info! */
 /* Depressing, isn't it? */
 {
     FILE *pp;
     char buf1[CF_BUFSIZE], buf2[CF_BUFSIZE], buf3[CF_BUFSIZE];
-    int i, nfs = false;
+    int i;
+    bool nfs = false, panfs = false, cifs = false;
 
+    // get mount command without parameters
     for (i = 0; VMOUNTCOMM[VSYSTEMHARDCLASS][i] != ' '; i++)
     {
         buf1[i] = VMOUNTCOMM[VSYSTEMHARDCLASS][i];
@@ -173,10 +223,22 @@ bool LoadMountInfo(Seq *list)
             }
         }
 
-        if (strstr(vbuff, "nfs"))
+        Seq *words = SeqStringFromString(vbuff, ' ');
+
+        if (SeqStringContains(words, "panfs"))
+        {
+            panfs = true;
+        }
+        else if (SeqStringContains(words, "nfs"))
         {
             nfs = true;
         }
+        else if (SeqStringContains(words, "cifs"))
+        {
+            cifs = true;
+        }
+
+        SeqDestroy(words);
 
         // security note: buff is CF_BUFSIZE, so that is the max that can be written to buf1, buf2 or buf3
         sscanf(vbuff, "%s%s%s", buf1, buf2, buf3);
@@ -223,20 +285,27 @@ bool LoadMountInfo(Seq *list)
 
 
 #if defined(__sun) || defined(__hpux)
-        if (IsAbsoluteFileName(buf3))
+        if (IsAbsoluteFileName(buf3) && !cifs)
         {
             strlcpy(host, "localhost", sizeof(host));
             strlcpy(mounton, buf1, sizeof(mounton));
         }
         else
         {
-            sscanf(buf1, "%255[^:]:%1023s", host, source);
+            if (cifs || panfs)
+            {
+                GetHostAndSource(buf1, host, source);
+            }
+            else
+            {
+                sscanf(buf1, "%255[^:]:%1023s", host, source);
+            }
             strlcpy(mounton, buf1, sizeof(mounton));
         }
 #elif defined(_AIX)
         /* skip header */
 
-        if (IsAbsoluteFileName(buf1))
+        if (IsAbsoluteFileName(buf1) && !cifs)
         {
             strlcpy(host, "localhost", sizeof(host));
             strlcpy(mounton, buf2, sizeof(mounton));
@@ -253,24 +322,42 @@ bool LoadMountInfo(Seq *list)
 #elif defined(sco) || defined(__SCO_DS)
         Log(LOG_LEVEL_ERR, "Don't understand SCO mount format, no data");
 #else
-        if (IsAbsoluteFileName(buf1))
+        if (IsAbsoluteFileName(buf1) && !cifs)
         {
+            Log(LOG_LEVEL_VERBOSE,
+                "'%s' is a local absolute file name, setting host to localhost",
+                buf1);
             strlcpy(host, "localhost", sizeof(host));
             strlcpy(mounton, buf3, sizeof(mounton));
         }
         else
         {
-            sscanf(buf1, "%255[^:]:%1023s", host, source);
+            if (cifs || panfs)
+            {
+                GetHostAndSource(buf1, host, source);
+            }
+            else
+            {
+                sscanf(buf1, "%255[^:]:%1023s", host, source);
+            }
             strlcpy(mounton, buf3, sizeof(mounton));
         }
 #endif
 
         Log(LOG_LEVEL_DEBUG, "LoadMountInfo: host '%s', source '%s', mounton '%s'", host, source, mounton);
 
-        if (nfs)
+        if (panfs)
+        {
+            AugmentMountInfo(list, host, source, mounton, "panfs");
+        }
+        else if (nfs)
         {
             AugmentMountInfo(list, host, source, mounton, "nfs");
         }
+        else if (cifs)
+        {
+            AugmentMountInfo(list, host, source, mounton, "cifs");
+        }
         else
         {
             AugmentMountInfo(list, host, source, mounton, NULL);
@@ -347,15 +434,17 @@ void DeleteMountInfo(Seq *list)
 
 /*******************************************************************/
 
-int VerifyInFstab(EvalContext *ctx, char *name, Attributes a, const Promise *pp, PromiseResult *result)
+int VerifyInFstab(EvalContext *ctx, char *name, const Attributes *a, const Promise *pp, PromiseResult *result)
 /* Ensure filesystem IS in fstab, and return no of changes */
 {
+    // FIXME: This ALWAYS returns 0 (!)
+    assert(a != NULL);
     char fstab[CF_BUFSIZE];
     char *host, *rmountpt, *mountpt, *fstype, *opts;
 
     if (!FSTABLIST)
     {
-        if (!LoadFileAsItemList(&FSTABLIST, VFSTAB[VSYSTEMHARDCLASS], a.edits))
+        if (!LoadFileAsItemList(&FSTABLIST, VFSTAB[VSYSTEMHARDCLASS], a->edits))
         {
             Log(LOG_LEVEL_ERR, "Couldn't open '%s'", VFSTAB[VSYSTEMHARDCLASS]);
             return false;
@@ -366,19 +455,19 @@ int VerifyInFstab(EvalContext *ctx, char *name, Attributes a, const Promise *pp,
         }
     }
 
-    if (a.mount.mount_options)
+    if (a->mount.mount_options)
     {
-        opts = Rlist2String(a.mount.mount_options, ",");
+        opts = Rlist2String(a->mount.mount_options, ",");
     }
     else
     {
         opts = xstrdup(VMOUNTOPTS[VSYSTEMHARDCLASS]);
     }
 
-    host = a.mount.mount_server;
-    rmountpt = a.mount.mount_source;
+    host = a->mount.mount_server;
+    rmountpt = a->mount.mount_source;
     mountpt = name;
-    fstype = a.mount.mount_type;
+    fstype = a->mount.mount_type;
 
 #if defined(__QNX__) || defined(__QNXNTO__)
     snprintf(fstab, CF_BUFSIZE, "%s:%s \t %s %s\t%s 0 0", host, rmountpt, mountpt, fstype, opts);
@@ -422,16 +511,17 @@ int VerifyInFstab(EvalContext *ctx, char *name, Attributes a, const Promise *pp,
 
 /*******************************************************************/
 
-int VerifyNotInFstab(EvalContext *ctx, char *name, Attributes a, const Promise *pp, PromiseResult *result)
+int VerifyNotInFstab(EvalContext *ctx, char *name, const Attributes *a, const Promise *pp, PromiseResult *result)
 /* Ensure filesystem is NOT in fstab, and return no of changes */
 {
+    // FIXME: This ALWAYS returns 0 (!)
     char regex[CF_BUFSIZE];
-    char *host, *mountpt, *opts;
+    char *host, *mountpt;
     Item *ip;
 
     if (!FSTABLIST)
     {
-        if (!LoadFileAsItemList(&FSTABLIST, VFSTAB[VSYSTEMHARDCLASS], a.edits))
+        if (!LoadFileAsItemList(&FSTABLIST, VFSTAB[VSYSTEMHARDCLASS], a->edits))
         {
             Log(LOG_LEVEL_ERR, "Couldn't open '%s'", VFSTAB[VSYSTEMHARDCLASS]);
             return false;
@@ -442,21 +532,12 @@ int VerifyNotInFstab(EvalContext *ctx, char *name, Attributes a, const Promise *
         }
     }
 
-    if (a.mount.mount_options)
-    {
-        opts = Rlist2String(a.mount.mount_options, ",");
-    }
-    else
-    {
-        opts = xstrdup(VMOUNTOPTS[VSYSTEMHARDCLASS]);
-    }
-
-    host = a.mount.mount_server;
+    host = a->mount.mount_server;
     mountpt = name;
 
     if (MatchFSInFstab(mountpt))
     {
-        if (a.mount.editfstab)
+        if (a->mount.editfstab)
         {
 #if defined(_AIX)
             FILE *pfp;
@@ -533,30 +614,27 @@ int VerifyNotInFstab(EvalContext *ctx, char *name, Attributes a, const Promise *
         }
     }
 
-    if (a.mount.mount_options)
-    {
-        free(opts);
-    }
-
     return 0;
 }
 
 /*******************************************************************/
 
-PromiseResult VerifyMount(EvalContext *ctx, char *name, Attributes a, const Promise *pp)
+PromiseResult VerifyMount(EvalContext *ctx, char *name, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
+
     char comm[CF_BUFSIZE];
     FILE *pfp;
     char *host, *rmountpt, *mountpt, *opts=NULL;
 
-    host = a.mount.mount_server;
-    rmountpt = a.mount.mount_source;
+    host = a->mount.mount_server;
+    rmountpt = a->mount.mount_source;
     mountpt = name;
 
     /* Check for options required for this mount - i.e., -o ro,rsize, etc. */
-    if (a.mount.mount_options)
+    if (a->mount.mount_options)
     {
-        opts = Rlist2String(a.mount.mount_options, ",");
+        opts = Rlist2String(a->mount.mount_options, ",");
     }
     else
     {
@@ -566,7 +644,18 @@ PromiseResult VerifyMount(EvalContext *ctx, char *name, Attributes a, const Prom
     PromiseResult result = PROMISE_RESULT_NOOP;
     if (!DONTDO)
     {
-        snprintf(comm, CF_BUFSIZE, "%s -o %s %s:%s %s", CommandArg0(VMOUNTCOMM[VSYSTEMHARDCLASS]), opts, host, rmountpt, mountpt);
+        if (StringSafeEqual(a->mount.mount_type, "panfs"))
+        {
+            snprintf(comm, CF_BUFSIZE, "%s -t panfs -o %s %s%s %s", CommandArg0(VMOUNTCOMM[VSYSTEMHARDCLASS]), opts, host, rmountpt, mountpt);
+        }
+        else if (StringSafeEqual(a->mount.mount_type, "cifs"))
+        {
+            snprintf(comm, CF_BUFSIZE, "%s -t cifs -o %s %s%s %s", CommandArg0(VMOUNTCOMM[VSYSTEMHARDCLASS]), opts, host, rmountpt, mountpt);
+        }
+        else
+        {
+            snprintf(comm, CF_BUFSIZE, "%s -o %s %s:%s %s", CommandArg0(VMOUNTCOMM[VSYSTEMHARDCLASS]), opts, host, rmountpt, mountpt);
+        }
 
         if ((pfp = cf_popen(comm, "r", true)) == NULL)
         {
@@ -613,7 +702,7 @@ PromiseResult VerifyMount(EvalContext *ctx, char *name, Attributes a, const Prom
 
 /*******************************************************************/
 
-PromiseResult VerifyUnmount(EvalContext *ctx, char *name, Attributes a, const Promise *pp)
+PromiseResult VerifyUnmount(EvalContext *ctx, char *name, const Attributes *a, const Promise *pp)
 {
     char comm[CF_BUFSIZE];
     FILE *pfp;
@@ -664,7 +753,7 @@ PromiseResult VerifyUnmount(EvalContext *ctx, char *name, Attributes a, const Pr
 
 /*******************************************************************/
 
-static int MatchFSInFstab(char *match)
+static bool MatchFSInFstab(char *match)
 {
     Item *ip;
     const char delimit[]=" \t\r\n\v\f";
