@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -24,12 +24,15 @@
 
 #include <package_module.h>
 #include <pipes.h>
+#include <exec_tools.h>
 #include <signals.h>
 #include <buffer.h>
 #include <string_lib.h>
+#include <path.h>
 #include <actuator.h>
 #include <file_lib.h>
 #include <known_dirs.h>
+#include <string_sequence.h>
 #include <locks.h>
 #include <rlist.h>
 #include <policy.h>
@@ -41,15 +44,22 @@
 static bool UpdateSinglePackageModuleCache(EvalContext *ctx,
                                     const PackageModuleWrapper *module_wrapper,
                                     UpdateType type, bool force_update);
-static char *GetPackageModuleRealPath(const char *package_manager_name);
+static void GetPackageModuleExecInfo(const PackageModuleBody *package_module, char **exec_path,
+                                     char **script_path, char **script_path_quoted, char **script_exec_opts);
 static int NegotiateSupportedAPIVersion(PackageModuleWrapper *wrapper);
 
 
 void DeletePackageModuleWrapper(PackageModuleWrapper *wrapper)
 {
-    free(wrapper->path);
-    free(wrapper->name);
-    free(wrapper);
+    if (wrapper != NULL)
+    {
+        free(wrapper->path);
+        free(wrapper->script_path);
+        free(wrapper->script_path_quoted);
+        free(wrapper->script_exec_opts);
+        free(wrapper->name);
+        free(wrapper);
+    }
 }
 
 PackageModuleWrapper *NewPackageModuleWrapper(PackageModuleBody *package_module)
@@ -58,17 +68,53 @@ PackageModuleWrapper *NewPackageModuleWrapper(PackageModuleBody *package_module)
 
     PackageModuleWrapper *wrapper = xmalloc(sizeof(PackageModuleWrapper));
 
-    wrapper->path = GetPackageModuleRealPath(package_module->name);
+    GetPackageModuleExecInfo(package_module, &(wrapper->path),
+                             &(wrapper->script_path), &(wrapper->script_path_quoted),
+                             &(wrapper->script_exec_opts));
     wrapper->name = SafeStringDuplicate(package_module->name);
     wrapper->package_module = package_module;
 
-    /* Check if file exists */
-    if (!wrapper->path || (access(wrapper->path, X_OK) != 0))
+    if (wrapper->path == NULL)
     {
+        Log(LOG_LEVEL_ERR, "No executable for the package module '%s'", package_module->name);
+        DeletePackageModuleWrapper(wrapper);
+        return NULL;
+    }
+
+    /* Check if the given files exist and have the required permissions. */
+    if (wrapper->script_path != NULL)
+    {
+        /* we were given a path to a script to execute with the given
+         * interpreter */
+        const char *interpreter_path = wrapper->path;
+        const char *script_path = wrapper->script_path;
+
+        if (access(interpreter_path, X_OK) != 0)
+        {
+            Log(LOG_LEVEL_ERR,
+                "can not find package wrapper interpreter in provided location '%s' "
+                "or access to file is restricted: %s",
+                wrapper->path, strerror(errno));
+            DeletePackageModuleWrapper(wrapper);
+            return NULL;
+        }
+        if (access(script_path, R_OK) != 0)
+        {
+            Log(LOG_LEVEL_ERR,
+                "can not find the package wrapper script in provided location '%s' "
+                "or access to file is restricted: %s",
+                wrapper->script_path, strerror(errno));
+            DeletePackageModuleWrapper(wrapper);
+            return NULL;
+        }
+    }
+    else if (access(wrapper->path, X_OK) != 0)
+    {
+        /* no script path specified --> the given path has to be executable */
         Log(LOG_LEVEL_ERR,
             "can not find package wrapper in provided location '%s' "
-            "or access to file is restricted: %d",
-            wrapper->path, errno);
+            "or access to file is restricted: %s",
+            wrapper->path, strerror(errno));
         DeletePackageModuleWrapper(wrapper);
         return NULL;
     }
@@ -91,6 +137,31 @@ PackageModuleWrapper *NewPackageModuleWrapper(PackageModuleBody *package_module)
     return wrapper;
 }
 
+static int PackageWrapperCommunicate(const PackageModuleWrapper *wrapper, const char *args,
+                                     const char *request, Rlist **response)
+{
+    char *all_args = NULL;
+    if (wrapper->script_path != NULL)
+    {
+        if (wrapper->script_exec_opts == NULL)
+        {
+            all_args = StringConcatenate(3, wrapper->script_path_quoted, " ", args);
+            args = all_args;
+        }
+        else
+        {
+            all_args = StringConcatenate(5, wrapper->script_exec_opts, " ",
+                                         wrapper->script_path_quoted, " ", args);
+            args = all_args;
+        }
+    }
+    int ret = PipeReadWriteData(wrapper->path, args, request, response,
+                                PACKAGE_PROMISE_SCRIPT_TIMEOUT_SEC,
+                                PACKAGE_PROMISE_TERMINATION_CHECK_SEC);
+    free(all_args);
+
+    return ret;
+}
 
 void UpdatePackagesCache(EvalContext *ctx, bool force_update)
 {
@@ -153,9 +224,7 @@ PackagePromiseGlobalLock AcquireGlobalPackagePromiseLock(EvalContext *ctx)
 
     package_promise_global_lock =
             AcquireLock(ctx, GLOBAL_PACKAGE_PROMISE_LOCK_NAME, VUQNAME, CFSTARTTIME,
-                        (TransactionContext) {.ifelapsed = 0,
-                                              .expireafter = VEXPIREAFTER},
-                        &pp, false);
+                        0, VEXPIREAFTER, &pp, false);
 
     return (PackagePromiseGlobalLock) {.g_lock = package_promise_global_lock,
                                        .lock_ctx = ctx};
@@ -324,10 +393,8 @@ static int NegotiateSupportedAPIVersion(PackageModuleWrapper *wrapper)
     int api_version = -1;
 
     Rlist *response = NULL;
-    if (PipeReadWriteData(wrapper->path, "supports-api-version", "",
-                          &response,
-                          PACKAGE_PROMISE_SCRIPT_TIMEOUT_SEC,
-                          PACKAGE_PROMISE_TERMINATION_CHECK_SEC) != 0)
+    if (PackageWrapperCommunicate(wrapper, "supports-api-version", "",
+                                  &response) != 0)
     {
         Log(LOG_LEVEL_INFO,
             "Error occurred while getting supported API version.");
@@ -370,10 +437,7 @@ PackageInfo *GetPackageData(const char *name, const char *version,
     free(arch);
 
     Rlist *response = NULL;
-    if (PipeReadWriteData(wrapper->path, "get-package-data", request,
-                          &response,
-                          PACKAGE_PROMISE_SCRIPT_TIMEOUT_SEC,
-                          PACKAGE_PROMISE_TERMINATION_CHECK_SEC) != 0)
+    if (PackageWrapperCommunicate(wrapper, "get-package-data", request, &response) != 0)
     {
         Log(LOG_LEVEL_INFO, "Some error occurred while communicating with "
             "package module while collecting package data.");
@@ -407,12 +471,53 @@ PackageInfo *GetPackageData(const char *name, const char *version,
     return package_data;
 }
 
-static char *GetPackageModuleRealPath(const char *package_module_name)
+static void GetPackageModuleExecInfo(const PackageModuleBody *package_module, char **exec_path,
+                                     char **script_path, char **script_path_quoted,
+                                     char **script_exec_opts)
 {
 
-    return StringFormat("%s%c%s%c%s%c%s", GetWorkDir(), FILE_SEPARATOR,
-                        "modules", FILE_SEPARATOR, "packages", FILE_SEPARATOR,
-                        package_module_name);
+    assert(exec_path != NULL);
+    assert(script_path != NULL);
+    assert(script_path_quoted != NULL);
+
+    char *package_module_path = NULL;
+
+    if (package_module->module_path != NULL && !StringSafeEqual(package_module->module_path, ""))
+    {
+        package_module_path = xstrdup(package_module->module_path);
+    }
+    else
+    {
+        package_module_path = StringFormat("%s%c%s%c%s%c%s", GetWorkDir(), FILE_SEPARATOR,
+                                           "modules", FILE_SEPARATOR, "packages", FILE_SEPARATOR,
+                                           package_module->name);
+    }
+
+    if (package_module->interpreter && !StringSafeEqual(package_module->interpreter, ""))
+    {
+        *script_path = package_module_path;
+        *script_path_quoted = Path_GetQuoted(*script_path);
+
+        if (strchr(package_module->interpreter, ' ') == NULL)
+        {
+            /* No spaces in the 'interpreter' string, easy! Just interpreter path given, no opts. */
+            *exec_path = xstrdup(package_module->interpreter);
+            *script_exec_opts = NULL;
+        }
+        else
+        {
+            /* we need to split the interpreter command from the (potential)
+             * interpreter args */
+            ArgGetExecutableAndArgs(package_module->interpreter, exec_path, script_exec_opts);
+        }
+    }
+    else
+    {
+        *exec_path = package_module_path;
+        *script_path = NULL;
+        *script_path_quoted = NULL;
+        *script_exec_opts = NULL;
+    }
 }
 
 static int IsPackageInCache(EvalContext *ctx,
@@ -679,10 +784,7 @@ bool UpdateCache(Rlist* options, const PackageModuleWrapper *wrapper,
         req_type = "list-updates-local";
     }
 
-    if (PipeReadWriteData(wrapper->path, req_type, options_str,
-                          &response,
-                          PACKAGE_PROMISE_SCRIPT_TIMEOUT_SEC,
-                          PACKAGE_PROMISE_TERMINATION_CHECK_SEC) != 0)
+    if (PackageWrapperCommunicate(wrapper, req_type, options_str, &response) != 0)
     {
         Log(LOG_LEVEL_VERBOSE, "Some error occurred while communicating with "
                 "package module while updating cache.");
@@ -776,10 +878,7 @@ PromiseResult RemovePackage(const char *name, Rlist* options,
     PromiseResult res = PROMISE_RESULT_CHANGE;
 
     Rlist *error_message = NULL;
-    if (PipeReadWriteData(wrapper->path, "remove", request,
-                          &error_message,
-                          PACKAGE_PROMISE_SCRIPT_TIMEOUT_SEC,
-                          PACKAGE_PROMISE_TERMINATION_CHECK_SEC) != 0)
+    if (PackageWrapperCommunicate(wrapper, "remove", request, &error_message) != 0)
     {
         Log(LOG_LEVEL_INFO,
             "Error communicating package module while removing package.");
@@ -838,10 +937,7 @@ static PromiseResult InstallPackageGeneric(Rlist *options,
         request);
 
     Rlist *error_message = NULL;
-    if (PipeReadWriteData(wrapper->path, package_install_command, request,
-                          &error_message,
-                          PACKAGE_PROMISE_SCRIPT_TIMEOUT_SEC,
-                          PACKAGE_PROMISE_TERMINATION_CHECK_SEC) != 0)
+    if (PackageWrapperCommunicate(wrapper, package_install_command, request, &error_message) != 0)
     {
         Log(LOG_LEVEL_INFO, "Some error occurred while communicating with "
             "package module while installing package.");
@@ -1022,7 +1118,8 @@ PromiseResult RepoInstall(EvalContext *ctx,
                           bool *verified)
 {
     Log(LOG_LEVEL_DEBUG, "Installing repo type package: %d", is_in_cache);
-
+    const char *const package_version = package_info->version;
+    const char *const package_name = package_info->name;
     /* Package is not present in cache. */
     if (is_in_cache == 0)
     {
@@ -1033,9 +1130,8 @@ PromiseResult RepoInstall(EvalContext *ctx,
             Log(LOG_LEVEL_INFO, "Can not update packages cache.");
         }
 
-        const char *version = package_info->version;
-        if (package_info->version &&
-                StringSafeEqual(package_info->version, "latest"))
+        const char *version = package_version;
+        if (StringSafeEqual(version, "latest"))
         {
             Log(LOG_LEVEL_DEBUG, "Clearing latest package version");
             version = NULL;
@@ -1043,13 +1139,13 @@ PromiseResult RepoInstall(EvalContext *ctx,
         if (action == cfa_warn || DONTDO)
         {
             Log(LOG_LEVEL_VERBOSE, "Should install repo type package: %s",
-                package_info->name);
+                package_name);
             return PROMISE_RESULT_WARN;
         }
 
         *verified = false; /* Verification will be done in RepoInstallPackage(). */
         return InstallPackage(policy_data->package_options, PACKAGE_TYPE_REPO,
-                              package_info->name, version, package_info->arch,
+                              package_name, version, package_info->arch,
                               wrapper);
     }
 
@@ -1058,8 +1154,7 @@ PromiseResult RepoInstall(EvalContext *ctx,
 
 
     /* We have 'latest' version in policy. */
-    if (package_info->version &&
-        StringSafeEqual(package_info->version, "latest"))
+    if (StringSafeEqual(package_version, "latest"))
     {
         /* This can return more than one latest version if we have packages
          * with different architectures installed. */
@@ -1069,7 +1164,7 @@ PromiseResult RepoInstall(EvalContext *ctx,
         {
             Log(LOG_LEVEL_VERBOSE,
                 "Package '%s' is already in the latest version. "
-                "Skipping installation.", package_info->name);
+                "Skipping installation.", package_name);
 
             return PROMISE_RESULT_NOOP;
         }
@@ -1087,27 +1182,28 @@ PromiseResult RepoInstall(EvalContext *ctx,
              * in updates available but we are interested only in updating
              * package with specific architecture. */
             if (package_info->arch &&
-                    !StringSafeEqual(package_info->arch, update_package->arch))
+                !StringSafeEqual(package_info->arch, update_package->arch))
             {
                 Log(LOG_LEVEL_DEBUG,
                     "Skipping update check of package '%s' as updates"
                     "architecure doesn't match specified in policy: %s != %s.",
-                    package_info->name, package_info->arch,
+                    package_name, package_info->arch,
                     update_package->arch);
                 continue;
             }
 
+            const char *const update_version = update_package->version;
+
             Log(LOG_LEVEL_DEBUG,
                 "Checking for package '%s' version '%s' in available updates",
-                package_info->name, update_package->version);
+                package_name, update_version);
 
             /* Just in case some package managers will report highest possible
              * version in updates list instead of removing entry if package is
              * already in the latest version. */
-            int is_in_cache = IsPackageInCache(ctx, wrapper, package_info->name,
-                                               update_package->version,
-                                               update_package->arch);
-            if (is_in_cache == 1)
+            const int update_in_cache = IsPackageInCache(
+                ctx, wrapper, package_name, update_version, update_package->arch);
+            if (update_in_cache == 1)
             {
                 Log(LOG_LEVEL_VERBOSE,
                     "Package version from updates matches one installed. "
@@ -1115,7 +1211,7 @@ PromiseResult RepoInstall(EvalContext *ctx,
                 res = PromiseResultUpdate(res, PROMISE_RESULT_NOOP);
                 continue;
             }
-            else if (is_in_cache == -1)
+            else if (update_in_cache == -1)
             {
                 Log(LOG_LEVEL_INFO,
                     "Skipping package installation due to error with checking "
@@ -1128,7 +1224,7 @@ PromiseResult RepoInstall(EvalContext *ctx,
                 if (action == cfa_warn || DONTDO)
                 {
                     Log(LOG_LEVEL_VERBOSE, "Should install repo type package: %s",
-                        package_info->name);
+                        package_name);
                     res = PromiseResultUpdate(res, PROMISE_RESULT_WARN);
                     continue;
                 }
@@ -1141,8 +1237,8 @@ PromiseResult RepoInstall(EvalContext *ctx,
                  * architecture being removed. */
                 BufferAppendF(install_buffer,
                               "Name=%s\nVersion=%s\nArchitecture=%s\n",
-                              package_info->name,
-                              update_package->version,
+                              package_name,
+                              update_version,
                               update_package->arch);
 
                 /* Here we are adding latest_versions elements to different
@@ -1193,8 +1289,7 @@ PromiseResult RepoInstall(EvalContext *ctx,
     /* No version or explicit version specified. */
     else
     {
-        Log(LOG_LEVEL_VERBOSE, "Package '%s' already installed",
-            package_info->name);
+        Log(LOG_LEVEL_VERBOSE, "Package '%s' already installed", package_name);
 
         return PROMISE_RESULT_NOOP;
     }
@@ -1496,13 +1591,21 @@ bool UpdateSinglePackageModuleCache(EvalContext *ctx,
             Log(LOG_LEVEL_VERBOSE,
                 "Forcing package list update due to missing database");
             force_update = true;
+
+            /* When the cfengine database containing the cache of package updates
+             * available is initialized we should use the network to get the
+             * current list of updates available. It's not unlikely that the OS
+             * does not yet have a local cache, which simply results in an error
+             * getting a list of package updates available. */
+            if (type == UPDATE_TYPE_LOCAL_UPDATES)
+            {
+                type = UPDATE_TYPE_UPDATES;
+            }
         }
 
         cache_updates_lock =
                 AcquireLock(ctx, cache_updates_lock_name, VUQNAME, CFSTARTTIME,
-                            (TransactionContext) { .ifelapsed = ifelapsed_time,
-                                                   .expireafter = VEXPIREAFTER},
-                            &pp, false);
+                            ifelapsed_time, VEXPIREAFTER, &pp, false);
     }
     free(db_name);
 
