@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -40,7 +40,7 @@ typedef enum
     PACKAGE_TYPE_FILE
 } PackageType;
 
-typedef struct 
+typedef struct
 {
     char *name;
     char *version;
@@ -58,6 +58,9 @@ typedef struct
 {
     char *name;
     char *path;
+    char *script_path;
+    char *script_path_quoted;
+    char *script_exec_opts;
     PackageModuleBody *package_module;
     int supported_api_version;
 } PackageModuleWrapper;
@@ -74,14 +77,14 @@ typedef enum {
     UPDATE_TYPE_LOCAL_UPDATES,
 } UpdateType;
 
-PromiseResult HandlePresentPromiseAction(EvalContext *ctx, 
+PromiseResult HandlePresentPromiseAction(EvalContext *ctx,
                                          const char *package_name,
                                          const NewPackages *policy_data,
                                          const PackageModuleWrapper *wrapper,
                                          enum cfopaction action);
 PromiseResult HandleAbsentPromiseAction(EvalContext *ctx,
                                         char *package_name,
-                                        const NewPackages *policy_data, 
+                                        const NewPackages *policy_data,
                                         const PackageModuleWrapper *wrapper,
                                         enum cfopaction action);
 
