@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -27,37 +27,53 @@
 #include <printsize.h>
 #include <actuator.h>
 #include <rlist.h>
+#include <string_lib.h>
 
-int VerifyCommandRetcode(EvalContext *ctx, int retcode, Attributes a, const Promise *pp, PromiseResult *result)
+bool VerifyCommandRetcode(EvalContext *ctx, int retcode, const Attributes *a, const Promise *pp, PromiseResult *result)
 {
+    assert(a != NULL);
+    assert(pp != NULL);
     bool result_retcode = true;
 
-    if (a.classes.retcode_kept ||
-        a.classes.retcode_repaired ||
-        a.classes.retcode_failed)
+    if (a->classes.retcode_kept ||
+        a->classes.retcode_repaired ||
+        a->classes.retcode_failed)
     {
         int matched = false;
         char retcodeStr[PRINTSIZE(retcode)];
         xsnprintf(retcodeStr, sizeof(retcodeStr), "%d", retcode);
 
-        if (RlistKeyIn(a.classes.retcode_kept, retcodeStr))
+        LogLevel info_or_verbose = LOG_LEVEL_INFO;
+
+        // inform constraint is only for commands promises,
+        // a->inform is actually false for other promise types, so
+        // checking the promise type here is important:
+        if (StringSafeEqual("commands", pp->parent_promise_type->name) && (!a->inform))
         {
-            cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_NOOP, pp, a,
+            // for commands promises which don't make changes to the system,
+            // you can use this to make the log messages verbose:
+            // inform => "false";
+            info_or_verbose = LOG_LEVEL_VERBOSE;
+        }
+
+        if (RlistKeyIn(a->classes.retcode_kept, retcodeStr))
+        {
+            cfPS(ctx, info_or_verbose, PROMISE_RESULT_NOOP, pp, a,
                  "Command related to promiser '%s' returned code defined as promise kept %d", pp->promiser,
                  retcode);
             matched = true;
         }
 
-        if (RlistKeyIn(a.classes.retcode_repaired, retcodeStr))
+        if (RlistKeyIn(a->classes.retcode_repaired, retcodeStr))
         {
-            cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_CHANGE, pp, a,
+            cfPS(ctx, info_or_verbose, PROMISE_RESULT_CHANGE, pp, a,
                  "Command related to promiser '%s' returned code defined as promise repaired %d", pp->promiser,
                  retcode);
             *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
             matched = true;
         }
 
-        if (RlistKeyIn(a.classes.retcode_failed, retcodeStr))
+        if (RlistKeyIn(a->classes.retcode_failed, retcodeStr))
         {
             cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a,
                  "Command related to promiser '%s' returned code defined as promise failed %d", pp->promiser,
@@ -69,7 +85,7 @@ int VerifyCommandRetcode(EvalContext *ctx, int retcode, Attributes a, const Prom
 
         if (!matched)
         {
-            cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_FAIL, pp, a,
+            cfPS(ctx, info_or_verbose, PROMISE_RESULT_FAIL, pp, a,
                  "Command related to promiser '%s' returned code not defined as promise kept, not kept or repaired; setting to failed: %d",
                  pp->promiser, retcode);
             *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
