@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -38,21 +38,23 @@
 #define CF_VALID_OPS_METHOD_OVERWRITE "=+-"
 #define CF_VALID_OPS_METHOD_APPEND "=+-"
 
-static int CheckACLSyntax(const char *file, Acl acl, const Promise *pp);
+static bool CheckACLSyntax(const char *file, Acl acl, const Promise *pp);
 
 static void SetACLDefaults(const char *path, Acl *acl);
-static int CheckACESyntax(char *ace, char *valid_nperms, char *valid_ops, int deny_support, int mask_support,
+static bool CheckACESyntax(char *ace, char *valid_nperms, char *valid_ops, int deny_support, int mask_support,
                         const Promise *pp);
-static int CheckModeSyntax(char **mode_p, char *valid_nperms, char *valid_ops, const Promise *pp);
-static int CheckPermTypeSyntax(char *permt, int deny_support, const Promise *pp);
+static bool CheckModeSyntax(char **mode_p, char *valid_nperms, char *valid_ops, const Promise *pp);
+static bool CheckPermTypeSyntax(char *permt, int deny_support, const Promise *pp);
 static int CheckAclDefault(const char *path, Acl *acl, const Promise *pp);
 
 
-PromiseResult VerifyACL(EvalContext *ctx, const char *file, Attributes a, const Promise *pp)
+PromiseResult VerifyACL(EvalContext *ctx, const char *file, const Attributes *attr, const Promise *pp)
 {
+    assert(attr != NULL);
+    Attributes a = *attr; // TODO: Remove this local copy
     if (!CheckACLSyntax(file, a.acl, pp))
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_INTERRUPTED, pp, a, "Syntax error in access control list for '%s'", file);
+        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_INTERRUPTED, pp, &a, "Syntax error in access control list for '%s'", file);
         PromiseRef(LOG_LEVEL_ERR, pp);
         return PROMISE_RESULT_INTERRUPTED;
     }
@@ -68,9 +70,9 @@ PromiseResult VerifyACL(EvalContext *ctx, const char *file, Attributes a, const
     case ACL_TYPE_GENERIC:
 
 #if defined(__linux__)
-        result = PromiseResultUpdate(result, CheckPosixLinuxACL(ctx, file, a.acl, a, pp));
+        result = PromiseResultUpdate(result, CheckPosixLinuxACL(ctx, file, a.acl, &a, pp));
 #elif defined(__MINGW32__)
-        result = PromiseResultUpdate(result, Nova_CheckNtACL(ctx, file, a.acl, a, pp));
+        result = PromiseResultUpdate(result, Nova_CheckNtACL(ctx, file, a.acl, &a, pp));
 #else
         Log(LOG_LEVEL_INFO, "ACLs are not yet supported on this system.");
 #endif
@@ -79,7 +81,7 @@ PromiseResult VerifyACL(EvalContext *ctx, const char *file, Attributes a, const
     case ACL_TYPE_POSIX:
 
 #if defined(__linux__)
-        result = PromiseResultUpdate(result, CheckPosixLinuxACL(ctx, file, a.acl, a, pp));
+        result = PromiseResultUpdate(result, CheckPosixLinuxACL(ctx, file, a.acl, &a, pp));
 #else
         Log(LOG_LEVEL_INFO, "Posix ACLs are not supported on this system");
 #endif
@@ -87,7 +89,7 @@ PromiseResult VerifyACL(EvalContext *ctx, const char *file, Attributes a, const
 
     case ACL_TYPE_NTFS_:
 #ifdef __MINGW32__
-        result = PromiseResultUpdate(result, Nova_CheckNtACL(ctx, file, a.acl, a, pp));
+        result = PromiseResultUpdate(result, Nova_CheckNtACL(ctx, file, a.acl, &a, pp));
 #else
         Log(LOG_LEVEL_INFO, "NTFS ACLs are not supported on this system");
 #endif
@@ -101,9 +103,9 @@ PromiseResult VerifyACL(EvalContext *ctx, const char *file, Attributes a, const
     return result;
 }
 
-static int CheckACLSyntax(const char *file, Acl acl, const Promise *pp)
+static bool CheckACLSyntax(const char *file, Acl acl, const Promise *pp)
 {
-    int valid = true;
+    bool valid = true;
     int deny_support = false;
     int mask_support = false;
     char *valid_ops = NULL;
@@ -257,16 +259,16 @@ static int CheckAclDefault(const char *path, Acl *acl, const Promise *pp)
     return valid;
 }
 
-static int CheckACESyntax(char *ace, char *valid_ops, char *valid_nperms, int deny_support, int mask_support,
-                          const Promise *pp)
+static bool CheckACESyntax(
+    char *ace,
+    char *valid_ops,
+    char *valid_nperms,
+    int deny_support,
+    int mask_support,
+    const Promise *pp)
 {
-    char *str;
-    int chkid;
-    int valid_mode;
-    int valid_permt;
-
-    str = ace;
-    chkid = false;
+    char *str = ace;
+    bool chkid = false;
 
 // first element must be "user", "group" or "all"
 
@@ -335,7 +337,7 @@ static int CheckACESyntax(char *ace, char *valid_ops, char *valid_nperms, int de
     }
 
 // check the mode-string (also skips to next field)
-    valid_mode = CheckModeSyntax(&str, valid_ops, valid_nperms, pp);
+    const bool valid_mode = CheckModeSyntax(&str, valid_ops, valid_nperms, pp);
 
     if (!valid_mode)
     {
@@ -352,7 +354,7 @@ static int CheckACESyntax(char *ace, char *valid_ops, char *valid_nperms, int de
     str++;
 
 // last field; must be a perm_type field
-    valid_permt = CheckPermTypeSyntax(str, deny_support, pp);
+    const bool valid_permt = CheckPermTypeSyntax(str, deny_support, pp);
 
     if (!valid_permt)
     {
@@ -363,18 +365,15 @@ static int CheckACESyntax(char *ace, char *valid_ops, char *valid_nperms, int de
     return true;
 }
 
-static int CheckModeSyntax(char **mode_p, char *valid_ops, char *valid_nperms, const Promise *pp)
+static bool CheckModeSyntax(char **mode_p, char *valid_ops, char *valid_nperms, const Promise *pp)
 /*
   Checks the syntax of a ':' or NULL terminated mode string.
   Moves the string pointer to the character following the mode
   (i.e. ':' or '\0')
 */
 {
-    char *mode;
-    int valid;
-
-    valid = false;
-    mode = *mode_p;
+    char *mode = *mode_p;
+    bool valid = false;
 
 // mode is allowed to be empty
 
@@ -431,16 +430,14 @@ static int CheckModeSyntax(char **mode_p, char *valid_ops, char *valid_nperms, c
     return valid;
 }
 
-static int CheckPermTypeSyntax(char *permt, int deny_support, const Promise *pp)
+static bool CheckPermTypeSyntax(char *permt, int deny_support, const Promise *pp)
 /*
   Checks if the given string corresponds to the perm_type syntax.
   Only "allow" or "deny", followed by NULL-termination are valid.
   In addition, "deny" is only valid for ACL types supporting it.
  */
 {
-    int valid;
-
-    valid = false;
+    bool valid = false;
 
     if (strcmp(permt, "allow") == 0)
     {
