@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -39,25 +39,24 @@
 #include <ornaments.h>
 #include <misc_lib.h>
 
-static int CheckDatabaseSanity(Attributes a, const Promise *pp);
-static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Promise *pp);
-static int VerifyDatabasePromise(CfdbConn *cfdb, char *database, Attributes a);
+static int CheckDatabaseSanity(const Attributes *a, const Promise *pp);
+static PromiseResult VerifySQLPromise(EvalContext *ctx, const Attributes *a, const Promise *pp);
+static bool VerifyDatabasePromise(CfdbConn *cfdb, char *database, const Attributes *a);
 
-static int ValidateSQLTableName(char *table_path, char *db, char *table);
-static int VerifyTablePromise(EvalContext *ctx, CfdbConn *cfdb, char *table_path, Rlist *columns, Attributes a, const Promise *pp, PromiseResult *result);
-static int ValidateSQLTableName(char *table_path, char *db, char *table);
+static bool ValidateSQLTableName(char *table_path, char *db, char *table);
+static bool VerifyTablePromise(EvalContext *ctx, CfdbConn *cfdb, char *table_path, Rlist *columns, const Attributes *a, const Promise *pp, PromiseResult *result);
 static void QueryTableColumns(char *s, char *db, char *table);
-static int NewSQLColumns(char *table, Rlist *columns, char ***name_table, char ***type_table, int **size_table,
+static bool NewSQLColumns(char *table, Rlist *columns, char ***name_table, char ***type_table, int **size_table,
                          int **done);
 static void DeleteSQLColumns(char **name_table, char **type_table, int *size_table, int *done, int len);
 static void CreateDBQuery(DatabaseType type, char *query);
-static int CreateTableColumns(CfdbConn *cfdb, char *table, Rlist *columns);
-static int CheckSQLDataType(char *type, char *ref_type, const Promise *pp);
+static bool CreateTableColumns(CfdbConn *cfdb, char *table, Rlist *columns);
+static bool CheckSQLDataType(char *type, char *ref_type, const Promise *pp);
 static int TableExists(CfdbConn *cfdb, char *name);
 static Rlist *GetSQLTables(CfdbConn *cfdb);
 static void ListTables(int type, char *query);
-static int ValidateRegistryPromiser(char *s, const Promise *pp);
-static int CheckRegistrySanity(Attributes a, const Promise *pp);
+static bool ValidateRegistryPromiser(char *s, const Promise *pp);
+static bool CheckRegistrySanity(const Attributes *a, const Promise *pp);
 
 /*****************************************************************************/
 
@@ -67,19 +66,19 @@ PromiseResult VerifyDatabasePromises(EvalContext *ctx, const Promise *pp)
 
     Attributes a = GetDatabaseAttributes(ctx, pp);
 
-    if (!CheckDatabaseSanity(a, pp))
+    if (!CheckDatabaseSanity(&a, pp))
     {
         return PROMISE_RESULT_FAIL;
     }
 
     if (strcmp(a.database.type, "sql") == 0)
     {
-        return VerifySQLPromise(ctx, a, pp);
+        return VerifySQLPromise(ctx, &a, pp);
     }
     else if (strcmp(a.database.type, "ms_registry") == 0)
     {
 #if defined(__MINGW32__)
-        return VerifyRegistryPromise(ctx, a, pp);
+        return VerifyRegistryPromise(ctx, &a, pp);
 #endif
         return PROMISE_RESULT_NOOP;
     }
@@ -93,8 +92,9 @@ PromiseResult VerifyDatabasePromises(EvalContext *ctx, const Promise *pp)
 /* Level                                                                     */
 /*****************************************************************************/
 
-static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Promise *pp)
+static PromiseResult VerifySQLPromise(EvalContext *ctx, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     char database[CF_MAXVARSIZE], table[CF_MAXVARSIZE], query[CF_BUFSIZE];
     char *sp;
     int count = 0;
@@ -104,7 +104,7 @@ static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Prom
 
     snprintf(lockname, CF_BUFSIZE - 1, "db-%s", pp->promiser);
 
-    thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME, a.transaction, pp, false);
+    thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME, a->transaction.ifelapsed, a->transaction.expireafter, pp, false);
     if (thislock.lock == NULL)
     {
         return PROMISE_RESULT_SKIPPED;
@@ -145,7 +145,7 @@ static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Prom
         strlcpy(database, pp->promiser, CF_MAXVARSIZE);
     }
 
-    if (a.database.operation == NULL)
+    if (a->database.operation == NULL)
     {
         cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a,
              "Missing database_operation in database promise");
@@ -154,22 +154,22 @@ static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Prom
         return PROMISE_RESULT_FAIL;
     }
 
-    if (strcmp(a.database.operation, "delete") == 0)
+    if (strcmp(a->database.operation, "delete") == 0)
     {
         /* Just deal with one */
-        strcpy(a.database.operation, "drop");
+        strcpy(a->database.operation, "drop");
     }
 
 /* Connect to the server */
 
-    CfConnectDB(&cfdb, a.database.db_server_type, a.database.db_server_host, a.database.db_server_owner,
-                a.database.db_server_password, database);
+    CfConnectDB(&cfdb, a->database.db_server_type, a->database.db_server_host, a->database.db_server_owner,
+                a->database.db_server_password, database);
 
     if (!cfdb.connected)
     {
         /* If we haven't said create then db should already exist */
 
-        if ((a.database.operation) && (strcmp(a.database.operation, "create") != 0))
+        if ((a->database.operation) && (strcmp(a->database.operation, "create") != 0))
         {
             Log(LOG_LEVEL_ERR, "Could not connect an existing database '%s' - check server configuration?", database);
             PromiseRef(LOG_LEVEL_ERR, pp);
@@ -181,20 +181,21 @@ static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Prom
 
 /* Check change of existential constraints */
 
-    if ((a.database.operation) && (strcmp(a.database.operation, "create") == 0))
+    if ((a->database.operation) && (strcmp(a->database.operation, "create") == 0))
     {
-        CfConnectDB(&cfdb, a.database.db_server_type, a.database.db_server_host, a.database.db_server_owner,
-                    a.database.db_server_password, a.database.db_connect_db);
+        CfConnectDB(&cfdb, a->database.db_server_type, a->database.db_server_host, a->database.db_server_owner,
+                    a->database.db_server_password, a->database.db_connect_db);
 
         if (!cfdb.connected)
         {
             Log(LOG_LEVEL_ERR, "Could not connect to the sql_db server for '%s'", database);
+            YieldCurrentLock(thislock);
             return PROMISE_RESULT_FAIL;
         }
 
         /* Don't drop the db if we really want to drop a table */
 
-        if ((strlen(table) == 0) || ((strlen(table) > 0) && (strcmp(a.database.operation, "drop") != 0)))
+        if ((strlen(table) == 0) || ((strlen(table) > 0) && (strcmp(a->database.operation, "drop") != 0)))
         {
             VerifyDatabasePromise(&cfdb, database, a);
         }
@@ -212,8 +213,8 @@ static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Prom
         return result;
     }
 
-    CfConnectDB(&cfdb, a.database.db_server_type, a.database.db_server_host, a.database.db_server_owner,
-                a.database.db_server_password, database);
+    CfConnectDB(&cfdb, a->database.db_server_type, a->database.db_server_host, a->database.db_server_owner,
+                a->database.db_server_password, database);
 
     if (!cfdb.connected)
     {
@@ -223,7 +224,7 @@ static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Prom
     {
         snprintf(query, CF_MAXVARSIZE - 1, "%s.%s", database, table);
 
-        if (VerifyTablePromise(ctx, &cfdb, query, a.database.columns, a, pp, &result))
+        if (VerifyTablePromise(ctx, &cfdb, query, a->database.columns, a, pp, &result))
         {
             cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_NOOP, pp, a, "Table '%s' is as promised", query);
         }
@@ -235,7 +236,7 @@ static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Prom
 
 /* Finally check any row constraints on this table */
 
-        if (a.database.rows)
+        if (a->database.rows)
         {
             Log(LOG_LEVEL_INFO,
                   "Database row operations are not currently supported. Please contact cfengine with suggestions.");
@@ -249,8 +250,9 @@ static PromiseResult VerifySQLPromise(EvalContext *ctx, Attributes a, const Prom
     return result;
 }
 
-static int VerifyDatabasePromise(CfdbConn *cfdb, char *database, Attributes a)
+static bool VerifyDatabasePromise(CfdbConn *cfdb, char *database, const Attributes *a)
 {
+    assert(a != NULL);
     char query[CF_BUFSIZE], name[CF_MAXVARSIZE];
     int found = false;
 
@@ -295,9 +297,9 @@ static int VerifyDatabasePromise(CfdbConn *cfdb, char *database, Attributes a)
         Log(LOG_LEVEL_VERBOSE, "Database '%s' does not seem to exist on this connection", database);
     }
 
-    if ((a.database.operation) && (strcmp(a.database.operation, "drop") == 0))
+    if ((a->database.operation) && (strcmp(a->database.operation, "drop") == 0))
     {
-        if (((a.transaction.action) != cfa_warn) && (!DONTDO))
+        if (((a->transaction.action) != cfa_warn) && (!DONTDO))
         {
             Log(LOG_LEVEL_VERBOSE, "Attempting to delete the database '%s'", database);
             snprintf(query, CF_MAXVARSIZE - 1, "drop database %s", database);
@@ -311,9 +313,9 @@ static int VerifyDatabasePromise(CfdbConn *cfdb, char *database, Attributes a)
         }
     }
 
-    if ((a.database.operation) && (strcmp(a.database.operation, "create") == 0))
+    if ((a->database.operation) && (strcmp(a->database.operation, "create") == 0))
     {
-        if (((a.transaction.action) != cfa_warn) && (!DONTDO))
+        if (((a->transaction.action) != cfa_warn) && (!DONTDO))
         {
             Log(LOG_LEVEL_VERBOSE, "Attempting to create the database '%s'", database);
             snprintf(query, CF_MAXVARSIZE - 1, "create database %s", database);
@@ -332,52 +334,53 @@ static int VerifyDatabasePromise(CfdbConn *cfdb, char *database, Attributes a)
 
 /*****************************************************************************/
 
-static int CheckDatabaseSanity(Attributes a, const Promise *pp)
+static int CheckDatabaseSanity(const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     Rlist *rp;
     int retval = true, commas = 0;
 
-    if ((a.database.type) && (strcmp(a.database.type, "ms_registry") == 0))
+    if ((a->database.type) && (strcmp(a->database.type, "ms_registry") == 0))
     {
         retval = CheckRegistrySanity(a, pp);
     }
-    else if ((a.database.type) && (strcmp(a.database.type, "sql") == 0))
+    else if ((a->database.type) && (strcmp(a->database.type, "sql") == 0))
     {
         if ((strchr(pp->promiser, '.') == NULL) && (strchr(pp->promiser, '/') == NULL)
             && (strchr(pp->promiser, '\\') == NULL))
         {
-            if (a.database.columns)
+            if (a->database.columns)
             {
                 Log(LOG_LEVEL_ERR, "Row values promised for an SQL table, but only the root database was promised");
                 retval = false;
             }
 
-            if (a.database.rows)
+            if (a->database.rows)
             {
                 Log(LOG_LEVEL_ERR, "Columns promised for an SQL table, but only the root database was promised");
                 retval = false;
             }
         }
 
-        if (a.database.db_server_host == NULL)
+        if (a->database.db_server_host == NULL)
         {
             Log(LOG_LEVEL_ERR, "No server host is promised for connecting to the SQL server");
             retval = false;
         }
 
-        if (a.database.db_server_owner == NULL)
+        if (a->database.db_server_owner == NULL)
         {
             Log(LOG_LEVEL_ERR, "No database login user is promised for connecting to the SQL server");
             retval = false;
         }
 
-        if (a.database.db_server_password == NULL)
+        if (a->database.db_server_password == NULL)
         {
             Log(LOG_LEVEL_ERR, "No database authentication password is promised for connecting to the SQL server");
             retval = false;
         }
 
-        for (rp = a.database.columns; rp != NULL; rp = rp->next)
+        for (rp = a->database.columns; rp != NULL; rp = rp->next)
         {
             commas = CountChar(RlistScalarValue(rp), ',');
 
@@ -390,12 +393,8 @@ static int CheckDatabaseSanity(Attributes a, const Promise *pp)
 
     }
 
-    if ((a.database.operation) && (strcmp(a.database.operation, "create") == 0))
-    {
-    }
-
-    if ((a.database.operation)
-        && ((strcmp(a.database.operation, "delete") == 0) || (strcmp(a.database.operation, "drop") == 0)))
+    if ((a->database.operation)
+        && ((strcmp(a->database.operation, "delete") == 0) || (strcmp(a->database.operation, "drop") == 0)))
     {
         if (pp->comment == NULL)
         {
@@ -408,42 +407,43 @@ static int CheckDatabaseSanity(Attributes a, const Promise *pp)
     return retval;
 }
 
-static int CheckRegistrySanity(Attributes a, const Promise *pp)
+static bool CheckRegistrySanity(const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     bool retval = true;
 
     ValidateRegistryPromiser(pp->promiser, pp);
 
-    if ((a.database.operation) && (strcmp(a.database.operation, "create") == 0))
+    if ((a->database.operation) && (strcmp(a->database.operation, "create") == 0))
     {
-        if (a.database.rows == NULL)
+        if (a->database.rows == NULL)
         {
             Log(LOG_LEVEL_INFO, "No row values promised for the MS registry database");
         }
 
-        if (a.database.columns != NULL)
+        if (a->database.columns != NULL)
         {
             Log(LOG_LEVEL_ERR, "Columns are only used to delete promised values for the MS registry database");
             retval = false;
         }
     }
 
-    if ((a.database.operation)
-        && ((strcmp(a.database.operation, "delete") == 0) || (strcmp(a.database.operation, "drop") == 0)))
+    if ((a->database.operation)
+        && ((strcmp(a->database.operation, "delete") == 0) || (strcmp(a->database.operation, "drop") == 0)))
     {
-        if (a.database.columns == NULL)
+        if (a->database.columns == NULL)
         {
             Log(LOG_LEVEL_INFO, "No columns were promised deleted in the MS registry database");
         }
 
-        if (a.database.rows != NULL)
+        if (a->database.rows != NULL)
         {
             Log(LOG_LEVEL_ERR, "Rows cannot be deleted in the MS registry database, only entire columns");
             retval = false;
         }
     }
 
-    for (Rlist *rp = a.database.rows; rp != NULL; rp = rp->next)
+    for (Rlist *rp = a->database.rows; rp != NULL; rp = rp->next)
     {
         if (CountChar(RlistScalarValue(rp), ',') != 2)
         {
@@ -452,7 +452,7 @@ static int CheckRegistrySanity(Attributes a, const Promise *pp)
         }
     }
 
-    for (Rlist *rp = a.database.columns; rp != NULL; rp = rp->next)
+    for (Rlist *rp = a->database.columns; rp != NULL; rp = rp->next)
     {
         if (CountChar(RlistScalarValue(rp), ',') > 0)
         {
@@ -464,7 +464,7 @@ static int CheckRegistrySanity(Attributes a, const Promise *pp)
     return retval;
 }
 
-static int ValidateRegistryPromiser(char *key, const Promise *pp)
+static bool ValidateRegistryPromiser(char *key, const Promise *pp)
 {
     static char *const valid[] = { "HKEY_CLASSES_ROOT", "HKEY_CURRENT_CONFIG",
         "HKEY_CURRENT_USER", "HKEY_LOCAL_MACHINE", "HKEY_USERS", NULL
@@ -477,6 +477,13 @@ static int ValidateRegistryPromiser(char *key, const Promise *pp)
 
     strlcpy(root_key, key, CF_MAXVARSIZE );
     sp = strchr(root_key, '\\');
+    if (sp == NULL)
+    {
+        Log(LOG_LEVEL_ERR, "Cannot locate '\\' in '%s'", root_key);
+        Log(LOG_LEVEL_ERR, "Failed validating registry promiser");
+        PromiseRef(LOG_LEVEL_ERR, pp);
+        return false;
+    }
     *sp = '\0';
 
     for (i = 0; valid[i] != NULL; i++)
@@ -496,11 +503,13 @@ static int ValidateRegistryPromiser(char *key, const Promise *pp)
 /* Linker troubles require this code to be here in the main body             */
 /*****************************************************************************/
 
-static int VerifyTablePromise(EvalContext *ctx, CfdbConn *cfdb, char *table_path, Rlist *columns, Attributes a,
+static bool VerifyTablePromise(EvalContext *ctx, CfdbConn *cfdb, char *table_path, Rlist *columns, const Attributes *a,
                               const Promise *pp, PromiseResult *result)
 {
+    assert(a != NULL);
     char name[CF_MAXVARSIZE], type[CF_MAXVARSIZE], query[CF_MAXVARSIZE], table[CF_MAXVARSIZE], db[CF_MAXVARSIZE];
-    int i, count, size, no_of_cols, *size_table, *done, identified, retval = true;
+    int i, count, size, no_of_cols, *size_table, *done, identified;
+    bool retval = true;
     char **name_table, **type_table;
 
     Log(LOG_LEVEL_VERBOSE, "Verifying promised table structure for '%s'", table_path);
@@ -522,9 +531,9 @@ static int VerifyTablePromise(EvalContext *ctx, CfdbConn *cfdb, char *table_path
     {
         Log(LOG_LEVEL_ERR, "The database did not contain the promised table '%s'", table_path);
 
-        if ((a.database.operation) && (strcmp(a.database.operation, "create") == 0))
+        if ((a->database.operation) && (strcmp(a->database.operation, "create") == 0))
         {
-            if ((!DONTDO) && ((a.transaction.action) != cfa_warn))
+            if ((!DONTDO) && ((a->transaction.action) != cfa_warn))
             {
                 cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_CHANGE, pp, a, "Database.table '%s' doesn't seem to exist, creating",
                      table_path);
@@ -636,7 +645,7 @@ static int VerifyTablePromise(EvalContext *ctx, CfdbConn *cfdb, char *table_path
                  "Column '%s' found in database.table '%s' is not part of its promise.", name, table_path);
             *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
 
-            if ((a.database.operation) && (strcmp(a.database.operation, "drop") == 0))
+            if ((a->database.operation) && (strcmp(a->database.operation, "drop") == 0))
             {
                 cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a,
                      "CFEngine will not promise to repair this, as the operation is potentially too destructive.");
@@ -652,7 +661,7 @@ static int VerifyTablePromise(EvalContext *ctx, CfdbConn *cfdb, char *table_path
 
 /* Now look for deviations - only if we have promised to create missing */
 
-    if ((a.database.operation) && (strcmp(a.database.operation, "drop") == 0))
+    if ((a->database.operation) && (strcmp(a->database.operation, "drop") == 0))
     {
         return retval;
     }
@@ -666,7 +675,7 @@ static int VerifyTablePromise(EvalContext *ctx, CfdbConn *cfdb, char *table_path
                 Log(LOG_LEVEL_ERR, "Promised column '%s' missing from database table '%s'", name_table[i],
                       pp->promiser);
 
-                if ((!DONTDO) && ((a.transaction.action) != cfa_warn))
+                if ((!DONTDO) && ((a->transaction.action) != cfa_warn))
                 {
                     if (size_table[i] > 0)
                     {
@@ -726,7 +735,7 @@ static int TableExists(CfdbConn *cfdb, char *name)
 
 /*****************************************************************************/
 
-static int CreateTableColumns(CfdbConn *cfdb, char *table, Rlist *columns)
+static bool CreateTableColumns(CfdbConn *cfdb, char *table, Rlist *columns)
 {
     char entry[CF_MAXVARSIZE], query[CF_BUFSIZE];
     int i, *size_table, *done;
@@ -828,7 +837,7 @@ static void CreateDBQuery(DatabaseType type, char *query)
 
 /*****************************************************************************/
 
-static int ValidateSQLTableName(char *table_path, char *db, char *table)
+static bool ValidateSQLTableName(char *table_path, char *db, char *table)
 {
     char *sp;
     int dot = false, back = false, fwd = false;
@@ -871,13 +880,13 @@ static int ValidateSQLTableName(char *table_path, char *db, char *table)
 static void QueryTableColumns(char *s, char *db, char *table)
 {
     snprintf(s, CF_MAXVARSIZE - 1,
-             "SELECT column_name,data_type,character_maximum_length FROM information_schema.columns WHERE table_name ='%s' AND table_schema = '%s'",
+             "SELECT column_name,data_type,character_maximum_length FROM information_schema->columns WHERE table_name ='%s' AND table_schema = '%s'",
              table, db);
 }
 
 /*****************************************************************************/
 
-static int NewSQLColumns(char *table, Rlist *columns, char ***name_table, char ***type_table, int **size_table,
+static bool NewSQLColumns(char *table, Rlist *columns, char ***name_table, char ***type_table, int **size_table,
                          int **done)
 {
     int i, no_of_cols = RlistLen(columns);
@@ -979,7 +988,7 @@ static void DeleteSQLColumns(char **name_table, char **type_table, int *size_tab
 
 /*****************************************************************************/
 
-static int CheckSQLDataType(char *type, char *ref_type, const Promise *pp)
+static bool CheckSQLDataType(char *type, char *ref_type, const Promise *pp)
 {
     static char *const aliases[3][2] =
         {
@@ -989,6 +998,7 @@ static int CheckSQLDataType(char *type, char *ref_type, const Promise *pp)
         };
 
     int i;
+    bool ret = true;
 
     for (i = 0; aliases[i][0] != NULL; i++)
     {
@@ -999,6 +1009,7 @@ static int CheckSQLDataType(char *type, char *ref_type, const Promise *pp)
             {
                 Log(LOG_LEVEL_VERBOSE, "Promised column in database '%s' has a non-matching type (%s != %s)",
                       pp->promiser, ref_type, type);
+                ret = false;
             }
         }
         else
@@ -1007,11 +1018,12 @@ static int CheckSQLDataType(char *type, char *ref_type, const Promise *pp)
             {
                 Log(LOG_LEVEL_VERBOSE, "Promised column in database '%s' has a non-matching type (%s != %s)",
                       pp->promiser, ref_type, type);
+                ret = false;
             }
         }
     }
 
-    return true;
+    return ret;
 }
 
 /*****************************************************************************/
