@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -74,20 +74,20 @@ char *CF_SUSPENDED[CF_MAX_CONCURRENT_ENVIRONMENTS];
 
 /*****************************************************************************/
 
-static int EnvironmentsSanityChecks(Attributes a, const Promise *pp);
-static PromiseResult VerifyEnvironments(EvalContext *ctx, Attributes a, const Promise *pp);
-static PromiseResult VerifyVirtDomain(EvalContext *ctx, char *uri, enum cfhypervisors envtype, Attributes a, const Promise *pp);
-static PromiseResult VerifyVirtNetwork(EvalContext *ctx, char *uri, enum cfhypervisors envtype, Attributes a, const Promise *pp);
-static PromiseResult CreateVirtDom(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp);
-static PromiseResult DeleteVirt(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp);
-static PromiseResult RunningVirt(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp);
-static PromiseResult SuspendedVirt(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp);
-static PromiseResult DownVirt(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp);
+static bool EnvironmentsSanityChecks(const Attributes *a, const Promise *pp);
+static PromiseResult VerifyEnvironments(EvalContext *ctx, const Attributes *a, const Promise *pp);
+static PromiseResult VerifyVirtDomain(EvalContext *ctx, char *uri, enum cfhypervisors envtype, const Attributes *a, const Promise *pp);
+static PromiseResult VerifyVirtNetwork(EvalContext *ctx, char *uri, enum cfhypervisors envtype, const Attributes *a, const Promise *pp);
+static PromiseResult CreateVirtDom(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp);
+static PromiseResult DeleteVirt(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp);
+static PromiseResult RunningVirt(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp);
+static PromiseResult SuspendedVirt(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp);
+static PromiseResult DownVirt(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp);
 static void EnvironmentErrorHandler(void);
 static void ShowRunList(virConnectPtr vc);
 static void ShowDormant(void);
-static PromiseResult CreateVirtNetwork(EvalContext *ctx, virConnectPtr vc, char **networks, Attributes a, const Promise *pp);
-static PromiseResult DeleteVirtNetwork(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp);
+static PromiseResult CreateVirtNetwork(EvalContext *ctx, virConnectPtr vc, char **networks, const Attributes *a, const Promise *pp);
+static PromiseResult DeleteVirtNetwork(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp);
 static enum cfhypervisors Str2Hypervisors(char *s);
 
 /*****************************************************************************/
@@ -126,9 +126,9 @@ PromiseResult VerifyEnvironmentsPromise(EvalContext *ctx, const Promise *pp)
     Attributes a = GetEnvironmentsAttributes(ctx, pp);
 
     PromiseResult result = PROMISE_RESULT_NOOP;
-    if (EnvironmentsSanityChecks(a, pp))
+    if (EnvironmentsSanityChecks(&a, pp))
     {
-        thislock = AcquireLock(ctx, "virtual", VUQNAME, CFSTARTTIME, a.transaction, pp, false);
+        thislock = AcquireLock(ctx, "virtual", VUQNAME, CFSTARTTIME, a.transaction.ifelapsed, a.transaction.expireafter, pp, false);
 
         if (thislock.lock == NULL)
         {
@@ -145,7 +145,7 @@ PromiseResult VerifyEnvironmentsPromise(EvalContext *ctx, const Promise *pp)
         }
         else
         {
-            result = VerifyEnvironments(ctx, a, pp);
+            result = VerifyEnvironments(ctx, &a, pp);
         }
 
         PromiseDestroy(pexp);
@@ -158,35 +158,37 @@ PromiseResult VerifyEnvironmentsPromise(EvalContext *ctx, const Promise *pp)
 
 /*****************************************************************************/
 
-static int EnvironmentsSanityChecks(Attributes a, const Promise *pp)
+static bool EnvironmentsSanityChecks(const Attributes *a, const Promise *pp)
 {
-    if (a.env.spec)
+    assert(a != NULL);
+    const Environments env = a->env;
+    if (env.spec)
     {
-        if (a.env.cpus != CF_NOINT || a.env.memory != CF_NOINT || a.env.disk != CF_NOINT)
+        if (env.cpus != CF_NOINT || env.memory != CF_NOINT || env.disk != CF_NOINT)
         {
             Log(LOG_LEVEL_ERR, "Conflicting promise of both a spec and cpu/memory/disk resources");
             return false;
         }
     }
 
-    if (a.env.host == NULL)
+    if (env.host == NULL)
     {
         Log(LOG_LEVEL_ERR, "No environment_host defined for environment promise");
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
 
-    switch (Str2Hypervisors(a.env.type))
+    switch (Str2Hypervisors(env.type))
     {
     case cfv_virt_xen_net:
     case cfv_virt_kvm_net:
     case cfv_virt_esx_net:
     case cfv_virt_test_net:
-        if (a.env.cpus != CF_NOINT || a.env.memory != CF_NOINT || a.env.disk != CF_NOINT || a.env.name
-            || a.env.addresses)
+        if (env.cpus != CF_NOINT || env.memory != CF_NOINT || env.disk != CF_NOINT || env.name
+            || env.addresses)
         {
-            Log(LOG_LEVEL_ERR, "Network environment promises computational resources (%d,%d,%d,%s)", a.env.cpus,
-                  a.env.memory, a.env.disk, a.env.name);
+            Log(LOG_LEVEL_ERR, "Network environment promises computational resources (%d,%d,%d,%s)", env.cpus,
+                  env.memory, env.disk, env.name);
             PromiseRef(LOG_LEVEL_ERR, pp);
         }
         break;
@@ -199,12 +201,13 @@ static int EnvironmentsSanityChecks(Attributes a, const Promise *pp)
 
 /*****************************************************************************/
 
-static PromiseResult VerifyEnvironments(EvalContext *ctx, Attributes a, const Promise *pp)
+static PromiseResult VerifyEnvironments(EvalContext *ctx, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     char hyper_uri[CF_MAXVARSIZE];
     enum cfhypervisors envtype = cfv_none;
-
-    switch (Str2Hypervisors(a.env.type))
+    Environments env= a->env;
+    switch (Str2Hypervisors(env.type))
     {
     case cfv_virt_xen:
     case cfv_virt_xen_net:
@@ -240,24 +243,24 @@ static PromiseResult VerifyEnvironments(EvalContext *ctx, Attributes a, const Pr
         break;
 
     default:
-        Log(LOG_LEVEL_ERR, "Environment type '%s' not currently supported", a.env.type);
+        Log(LOG_LEVEL_ERR, "Environment type '%s' not currently supported", env.type);
         return PROMISE_RESULT_NOOP;
         break;
     }
 
-    Log(LOG_LEVEL_VERBOSE, "Selecting environment type '%s' '%s'", a.env.type, hyper_uri);
+    Log(LOG_LEVEL_VERBOSE, "Selecting environment type '%s' '%s'", env.type, hyper_uri);
 
-    ClassRef environment_host_ref = ClassRefParse(a.env.host);
+    ClassRef environment_host_ref = ClassRefParse(env.host);
     if (!EvalContextClassGet(ctx, environment_host_ref.ns, environment_host_ref.name))
     {
-        switch (a.env.state)
+        switch (env.state)
         {
         case ENVIRONMENT_STATE_CREATE:
         case ENVIRONMENT_STATE_RUNNING:
             Log(LOG_LEVEL_VERBOSE,
                 "This host ''%s' is not the promised host for the environment '%s', so setting its intended state to 'down'",
-                  VFQNAME, a.env.host);
-            a.env.state = ENVIRONMENT_STATE_DOWN;
+                  VFQNAME, env.host);
+            env.state = ENVIRONMENT_STATE_DOWN;
             break;
         default:
             Log(LOG_LEVEL_VERBOSE,
@@ -270,7 +273,7 @@ static PromiseResult VerifyEnvironments(EvalContext *ctx, Attributes a, const Pr
 
     PromiseResult result = PROMISE_RESULT_NOOP;
 #if defined(__linux__)
-    switch (Str2Hypervisors(a.env.type))
+    switch (Str2Hypervisors(env.type))
     {
     case cfv_virt_xen:
     case cfv_virt_kvm:
@@ -293,7 +296,7 @@ static PromiseResult VerifyEnvironments(EvalContext *ctx, Attributes a, const Pr
         break;
     }
 #elif defined(__APPLE__)
-    switch (Str2Hypervisors(a.env.type))
+    switch (Str2Hypervisors(env.type))
     {
     case cfv_virt_vbox:
     case cfv_virt_test:
@@ -309,7 +312,7 @@ static PromiseResult VerifyEnvironments(EvalContext *ctx, Attributes a, const Pr
         break;
     }
 #elif defined(__sun)
-    switch (Str2Hypervisors(a.env.type))
+    switch (Str2Hypervisors(env.type))
     {
     case cfv_zone:
         result = PromiseResultUpdate(result, VerifyZone(a, pp));
@@ -327,8 +330,9 @@ static PromiseResult VerifyEnvironments(EvalContext *ctx, Attributes a, const Pr
 /* Level                                                                     */
 /*****************************************************************************/
 
-static PromiseResult VerifyVirtDomain(EvalContext *ctx, char *uri, enum cfhypervisors envtype, Attributes a, const Promise *pp)
+static PromiseResult VerifyVirtDomain(EvalContext *ctx, char *uri, enum cfhypervisors envtype, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     int num, i;
 
 /* set up the library error handler */
@@ -357,7 +361,7 @@ static PromiseResult VerifyVirtDomain(EvalContext *ctx, char *uri, enum cfhyperv
     ShowDormant();
 
     PromiseResult result = PROMISE_RESULT_NOOP;
-    switch (a.env.state)
+    switch (a->env.state)
     {
     case ENVIRONMENT_STATE_CREATE:
         result = PromiseResultUpdate(result, CreateVirtDom(ctx, CFVC[envtype], a, pp));
@@ -384,8 +388,9 @@ static PromiseResult VerifyVirtDomain(EvalContext *ctx, char *uri, enum cfhyperv
 
 /*****************************************************************************/
 
-static PromiseResult VerifyVirtNetwork(EvalContext *ctx, char *uri, enum cfhypervisors envtype, Attributes a, const Promise *pp)
+static PromiseResult VerifyVirtNetwork(EvalContext *ctx, char *uri, enum cfhypervisors envtype, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     int num, i;
     char *networks[CF_MAX_CONCURRENT_ENVIRONMENTS];
 
@@ -408,7 +413,7 @@ static PromiseResult VerifyVirtNetwork(EvalContext *ctx, char *uri, enum cfhyper
     Log(LOG_LEVEL_VERBOSE, "Detected %d active networks", num);
 
     PromiseResult result = PROMISE_RESULT_NOOP;
-    switch (a.env.state)
+    switch (a->env.state)
     {
     case ENVIRONMENT_STATE_CREATE:
         result = PromiseResultUpdate(result, CreateVirtNetwork(ctx, CFVC[envtype], networks, a, pp));
@@ -430,8 +435,9 @@ static PromiseResult VerifyVirtNetwork(EvalContext *ctx, char *uri, enum cfhyper
 /* Level                                                                     */
 /*****************************************************************************/
 
-static PromiseResult CreateVirtDom(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp)
+static PromiseResult CreateVirtDom(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     int alloc_file = false;
     char *xml_file;
     const char *name;
@@ -473,9 +479,10 @@ static PromiseResult CreateVirtDom(EvalContext *ctx, virConnectPtr vc, Attribute
         }
     }
 
-    if(a.env.spec)
+    const Environments env = a->env;
+    if (env.spec)
     {
-        xml_file = xstrdup(a.env.spec);
+        xml_file = xstrdup(env.spec);
         alloc_file = true;
     }
     else
@@ -490,7 +497,7 @@ static PromiseResult CreateVirtDom(EvalContext *ctx, virConnectPtr vc, Attribute
         cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_CHANGE, pp, a, "Created a virtual domain '%s'", pp->promiser);
         result = PromiseResultUpdate(result, PROMISE_RESULT_CHANGE);
 
-        if (a.env.cpus != CF_NOINT)
+        if (env.cpus != CF_NOINT)
         {
             int maxcpus;
 
@@ -500,24 +507,24 @@ static PromiseResult CreateVirtDom(EvalContext *ctx, virConnectPtr vc, Attribute
             }
             else
             {
-                if (a.env.cpus > maxcpus)
+                if (env.cpus > maxcpus)
                 {
                     Log(LOG_LEVEL_INFO,
                           "The promise to allocate %d CPUs in domain '%s' cannot be kept - only %d exist on the host",
-                          a.env.cpus, pp->promiser, maxcpus);
+                          env.cpus, pp->promiser, maxcpus);
                 }
-                else if (virDomainSetVcpus(dom, (unsigned int) a.env.cpus) == -1)
+                else if (virDomainSetVcpus(dom, (unsigned int) env.cpus) == -1)
                 {
-                    Log(LOG_LEVEL_INFO, "Unable to adjust CPU count to %d", a.env.cpus);
+                    Log(LOG_LEVEL_INFO, "Unable to adjust CPU count to %d", env.cpus);
                 }
                 else
                 {
-                    Log(LOG_LEVEL_INFO, "Verified that environment CPU count is now %d", a.env.cpus);
+                    Log(LOG_LEVEL_INFO, "Verified that environment CPU count is now %d", env.cpus);
                 }
             }
         }
 
-        if (a.env.memory != CF_NOINT)
+        if (env.memory != CF_NOINT)
         {
             unsigned long maxmem;
 
@@ -527,23 +534,23 @@ static PromiseResult CreateVirtDom(EvalContext *ctx, virConnectPtr vc, Attribute
             }
             else
             {
-                if (virDomainSetMaxMemory(dom, (unsigned long) a.env.memory) == -1)
+                if (virDomainSetMaxMemory(dom, (unsigned long) env.memory) == -1)
                 {
-                    Log(LOG_LEVEL_INFO, " Unable to set the memory limit to %d", a.env.memory);
+                    Log(LOG_LEVEL_INFO, " Unable to set the memory limit to %d", env.memory);
                 }
                 else
                 {
-                    Log(LOG_LEVEL_INFO, "Setting the memory limit to %d", a.env.memory);
+                    Log(LOG_LEVEL_INFO, "Setting the memory limit to %d", env.memory);
                 }
 
-                if (virDomainSetMemory(dom, (unsigned long) a.env.memory) == -1)
+                if (virDomainSetMemory(dom, (unsigned long) env.memory) == -1)
                 {
-                    Log(LOG_LEVEL_INFO, " Unable to set the current memory to %d", a.env.memory);
+                    Log(LOG_LEVEL_INFO, " Unable to set the current memory to %d", env.memory);
                 }
             }
         }
 
-        if (a.env.disk != CF_NOINT)
+        if (env.disk != CF_NOINT)
         {
             Log(LOG_LEVEL_VERBOSE, "Info: env_disk parameter is not currently supported on this platform");
         }
@@ -573,8 +580,9 @@ static PromiseResult CreateVirtDom(EvalContext *ctx, virConnectPtr vc, Attribute
 
 /*****************************************************************************/
 
-static PromiseResult DeleteVirt(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp)
+static PromiseResult DeleteVirt(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     virDomainPtr dom;
 
     dom = virDomainLookupByName(vc, pp->promiser);
@@ -605,8 +613,9 @@ static PromiseResult DeleteVirt(EvalContext *ctx, virConnectPtr vc, Attributes a
 
 /*****************************************************************************/
 
-static PromiseResult RunningVirt(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp)
+static PromiseResult RunningVirt(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     virDomainPtr dom;
     virDomainInfo info;
 
@@ -692,36 +701,38 @@ static PromiseResult RunningVirt(EvalContext *ctx, virConnectPtr vc, Attributes
             break;
         }
 
-        if (a.env.cpus > 0)
+        const Environments env = a->env;
+
+        if (env.cpus > 0)
         {
-            if (virDomainSetVcpus(dom, a.env.cpus) == -1)
+            if (virDomainSetVcpus(dom, env.cpus) == -1)
             {
-                Log(LOG_LEVEL_INFO, " Unable to set the number of cpus to %d", a.env.cpus);
+                Log(LOG_LEVEL_INFO, " Unable to set the number of cpus to %d", env.cpus);
             }
             else
             {
-                Log(LOG_LEVEL_INFO, "Setting the number of virtual cpus to %d", a.env.cpus);
+                Log(LOG_LEVEL_INFO, "Setting the number of virtual cpus to %d", env.cpus);
             }
         }
 
-        if (a.env.memory != CF_NOINT)
+        if (env.memory != CF_NOINT)
         {
-            if (virDomainSetMaxMemory(dom, (unsigned long) a.env.memory) == -1)
+            if (virDomainSetMaxMemory(dom, (unsigned long) env.memory) == -1)
             {
-                Log(LOG_LEVEL_INFO, " Unable to set the memory limit to %d", a.env.memory);
+                Log(LOG_LEVEL_INFO, " Unable to set the memory limit to %d", env.memory);
             }
             else
             {
-                Log(LOG_LEVEL_INFO, "Setting the memory limit to %d", a.env.memory);
+                Log(LOG_LEVEL_INFO, "Setting the memory limit to %d", env.memory);
             }
 
-            if (virDomainSetMemory(dom, (unsigned long) a.env.memory) == -1)
+            if (virDomainSetMemory(dom, (unsigned long) env.memory) == -1)
             {
-                Log(LOG_LEVEL_INFO, " Unable to set the current memory to %d", a.env.memory);
+                Log(LOG_LEVEL_INFO, " Unable to set the current memory to %d", env.memory);
             }
         }
 
-        if (a.env.disk != CF_NOINT)
+        if (env.disk != CF_NOINT)
         {
             Log(LOG_LEVEL_VERBOSE, "Info: env_disk parameter is not currently supported on this platform");
         }
@@ -739,8 +750,9 @@ static PromiseResult RunningVirt(EvalContext *ctx, virConnectPtr vc, Attributes
 
 /*****************************************************************************/
 
-static PromiseResult SuspendedVirt(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp)
+static PromiseResult SuspendedVirt(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     virDomainPtr dom;
     virDomainInfo info;
 
@@ -824,7 +836,7 @@ static PromiseResult SuspendedVirt(EvalContext *ctx, virConnectPtr vc, Attribute
 
 /*****************************************************************************/
 
-static PromiseResult DownVirt(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp)
+static PromiseResult DownVirt(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp)
 {
     virDomainPtr dom;
     virDomainInfo info;
@@ -877,7 +889,7 @@ static PromiseResult DownVirt(EvalContext *ctx, virConnectPtr vc, Attributes a,
                      pp->promiser);
                 result = PromiseResultUpdate(result, PROMISE_RESULT_INTERRUPTED);
                 virDomainFree(dom);
-                return false;
+                return result;
             }
 
             cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_CHANGE, pp, a, "Virtual domain '%s' is in a crashed state, terminating",
@@ -904,8 +916,9 @@ static PromiseResult DownVirt(EvalContext *ctx, virConnectPtr vc, Attributes a,
 
 /*****************************************************************************/
 
-static PromiseResult CreateVirtNetwork(EvalContext *ctx, virConnectPtr vc, char **networks, Attributes a, const Promise *pp)
+static PromiseResult CreateVirtNetwork(EvalContext *ctx, virConnectPtr vc, char **networks, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     virNetworkPtr network;
     char *xml_file;
     char defaultxml[CF_MAXVARSIZE];
@@ -936,9 +949,9 @@ static PromiseResult CreateVirtNetwork(EvalContext *ctx, virConnectPtr vc, char
         return PROMISE_RESULT_NOOP;
     }
 
-    if (a.env.spec)
+    if (a->env.spec)
     {
-        xml_file = xstrdup(a.env.spec);
+        xml_file = xstrdup(a->env.spec);
     }
     else
     {
@@ -966,7 +979,7 @@ static PromiseResult CreateVirtNetwork(EvalContext *ctx, virConnectPtr vc, char
 
 /*****************************************************************************/
 
-static PromiseResult DeleteVirtNetwork(EvalContext *ctx, virConnectPtr vc, Attributes a, const Promise *pp)
+static PromiseResult DeleteVirtNetwork(EvalContext *ctx, virConnectPtr vc, const Attributes *a, const Promise *pp)
 {
     virNetworkPtr network;
 
