@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -34,7 +34,7 @@
 #include <files_interfaces.h>
 #include <files_lib.h>
 #include <files_operators.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <files_edit.h>
 #include <files_editxml.h>
 #include <files_editline.h>
@@ -62,9 +62,10 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
 
 /*****************************************************************************/
 
-static int FileSanityChecks(char *path, Attributes a, const Promise *pp)
+static bool FileSanityChecks(char *path, const Attributes *a, const Promise *pp)
 {
-    if ((a.havelink) && (a.havecopy))
+    assert(a != NULL);
+    if ((a->havelink) && (a->havecopy))
     {
         Log(LOG_LEVEL_ERR,
             "Promise constraint conflicts - '%s' file cannot both be a copy of and a link to the source", path);
@@ -76,14 +77,14 @@ static int FileSanityChecks(char *path, Attributes a, const Promise *pp)
      * body, so we can't distinguish between link and copy source. In
      * post-verification all bodies are already expanded, so we don't have the
      * information either */
-    if ((a.havelink) && (!a.link.source))
+    if ((a->havelink) && (!a->link.source))
     {
         Log(LOG_LEVEL_ERR, "Promise to establish a link at '%s' has no source", path);
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
 
-    if ((a.haveeditline) && (a.haveeditxml))
+    if ((a->haveeditline) && (a->haveeditxml))
     {
         Log(LOG_LEVEL_ERR, "Promise constraint conflicts - '%s' editing file as both line and xml makes no sense",
               path);
@@ -91,21 +92,21 @@ static int FileSanityChecks(char *path, Attributes a, const Promise *pp)
         return false;
     }
 
-    if ((a.havedepthsearch) && (a.haveedit))
+    if ((a->havedepthsearch) && (a->haveedit))
     {
         Log(LOG_LEVEL_ERR, "Recursive depth_searches are not compatible with general file editing");
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
 
-    if ((a.havedelete) && ((a.create) || (a.havecopy) || (a.haveedit) || (a.haverename)))
+    if ((a->havedelete) && ((a->create) || (a->havecopy) || (a->haveedit) || (a->haverename)))
     {
         Log(LOG_LEVEL_ERR, "Promise constraint conflicts - '%s' cannot be deleted and exist at the same time", path);
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
 
-    if ((a.haverename) && ((a.create) || (a.havecopy) || (a.haveedit)))
+    if ((a->haverename) && ((a->create) || (a->havecopy) || (a->haveedit)))
     {
         Log(LOG_LEVEL_ERR,
             "Promise constraint conflicts - '%s' cannot be renamed/moved and exist there at the same time", path);
@@ -113,7 +114,7 @@ static int FileSanityChecks(char *path, Attributes a, const Promise *pp)
         return false;
     }
 
-    if ((a.havedelete) && (a.havedepthsearch) && (!a.haveselect))
+    if ((a->havedelete) && (a->havedepthsearch) && (!a->haveselect))
     {
         Log(LOG_LEVEL_ERR,
             "Dangerous or ambiguous promise - '%s' specifies recursive deletion but has no file selection criteria",
@@ -122,21 +123,21 @@ static int FileSanityChecks(char *path, Attributes a, const Promise *pp)
         return false;
     }
 
-    if ((a.haveselect) && (!a.select.result))
+    if ((a->haveselect) && (!a->select.result))
     {
         Log(LOG_LEVEL_ERR, "File select constraint body promised no result (check body definition)");
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
 
-    if ((a.havedelete) && (a.haverename))
+    if ((a->havedelete) && (a->haverename))
     {
         Log(LOG_LEVEL_ERR, "File '%s' cannot promise both deletion and renaming", path);
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
 
-    if ((a.havecopy) && (a.havedepthsearch) && (a.havedelete))
+    if ((a->havecopy) && (a->havedepthsearch) && (a->havedelete))
     {
         Log(LOG_LEVEL_WARNING,
             "depth_search of '%s' applies to both delete and copy, but these refer to different searches (source/destination)",
@@ -144,14 +145,14 @@ static int FileSanityChecks(char *path, Attributes a, const Promise *pp)
         PromiseRef(LOG_LEVEL_INFO, pp);
     }
 
-    if ((a.transaction.background) && (a.transaction.audit))
+    if ((a->transaction.background) && (a->transaction.audit))
     {
         Log(LOG_LEVEL_ERR, "Auditing cannot be performed on backgrounded promises (this might change).");
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
 
-    if (((a.havecopy) || (a.havelink)) && (a.transformer))
+    if (((a->havecopy) || (a->havelink)) && (a->transformer))
     {
         Log(LOG_LEVEL_ERR, "File object(s) '%s' cannot both be a copy of source and transformed simultaneously",
               pp->promiser);
@@ -159,21 +160,21 @@ static int FileSanityChecks(char *path, Attributes a, const Promise *pp)
         return false;
     }
 
-    if ((a.haveselect) && (a.select.result == NULL))
+    if ((a->haveselect) && (a->select.result == NULL))
     {
         Log(LOG_LEVEL_ERR, "Missing file_result attribute in file_select body");
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
 
-    if ((a.havedepthsearch) && (a.change.report_diffs))
+    if ((a->havedepthsearch) && (a->change.report_diffs))
     {
         Log(LOG_LEVEL_ERR, "Difference reporting is not allowed during a depth_search");
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
 
-    if ((a.haveedit) && (a.file_type) && (!strncmp(a.file_type, "fifo", 5)))
+    if ((a->haveedit) && (a->file_type) && (!strncmp(a->file_type, "fifo", 5)))
     {
         Log(LOG_LEVEL_ERR, "Editing is not allowed on fifos");
         PromiseRef(LOG_LEVEL_ERR, pp);
@@ -183,14 +184,15 @@ static int FileSanityChecks(char *path, Attributes a, const Promise *pp)
     return true;
 }
 
-static bool AttrHasNoAction(Attributes attr)
+static bool AttrHasNoAction(const Attributes *attr)
 {
+    assert(attr != NULL);
     /* Hopefully this includes all "actions" for a files promise. See struct
      * Attributes for reference. */
-    if (!(attr.transformer || attr.haverename || attr.havedelete ||
-          attr.havecopy || attr.create || attr.touch || attr.havelink ||
-          attr.haveperms || attr.havechange || attr.acl.acl_entries ||
-          attr.haveedit || attr.haveeditline || attr.haveeditxml))
+    if (!(attr->transformer || attr->haverename || attr->havedelete ||
+          attr->havecopy || attr->create || attr->touch || attr->havelink ||
+          attr->haveperms || attr->havechange || attr->acl.acl_entries ||
+          attr->haveedit || attr->haveeditline || attr->haveeditxml))
     {
         return true;
     }
@@ -281,13 +283,13 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
 
     Attributes attr = GetFilesAttributes(ctx, pp);
 
-    if (!FileSanityChecks(path, attr, pp))
+    if (!FileSanityChecks(path, &attr, pp))
     {
         ClearFilesAttributes(&attr);
         return PROMISE_RESULT_NOOP;
     }
 
-    thislock = AcquireLock(ctx, path, VUQNAME, CFSTARTTIME, attr.transaction, pp, false);
+    thislock = AcquireLock(ctx, path, VUQNAME, CFSTARTTIME, attr.transaction.ifelapsed, attr.transaction.expireafter, pp, false);
     if (thislock.lock == NULL)
     {
         ClearFilesAttributes(&attr);
@@ -306,7 +308,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
     if (template_data_constraint != NULL &&
         template_data_constraint->rval.type != RVAL_TYPE_CONTAINER)
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a,
+        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, &a,
              "No template data for the promise '%s'", pp->promiser);
         result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
         goto exit;
@@ -316,7 +318,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
     {
         if ((a.create) || (a.touch))
         {
-            if (!CfCreateFile(ctx, path, pp, a, &result))
+            if (!CfCreateFile(ctx, path, pp, &a, &result))
             {
                 goto exit;
             }
@@ -332,7 +334,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
     {
         if ((a.create) || (a.touch))
         {
-            cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, a, "File '%s' exists as promised", path);
+            cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, &a, "File '%s' exists as promised", path);
         }
         exists = true;
         link = true;
@@ -340,7 +342,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
 
     if ((a.havedelete) && (!exists))
     {
-        cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, a, "File '%s' does not exist as promised", path);
+        cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, &a, "File '%s' does not exist as promised", path);
         goto exit;
     }
 
@@ -380,7 +382,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
      * (if we have one) then just exit. But continue if it's a directory and
      * depth_search is on, so that we can file_select into it. */
     if (exists
-        && (a.haveselect && !SelectLeaf(ctx, path, &oslb, a.select))
+        && (a.haveselect && !SelectLeaf(ctx, path, &oslb, &(a.select)))
         && !(a.havedepthsearch && S_ISDIR(oslb.st_mode)))
     {
         goto exit;
@@ -390,7 +392,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
     {
         if ((a.create) || (a.touch))
         {
-            if (!CfCreateFile(ctx, path, pp, a, &result))
+            if (!CfCreateFile(ctx, path, pp, &a, &result))
             {
                 goto exit;
             }
@@ -443,7 +445,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
     {
         lstat(path, &oslb);     /* if doesn't exist have to stat again anyway */
 
-        DepthSearch(ctx, path, &oslb, 0, a, pp, oslb.st_dev, &result);
+        DepthSearch(ctx, path, &oslb, 0, &a, pp, oslb.st_dev, &result);
 
         /* normally searches do not include the base directory */
 
@@ -454,7 +456,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
             /* Handle this node specially */
 
             a.havedepthsearch = false;
-            DepthSearch(ctx, path, &oslb, 0, a, pp, oslb.st_dev, &result);
+            DepthSearch(ctx, path, &oslb, 0, &a, pp, oslb.st_dev, &result);
             a.havedepthsearch = save_search;
         }
         else
@@ -462,7 +464,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
             /* unless child nodes were repaired, set a promise kept class */
             if (result == PROMISE_RESULT_NOOP)
             {
-                cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, a, "Basedir '%s' not promising anything", path);
+                cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, &a, "Basedir '%s' not promising anything", path);
             }
         }
     }
@@ -471,18 +473,18 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
 
     if (a.havecopy)
     {
-        result = PromiseResultUpdate(result, ScheduleCopyOperation(ctx, path, a, pp));
+        result = PromiseResultUpdate(result, ScheduleCopyOperation(ctx, path, &a, pp));
     }
 
 /* Phase 2b link after copy in case need file first */
 
     if ((a.havelink) && (a.link.link_children))
     {
-        result = PromiseResultUpdate(result, ScheduleLinkChildrenOperation(ctx, path, a.link.source, 1, a, pp));
+        result = PromiseResultUpdate(result, ScheduleLinkChildrenOperation(ctx, path, a.link.source, 1, &a, pp));
     }
     else if (a.havelink)
     {
-        result = PromiseResultUpdate(result, ScheduleLinkOperation(ctx, path, a.link.source, a, pp));
+        result = PromiseResultUpdate(result, ScheduleLinkOperation(ctx, path, a.link.source, &a, pp));
     }
 
 /* Phase 3 - content editing */
@@ -491,11 +493,11 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
     {
         if (exists)
         {
-            result = PromiseResultUpdate(result, ScheduleEditOperation(ctx, path, a, pp));
+            result = PromiseResultUpdate(result, ScheduleEditOperation(ctx, path, &a, pp));
         }
         else
         {
-            cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Promised to edit '%s', but file does not exist", path);
+            cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, &a, "Promised to edit '%s', but file does not exist", path);
             result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
         }
     }
@@ -505,20 +507,20 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
     exists = (stat(path, &osb) != -1);
 
     if (exists && (S_ISREG(osb.st_mode))
-        && (!a.haveselect || SelectLeaf(ctx, path, &osb, a.select)))
+        && (!a.haveselect || SelectLeaf(ctx, path, &osb, &(a.select))))
     {
-        VerifyFileLeaf(ctx, path, &osb, a, pp, &result);
+        VerifyFileLeaf(ctx, path, &osb, &a, pp, &result);
     }
 
     if (!exists && a.havechange)
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Promised to monitor '%s' for changes, but file does not exist", path);
+        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, &a, "Promised to monitor '%s' for changes, but file does not exist", path);
         result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
 
 exit:
 
-    if (AttrHasNoAction(a))
+    if (AttrHasNoAction(&a))
     {
         Log(LOG_LEVEL_INFO,
             "No action was requested for file '%s'. Maybe a typo in the policy?", path);
@@ -535,21 +537,23 @@ exit:
 /*****************************************************************************/
 
 static PromiseResult RenderTemplateCFEngine(EvalContext *ctx, const Promise *pp,
-                                            const Rlist *bundle_args, Attributes a,
+                                            const Rlist *bundle_args, const Attributes *attr,
                                             EditContext *edcontext)
 {
+    assert(attr != NULL);
+    Attributes a = *attr; // TODO: Try to remove this copy
     PromiseResult result = PROMISE_RESULT_NOOP;
 
     Policy *tmp_policy = PolicyNew();
     Bundle *bp = NULL;
-    if ((bp = MakeTemporaryBundleFromTemplate(ctx, tmp_policy, a, pp, &result)))
+    if ((bp = MakeTemporaryBundleFromTemplate(ctx, tmp_policy, &a, pp, &result)))
     {
         a.haveeditline = true;
 
         EvalContextStackPushBundleFrame(ctx, bp, bundle_args, a.edits.inherit);
         BundleResolve(ctx, bp);
 
-        ScheduleEditLineOperations(ctx, bp, a, pp, edcontext);
+        ScheduleEditLineOperations(ctx, bp, &a, pp, edcontext);
 
         EvalContextStackPopFrame(ctx);
 
@@ -566,8 +570,9 @@ static PromiseResult RenderTemplateCFEngine(EvalContext *ctx, const Promise *pp,
 
 static bool SaveBufferCallback(const char *dest_filename, void *param, NewLineMode new_line_mode)
 {
-    FILE *fp = safe_fopen(dest_filename, (new_line_mode == NewLineMode_Native) ? "wt" : "w");
-    if (!fp)
+    FILE *fp = safe_fopen(
+        dest_filename, (new_line_mode == NewLineMode_Native) ? "wt" : "w");
+    if (fp == NULL)
     {
         Log(LOG_LEVEL_ERR, "Unable to open destination file '%s' for writing. (fopen: %s)",
             dest_filename, GetErrorStr());
@@ -597,14 +602,19 @@ static bool SaveBufferCallback(const char *dest_filename, void *param, NewLineMo
 }
 
 
-static PromiseResult RenderTemplateMustache(EvalContext *ctx, const Promise *pp, Attributes a,
+static PromiseResult RenderTemplateMustache(EvalContext *ctx, const Promise *pp, const Attributes *attr,
                                             EditContext *edcontext, const char *template)
 {
+    assert(attr != NULL);
+    Attributes a = *attr; // TODO: Try to remove this copy
     PromiseResult result = PROMISE_RESULT_NOOP;
 
+    JsonElement *destroy_this = NULL;
+
     if (a.template_data == NULL)
     {
         a.template_data = DefaultTemplateData(ctx, NULL);
+        destroy_this = a.template_data;
     }
 
     unsigned char existing_output_digest[EVP_MAX_MD_SIZE + 1] = { 0 };
@@ -634,22 +644,24 @@ static PromiseResult RenderTemplateMustache(EvalContext *ctx, const Promise *pp,
         {
             if (a.transaction.action == cfa_warn || DONTDO)
             {
-                cfPS(ctx, LOG_LEVEL_WARNING, PROMISE_RESULT_WARN, pp, a,
+                cfPS(ctx, LOG_LEVEL_WARNING, PROMISE_RESULT_WARN, pp, &a,
                      "Need to update rendering of '%s' from mustache template '%s' but policy is dry-run",
                      pp->promiser, message);
                 result = PromiseResultUpdate(result, PROMISE_RESULT_WARN);
             }
             else
             {
-                if (SaveAsFile(SaveBufferCallback, output_buffer, edcontext->filename, a, edcontext->new_line_mode))
+                if (SaveAsFile(SaveBufferCallback, output_buffer, edcontext->filename, &a, edcontext->new_line_mode))
                 {
-                    cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_CHANGE, pp, a, "Updated rendering of '%s' from mustache template '%s'",
+                    cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_CHANGE, pp, &a, "Updated rendering of '%s' from mustache template '%s'",
                          pp->promiser, message);
                     result = PromiseResultUpdate(result, PROMISE_RESULT_CHANGE);
+
+                    edcontext->num_rewrites++;
                 }
                 else
                 {
-                    cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Failed to update rendering of '%s' from mustache template '%s'",
+                    cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, &a, "Failed to update rendering of '%s' from mustache template '%s'",
                          pp->promiser, message);
                     result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
                 }
@@ -658,31 +670,33 @@ static PromiseResult RenderTemplateMustache(EvalContext *ctx, const Promise *pp,
 
         BufferDestroy(output_buffer);
         free(message);
-
+        JsonDestroy(destroy_this);
         return result;
     }
     else
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Error rendering mustache template '%s'", a.edit_template);
+        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, &a, "Error rendering mustache template '%s'", a.edit_template);
         result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
         BufferDestroy(output_buffer);
+        JsonDestroy(destroy_this);
         return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
 }
 
-static PromiseResult RenderTemplateMustacheFromFile(EvalContext *ctx, const Promise *pp, Attributes a,
+static PromiseResult RenderTemplateMustacheFromFile(EvalContext *ctx, const Promise *pp, const Attributes *a,
                                             EditContext *edcontext)
 {
+    assert(a != NULL);
     PromiseResult result = PROMISE_RESULT_NOOP;
 
-    if (!FileCanOpen(a.edit_template, "r"))
+    if (!FileCanOpen(a->edit_template, "r"))
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Template file '%s' could not be opened for reading", a.edit_template);
+        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Template file '%s' could not be opened for reading", a->edit_template);
         return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
 
 
-    int template_fd = safe_open(a.edit_template, O_RDONLY | O_TEXT);
+    int template_fd = safe_open(a->edit_template, O_RDONLY | O_TEXT);
     Writer *template_writer = NULL;
     if (template_fd >= 0)
     {
@@ -691,7 +705,7 @@ static PromiseResult RenderTemplateMustacheFromFile(EvalContext *ctx, const Prom
     }
     if (template_writer == NULL)
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Could not read template file '%s'", a.edit_template);
+        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Could not read template file '%s'", a->edit_template);
         return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
 
@@ -702,11 +716,11 @@ static PromiseResult RenderTemplateMustacheFromFile(EvalContext *ctx, const Prom
     return result;
 }
 
-static PromiseResult RenderTemplateMustacheFromString(EvalContext *ctx, const Promise *pp, Attributes a,
+static PromiseResult RenderTemplateMustacheFromString(EvalContext *ctx, const Promise *pp, const Attributes *a,
                                             EditContext *edcontext)
 {
-
-    if ( a.edit_template_string == NULL  )
+    assert(a != NULL);
+    if ( a->edit_template_string == NULL  )
     {
         PromiseResult result = PROMISE_RESULT_NOOP;
 
@@ -714,11 +728,12 @@ static PromiseResult RenderTemplateMustacheFromString(EvalContext *ctx, const Pr
         return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
 
-    return  RenderTemplateMustache(ctx, pp, a, edcontext, a.edit_template_string);
+    return  RenderTemplateMustache(ctx, pp, a, edcontext, a->edit_template_string);
 }
 
-PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, Attributes a, const Promise *pp)
+PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     void *vp;
     FnCall *fp;
     Rlist *args = NULL;
@@ -726,7 +741,7 @@ PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, Attributes
     CfLock thislock;
 
     snprintf(lockname, CF_BUFSIZE - 1, "fileedit-%s", filename);
-    thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME, a.transaction, pp, false);
+    thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME, a->transaction.ifelapsed, a->transaction.expireafter, pp, false);
 
     if (thislock.lock == NULL)
     {
@@ -745,7 +760,7 @@ PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, Attributes
 
     const Policy *policy = PolicyFromPromise(pp);
 
-    if (a.haveeditline)
+    if (a->haveeditline)
     {
         if ((vp = PromiseGetConstraintAsRval(pp, "edit_line", RVAL_TYPE_FNCALL)))
         {
@@ -768,7 +783,7 @@ PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, Attributes
         const Bundle *bp = EvalContextResolveBundleExpression(ctx, policy, edit_bundle_name, "edit_line");
         if (bp)
         {
-            EvalContextStackPushBundleFrame(ctx, bp, args, a.edits.inherit);
+            EvalContextStackPushBundleFrame(ctx, bp, args, a->edits.inherit);
 
             BundleResolve(ctx, bp);
 
@@ -783,7 +798,7 @@ PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, Attributes
     }
 
 
-    if (a.haveeditxml)
+    if (a->haveeditxml)
     {
         if ((vp = PromiseGetConstraintAsRval(pp, "edit_xml", RVAL_TYPE_FNCALL)))
         {
@@ -806,7 +821,7 @@ PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, Attributes
         const Bundle *bp = EvalContextResolveBundleExpression(ctx, policy, edit_bundle_name, "edit_xml");
         if (bp)
         {
-            EvalContextStackPushBundleFrame(ctx, bp, args, a.edits.inherit);
+            EvalContextStackPushBundleFrame(ctx, bp, args, a->edits.inherit);
             BundleResolve(ctx, bp);
 
             ScheduleEditXmlOperations(ctx, bp, a, pp, edcontext);
@@ -815,34 +830,34 @@ PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, Attributes
         }
     }
 
-    if (strcmp("cfengine", a.template_method) == 0)
+    if (strcmp("cfengine", a->template_method) == 0)
     {
-        if (a.edit_template)
+        if (a->edit_template)
         {
             Log(LOG_LEVEL_VERBOSE, "Rendering '%s' using template '%s' with method '%s'",
-                filename, a.edit_template, a.template_method);
+                filename, a->edit_template, a->template_method);
 
             PromiseResult render_result = RenderTemplateCFEngine(ctx, pp, args, a, edcontext);
             result = PromiseResultUpdate(result, render_result);
         }
     }
-    else if (strcmp("mustache", a.template_method) == 0)
+    else if (strcmp("mustache", a->template_method) == 0)
     {
-        if (a.edit_template)
+        if (a->edit_template)
         {
             Log(LOG_LEVEL_VERBOSE, "Rendering '%s' using template '%s' with method '%s'",
-                filename, a.edit_template, a.template_method);
+                filename, a->edit_template, a->template_method);
 
             PromiseResult render_result = RenderTemplateMustacheFromFile(ctx, pp, a, edcontext);
             result = PromiseResultUpdate(result, render_result);
         }
     }
-    else if (strcmp("inline_mustache", a.template_method) == 0)
+    else if (strcmp("inline_mustache", a->template_method) == 0)
     {
-        if (a.edit_template_string)
+        if (a->edit_template_string)
         {
             Log(LOG_LEVEL_VERBOSE, "Rendering '%s' with method '%s'",
-                filename, a.template_method);
+                filename, a->template_method);
 
             PromiseResult render_result = RenderTemplateMustacheFromString(ctx, pp, a, edcontext);
             result = PromiseResultUpdate(result, render_result);
