@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -30,12 +30,12 @@
 #include <client_code.h>
 #include <files_interfaces.h>
 #include <files_lib.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <misc_lib.h>
 #include <eval_context.h>
 #include <known_dirs.h>
 
-int CompareFileHashes(const char *file1, const char *file2, struct stat *sstat, struct stat *dstat, FileCopy fc, AgentConnection *conn)
+bool CompareFileHashes(const char *file1, const char *file2, const struct stat *sstat, const struct stat *dstat, const FileCopy *fc, AgentConnection *conn)
 {
     unsigned char digest1[EVP_MAX_MD_SIZE + 1] = { 0 }, digest2[EVP_MAX_MD_SIZE + 1] = { 0 };
     int i;
@@ -64,12 +64,12 @@ int CompareFileHashes(const char *file1, const char *file2, struct stat *sstat,
     }
     else
     {
-        assert(fc.servers && strcmp(RlistScalarValue(fc.servers), "localhost"));
-        return CompareHashNet(file1, file2, fc.encrypt, conn);  /* client.c */
+        assert(fc->servers && strcmp(RlistScalarValue(fc->servers), "localhost"));
+        return CompareHashNet(file1, file2, fc->encrypt, conn);  /* client.c */
     }
 }
 
-int CompareBinaryFiles(const char *file1, const char *file2, struct stat *sstat, struct stat *dstat, FileCopy fc, AgentConnection *conn)
+bool CompareBinaryFiles(const char *file1, const char *file2, const struct stat *sstat, const struct stat *dstat, const FileCopy *fc, AgentConnection *conn)
 {
     int fd1, fd2, bytes1, bytes2;
     char buff1[BUFSIZ], buff2[BUFSIZ];
@@ -82,8 +82,8 @@ int CompareBinaryFiles(const char *file1, const char *file2, struct stat *sstat,
 
     if (conn == NULL)
     {
-        fd1 = safe_open(file1, O_RDONLY | O_BINARY, 0400);
-        fd2 = safe_open(file2, O_RDONLY | O_BINARY, 0400);
+        fd1 = safe_open(file1, O_RDONLY | O_BINARY);
+        fd2 = safe_open(file2, O_RDONLY | O_BINARY);
 
         do
         {
@@ -106,8 +106,8 @@ int CompareBinaryFiles(const char *file1, const char *file2, struct stat *sstat,
     }
     else
     {
-        assert(fc.servers && strcmp(RlistScalarValue(fc.servers), "localhost"));
+        assert(fc->servers && strcmp(RlistScalarValue(fc->servers), "localhost"));
         Log(LOG_LEVEL_DEBUG, "Using network checksum instead");
-        return CompareHashNet(file1, file2, fc.encrypt, conn);  /* client.c */
+        return CompareHashNet(file1, file2, fc->encrypt, conn);  /* client.c */
     }
 }
