@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -30,7 +30,6 @@
 #include <expand.h>
 #include <files_names.h>
 #include <scope.h>
-#include <hashes.h>
 #include <unix.h>
 #include <attributes.h>
 #include <locks.h>
@@ -72,7 +71,7 @@ PromiseResult VerifyMethodsPromise(EvalContext *ctx, const Promise *pp)
         destroy_name = true;
     }
 
-    PromiseResult result = VerifyMethod(ctx, method_name, a, pp);
+    PromiseResult result = VerifyMethod(ctx, method_name, &a, pp);
 
     if (destroy_name)
     {
@@ -84,7 +83,7 @@ PromiseResult VerifyMethodsPromise(EvalContext *ctx, const Promise *pp)
 
 /*****************************************************************************/
 
-PromiseResult VerifyMethod(EvalContext *ctx, const Rval call, Attributes a, const Promise *pp)
+PromiseResult VerifyMethod(EvalContext *ctx, const Rval call, const Attributes *a, const Promise *pp)
 {
     const Rlist *args = NULL;
     Buffer *method_name = BufferNew();
@@ -121,7 +120,7 @@ PromiseResult VerifyMethod(EvalContext *ctx, const Rval call, Attributes a, cons
     char lockname[CF_BUFSIZE];
     GetLockName(lockname, "method", pp->promiser, args);
 
-    CfLock thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME, a.transaction, pp, false);
+    CfLock thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME, a->transaction.ifelapsed, a->transaction.expireafter, pp, false);
     if (thislock.lock == NULL)
     {
         BufferDestroy(method_name);
@@ -141,7 +140,7 @@ PromiseResult VerifyMethod(EvalContext *ctx, const Rval call, Attributes a, cons
 
     if (bp)
     {
-        if (a.transaction.action == cfa_warn) // don't skip for dry-runs (ie ignore DONTDO)
+        if (a->transaction.action == cfa_warn) // don't skip for dry-runs (ie ignore DONTDO)
         {
             result = PROMISE_RESULT_WARN;
             cfPS(ctx, LOG_LEVEL_WARNING, result, pp, a, "Bundle '%s' should be invoked, but only a warning was promised!", BufferData(method_name));
@@ -150,7 +149,7 @@ PromiseResult VerifyMethod(EvalContext *ctx, const Rval call, Attributes a, cons
         {
             BundleBanner(bp, args);
             EvalContextSetBundleArgs(ctx, args);
-            EvalContextStackPushBundleFrame(ctx, bp, args, a.inherit);
+            EvalContextStackPushBundleFrame(ctx, bp, args, a->inherit);
 
             /* Clear all array-variables that are already set in the sub-bundle.
                Otherwise, array-data accumulates between multiple bundle evaluations.
