@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -29,28 +29,29 @@
 #include <locks.h>
 #include <ornaments.h>
 
-static bool NewPackagePromiseSanityCheck(Attributes a)
+static bool NewPackagePromiseSanityCheck(const Attributes *a)
 {
-    if (!a.new_packages.module_body || !a.new_packages.module_body->name)
+    assert(a != NULL);
+    if (!a->new_packages.module_body || !a->new_packages.module_body->name)
     {
         Log(LOG_LEVEL_ERR, "Can not find package module body in policy.");
         return false;
     }
-    
-    if (a.new_packages.module_body->updates_ifelapsed == CF_NOINT ||
-        a.new_packages.module_body->installed_ifelapsed == CF_NOINT)
+
+    if (a->new_packages.module_body->updates_ifelapsed == CF_NOINT ||
+        a->new_packages.module_body->installed_ifelapsed == CF_NOINT)
     {
         Log(LOG_LEVEL_ERR,
                 "Invalid or missing arguments in package_module body '%s':  "
                 "query_installed_ifelapsed = %d query_updates_ifelapsed = %d",
-                a.new_packages.module_body->name, 
-                a.new_packages.module_body->installed_ifelapsed,
-                a.new_packages.module_body->updates_ifelapsed);
+                a->new_packages.module_body->name,
+                a->new_packages.module_body->installed_ifelapsed,
+                a->new_packages.module_body->updates_ifelapsed);
             return false;
         return false;
     }
-    
-    if (a.new_packages.package_policy == NEW_PACKAGE_ACTION_NONE)
+
+    if (a->new_packages.package_policy == NEW_PACKAGE_ACTION_NONE)
     {
         Log(LOG_LEVEL_ERR, "Unsupported package policy in package promise.");
         return false;
@@ -59,12 +60,13 @@ static bool NewPackagePromiseSanityCheck(Attributes a)
 }
 
 PromiseResult HandleNewPackagePromiseType(EvalContext *ctx, const Promise *pp,
-                                          Attributes a, char **promise_log_msg,
+                                          const Attributes *a, char **promise_log_msg,
                                           LogLevel *log_lvl)
 {
+    assert(a != NULL);
     Log(LOG_LEVEL_DEBUG, "New package promise handler");
-    
-    
+
+
     if (!NewPackagePromiseSanityCheck(a))
     {
         *promise_log_msg =
@@ -72,15 +74,15 @@ PromiseResult HandleNewPackagePromiseType(EvalContext *ctx, const Promise *pp,
         *log_lvl = LOG_LEVEL_ERR;
         return PROMISE_RESULT_FAIL;
     }
-    
+
     PromiseBanner(ctx, pp);
-    
+
     PackagePromiseGlobalLock global_lock = AcquireGlobalPackagePromiseLock(ctx);
-    
+
     CfLock package_promise_lock;
     char promise_lock[CF_BUFSIZE];
     snprintf(promise_lock, sizeof(promise_lock), "new-package-%s-%s",
-             pp->promiser, a.new_packages.module_body->name);
+             pp->promiser, a->new_packages.module_body->name);
 
     if (global_lock.g_lock.lock == NULL)
     {
@@ -88,51 +90,51 @@ PromiseResult HandleNewPackagePromiseType(EvalContext *ctx, const Promise *pp,
                 SafeStringDuplicate("Can not acquire global lock for package "
                                     "promise. Skipping promise evaluation");
         *log_lvl = LOG_LEVEL_INFO;
-        
+
         return PROMISE_RESULT_SKIPPED;
     }
-    
+
     package_promise_lock =
             AcquireLock(ctx, promise_lock, VUQNAME, CFSTARTTIME,
-            a.transaction, pp, false);
+            a->transaction.ifelapsed, a->transaction.expireafter, pp, false);
     if (package_promise_lock.lock == NULL)
     {
         Log(LOG_LEVEL_DEBUG, "Skipping promise execution due to locking.");
         YieldGlobalPackagePromiseLock(global_lock);
-        
+
         *promise_log_msg =
                 StringFormat("Can not acquire lock for '%s' package promise. "
                              "Skipping promise evaluation",  pp->promiser);
         *log_lvl = LOG_LEVEL_VERBOSE;
-        
+
         return PROMISE_RESULT_SKIPPED;
     }
-    
+
     PackageModuleWrapper *package_module =
-            NewPackageModuleWrapper(a.new_packages.module_body);
-    
+            NewPackageModuleWrapper(a->new_packages.module_body);
+
     if (!package_module)
     {
         *promise_log_msg =
                 StringFormat("Some error occurred while contacting package "
                              "module - promise: %s", pp->promiser);
         *log_lvl = LOG_LEVEL_ERR;
-        
+
         YieldCurrentLock(package_promise_lock);
         YieldGlobalPackagePromiseLock(global_lock);
-    
+
         return PROMISE_RESULT_FAIL;
     }
-    
+
     PromiseResult result = PROMISE_RESULT_FAIL;
-    
-    switch (a.new_packages.package_policy)
+
+    switch (a->new_packages.package_policy)
     {
         case NEW_PACKAGE_ACTION_ABSENT:
-            result = HandleAbsentPromiseAction(ctx, pp->promiser, 
-                                               &a.new_packages,
+            result = HandleAbsentPromiseAction(ctx, pp->promiser,
+                                               &a->new_packages,
                                                package_module,
-                                               a.transaction.action);
+                                               a->transaction.action);
 
             switch (result)
             {
@@ -170,10 +172,10 @@ PromiseResult HandleNewPackagePromiseType(EvalContext *ctx, const Promise *pp,
             }
             break;
         case NEW_PACKAGE_ACTION_PRESENT:
-            result = HandlePresentPromiseAction(ctx, pp->promiser, 
-                                                &a.new_packages,
+            result = HandlePresentPromiseAction(ctx, pp->promiser,
+                                                &a->new_packages,
                                                 package_module,
-                                                a.transaction.action);
+                                                a->transaction.action);
 
             switch (result)
             {
@@ -214,14 +216,14 @@ PromiseResult HandleNewPackagePromiseType(EvalContext *ctx, const Promise *pp,
         case NEW_PACKAGE_ACTION_NONE:
         default:
             ProgrammingError("Unsupported package action: %d",
-                             a.new_packages.package_policy);
+                             a->new_packages.package_policy);
             break;
     }
-    
+
     DeletePackageModuleWrapper(package_module);
-    
+
     YieldCurrentLock(package_promise_lock);
     YieldGlobalPackagePromiseLock(global_lock);
-    
+
     return result;
 }
