@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -42,48 +42,50 @@
 #include <scope.h>
 #include <ornaments.h>
 
-static PromiseResult VerifyProcesses(EvalContext *ctx, Attributes a, const Promise *pp);
-static bool ProcessSanityChecks(Attributes a, const Promise *pp);
-static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promise *pp);
-static int FindPidMatches(Item **killlist, Attributes a, const char *promiser);
+static PromiseResult VerifyProcesses(EvalContext *ctx, const Attributes *a, const Promise *pp);
+static bool ProcessSanityChecks(const Attributes *a, const Promise *pp);
+static PromiseResult VerifyProcessOp(EvalContext *ctx, const Attributes *a, const Promise *pp);
+static int FindPidMatches(Item **killlist, const Attributes *a, const char *promiser);
 
 PromiseResult VerifyProcessesPromise(EvalContext *ctx, const Promise *pp)
 {
     Attributes a = GetProcessAttributes(ctx, pp);
-    ProcessSanityChecks(a, pp);
+    ProcessSanityChecks(&a, pp);
 
-    return VerifyProcesses(ctx, a, pp);
+    return VerifyProcesses(ctx, &a, pp);
 }
 
 /*****************************************************************************/
 /* Level                                                                     */
 /*****************************************************************************/
 
-static bool ProcessSanityChecks(Attributes a, const Promise *pp)
+static bool ProcessSanityChecks(const Attributes *a, const Promise *pp)
 {
-    int promised_zero, ret = true;
+    assert(a != NULL);
+    assert(pp != NULL);
+    // Note that an undefined match_value results in process_count.min_range and max_range being set to CF_NOINT
+    bool promised_zero = ((a->process_count.min_range == 0) && (a->process_count.max_range == 0));
+    bool ret = true;
 
-    promised_zero = ((a.process_count.min_range == 0) && (a.process_count.max_range == 0));
-
-    if (a.restart_class)
+    if (a->restart_class)
     {
-        if ((RlistKeyIn(a.signals, "term")) || (RlistKeyIn(a.signals, "kill")))
+        if ((RlistKeyIn(a->signals, "term")) || (RlistKeyIn(a->signals, "kill")))
         {
             Log(LOG_LEVEL_WARNING, "Promise '%s' kills then restarts - never strictly converges",
                 pp->promiser);
             PromiseRef(LOG_LEVEL_INFO, pp);
         }
 
-        if (a.haveprocess_count)
+        if (a->haveprocess_count)
         {
-            Log(LOG_LEVEL_ERR,
-                "process_count and restart_class should not be used in the same promise as this makes no sense");
-            PromiseRef(LOG_LEVEL_INFO, pp);
+            Log(LOG_LEVEL_VERBOSE,
+                "Note both process_count and restart_class define classes. Please exercises caution that there is not a logic error. Review process_count match_range, in_range_define, and out_of_range_define with respect to restart_class.");
+            PromiseRef(LOG_LEVEL_VERBOSE, pp);
             ret = false;
         }
     }
 
-    if (promised_zero && (a.restart_class))
+    if (promised_zero && (a->restart_class))
     {
         Log(LOG_LEVEL_ERR, "Promise constraint conflicts - '%s' processes cannot have zero count if restarted",
             pp->promiser);
@@ -91,33 +93,50 @@ static bool ProcessSanityChecks(Attributes a, const Promise *pp)
         ret = false;
     }
 
-    if ((a.haveselect) && (!a.process_select.process_result))
+    if ((a->haveselect) && (!a->process_select.process_result))
     {
         Log(LOG_LEVEL_ERR, "Process select constraint body promised no result (check body definition)");
         PromiseRef(LOG_LEVEL_ERR, pp);
-        return false;
+        ret = false;
     }
 
+    // CF_NOINT is the value assigned when match_range is not specified
+    if ((a->haveprocess_count) && ((a->process_count.min_range == CF_NOINT) || (a->process_count.max_range == CF_NOINT)))
+    {
+        Log(LOG_LEVEL_ERR, "process_count used without match_range, this promise will not define a class based on a count of matching promises.");
+        PromiseRef(LOG_LEVEL_ERR, pp);
+        ret = false;
+    }
+
+    if ((a->haveprocess_count) && !((a->process_count.in_range_define) || (a->process_count.out_of_range_define)))
+    {
+        Log(LOG_LEVEL_ERR, "process_count body is insufficiently defined. process_count must specify at least one of in_range_define or out_of_range_define. ");
+        PromiseRef(LOG_LEVEL_ERR, pp);
+        ret = false;
+    }
+
+
     return ret;
 }
 
 /*****************************************************************************/
 
-static PromiseResult VerifyProcesses(EvalContext *ctx, Attributes a, const Promise *pp)
+static PromiseResult VerifyProcesses(EvalContext *ctx, const Attributes *a, const Promise *pp)
 {
     CfLock thislock;
     char lockname[CF_BUFSIZE];
 
-    if (a.restart_class)
+    if (a->restart_class)
     {
-        snprintf(lockname, CF_BUFSIZE - 1, "proc-%s-%s", pp->promiser, a.restart_class);
+        snprintf(lockname, CF_BUFSIZE - 1, "proc-%s-%s", pp->promiser, a->restart_class);
     }
     else
     {
         snprintf(lockname, CF_BUFSIZE - 1, "proc-%s-norestart", pp->promiser);
     }
 
-    thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME, a.transaction, pp, false);
+    thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME,
+        a->transaction.ifelapsed, a->transaction.expireafter, pp, false);
     if (thislock.lock == NULL)
     {
         return PROMISE_RESULT_SKIPPED;
@@ -131,8 +150,38 @@ static PromiseResult VerifyProcesses(EvalContext *ctx, Attributes a, const Promi
     return result;
 }
 
-static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promise *pp)
+static void ProcessCountMaybeDefineClass(
+    EvalContext *const ctx,
+    const Rlist *const rp,
+    const Promise *const pp,
+    const char *const attribute_name)
 {
+    assert(ctx != NULL);
+    assert(rp != NULL);
+    assert(pp != NULL);
+    assert(attribute_name != NULL && strlen(attribute_name) > 0);
+
+    if (rp->val.type == RVAL_TYPE_FNCALL)
+    {
+        Log(LOG_LEVEL_ERR,
+            "Body (process_count), for processes promise %s, has a failing function call (%s) in the %s attribute - cannot define the class",
+            pp->promiser,
+            RlistFnCallValue(rp)->name,
+            attribute_name);
+        return;
+    }
+
+    assert(rp->val.type == RVAL_TYPE_SCALAR);
+    ClassRef ref = ClassRefParse(RlistScalarValue(rp));
+    EvalContextClassPutSoft(
+        ctx, RlistScalarValue(rp), CONTEXT_SCOPE_NAMESPACE, "source=promise");
+    ClassRefDestroy(ref);
+}
+
+static PromiseResult VerifyProcessOp(EvalContext *ctx, const Attributes *a, const Promise *pp)
+{
+    assert(a != NULL);
+    assert(pp != NULL);
     bool do_signals = true;
     int out_of_range;
     int killed = 0;
@@ -144,27 +193,23 @@ static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promi
 /* promise based on number of matches */
 
     PromiseResult result = PROMISE_RESULT_NOOP;
-    if (a.process_count.min_range != CF_NOINT)  /* if a range is specified */
+    if (a->process_count.min_range != CF_NOINT)  /* if a range is specified */
     {
-        if ((matches < a.process_count.min_range) || (matches > a.process_count.max_range))
+        if ((matches < a->process_count.min_range) || (matches > a->process_count.max_range))
         {
             cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_CHANGE, pp, a, "Process count for '%s' was out of promised range (%d found)", pp->promiser, matches);
             result = PromiseResultUpdate(result, PROMISE_RESULT_CHANGE);
-            for (const Rlist *rp = a.process_count.out_of_range_define; rp != NULL; rp = rp->next)
+            for (const Rlist *rp = a->process_count.out_of_range_define; rp != NULL; rp = rp->next)
             {
-                ClassRef ref = ClassRefParse(RlistScalarValue(rp));
-                EvalContextClassPutSoft(ctx, RlistScalarValue(rp), CONTEXT_SCOPE_NAMESPACE, "source=promise");
-                ClassRefDestroy(ref);
+                ProcessCountMaybeDefineClass(ctx, rp, pp, "out_of_range_define");
             }
             out_of_range = true;
         }
         else
         {
-            for (const Rlist *rp = a.process_count.in_range_define; rp != NULL; rp = rp->next)
+            for (const Rlist *rp = a->process_count.in_range_define; rp != NULL; rp = rp->next)
             {
-                ClassRef ref = ClassRefParse(RlistScalarValue(rp));
-                EvalContextClassPutSoft(ctx, RlistScalarValue(rp), CONTEXT_SCOPE_NAMESPACE, "source=promise");
-                ClassRefDestroy(ref);
+                ProcessCountMaybeDefineClass(ctx, rp, pp, "in_range_define");
             }
             cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, a, "Process promise for '%s' is kept", pp->promiser);
             out_of_range = false;
@@ -181,7 +226,7 @@ static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promi
         return result;
     }
 
-    if (a.transaction.action == cfa_warn)
+    if (a->transaction.action == cfa_warn)
     {
         do_signals = false;
         result = PromiseResultUpdate(result, PROMISE_RESULT_WARN);
@@ -195,7 +240,7 @@ static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promi
 
     if (do_signals && (matches > 0))
     {
-        if (a.process_stop != NULL)
+        if (a->process_stop != NULL)
         {
             if (DONTDO)
             {
@@ -205,15 +250,30 @@ static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promi
             }
             else
             {
-                if (IsExecutable(CommandArg0(a.process_stop)))
+                if (IsExecutable(CommandArg0(a->process_stop)))
                 {
-                    ShellCommandReturnsZero(a.process_stop, SHELL_TYPE_NONE);
+                    Log(LOG_LEVEL_DEBUG, "Found process_stop command '%s' is executable.", a->process_stop);
+
+                    if (ShellCommandReturnsZero(a->process_stop, SHELL_TYPE_NONE))
+                    {
+                        cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_CHANGE, pp, a,
+                             "Promise to stop '%s' repaired, '%s' returned zero",
+                             pp->promiser, a->process_stop);
+                        result = PromiseResultUpdate(result, PROMISE_RESULT_CHANGE);
+                    }
+                    else
+                    {
+                        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a,
+                             "Promise to stop '%s' failed, '%s' returned nonzero",
+                             pp->promiser, a->process_stop);
+                        result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
+                    }
                 }
                 else
                 {
                     cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a,
-                         "Process promise to stop '%s' could not be kept because '%s' the stop operator failed",
-                         pp->promiser, a.process_stop);
+                         "Process promise to stop '%s' could not be kept because '%s' is not executable",
+                         pp->promiser, a->process_stop);
                     result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
                     DeleteItemList(killlist);
                     return result;
@@ -226,7 +286,7 @@ static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promi
 
 /* delegated promise to restart killed or non-existent entries */
 
-    need_to_restart = (a.restart_class != NULL) && (killed || (matches == 0));
+    need_to_restart = (a->restart_class != NULL) && (killed || (matches == 0));
 
     DeleteItemList(killlist);
 
@@ -237,7 +297,7 @@ static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promi
     }
     else
     {
-        if (a.transaction.action == cfa_warn)
+        if (a->transaction.action == cfa_warn)
         {
             cfPS(ctx, LOG_LEVEL_WARNING, PROMISE_RESULT_WARN, pp, a,
                  "Need to keep restart promise for '%s', but only a warning is promised", pp->promiser);
@@ -247,9 +307,9 @@ static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promi
         {
             PromiseResult status = killed ? PROMISE_RESULT_CHANGE : PROMISE_RESULT_NOOP;
             cfPS(ctx, LOG_LEVEL_VERBOSE, status, pp, a,
-                 "C:     +  Global class: %s ", a.restart_class);
+                 "C:     +  Global class: %s ", a->restart_class);
             result = PromiseResultUpdate(result, status);
-            EvalContextClassPutSoft(ctx, a.restart_class, CONTEXT_SCOPE_NAMESPACE, "source=promise");
+            EvalContextClassPutSoft(ctx, a->restart_class, CONTEXT_SCOPE_NAMESPACE, "source=promise");
         }
     }
 
@@ -257,7 +317,7 @@ static PromiseResult VerifyProcessOp(EvalContext *ctx, Attributes a, const Promi
 }
 
 #ifndef __MINGW32__
-int DoAllSignals(EvalContext *ctx, Item *siglist, Attributes a, const Promise *pp, PromiseResult *result)
+int DoAllSignals(EvalContext *ctx, Item *siglist, const Attributes *a, const Promise *pp, PromiseResult *result)
 {
     Item *ip;
     Rlist *rp;
@@ -270,7 +330,7 @@ int DoAllSignals(EvalContext *ctx, Item *siglist, Attributes a, const Promise *p
         return 0;
     }
 
-    if (a.signals == NULL)
+    if (a->signals == NULL)
     {
         Log(LOG_LEVEL_VERBOSE, "No signals to send for '%s'", pp->promiser);
         return 0;
@@ -280,7 +340,7 @@ int DoAllSignals(EvalContext *ctx, Item *siglist, Attributes a, const Promise *p
     {
         pid = ip->counter;
 
-        for (rp = a.signals; rp != NULL; rp = rp->next)
+        for (rp = a->signals; rp != NULL; rp = rp->next)
         {
             int signal = SignalFromString(RlistScalarValue(rp));
 
@@ -324,22 +384,22 @@ int DoAllSignals(EvalContext *ctx, Item *siglist, Attributes a, const Promise *p
 }
 #endif
 
-static int FindPidMatches(Item **killlist, Attributes a, const char *promiser)
+static int FindPidMatches(Item **killlist, const Attributes *a, const char *promiser)
 {
     int matches = 0;
     pid_t cfengine_pid = getpid();
 
-    Item *matched = SelectProcesses(promiser, a.process_select, a.haveselect);
+    Item *matched = SelectProcesses(promiser, &(a->process_select), a->haveselect);
 
     for (Item *ip = matched; ip != NULL; ip = ip->next)
     {
         pid_t pid = ip->counter;
 
-        if (a.signals) /* There are some processes we don't want to signal. */
+        if (a->signals) /* There are some processes we don't want to signal. */
         {
             if (pid == 1)
             {
-                if (RlistLen(a.signals) == 1 && RlistKeyIn(a.signals, "hup"))
+                if (RlistLen(a->signals) == 1 && RlistKeyIn(a->signals, "hup"))
                 {
                     Log(LOG_LEVEL_VERBOSE, "Okay to send only HUP to init");
                 }
@@ -362,16 +422,16 @@ static int FindPidMatches(Item **killlist, Attributes a, const char *promiser)
             }
         }
 
-        bool promised_zero = (a.process_count.min_range == 0) && (a.process_count.max_range == 0);
+        bool promised_zero = (a->process_count.min_range == 0) && (a->process_count.max_range == 0);
 
-        if ((a.transaction.action == cfa_warn) && promised_zero)
+        if ((a->transaction.action == cfa_warn) && promised_zero)
         {
             Log(LOG_LEVEL_WARNING, "Process alert '%s'", GetProcessTableLegend());
             Log(LOG_LEVEL_WARNING, "Process alert '%s'", ip->name);
             continue;
         }
 
-        if (a.transaction.action == cfa_warn)
+        if (a->transaction.action == cfa_warn)
         {
             Log(LOG_LEVEL_WARNING, "Matched '%s'", ip->name);
         }
