@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -39,10 +39,10 @@
 #include <ornaments.h>
 #include <eval_context.h>
 
-static int ServicesSanityChecks(Attributes a, const Promise *pp);
+static bool ServicesSanityChecks(const Attributes *a, const Promise *pp);
 static void SetServiceDefaults(Attributes *a);
-static PromiseResult DoVerifyServices(EvalContext *ctx, Attributes a, const Promise *pp);
-static PromiseResult VerifyServices(EvalContext *ctx, Attributes a, const Promise *pp);
+static PromiseResult DoVerifyServices(EvalContext *ctx, const Attributes *attr, const Promise *pp);
+static PromiseResult VerifyServices(EvalContext *ctx, const Attributes *a, const Promise *pp);
 
 
 /*****************************************************************************/
@@ -53,9 +53,9 @@ PromiseResult VerifyServicesPromise(EvalContext *ctx, const Promise *pp)
 
     SetServiceDefaults(&a);
 
-    if (ServicesSanityChecks(a, pp))
+    if (ServicesSanityChecks(&a, pp))
     {
-        return VerifyServices(ctx, a, pp);
+        return VerifyServices(ctx, &a, pp);
     }
     else
     {
@@ -65,11 +65,11 @@ PromiseResult VerifyServicesPromise(EvalContext *ctx, const Promise *pp)
 
 /*****************************************************************************/
 
-static int ServicesSanityChecks(Attributes a, const Promise *pp)
+static bool ServicesSanityChecks(const Attributes *a, const Promise *pp)
 {
     Rlist *dep;
 
-    for (dep = a.service.service_depend; dep != NULL; dep = dep->next)
+    for (dep = a->service.service_depend; dep != NULL; dep = dep->next)
     {
         if (strcmp(pp->promiser, RlistScalarValue(dep)) == 0)
         {
@@ -79,7 +79,7 @@ static int ServicesSanityChecks(Attributes a, const Promise *pp)
         }
     }
 
-    if (a.service.service_type == NULL)
+    if (a->service.service_type == NULL)
     {
         Log(LOG_LEVEL_ERR, "Service type for service '%s' is not known", pp->promiser);
         PromiseRef(LOG_LEVEL_ERR, pp);
@@ -88,10 +88,10 @@ static int ServicesSanityChecks(Attributes a, const Promise *pp)
 
 #ifdef __MINGW32__
 
-    if (strcmp(a.service.service_type, "windows") != 0)
+    if (strcmp(a->service.service_type, "windows") != 0)
     {
         Log(LOG_LEVEL_ERR, "Service type for promiser '%s' must be 'windows' on this system, but is '%s'",
-            pp->promiser, a.service.service_type);
+            pp->promiser, a->service.service_type);
         PromiseRef(LOG_LEVEL_ERR, pp);
         return false;
     }
@@ -141,11 +141,12 @@ static void SetServiceDefaults(Attributes *a)
 /* Level                                                                     */
 /*****************************************************************************/
 
-static PromiseResult VerifyServices(EvalContext *ctx, Attributes a, const Promise *pp)
+static PromiseResult VerifyServices(EvalContext *ctx, const Attributes *a, const Promise *pp)
 {
+    assert(a != NULL);
     CfLock thislock;
 
-    thislock = AcquireLock(ctx, pp->promiser, VUQNAME, CFSTARTTIME, a.transaction, pp, false);
+    thislock = AcquireLock(ctx, pp->promiser, VUQNAME, CFSTARTTIME, a->transaction.ifelapsed, a->transaction.expireafter, pp, false);
     if (thislock.lock == NULL)
     {
         return PROMISE_RESULT_SKIPPED;
@@ -154,7 +155,7 @@ static PromiseResult VerifyServices(EvalContext *ctx, Attributes a, const Promis
     PromiseBanner(ctx, pp);
 
     PromiseResult result = PROMISE_RESULT_SKIPPED;
-    if (strcmp(a.service.service_type, "windows") == 0)
+    if (strcmp(a->service.service_type, "windows") == 0)
     {
 #ifdef __MINGW32__
         result = PromiseResultUpdate(result, VerifyWindowsService(ctx, a, pp));
@@ -193,14 +194,16 @@ static FnCall *DefaultServiceBundleCall(const Promise *pp, const char *service_p
     }
     else
     {
-        call = FnCallNew("standard_services", args);
+        call = FnCallNew("default:standard_services", args);
     }
 
     return call;
 }
 
-static PromiseResult DoVerifyServices(EvalContext *ctx, Attributes a, const Promise *pp)
+static PromiseResult DoVerifyServices(EvalContext *ctx, const Attributes *attr, const Promise *pp)
 {
+    assert(attr != NULL);
+    Attributes a = *attr; // TODO: Remove this local copy
     Rval call;
     {
         const Constraint *cp = PromiseGetConstraint(pp, "service_bundle");
@@ -218,7 +221,7 @@ static PromiseResult DoVerifyServices(EvalContext *ctx, Attributes a, const Prom
     EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "service_policy", a.service.service_policy, CF_DATA_TYPE_STRING, "source=promise");
 
     PromiseResult result = PROMISE_RESULT_NOOP;
-    result = PromiseResultUpdate(result, VerifyMethod(ctx, call, a, pp));  // Send list of classes to set privately?
+    result = PromiseResultUpdate(result, VerifyMethod(ctx, call, &a, pp));  // Send list of classes to set privately?
 
     RvalDestroy(call);
 
