@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -31,7 +31,7 @@
 #include <string_lib.h>
 #include <misc_lib.h>
 
-static int UserSanityCheck(Attributes a, const Promise *pp);
+static bool UserSanityCheck(const Attributes *a, const Promise *pp);
 
 PromiseResult VerifyUsersPromise(EvalContext *ctx, const Promise *pp)
 {
@@ -40,7 +40,7 @@ PromiseResult VerifyUsersPromise(EvalContext *ctx, const Promise *pp)
 
     Attributes a = GetUserAttributes(ctx, pp);
 
-    if (!UserSanityCheck(a, pp))
+    if (!UserSanityCheck(&a, pp))
     {
         return PROMISE_RESULT_FAIL;
     }
@@ -49,28 +49,29 @@ PromiseResult VerifyUsersPromise(EvalContext *ctx, const Promise *pp)
 
     snprintf(lockname, CF_BUFSIZE - 1, "user-%s-%d", pp->promiser, a.users.policy);
 
-    thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME, a.transaction, pp, false);
+    thislock = AcquireLock(ctx, lockname, VUQNAME, CFSTARTTIME,
+        a.transaction.ifelapsed, a.transaction.expireafter, pp, false);
     if (thislock.lock == NULL)
     {
         return PROMISE_RESULT_SKIPPED;
     }
 
     PromiseResult result = PROMISE_RESULT_NOOP;
-    VerifyOneUsersPromise(pp->promiser, a.users, &result, a.transaction.action, ctx, &a, pp);
+    VerifyOneUsersPromise(pp->promiser, &(a.users), &result, a.transaction.action, ctx, &a, pp);
 
     switch (result) {
     case PROMISE_RESULT_NOOP:
-        cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, a, "User promise kept");
+        cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, &a, "User promise kept");
         break;
     case PROMISE_RESULT_FAIL:
     case PROMISE_RESULT_DENIED:
     case PROMISE_RESULT_TIMEOUT:
     case PROMISE_RESULT_INTERRUPTED:
     case PROMISE_RESULT_WARN:
-        cfPS(ctx, LOG_LEVEL_INFO, result, pp, a, "User promise not kept");
+        cfPS(ctx, LOG_LEVEL_INFO, result, pp, &a, "User promise not kept");
         break;
     case PROMISE_RESULT_CHANGE:
-        cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_CHANGE, pp, a, "User promise repaired");
+        cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_CHANGE, pp, &a, "User promise repaired");
         break;
     default:
         ProgrammingError("Unknown promise result");
@@ -84,9 +85,10 @@ PromiseResult VerifyUsersPromise(EvalContext *ctx, const Promise *pp)
 
 /** Pre-check of promise contents **/
 
-static int UserSanityCheck(Attributes a, const Promise *pp)
+static bool UserSanityCheck(const Attributes *a, const Promise *pp)
 {
-    User *u = &a.users;
+    assert(a != NULL);
+    const User *u = &(a->users);
     switch (u->policy)
     {
     case USER_STATE_PRESENT:
