@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -121,7 +121,7 @@ static int PasswordSupplier(int num_msg, const struct pam_message **msg,
  */
 static bool GetAIXShadowHash(const char *puser, const char **result)
 {
-    FILE *fptr = fopen("/etc/security/passwd", "r");
+    FILE *fptr = safe_fopen("/etc/security/passwd", "r");
     if (fptr == NULL)
     {
         return false;
@@ -209,7 +209,7 @@ end:
 // is a local user, and not for example from LDAP.
 static struct spwd *GetSpEntry(const char *puser)
 {
-    FILE *fptr = fopen("/etc/shadow", "r");
+    FILE *fptr = safe_fopen("/etc/shadow", "r");
     if (!fptr)
     {
         Log(LOG_LEVEL_ERR, "Could not open '/etc/shadow': %s", GetErrorStr());
@@ -483,7 +483,7 @@ static bool ChangePasswordHashUsingLckpwdf(const char *puser, const char *passwo
         goto unlock_passwd;
     }
 
-    FILE *passwd_fd = fopen(passwd_file, "r");
+    FILE *passwd_fd = safe_fopen(passwd_file, "r");
     if (!passwd_fd)
     {
         Log(LOG_LEVEL_ERR, "Could not open password database '%s'. (fopen: '%s')", passwd_file, GetErrorStr());
@@ -774,19 +774,32 @@ static bool SetAccountLocked(const char *puser, const char *hash, bool lock)
 
     return SetAccountLockExpiration(puser, lock);
 }
+static void TransformGidsToGroups(StringSet **list);
 
-static bool GroupGetUserMembership (const char *user, StringSet *result)
+static bool GetGroupInfo (const char *user, const User *u, StringSet **groups_to_set, StringSet **groups_missing, StringSet **current_secondary_groups)
 {
     bool ret = true;
     struct group *group_info;
 
-    FILE *fptr = fopen("/etc/group", "r");
+    FILE *fptr = safe_fopen("/etc/group", "r");
     if (!fptr)
     {
         Log(LOG_LEVEL_ERR, "Could not open '/etc/group': %s", GetErrorStr());
         return false;
     }
 
+    StringSet *wanted_groups = StringSetNew();
+    if (u->groups_secondary_given)
+    {
+        for (Rlist *ptr = u->groups_secondary; ptr != NULL; ptr = ptr->next)
+        {
+            StringSetAdd(*groups_missing, xstrdup(RvalScalarValue(ptr->val)));
+            StringSetAdd(wanted_groups, xstrdup(RvalScalarValue(ptr->val)));
+        }
+        TransformGidsToGroups(groups_missing);
+        TransformGidsToGroups(&wanted_groups);
+    }
+
     while (true)
     {
         errno = 0;
@@ -806,16 +819,40 @@ static bool GroupGetUserMembership (const char *user, StringSet *result)
             }
             break;
         }
-        for (int i = 0; group_info->gr_mem[i] != NULL; i++)
+
+
+        if (StringSetContains(wanted_groups, group_info->gr_name))
         {
-            if (strcmp(user, group_info->gr_mem[i]) == 0)
+            StringSetRemove(*groups_missing, group_info->gr_name);
+        }
+
+        // At least on FreeBSD, gr_mem can be NULL:
+        if (group_info->gr_mem != NULL)
+        {
+            bool found = false;
+            for (int i = 0; !found && group_info->gr_mem[i] != NULL; i++)
             {
-                StringSetAdd(result, xstrdup(group_info->gr_name));
-                break;
+                if (strcmp(user, group_info->gr_mem[i]) == 0)
+                {
+                    found = true;
+                    StringSetAdd(*current_secondary_groups, xstrdup(group_info->gr_name));
+                    if (StringSetContains(wanted_groups, group_info->gr_name))
+                    {
+                        StringSetAdd(*groups_to_set, xstrdup(group_info->gr_name));
+                    }
+                }
+            }
+            if (!found && StringSetContains(wanted_groups, group_info->gr_name))
+            {
+                StringSetAdd(*groups_to_set, xstrdup(group_info->gr_name));
             }
         }
+#ifdef __FreeBSD__
+        free(group_info);
+#endif
     }
 
+    StringSetDestroy(wanted_groups);
     fclose(fptr);
 
     return ret;
@@ -836,7 +873,7 @@ static bool EqualGroupName(const char *key, const struct group *entry)
 static struct group *GetGrEntry(const char *key,
                                 bool (*equal_fn)(const char *key, const struct group *entry))
 {
-    FILE *fptr = fopen("/etc/group", "r");
+    FILE *fptr = safe_fopen("/etc/group", "r");
     if (!fptr)
     {
         Log(LOG_LEVEL_ERR, "Could not open '/etc/group': %s", GetErrorStr());
@@ -924,64 +961,69 @@ static void TransformGidsToGroups(StringSet **list)
     StringSetDestroy(old_list);
 }
 
-static bool VerifyIfUserNeedsModifs (const char *puser, User u, const struct passwd *passwd_info,
-                             uint32_t *changemap)
+static bool VerifyIfUserNeedsModifs (const char *puser, const User *u, const struct passwd *passwd_info,
+                             uint32_t *changemap, StringSet *groups_to_set, StringSet *current_secondary_groups)
 {
-    if (u.description != NULL && strcmp (u.description, passwd_info->pw_gecos))
+    assert(u != NULL);
+    if (u->description != NULL && strcmp (u->description, passwd_info->pw_gecos))
     {
         CFUSR_SETBIT (*changemap, i_comment);
     }
-    if (u.uid != NULL && (atoi (u.uid) != passwd_info->pw_uid))
+    if (u->uid != NULL && (atoi (u->uid) != passwd_info->pw_uid))
     {
         CFUSR_SETBIT (*changemap, i_uid);
     }
-    if (u.home_dir != NULL && strcmp (u.home_dir, passwd_info->pw_dir))
+    if (u->home_dir != NULL && strcmp (u->home_dir, passwd_info->pw_dir))
     {
         CFUSR_SETBIT (*changemap, i_home);
     }
-    if (u.shell != NULL && strcmp (u.shell, passwd_info->pw_shell))
+    if (u->shell != NULL && strcmp (u->shell, passwd_info->pw_shell))
     {
         CFUSR_SETBIT (*changemap, i_shell);
     }
     bool account_is_locked = IsAccountLocked(puser, passwd_info);
-    if ((!account_is_locked && u.policy == USER_STATE_LOCKED)
-        || (account_is_locked && u.policy != USER_STATE_LOCKED))
+    if ((!account_is_locked && u->policy == USER_STATE_LOCKED)
+        || (account_is_locked && u->policy != USER_STATE_LOCKED))
     {
         CFUSR_SETBIT(*changemap, i_locked);
     }
     // Don't bother with passwords if the account is going to be locked anyway.
-    if (u.password != NULL && strcmp (u.password, "")
-        && u.policy != USER_STATE_LOCKED)
+    if (u->password != NULL && strcmp (u->password, "")
+        && u->policy != USER_STATE_LOCKED)
     {
-        if (!IsPasswordCorrect(puser, u.password, u.password_format, passwd_info))
+        if (!IsPasswordCorrect(puser, u->password, u->password_format, passwd_info))
         {
             CFUSR_SETBIT (*changemap, i_password);
         }
     }
-
-    if (SafeStringLength(u.group_primary))
+    if (u->groups_secondary_given && !StringSetIsEqual(groups_to_set, current_secondary_groups))
     {
-        bool group_could_be_gid = (strlen(u.group_primary) == strspn(u.group_primary, "0123456789"));
+        CFUSR_SETBIT (*changemap, i_groups);
+    }
+
+    if (SafeStringLength(u->group_primary))
+    {
+        bool group_could_be_gid = (strlen(u->group_primary) == strspn(u->group_primary, "0123456789"));
         int gid;
 
         // We try name first, even if it looks like a gid. Only fall back to gid.
         struct group *group_info;
         errno = 0;
-        group_info = GetGrEntry(u.group_primary, &EqualGroupName);
+        group_info = GetGrEntry(u->group_primary, &EqualGroupName);
         if (!group_info && errno != 0)
         {
-            Log(LOG_LEVEL_ERR, "Could not obtain information about group '%s': %s", u.group_primary, GetErrorStr());
+            Log(LOG_LEVEL_ERR, "Could not obtain information about group '%s': %s", u->group_primary, GetErrorStr());
             gid = -1;
         }
         else if (!group_info)
         {
             if (group_could_be_gid)
             {
-                gid = atoi(u.group_primary);
+                gid = atoi(u->group_primary);
             }
             else
             {
-                Log(LOG_LEVEL_ERR, "No such group '%s'.", u.group_primary);
+                Log(LOG_LEVEL_ERR, "No such group '%s'.", u->group_primary);
                 gid = -1;
             }
         }
@@ -996,26 +1038,6 @@ static bool VerifyIfUserNeedsModifs (const char *puser, User u, const struct pas
         }
     }
 
-    if (u.groups_secondary_given)
-    {
-        StringSet *wanted_groups = StringSetNew();
-        for (Rlist *ptr = u.groups_secondary; ptr; ptr = ptr->next)
-        {
-            StringSetAdd(wanted_groups, xstrdup(RvalScalarValue(ptr->val)));
-        }
-        TransformGidsToGroups(&wanted_groups);
-        StringSet *current_groups = StringSetNew();
-        if (!GroupGetUserMembership (puser, current_groups))
-        {
-            CFUSR_SETBIT (*changemap, i_groups);
-        }
-        else if (!StringSetIsEqual (current_groups, wanted_groups))
-        {
-            CFUSR_SETBIT (*changemap, i_groups);
-        }
-        StringSetDestroy(current_groups);
-        StringSetDestroy(wanted_groups);
-    }
 
     ////////////////////////////////////////////
     if (*changemap == 0)
@@ -1087,9 +1109,10 @@ static bool ExecuteUserCommand(const char *puser, const char *cmd, size_t sizeof
     return true;
 }
 
-static bool DoCreateUser(const char *puser, User u, enum cfopaction action,
+static bool DoCreateUser(const char *puser, const User *u, enum cfopaction action,
                          EvalContext *ctx, const Attributes *a, const Promise *pp)
 {
+    assert(u != NULL);
     char cmd[CF_BUFSIZE];
     char sec_group_args[CF_BUFSIZE];
     if (puser == NULL || !strcmp (puser, ""))
@@ -1098,34 +1121,34 @@ static bool DoCreateUser(const char *puser, User u, enum cfopaction action,
     }
     strcpy (cmd, USERADD);
 
-    if (u.uid != NULL && strcmp (u.uid, ""))
+    if (u->uid != NULL && strcmp (u->uid, ""))
     {
         StringAppend(cmd, " -u \"", sizeof(cmd));
-        StringAppend(cmd, u.uid, sizeof(cmd));
+        StringAppend(cmd, u->uid, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
 
-    if (u.description != NULL)
+    if (u->description != NULL)
     {
         StringAppend(cmd, " -c \"", sizeof(cmd));
-        StringAppend(cmd, u.description, sizeof(cmd));
+        StringAppend(cmd, u->description, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
 
-    if (u.group_primary != NULL && strcmp (u.group_primary, ""))
+    if (u->group_primary != NULL && strcmp (u->group_primary, ""))
     {
         // TODO: Should check that group exists
         StringAppend(cmd, " -g \"", sizeof(cmd));
-        StringAppend(cmd, u.group_primary, sizeof(cmd));
+        StringAppend(cmd, u->group_primary, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
 
-    if (u.groups_secondary_given)
+    if (u->groups_secondary_given)
     {
         // TODO: Should check that groups exist
         strlcpy(sec_group_args, " -G \"", sizeof(sec_group_args));
         char sep[2] = { '\0', '\0' };
-        for (Rlist *i = u.groups_secondary; i; i = i->next)
+        for (Rlist *i = u->groups_secondary; i; i = i->next)
         {
             StringAppend(sec_group_args, sep, sizeof(sec_group_args));
             StringAppend(sec_group_args, RvalScalarValue(i->val), sizeof(sec_group_args));
@@ -1135,16 +1158,16 @@ static bool DoCreateUser(const char *puser, User u, enum cfopaction action,
         StringAppend(cmd, sec_group_args, sizeof(cmd));
     }
 
-    if (u.home_dir != NULL && strcmp (u.home_dir, ""))
+    if (u->home_dir != NULL && strcmp (u->home_dir, ""))
     {
         StringAppend(cmd, " -d \"", sizeof(cmd));
-        StringAppend(cmd, u.home_dir, sizeof(cmd));
+        StringAppend(cmd, u->home_dir, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
-    if (u.shell != NULL && strcmp (u.shell, ""))
+    if (u->shell != NULL && strcmp (u->shell, ""))
     {
         StringAppend(cmd, " -s \"", sizeof(cmd));
-        StringAppend(cmd, u.shell, sizeof(cmd));
+        StringAppend(cmd, u->shell, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
 
@@ -1176,7 +1199,7 @@ static bool DoCreateUser(const char *puser, User u, enum cfopaction action,
             return false;
         }
 
-        if (u.groups_secondary_given)
+        if (u->groups_secondary_given)
         {
             // Work around issue on AIX. Always set secondary groups a second time, because AIX
             // likes to assign the primary group as the secondary group as well, even if we didn't
@@ -1198,7 +1221,7 @@ static bool DoCreateUser(const char *puser, User u, enum cfopaction action,
             return false;
         }
 
-        if (u.policy == USER_STATE_LOCKED)
+        if (u->policy == USER_STATE_LOCKED)
         {
             if (!SetAccountLocked(puser, "x", true))
             {
@@ -1209,12 +1232,17 @@ static bool DoCreateUser(const char *puser, User u, enum cfopaction action,
         if (a->havebundle)
         {
             const Constraint *method_attrib = PromiseGetConstraint(pp, "home_bundle");
-            VerifyMethod(ctx, method_attrib->rval, *a, pp);
+            if (method_attrib == NULL)
+            {
+                Log(LOG_LEVEL_ERR, "Cannot create user (home_bundle not found)");
+                return false;
+            }
+            VerifyMethod(ctx, method_attrib->rval, a, pp);
         }
 
-        if (u.policy != USER_STATE_LOCKED && u.password != NULL && strcmp (u.password, ""))
+        if (u->policy != USER_STATE_LOCKED && u->password != NULL && strcmp (u->password, ""))
         {
-            if (!ChangePassword(puser, u.password, u.password_format))
+            if (!ChangePassword(puser, u->password, u->password_format))
             {
                 return false;
             }
@@ -1242,8 +1270,9 @@ static bool DoRemoveUser (const char *puser, enum cfopaction action)
     return ExecuteUserCommand(puser, cmd, sizeof(cmd), "removing", "Removing");
 }
 
-static bool DoModifyUser (const char *puser, User u, const struct passwd *passwd_info, uint32_t changemap, enum cfopaction action)
+static bool DoModifyUser (const char *puser, const User *u, const struct passwd *passwd_info, uint32_t changemap, enum cfopaction action, StringSet *groups_to_set)
 {
+    assert(u != NULL);
     char cmd[CF_BUFSIZE];
 
     strcpy (cmd, USERMOD);
@@ -1251,55 +1280,38 @@ static bool DoModifyUser (const char *puser, User u, const struct passwd *passwd
     if (CFUSR_CHECKBIT (changemap, i_uid) != 0)
     {
         StringAppend(cmd, " -u \"", sizeof(cmd));
-        StringAppend(cmd, u.uid, sizeof(cmd));
+        StringAppend(cmd, u->uid, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
 
     if (CFUSR_CHECKBIT (changemap, i_comment) != 0)
     {
         StringAppend(cmd, " -c \"", sizeof(cmd));
-        StringAppend(cmd, u.description, sizeof(cmd));
+        StringAppend(cmd, u->description, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
 
     if (CFUSR_CHECKBIT (changemap, i_group) != 0)
     {
         StringAppend(cmd, " -g \"", sizeof(cmd));
-        StringAppend(cmd, u.group_primary, sizeof(cmd));
+        StringAppend(cmd, u->group_primary, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
 
-#ifndef __hpux
-    // HP-UX does not support -G with empty argument
-    if (CFUSR_CHECKBIT (changemap, i_groups) != 0)
-    {
-        /* Work around bug on SUSE. If secondary groups contain a group that is
-           the same group as the primary group, the secondary group is not set.
-           This happens even if the primary group is changed in the same call.
-           Therefore, set an empty group list first, and then set it to the real
-           list later.
-        */
-        StringAppend(cmd, " -G \"\"", sizeof(cmd));
-    }
-#endif
-
     if (CFUSR_CHECKBIT (changemap, i_home) != 0)
     {
         StringAppend(cmd, " -d \"", sizeof(cmd));
-        StringAppend(cmd, u.home_dir, sizeof(cmd));
+        StringAppend(cmd, u->home_dir, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
 
     if (CFUSR_CHECKBIT (changemap, i_shell) != 0)
     {
         StringAppend(cmd, " -s \"", sizeof(cmd));
-        StringAppend(cmd, u.shell, sizeof(cmd));
+        StringAppend(cmd, u->shell, sizeof(cmd));
         StringAppend(cmd, "\"", sizeof(cmd));
     }
 
-    StringAppend(cmd, " ", sizeof(cmd));
-    StringAppend(cmd, puser, sizeof(cmd));
-
     if (CFUSR_CHECKBIT (changemap, i_password) != 0)
     {
         if (action == cfa_warn || DONTDO)
@@ -1309,7 +1321,7 @@ static bool DoModifyUser (const char *puser, User u, const struct passwd *passwd
         }
         else
         {
-            if (!ChangePassword(puser, u.password, u.password_format))
+            if (!ChangePassword(puser, u->password, u->password_format))
             {
                 return false;
             }
@@ -1321,7 +1333,7 @@ static bool DoModifyUser (const char *puser, User u, const struct passwd *passwd
         if (action == cfa_warn || DONTDO)
         {
             Log(LOG_LEVEL_WARNING, "Need to %s account for user '%s'.",
-                (u.policy == USER_STATE_LOCKED) ? "lock" : "unlock", puser);
+                (u->policy == USER_STATE_LOCKED) ? "lock" : "unlock", puser);
             return false;
         }
         else
@@ -1341,13 +1353,26 @@ static bool DoModifyUser (const char *puser, User u, const struct passwd *passwd
                 // account will already have been unlocked anyway.
                 hash = NULL;
             }
-            if (!SetAccountLocked(puser, hash, (u.policy == USER_STATE_LOCKED)))
+            if (!SetAccountLocked(puser, hash, (u->policy == USER_STATE_LOCKED)))
             {
                 return false;
             }
         }
     }
 
+    if (CFUSR_CHECKBIT (changemap, i_groups) != 0)
+    {
+        StringAppend(cmd, " -G \"", sizeof(cmd));
+        Buffer *buf = BufferNew();
+        buf = StringSetToBuffer(groups_to_set, ',');
+        StringAppend(cmd, buf->buffer, sizeof(cmd));
+        BufferDestroy(buf);
+        StringAppend(cmd, "\" ", sizeof(cmd));
+    }
+#ifndef HAVE_PW
+        StringAppend(cmd, " ", sizeof(cmd));
+        StringAppend(cmd, puser, sizeof(cmd));
+#endif
     // If password and locking were the only things changed, don't run the command.
     CFUSR_CLEARBIT(changemap, i_password);
     CFUSR_CLEARBIT(changemap, i_locked);
@@ -1358,38 +1383,10 @@ static bool DoModifyUser (const char *puser, User u, const struct passwd *passwd
     }
     else if (changemap != 0)
     {
-#ifdef __hpux
-        // This is to overcome the Suse hack above which does not work on HP-UX and thus we
-        // risk getting an empty command if change of secondary groups is the only change
-        // Only run for other changes than i_groups, otherwise the command will be empty
-        uint32_t changemap_without_groups = changemap;
-        CFUSR_CLEARBIT(changemap_without_groups, i_groups);
-        if(changemap_without_groups != 0)
-#endif
+
+        if (!ExecuteUserCommand(puser, cmd, sizeof(cmd), "modifying", "Modifying"))
         {
-            if (!ExecuteUserCommand(puser, cmd, sizeof(cmd), "modifying", "Modifying"))
-            {
-                return false;
-            }
-        }
-        if (CFUSR_CHECKBIT (changemap, i_groups) != 0)
-        {
-            // Set real group list (see -G comment earlier).
-            strcpy(cmd, USERMOD);
-            StringAppend(cmd, " -G \"", sizeof(cmd));
-            char sep[2] = { '\0', '\0' };
-            for (Rlist *i = u.groups_secondary; i; i = i->next)
-            {
-                StringAppend(cmd, sep, sizeof(cmd));
-                StringAppend(cmd, RvalScalarValue(i->val), sizeof(cmd));
-                sep[0] = ',';
-            }
-            StringAppend(cmd, "\" ", sizeof(cmd));
-            StringAppend(cmd, puser, sizeof(cmd));
-            if (!ExecuteUserCommand(puser, cmd, sizeof(cmd), "modifying", "Modifying"))
-            {
-                return false;
-            }
+            return false;
         }
     }
     return true;
@@ -1399,7 +1396,7 @@ static bool DoModifyUser (const char *puser, User u, const struct passwd *passwd
 // is a local user, and not for example from LDAP.
 static struct passwd *GetPwEntry(const char *puser)
 {
-    FILE *fptr = fopen("/etc/passwd", "r");
+    FILE *fptr = safe_fopen("/etc/passwd", "r");
     if (!fptr)
     {
         Log(LOG_LEVEL_ERR, "Could not open '/etc/passwd': %s", GetErrorStr());
@@ -1433,12 +1430,16 @@ static struct passwd *GetPwEntry(const char *puser)
     }
 }
 
-void VerifyOneUsersPromise (const char *puser, User u, PromiseResult *result, enum cfopaction action,
+void VerifyOneUsersPromise (const char *puser, const User *u, PromiseResult *result, enum cfopaction action,
                             EvalContext *ctx, const Attributes *a, const Promise *pp)
 {
+    assert(u != NULL);
     bool res;
 
     struct passwd *passwd_info;
+    StringSet *groups_to_set = StringSetNew();
+    StringSet *current_secondary_groups = StringSetNew();
+    StringSet *groups_missing = StringSetNew();
     passwd_info = GetPwEntry(puser);
     if (!passwd_info && errno != 0)
     {
@@ -1446,26 +1447,34 @@ void VerifyOneUsersPromise (const char *puser, User u, PromiseResult *result, en
         return;
     }
 
-    if (u.policy == USER_STATE_PRESENT || u.policy == USER_STATE_LOCKED)
+    if (u->policy == USER_STATE_PRESENT || u->policy == USER_STATE_LOCKED)
     {
         if (passwd_info)
         {
-            uint32_t cmap = 0;
-            if (VerifyIfUserNeedsModifs (puser, u, passwd_info, &cmap))
+            res = GetGroupInfo(puser, u, &groups_to_set, &groups_missing, &current_secondary_groups);
+            if (res)
             {
-                res = DoModifyUser (puser, u, passwd_info, cmap, action);
-                if (res)
+                uint32_t cmap = 0;
+                if (VerifyIfUserNeedsModifs (puser, u, passwd_info, &cmap, groups_to_set, current_secondary_groups))
                 {
-                    *result = PROMISE_RESULT_CHANGE;
+                    res = DoModifyUser (puser, u, passwd_info, cmap, action, groups_to_set);
+                    if (res)
+                    {
+                        *result = PROMISE_RESULT_CHANGE;
+                    }
+                    else
+                    {
+                        *result = PROMISE_RESULT_FAIL;
+                    }
                 }
                 else
                 {
-                    *result = PROMISE_RESULT_FAIL;
+                    *result = PROMISE_RESULT_NOOP;
                 }
             }
             else
             {
-                *result = PROMISE_RESULT_NOOP;
+                *result = PROMISE_RESULT_FAIL;
             }
         }
         else
@@ -1481,7 +1490,7 @@ void VerifyOneUsersPromise (const char *puser, User u, PromiseResult *result, en
             }
         }
     }
-    else if (u.policy == USER_STATE_ABSENT)
+    else if (u->policy == USER_STATE_ABSENT)
     {
         if (passwd_info)
         {
