@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -27,11 +27,11 @@
 
 #include <files_names.h>
 #include <files_interfaces.h>
-#include <hashes.h>
 #include <string_lib.h>
 #include <pipes.h>
 #include <unix.h>
 #include <mutex.h>
+#include <global_mutex.h>
 #include <signals.h>
 #include <exec_tools.h>
 #include <misc_lib.h>
@@ -41,8 +41,9 @@
 #include <known_dirs.h>
 #include <bootstrap.h>
 #include <policy_server.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <item_lib.h>
+#include <processes_select.h>   /* LoadProcessTable()/SelectProcesses() */
 
 #include <cf-windows-functions.h>
 
@@ -53,7 +54,7 @@ static const int INF_LINES = -2;
 /*******************************************************************/
 
 static void MailResult(const ExecConfig *config, const char *file);
-static int Dialogue(int sd, const char *s);
+static bool Dialogue(int sd, const char *s);
 
 /******************************************************************************/
 
@@ -215,8 +216,8 @@ void LocalExec(const ExecConfig *config)
 
 /* What if no more processes? Could sacrifice and exec() - but we need a sentinel */
 
-    FILE *fp = fopen(filename, "w");
-    if (!fp)
+    FILE *fp = safe_fopen(filename, "w");
+    if (fp == NULL)
     {
         Log(LOG_LEVEL_ERR, "Couldn't open '%s' - aborting exec. (fopen: %s)", filename, GetErrorStr());
         return;
@@ -263,11 +264,53 @@ void LocalExec(const ExecConfig *config)
             Log(LOG_LEVEL_NOTICE, errmsg, config->agent_expireafter);
             count++;
 
-            pid_t pid_agent;
+            pid_t pid_shell;
 
-            if (PipeToPid(&pid_agent, pp))
+            if (PipeToPid(&pid_shell, pp))
             {
-                ProcessSignalTerminate(pid_agent);
+                /* Default to killing the shell process (if we fail to get
+                 * more precise target). */
+                pid_t pid_to_kill = pid_shell;
+
+#ifndef __MINGW32__
+                /* The agent command is executed in a shell. Trying to kill the
+                 * shell may end up sending it SIGKILL which is not propagated
+                 * to the subprocesses of the shell and thus the cf-agent
+                 * process. The shell, however, creates a new process group
+                 * (with the PGID equal to the PID of the child process) for the
+                 * agent which then allows us to kill the whole process group
+                 * here. */
+
+                /* We need to determine the PID of the agent (and thus its
+                 * process group) first.*/
+                ClearProcessTable();
+                if (LoadProcessTable())
+                {
+                    ProcessSelect ps = PROCESS_SELECT_INIT;
+                    ps.min_ppid = pid_shell;
+                    ps.max_ppid = pid_shell;
+                    Item *procs = SelectProcesses(".*" /* any command */, &ps, true /* apply ps */);
+                    if (procs != NULL)
+                    {
+                        pid_to_kill = procs->counter;
+
+                        /* There should only be one child process of the shell
+                         * running by default. But it doesn't apply to all
+                         * values of exec_command in general. */
+                        assert(procs->next == NULL);
+                    }
+                }
+
+                /* kill(-pid) is actually kill(pgid=pid) and kill(pid, 0) just
+                 * checks if it's possible to send signals to pid (or pgid in
+                 * our case). Kill the whole process group if possible. */
+                if ((getpgid(pid_to_kill) == pid_to_kill)
+                    && (kill(-pid_to_kill, 0) == 0))
+                {
+                    pid_to_kill = -pid_to_kill;
+                }
+#endif
+                ProcessSignalTerminate(pid_to_kill);
             }
             else
             {
@@ -301,8 +344,19 @@ void LocalExec(const ExecConfig *config)
 
         if (*sp != '\0') /* line isn't entirely blank */
         {
-            char *line_escaped = xmalloc(2 * line_size);
-            ReplaceStr(line, line_escaped, 2 * line_size, "%", "%%");
+            /* Need space for 2x buffer and null byte. */
+            char line_escaped[res * 2 + 1];
+            memcpy(line_escaped, line, res + 1);
+
+            ssize_t replace_res =
+                StringReplace(line_escaped, sizeof(line_escaped), "%", "%%");
+            if (replace_res == -1)
+            {
+                ProgrammingError("StringReplace(): buffer overflow in %s",
+                                 __FILE__);
+                line[0] = '\0';
+                continue;
+            }
 
             fprintf(fp, "%s\n", line_escaped);
             count++;
@@ -321,20 +375,18 @@ void LocalExec(const ExecConfig *config)
             }
 
             line[0] = '\0';
-            free(line_escaped);
         }
     }
 
     free(line);
     cf_pclose(pp);
-    Log(LOG_LEVEL_DEBUG, "Closing fp");
-    fclose(fp);
-
     Log(LOG_LEVEL_VERBOSE,
         complete ? "Command is complete" : "Terminated command");
 
     if (count)
     {
+        Log(LOG_LEVEL_DEBUG, "Closing fp");
+        fclose(fp);
         Log(LOG_LEVEL_VERBOSE, "Mailing result");
         MailResult(config, filename);
     }
@@ -342,6 +394,7 @@ void LocalExec(const ExecConfig *config)
     {
         Log(LOG_LEVEL_VERBOSE, "No output");
         unlink(filename);
+        fclose(fp);
     }
 }
 
@@ -501,11 +554,7 @@ static bool CompareResultEqualOrFiltered(const ExecConfig *config,
         fclose(new_fp);
     }
 
-    if (!ThreadLock(cft_count))
-    {
-        Log(LOG_LEVEL_ERR, "Severe lock error when mailing in exec");
-        return 1;
-    }
+    ThreadLock(cft_count);
 
 /* replace old file with new*/
 
@@ -577,19 +626,29 @@ static void MailResult(const ExecConfig *config, const char *file)
     time_t now = time(NULL);
 #endif
 
+    FILE *fp = safe_fopen(file, "r");
+    if (fp == NULL)
+    {
+        Log(LOG_LEVEL_ERR, "Mail report: couldn't open file '%s'. (fopen: %s)", file, GetErrorStr());
+        return;
+    }
+
     Log(LOG_LEVEL_VERBOSE, "Mail report: sending result...");
 
     {
+        int fd = fileno(fp);
         struct stat statbuf;
-        if (stat(file, &statbuf) == -1)
+        if (fstat(fd, &statbuf) == -1)
         {
             Log(LOG_LEVEL_ERR, "Mail report: failed to stat file '%s' [errno: %d]", file, errno);
+            fclose(fp);
             return;
         }
 
         if (statbuf.st_size == 0)
         {
             unlink(file);
+            fclose(fp);
             Log(LOG_LEVEL_DEBUG, "Mail report: nothing to report in file '%s'", file);
             return;
         }
@@ -603,6 +662,7 @@ static void MailResult(const ExecConfig *config, const char *file)
         if (CompareResultEqualOrFiltered(config, file, prev_file))
         {
             Log(LOG_LEVEL_VERBOSE, "Mail report: previous output is the same as current so do not mail it");
+            fclose(fp);
             return;
         }
     }
@@ -611,24 +671,19 @@ static void MailResult(const ExecConfig *config, const char *file)
     {
         /* Syslog should have done this */
         Log(LOG_LEVEL_VERBOSE, "Mail report: empty mail server or address - skipping");
+        fclose(fp);
         return;
     }
 
     if (config->mail_max_lines == 0)
     {
         Log(LOG_LEVEL_DEBUG, "Mail report: not mailing because EmailMaxLines was zero");
+        fclose(fp);
         return;
     }
 
     Log(LOG_LEVEL_DEBUG, "Mail report: mailing results of '%s' to '%s'", file, config->mail_to_address);
 
-    FILE *fp = fopen(file, "r");
-    if (fp == NULL)
-    {
-        Log(LOG_LEVEL_ERR, "Mail report: couldn't open file '%s'. (fopen: %s)", file, GetErrorStr());
-        return;
-    }
-
     int sd = ConnectToSmtpSocket(config);
     if (sd < 0)
     {
@@ -718,7 +773,11 @@ static void MailResult(const ExecConfig *config, const char *file)
     free(existing_policy_server);
 
 #if defined __linux__ || defined __NetBSD__ || defined __FreeBSD__ || defined __OpenBSD__
-    strftime(vbuff, CF_BUFSIZE, "Date: %a, %d %b %Y %H:%M:%S %z\r\n", localtime(&now));
+    {
+        struct tm tm_value;
+        struct tm *const tm_pointer = localtime_r(&now, &tm_value);
+        strftime(vbuff, CF_BUFSIZE, "Date: %a, %d %b %Y %H:%M:%S %z\r\n", tm_pointer);
+    }
     send(sd, vbuff, strlen(vbuff), 0);
 #endif
 
@@ -798,7 +857,7 @@ static void MailResult(const ExecConfig *config, const char *file)
     Log(LOG_LEVEL_ERR, "Mail report: cannot mail to %s.", config->mail_to_address);
 }
 
-static int Dialogue(int sd, const char *s)
+static bool Dialogue(int sd, const char *s)
 {
     if ((s != NULL) && (*s != '\0'))
     {
