@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -40,12 +40,16 @@
 #include <loading.h>
 #include <printsize.h>
 #include <cleanup.h>
+#include <repair.h>
+#include <dbm_api.h>            /* CheckDBRepairFlagFile() */
+#include <string_lib.h>
 
 #include <cf-windows-functions.h>
 
 #define CF_EXEC_IFELAPSED 0
 #define CF_EXEC_EXPIREAFTER 1
 
+static bool PERFORM_DB_CHECK = false;
 static int NO_FORK = false; /* GLOBAL_A */
 static int ONCE = false; /* GLOBAL_A */
 static int WINSERVICE = true; /* GLOBAL_A */
@@ -76,7 +80,8 @@ static const char *const CF_EXECD_MANPAGE_LONG_DESCRIPTION =
     "cf-execd is the scheduling daemon for cf-agent. It runs cf-agent locally according to a schedule specified in "
     "policy code (executor control body). After a cf-agent run is completed, cf-execd gathers output from cf-agent, "
     "and may be configured to email the output to a specified address. It may also be configured to splay (randomize) the "
-    "execution schedule to prevent synchronized cf-agent runs across a network.";
+    "execution schedule to prevent synchronized cf-agent runs across a network. "
+    "Note: this daemon reloads it's config when the SIGHUP signal is received.";
 
 static const struct option OPTIONS[] =
 {
@@ -98,6 +103,7 @@ static const struct option OPTIONS[] =
     {"ld-library-path", required_argument, 0, 'L'},
     {"color", optional_argument, 0, 'C'},
     {"timestamp", no_argument, 0, 'l'},
+    {"skip-db-check", optional_argument, 0, 0 },
     {NULL, 0, 0, '\0'}
 };
 
@@ -105,7 +111,7 @@ static const char *const HINTS[] =
 {
     "Print the help message",
     "Enable debugging output",
-    "Output verbose information about the behaviour of the agent",
+    "Output verbose information about the behaviour of cf-execd",
     "All talk and no action mode - make no changes, only inform of promises not kept",
     "Output the version of the software",
     "Specify an alternative input file than the default. This option is overridden by FILE if supplied as argument.",
@@ -121,6 +127,7 @@ static const char *const HINTS[] =
     "Set the internal value of LD_LIBRARY_PATH for child processes",
     "Enable colorized output. Possible values: 'always', 'auto', 'never'. If option is used, the default value is 'auto'",
     "Log timestamps on each line of log output",
+    "Do not run database integrity checks and repairs at startup",
     NULL
 };
 
@@ -129,6 +136,12 @@ static const char *const HINTS[] =
 int main(int argc, char *argv[])
 {
     GenericAgentConfig *config = CheckOpts(argc, argv);
+    bool force_repair = CheckDBRepairFlagFile();
+    if (force_repair || PERFORM_DB_CHECK)
+    {
+        repair_lmdb_default(force_repair);
+    }
+
     EvalContext *ctx = EvalContextNew();
     GenericAgentConfigApply(ctx, config);
 
@@ -163,7 +176,6 @@ int main(int argc, char *argv[])
     GenericAgentFinalize(ctx, config);
     ExecConfigDestroy(exec_config);
     ExecdConfigDestroy(execd_config);
-
     return 0;
 }
 
@@ -175,13 +187,13 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
 {
     extern char *optarg;
     int c;
-    char ld_library_path[CF_BUFSIZE];
 
     GenericAgentConfig *config = GenericAgentConfigNewDefault(AGENT_TYPE_EXECUTOR, GetTTYInteractive());
 
 
+    int longopt_idx;
     while ((c = getopt_long(argc, argv, "dvnKIf:g:D:N:VxL:hFOV1gMWC::l",
-                            OPTIONS, NULL))
+                            OPTIONS, &longopt_idx))
            != -1)
     {
         switch (c)
@@ -208,8 +220,8 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
                 else
                 {
-                    StringSetJoin(config->heap_soft, defined_classes);
-                    free(defined_classes);
+                    StringSetJoin(config->heap_soft, defined_classes, xstrdup);
+                    StringSetDestroy(defined_classes);
                 }
             }
             break;
@@ -223,8 +235,8 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
                 else
                 {
-                    StringSetJoin(config->heap_negated, negated_classes);
-                    free(negated_classes);
+                    StringSetJoin(config->heap_negated, negated_classes, xstrdup);
+                    StringSetDestroy(negated_classes);
                 }
             }
             break;
@@ -248,10 +260,11 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             break;
 
         case 'L':
-            snprintf(ld_library_path, CF_BUFSIZE - 1, "LD_LIBRARY_PATH=%s", optarg);
-            putenv(xstrdup(ld_library_path));
-            break;
-
+            {
+                Log(LOG_LEVEL_VERBOSE, "Setting 'LD_LIBRARY_PATH=%s'", optarg);
+                setenv_wrapper("LD_LIBRARY_PATH", optarg, 1);
+                break;
+            }
         case 'W':
             WINSERVICE = false;
             break;
@@ -276,7 +289,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
         case 'h':
         {
             Writer *w = FileWriter(stdout);
-            WriterWriteHelp(w, "cf-execd", OPTIONS, HINTS, true, NULL);
+            WriterWriteHelp(w, "cf-execd", OPTIONS, HINTS, NULL, false, true);
             FileWriterDetach(w);
         }
         DoCleanupAndExit(EXIT_SUCCESS);
@@ -288,6 +301,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                          CF_EXECD_SHORT_DESCRIPTION,
                          CF_EXECD_MANPAGE_LONG_DESCRIPTION,
                          OPTIONS, HINTS,
+                         NULL, false,
                          true);
             FileWriterDetach(out);
             DoCleanupAndExit(EXIT_SUCCESS);
@@ -308,10 +322,37 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             LoggingEnableTimestamps(true);
             break;
 
+        case 0:
+        {
+            const char *const option_name = OPTIONS[longopt_idx].name;
+            if (StringSafeEqual(option_name, "skip-db-check"))
+            {
+                if (optarg == NULL)
+                {
+                    PERFORM_DB_CHECK = false; // Skip (no arg), check = false
+                }
+                else if (StringSafeEqual_IgnoreCase(optarg, "yes"))
+                {
+                    PERFORM_DB_CHECK = false; // Skip = yes, check = false
+                }
+                else if (StringSafeEqual_IgnoreCase(optarg, "no"))
+                {
+                    PERFORM_DB_CHECK = true; // Skip = no, check = true
+                }
+                else
+                {
+                    Log(LOG_LEVEL_ERR,
+                        "Invalid argument for --skip-db-check(yes/no): '%s'",
+                        optarg);
+                    DoCleanupAndExit(EXIT_FAILURE);
+                }
+            }
+            break;
+        }
         default:
         {
             Writer *w = FileWriter(stdout);
-            WriterWriteHelp(w, "cf-execd", OPTIONS, HINTS, true, NULL);
+            WriterWriteHelp(w, "cf-execd", OPTIONS, HINTS, NULL, false, true);
             FileWriterDetach(w);
         }
         DoCleanupAndExit(EXIT_FAILURE);
@@ -392,6 +433,7 @@ void StartServer(EvalContext *ctx, Policy *policy, GenericAgentConfig *config, E
     WritePID("cf-execd.pid");
     signal(SIGINT, HandleSignalsForDaemon);
     signal(SIGTERM, HandleSignalsForDaemon);
+    signal(SIGBUS, HandleSignalsForDaemon);
     signal(SIGHUP, HandleSignalsForDaemon);
     signal(SIGPIPE, SIG_IGN);
     signal(SIGUSR1, HandleSignalsForDaemon);
@@ -508,8 +550,8 @@ static bool LocalExecInThread(const ExecConfig *config)
 static void Apoptosis(void)
 {
     char promiser_buf[CF_SMALLBUF];
-    snprintf(promiser_buf, sizeof(promiser_buf), "%s%cbin%ccf-execd",
-             GetWorkDir(), FILE_SEPARATOR, FILE_SEPARATOR);
+    snprintf(promiser_buf, sizeof(promiser_buf), "%s%ccf-execd",
+             GetBinDir(), FILE_SEPARATOR);
 
     if (LoadProcessTable())
     {
@@ -519,12 +561,11 @@ static void Apoptosis(void)
         Rlist *owners = NULL;
         RlistPrepend(&owners, myuid, RVAL_TYPE_SCALAR);
 
-        ProcessSelect process_select = {
-            .owner = owners,
-            .process_result = "process_owner",
-        };
+        ProcessSelect process_select = PROCESS_SELECT_INIT;
+        process_select.owner = owners;
+        process_select.process_result = "process_owner";
 
-        Item *killlist = SelectProcesses(promiser_buf, process_select, true);
+        Item *killlist = SelectProcesses(promiser_buf, &(process_select), true);
         RlistDestroy(owners);
 
         for (Item *ip = killlist; ip != NULL; ip = ip->next)
