@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -34,7 +34,7 @@
 #include <scope.h>
 #include <files_copy.h>
 #include <files_interfaces.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <keyring.h>
 #include <communication.h>
 #include <eval_context.h>
@@ -120,12 +120,23 @@ void ParseKeyArg(char *keyarg, char **filename, char **ipaddr, char **username)
 
 extern bool cf_key_interrupted;
 
-bool ShowHost(const char *hostkey, const char *address, bool incoming,
-                     const KeyHostSeen *quality, void *ctx)
-{
-    int *count = ctx;
-    char timebuf[26];
+#define HOST_FMT_TRUNCATE "%-10.10s %-40.40s %-25.25s %-26.26s %-s\n"
+#define HOST_FMT_NO_TRUNCATE "%s\t%s\t%s\t%s\t%s\n"
 
+typedef struct _HostPrintState
+{
+    int count;
+    bool truncate;
+} HostPrintState;
+
+static bool ShowHost(
+    const char *const hostkey,
+    const char *const address,
+    bool incoming,
+    const KeyHostSeen *const quality,
+    void *const ctx)
+{
+    HostPrintState *const state = ctx;
     char hostname[NI_MAXHOST];
     if (LOOKUP_HOSTS)
     {
@@ -139,9 +150,11 @@ bool ShowHost(const char *hostkey, const char *address, bool incoming,
     {
         strlcpy(hostname, address, sizeof(hostname));
     }
+    ++(state->count);
 
-    (*count)++;
-    printf("%-10.10s %-40.40s %-25.25s %-26.26s %-s\n",
+    bool truncate = state->truncate;
+    char timebuf[26];
+    printf(truncate ? HOST_FMT_TRUNCATE : HOST_FMT_NO_TRUNCATE,
            incoming ? "Incoming" : "Outgoing",
            address, hostname,
            cf_strtimestamp_local(quality->lastseen, timebuf), hostkey);
@@ -149,19 +162,27 @@ bool ShowHost(const char *hostkey, const char *address, bool incoming,
     return !cf_key_interrupted;
 }
 
-void ShowLastSeenHosts()
+void ShowLastSeenHosts(bool truncate)
 {
-    int count = 0;
+    HostPrintState state = { 0 };
+    state.count = 0;
+    state.truncate = truncate;
 
-    printf("%-10.10s %-40.40s %-25.25s %-26.26s %-s\n", "Direction", "IP", "Name", "Last connection", "Key");
+    printf(
+        truncate ? HOST_FMT_TRUNCATE : HOST_FMT_NO_TRUNCATE,
+        "Direction",
+        "IP",
+        "Name",
+        "Last connection",
+        "Key");
 
-    if (!ScanLastSeenQuality(ShowHost, &count))
+    if (!ScanLastSeenQuality(ShowHost, &state))
     {
         Log(LOG_LEVEL_ERR, "Unable to show lastseen database");
         return;
     }
 
-    printf("Total Entries: %d\n", count);
+    printf("Total Entries: %d\n", state.count);
 }
 
 /**
@@ -211,27 +232,18 @@ int RemoveKeys(const char *input, bool must_be_coherent)
 
 static bool KeepKeyPromisesRSA(RSA *pair, const char *public_key_file, const char *private_key_file)
 {
-    int fd = safe_open(private_key_file, O_WRONLY | O_CREAT | O_TRUNC, 0600);
-
-    if (fd < 0)
-    {
-        Log(LOG_LEVEL_ERR, "Couldn't open private key file '%s' (open: %s)", private_key_file, GetErrorStr());
-        return false;
-    }
-
-    FILE *fp = fdopen(fd, "w");
+    FILE *fp = safe_fopen_create_perms(private_key_file, "w", 0600);
     if (fp == NULL)
     {
-        Log(LOG_LEVEL_ERR, "Error while writing private key file '%s' (fdopen: %s)", private_key_file, GetErrorStr());
-        close(fd);
+        Log(LOG_LEVEL_ERR,
+            "Error while writing private key file '%s' (fopen: %s)",
+            private_key_file, GetErrorStr());
         return false;
     }
 
     Log(LOG_LEVEL_VERBOSE, "Writing private key to '%s'", private_key_file);
 
-    const EVP_CIPHER *cipher = EVP_des_ede3_cbc();
-    int res = PEM_write_RSAPrivateKey(fp, pair, cipher, (void *)PRIVKEY_PASSPHRASE,
-                                 PRIVKEY_PASSPHRASE_LEN, NULL, NULL);
+    int res = PEM_write_RSAPrivateKey(fp, pair, NULL, NULL, 0, NULL, NULL);
     fclose(fp);
 
     if (res == 0)
@@ -242,22 +254,15 @@ static bool KeepKeyPromisesRSA(RSA *pair, const char *public_key_file, const cha
         return false;
     }
 
-    fd = safe_open(public_key_file, O_WRONLY | O_CREAT | O_TRUNC, 0600);
-
-    if (fd < 0)
+    fp = safe_fopen_create_perms(public_key_file, "w", 0600);
+    if (fp == NULL)
     {
-        Log(LOG_LEVEL_ERR, "Couldn't open public key file '%s' (open: %s)",
+        Log(LOG_LEVEL_ERR,
+            "Error while writing public key file '%s' (fopen: %s)",
             public_key_file, GetErrorStr());
         return false;
     }
 
-    if ((fp = fdopen(fd, "w")) == NULL)
-    {
-        Log(LOG_LEVEL_ERR, "Error while writing public key file '%s' (fdopen: %s)", public_key_file, GetErrorStr());
-        close(fd);
-        return false;
-    }
-
     Log(LOG_LEVEL_VERBOSE, "Writing public key to file '%s'", public_key_file);
 
     if (!PEM_write_RSAPublicKey(fp, pair))
