@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -30,7 +30,7 @@
 #include <scope.h>
 #include <files_copy.h>
 #include <files_interfaces.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <keyring.h>
 #include <eval_context.h>
 #include <crypto.h>
@@ -42,7 +42,8 @@
 
 #include <cf-key-functions.h>
 
-int SHOWHOSTS = false;                                          /* GLOBAL_A */
+bool SHOWHOSTS = false;                                         /* GLOBAL_A */
+bool NO_TRUNCATE = false;                                       /* GLOBAL_A */
 bool FORCEREMOVAL = false;                                      /* GLOBAL_A */
 bool REMOVEKEYS = false;                                        /* GLOBAL_A */
 bool LICENSE_INSTALL = false;                                   /* GLOBAL_A */
@@ -78,6 +79,7 @@ static const struct option OPTIONS[] =
     {"output-file", required_argument, 0, 'f'},
     {"key-type", required_argument, 0, 'T'},
     {"show-hosts", no_argument, 0, 's'},
+    {"no-truncate", no_argument, 0, 'N'},
     {"remove-keys", required_argument, 0, 'r'},
     {"force-removal", no_argument, 0, 'x'},
     {"install-license", required_argument, 0, 'l'},
@@ -94,14 +96,15 @@ static const char *const HINTS[] =
     "Print the help message",
     "Print basic information about key generation",
     "Enable debugging output",
-    "Output verbose information about the behaviour of the agent",
+    "Output verbose information about the behaviour of cf-key",
     "Output the version of the software",
     "Specify how detailed logs should be. Possible values: 'error', 'warning', 'notice', 'info', 'verbose', 'debug'",
     "Specify an alternative output file than the default.",
     "Specify a RSA key size in bits, the default value is 2048.",
     "Show lastseen hostnames and IP addresses",
-    "Remove keys for specified hostname/IP",
-    "Force removal of keys (USE AT YOUR OWN RISK)",
+    "Don't truncate -s / --show-hosts output",
+    "Remove keys for specified hostname/IP/MD5/SHA (cf-key -r SHA=12345, cf-key -r MD5=12345, cf-key -r host001, cf-key -r 203.0.113.1)",
+    "Force removal of keys",
     "Install license file on Enterprise server (CFEngine Enterprise Only)",
     "Print digest of the specified public key",
     "Make cf-serverd/cf-agent trust the specified public key. Argument value is of the form [[USER@]IPADDR:]FILENAME where FILENAME is the local path of the public key for client at IPADDR address.",
@@ -129,6 +132,7 @@ static void SetupSignalsForCfKey(CfKeySigHandler sighandler)
     signal(SIGTERM, sighandler);
     signal(SIGHUP, SIG_IGN);
     signal(SIGPIPE, SIG_IGN);
+    signal(SIGBUS, HandleSignalsForAgent);
     signal(SIGUSR1, HandleSignalsForAgent);
     signal(SIGUSR2, HandleSignalsForAgent);
 }
@@ -146,7 +150,7 @@ int main(int argc, char *argv[])
     if (SHOWHOSTS)
     {
         SetupSignalsForCfKey(handleShowKeysSignal);
-        ShowLastSeenHosts();
+        ShowLastSeenHosts(!NO_TRUNCATE);
         return 0;
     }
 
@@ -240,13 +244,20 @@ int main(int argc, char *argv[])
 /* Level                                                                     */
 /*****************************************************************************/
 
+static void PrintHelp()
+{
+    Writer *w = FileWriter(stdout);
+    WriterWriteHelp(w, "cf-key", OPTIONS, HINTS, NULL, false, false);
+    FileWriterDetach(w);
+}
+
 static GenericAgentConfig *CheckOpts(int argc, char **argv)
 {
     extern char *optarg;
     int c;
     GenericAgentConfig *config = GenericAgentConfigNewDefault(AGENT_TYPE_KEYGEN, GetTTYInteractive());
 
-    while ((c = getopt_long(argc, argv, "dvIf:g:T:VMp:sr:xt:hl:C::n",
+    while ((c = getopt_long(argc, argv, "dvIf:g:T:VMp:sNr:xt:hl:C::n",
                             OPTIONS, NULL))
            != -1)
     {
@@ -292,6 +303,10 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             SHOWHOSTS = true;
             break;
 
+        case 'N':
+            NO_TRUNCATE = true;
+            break;
+
         case 'x':
             FORCEREMOVAL = true;
             break;
@@ -311,11 +326,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             break;
 
         case 'h':
-            {
-                Writer *w = FileWriter(stdout);
-                WriterWriteHelp(w, "cf-key", OPTIONS, HINTS, false, NULL);
-                FileWriterDetach(w);
-            }
+            PrintHelp();
             DoCleanupAndExit(EXIT_SUCCESS);
 
         case 'M':
@@ -325,6 +336,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                              CF_KEY_SHORT_DESCRIPTION,
                              CF_KEY_MANPAGE_LONG_DESCRIPTION,
                              OPTIONS, HINTS,
+                             NULL, false,
                              false);
                 FileWriterDetach(out);
                 DoCleanupAndExit(EXIT_SUCCESS);
@@ -346,15 +358,18 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             break;
 
         default:
-            {
-                Writer *w = FileWriter(stdout);
-                WriterWriteHelp(w, "cf-key", OPTIONS, HINTS, false, NULL);
-                FileWriterDetach(w);
-            }
+            PrintHelp();
             DoCleanupAndExit(EXIT_FAILURE);
 
         }
     }
 
+    if (NO_TRUNCATE && !SHOWHOSTS)
+    {
+        PrintHelp();
+        Log(LOG_LEVEL_ERR, "--no-truncate / -N option is only for --show-hosts / -s");
+        DoCleanupAndExit(EXIT_FAILURE);
+    }
+
     return config;
 }
