@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -42,6 +42,7 @@
 #include <exec_tools.h>
 #include <generic_agent.h> // WritePID
 #include <files_lib.h>
+#include <file_lib.h> // SetUmask()
 #include <unix.h>
 #include <verify_measurements.h>
 #include <verify_classes.h>
@@ -204,7 +205,6 @@ static void GetDatabaseAge()
 
 static void LoadHistogram(void)
 {
-    FILE *fp;
     int i, day, position;
     double maxval[CF_OBSERVABLES];
 
@@ -212,7 +212,8 @@ static void LoadHistogram(void)
 
     snprintf(filename, CF_BUFSIZE, "%s%chistograms", GetStateDir(), FILE_SEPARATOR);
 
-    if ((fp = fopen(filename, "r")) == NULL)
+    FILE *fp = safe_fopen(filename, "r");
+    if (fp == NULL)
     {
         Log(LOG_LEVEL_VERBOSE,
             "Unable to load histogram data from '%s' (fopen: %s)",
@@ -302,12 +303,7 @@ void MonitorStartServer(EvalContext *ctx, const Policy *policy)
 
 #endif /* !__MINGW32__ */
 
-    TransactionContext tc = {
-        .ifelapsed = 0,
-        .expireafter = 0,
-    };
-
-    thislock = AcquireLock(ctx, pp->promiser, VUQNAME, CFSTARTTIME, tc, pp, false);
+    thislock = AcquireLock(ctx, pp->promiser, VUQNAME, CFSTARTTIME, 0, 0, pp, false);
     if (thislock.lock == NULL)
     {
         PolicyDestroy(monitor_cfengine_policy);
@@ -551,17 +547,17 @@ static void LeapDetection(void)
 
 static void PublishEnvironment(Item *classes)
 {
-    FILE *fp;
-    Item *ip;
-
     unlink(ENVFILE_NEW);
 
-    if ((fp = fopen(ENVFILE_NEW, "a")) == NULL)
+    const mode_t old_umask = SetUmask(0077);
+    FILE *fp = safe_fopen(ENVFILE_NEW, "a");
+    RestoreUmask(old_umask);
+    if (fp == NULL)
     {
         return;
     }
 
-    for (ip = classes; ip != NULL; ip = ip->next)
+    for (Item *ip = classes; ip != NULL; ip = ip->next)
     {
         fprintf(fp, "%s\n", ip->name);
     }
@@ -802,7 +798,6 @@ static void UpdateDistributions(EvalContext *ctx, char *timekey, Averages *av)
 {
     int position, day, i;
     char filename[CF_BUFSIZE];
-    FILE *fp;
 
 /* Take an interval of 4 standard deviations from -2 to +2, divided into CF_GRAINS
    parts. Centre each measurement on CF_GRAINS/2 and scale each measurement by the
@@ -826,7 +821,8 @@ static void UpdateDistributions(EvalContext *ctx, char *timekey, Averages *av)
 
         snprintf(filename, CF_BUFSIZE, "%s%chistograms", GetStateDir(), FILE_SEPARATOR);
 
-        if ((fp = fopen(filename, "w")) == NULL)
+        FILE *fp = safe_fopen(filename, "w");
+        if (fp == NULL)
         {
             Log(LOG_LEVEL_ERR, "Unable to save histograms to '%s' (fopen: %s)", filename, GetErrorStr());
             return;
