@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -24,6 +24,7 @@
 
 #include <cf3.defs.h>
 
+#include <file_lib.h>
 #include <mon.h>
 
 #if !defined(__MINGW32__)
@@ -45,10 +46,10 @@ void MonCPUGatherData(double *cf_this)
     char cpuname[CF_MAXVARSIZE], buf[CF_BUFSIZE];
     long cpuidx, userticks = 0, niceticks = 0, systemticks = 0, idle = 0, iowait = 0, irq = 0, softirq = 0;
     long total_time = 1;
-    FILE *fp;
     enum observables slot = ob_spare;
 
-    if ((fp = fopen("/proc/stat", "r")) == NULL)
+    FILE *fp = safe_fopen("/proc/stat", "r");
+    if (fp == NULL)
     {
         Log(LOG_LEVEL_VERBOSE, "Could not open /proc/stat while gathering CPU data (fopen: %s)", GetErrorStr());
         return;
@@ -62,8 +63,8 @@ void MonCPUGatherData(double *cf_this)
         {
             break;
         }
-
-        if (sscanf(buf, "%s%ld%ld%ld%ld%ld%ld%ld", cpuname, &userticks, &niceticks, &systemticks, &idle, &iowait, &irq,
+        assert(CF_MAXVARSIZE == 1024);
+        if (sscanf(buf, "%1023s%ld%ld%ld%ld%ld%ld%ld", cpuname, &userticks, &niceticks, &systemticks, &idle, &iowait, &irq,
                &softirq) != 8)
         {
             Log(LOG_LEVEL_VERBOSE, "Could not scan /proc/stat line: %60s", buf);
