@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -26,6 +26,7 @@
 
 #include <string_lib.h>
 #include <files_names.h>
+#include <file_lib.h>
 #include <mon_cumulative.h>
 #include <monitoring.h>
 #include <probes.h>
@@ -49,7 +50,6 @@ static void SysfsizeDeviceName(char *name)
 static int DiskSectorSize(const char *sysfsname)
 {
     char sysfspath[CF_BUFSIZE];
-    FILE *fh;
     int size = 512;             /* Long-time default value */
 
     if (snprintf(sysfspath, CF_BUFSIZE, SYSFSBLOCK "%s/queue/logical_block_size", sysfsname) >= CF_BUFSIZE)
@@ -58,7 +58,8 @@ static int DiskSectorSize(const char *sysfsname)
         return -1;
     }
 
-    if (!(fh = fopen(sysfspath, "r")))
+    FILE *fh = safe_fopen(sysfspath, "r");
+    if (fh == NULL)
     {
         /*
          * Older Linux systems don't work correctly with new 4K drives. It's safe to
@@ -238,9 +239,9 @@ static void MonIoDiskstatsGatherData(double *cf_this)
                                       0.0, 1000.0, true);
     int writes_slot = NovaRegisterSlot(MON_IO_WRITES, "Number of I/O writes", "writes per second",
                                        0.0f, 1000.0f, true);
-    int readdata_slot = NovaRegisterSlot(MON_IO_READDATA, "Amount of data read", "megabytes/s",
+    int readdata_slot = NovaRegisterSlot(MON_IO_READDATA, "Aggregate mount of data read across all devices", "megabytes/s",
                                          0.0, 1000.0, true);
-    int writtendata_slot = NovaRegisterSlot(MON_IO_WRITTENDATA, "Amount of data written", "megabytes",
+    int writtendata_slot = NovaRegisterSlot(MON_IO_WRITTENDATA, "Aggregate amount of data written across all devices", "megabytes/s",
                                             0.0, 1000.0, true);
 
     if (reads_slot != -1 && totalreads != 0)
