@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -30,6 +30,7 @@
 #include <files_names.h>
 #include <files_interfaces.h>
 #include <files_lib.h>
+#include <file_lib.h> // SetUmask()
 #include <pipes.h>
 #include <known_dirs.h>
 
@@ -199,7 +200,7 @@ void MonNetworkGatherData(double *cf_this)
 
     DeleteItemList(ALL_INCOMING);
     ALL_INCOMING = NULL;
-    
+
     DeleteItemList(MON_TCP4);
     DeleteItemList(MON_TCP6);
     DeleteItemList(MON_UDP4);
@@ -210,6 +211,9 @@ void MonNetworkGatherData(double *cf_this)
 
     if (!FileCanOpen(comm, "r"))
     {
+        Log(LOG_LEVEL_VERBOSE,
+            "Cannot open '%s', aborting gathering of network data (monitoring)",
+            comm);
         return;
     }
 
@@ -217,7 +221,9 @@ void MonNetworkGatherData(double *cf_this)
 
     if ((pp = cf_popen(comm, "r", true)) == NULL)
     {
-        /* FIXME: no logging */
+        Log(LOG_LEVEL_VERBOSE,
+            "Opening '%s' failed, aborting gathering of network data (monitoring)",
+            comm);
         return;
     }
 
@@ -234,7 +240,10 @@ void MonNetworkGatherData(double *cf_this)
         {
             if (!feof(pp))
             {
-                /* FIXME: no logging */
+                Log(LOG_LEVEL_DEBUG,
+                    "Error occured while reading '%s' "
+                    "(CfReadLine/getline returned -1 but no EOF found)",
+                    comm);
                 cf_pclose(pp);
                 free(vbuff);
                 return;
@@ -421,7 +430,9 @@ void MonNetworkGatherData(double *cf_this)
         }
 
         SetNetworkEntropyClasses(CanonifyName(ECGSOCKS[i].name), "in", in[i]);
+        const mode_t old_umask = SetUmask(0077);
         RawSaveItemList(in[i], vbuff, NewLineMode_Unix);
+        RestoreUmask(old_umask);
         DeleteItemList(in[i]);
         Log(LOG_LEVEL_DEBUG, "Saved in netstat data in '%s'", vbuff);
     }
@@ -446,7 +457,9 @@ void MonNetworkGatherData(double *cf_this)
         }
 
         SetNetworkEntropyClasses(CanonifyName(ECGSOCKS[i].name), "out", out[i]);
+        const mode_t old_umask = SetUmask(0077);
         RawSaveItemList(out[i], vbuff, NewLineMode_Unix);
+        RestoreUmask(old_umask);
         Log(LOG_LEVEL_DEBUG, "Saved out netstat data in '%s'", vbuff);
         DeleteItemList(out[i]);
     }
