@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -251,10 +251,12 @@ static void AnalyzeArrival(Item *ip_addresses, long iteration, char *arrival, do
     {
         arr++;
     }
-
     if ((strstr(arrival, "proto TCP")) || (strstr(arrival, "ack")))
     {
-        sscanf(arr, "%s %*c %s %c ", src, dest, &flag);
+        assert(sizeof(src) == CF_BUFSIZE);
+        assert(sizeof(dest) == CF_BUFSIZE);
+        assert(CF_BUFSIZE == 4096);
+        sscanf(arr, "%4095s %*c %4095s %c ", src, dest, &flag);
         DePort(src);
         DePort(dest);
         isme_dest = IsInterfaceAddress(ip_addresses, dest);
@@ -308,7 +310,10 @@ static void AnalyzeArrival(Item *ip_addresses, long iteration, char *arrival, do
     }
     else if (strstr(arrival, ".53"))
     {
-        sscanf(arr, "%s %*c %s %c ", src, dest, &flag);
+        assert(sizeof(src) == CF_BUFSIZE);
+        assert(sizeof(dest) == CF_BUFSIZE);
+        assert(CF_BUFSIZE == 4096);
+        sscanf(arr, "%4095s %*c %4095s %c ", src, dest, &flag);
         DePort(src);
         DePort(dest);
         isme_dest = IsInterfaceAddress(ip_addresses, dest);
@@ -328,7 +333,10 @@ static void AnalyzeArrival(Item *ip_addresses, long iteration, char *arrival, do
     }
     else if (strstr(arrival, "proto UDP"))
     {
-        sscanf(arr, "%s %*c %s %c ", src, dest, &flag);
+        assert(sizeof(src) == CF_BUFSIZE);
+        assert(sizeof(dest) == CF_BUFSIZE);
+        assert(CF_BUFSIZE == 4096);
+        sscanf(arr, "%4095s %*c %4095s %c ", src, dest, &flag);
         DePort(src);
         DePort(dest);
         isme_dest = IsInterfaceAddress(ip_addresses, dest);
@@ -348,7 +356,10 @@ static void AnalyzeArrival(Item *ip_addresses, long iteration, char *arrival, do
     }
     else if (strstr(arrival, "proto ICMP"))
     {
-        sscanf(arr, "%s %*c %s %c ", src, dest, &flag);
+        assert(sizeof(src) == CF_BUFSIZE);
+        assert(sizeof(dest) == CF_BUFSIZE);
+        assert(CF_BUFSIZE == 4096);
+        sscanf(arr, "%4095s %*c %4095s %c ", src, dest, &flag);
         DePort(src);
         DePort(dest);
         isme_dest = IsInterfaceAddress(ip_addresses, dest);
@@ -374,8 +385,9 @@ static void AnalyzeArrival(Item *ip_addresses, long iteration, char *arrival, do
         cf_this[ob_tcpmisc_in]++;
 
         /* Here we don't know what source will be, but .... */
-
-        sscanf(arrival, "%s", src);
+        assert(sizeof(src) == CF_BUFSIZE);
+        assert(CF_BUFSIZE == 4096);
+        sscanf(arrival, "%4095s", src);
 
         if (!isdigit((int) *src))
         {
@@ -407,7 +419,6 @@ static void AnalyzeArrival(Item *ip_addresses, long iteration, char *arrival, do
 static void SaveTCPEntropyData(Item *list, int i, char *inout)
 {
     Item *ip;
-    FILE *fp;
     char filename[CF_BUFSIZE];
 
     Log(LOG_LEVEL_VERBOSE, "TCP Save '%s'", TCPNAMES[i]);
@@ -429,7 +440,8 @@ static void SaveTCPEntropyData(Item *list, int i, char *inout)
 
     Log(LOG_LEVEL_VERBOSE, "TCP Save '%s'", filename);
 
-    if ((fp = fopen(filename, "w")) == NULL)
+    FILE *fp = safe_fopen(filename, "w");
+    if (fp == NULL)
     {
         Log(LOG_LEVEL_ERR, "Couldn't save TCP entropy to '%s' (fopen: %s)", filename, GetErrorStr());
         return;
