@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -27,6 +27,7 @@
 #include <mon.h>
 #include <item_lib.h>
 #include <files_interfaces.h>
+#include <file_lib.h> // SetUmask()
 #include <pipes.h>
 #include <systype.h>
 #include <known_dirs.h>
@@ -61,7 +62,9 @@ void MonProcessesGatherData(double *cf_this)
     xsnprintf(vbuff, sizeof(vbuff), "%s/cf_users", GetStateDir());
     MapName(vbuff);
 
+    const mode_t old_umask = SetUmask(0077);
     RawSaveItemList(userList, vbuff, NewLineMode_Unix);
+    RestoreUmask(old_umask);
     DeleteItemList(userList);
 
     Log(LOG_LEVEL_VERBOSE, "(Users,root,other) = (%d,%d,%d)",
