@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -23,7 +23,7 @@
 */
 
 
-#define CF_NET_VERSION "0.1.1"
+#define CF_NET_VERSION "0.1.2"
 
 #include <platform.h>
 #include <libgen.h>
@@ -34,7 +34,6 @@
 #include <addr_lib.h>           // ParseHostPort
 #include <net.h>                // SocketConnect() SendTransaction()
 #include <time.h>               // time_t, time, difftime
-#include <stat_cache.h>         // cf_remote_stat
 #include <string_lib.h>         // ToLowerStrInplace
 #include <writer.h>
 #include <policy_server.h>      // PolicyServerReadFile
@@ -42,6 +41,8 @@
 #include <cf-windows-functions.h> // TODO: move this out of libpromises
 #include <known_dirs.h>           // TODO: move this 'out of libpromises
 #include <cleanup.h>
+#include <protocol.h>
+#include <sequence.h>
 
 #define ARG_UNUSED __attribute__((unused))
 
@@ -51,6 +52,8 @@ typedef struct
     bool verbose;
     bool inform;
     bool used_default;
+    char *min_tls_version;
+    char *allow_ciphers;
 } CFNetOptions;
 
 //*******************************************************************
@@ -99,6 +102,8 @@ static const struct option OPTIONS[] =
     {"verbose",     no_argument,        0, 'v'},
     {"log-level",   required_argument,  0, 'g'},
     {"inform",      no_argument,        0, 'I'},
+    {"tls-version", required_argument,  0, 't'},
+    {"ciphers",     required_argument,  0, 'c'},
     {NULL,          0,                  0, '\0'}
 };
 
@@ -111,6 +116,8 @@ static const char *const HINTS[] =
     "Enable verbose output",
     "Specify how detailed logs should be. Possible values: 'error', 'warning', 'notice', 'info', 'verbose', 'debug'",
     "Enable basic information output",
+    "Minimum TLS version to use",
+    "TLS ciphers to use (comma-separated list)",
     NULL
 };
 
@@ -150,6 +157,7 @@ static const char *command_strings[] =
 // INIT:
 static void CFNetSetDefault(CFNetOptions *opts);
 static void CFNetInit();
+static void CFNetOptionsClear(CFNetOptions *opts);
 
 // MAIN LOGIC:
 static int CFNetParse(int argc, char **argv,                        // INPUTS
@@ -196,7 +204,10 @@ int main(int argc, char **argv)
     }
     ret = CFNetRun(&opts, args, hostnames);  // Commands return exit code
     free(hostnames);
-    return ret;
+    CFNetOptionsClear(&opts);
+
+    ret = (ret == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+    DoCleanupAndExit(ret);
 }
 
 //*******************************************************************
@@ -209,9 +220,18 @@ static void CFNetSetDefault(CFNetOptions *opts){
     opts->verbose     = false;
     opts->inform      = false;
     opts->used_default= false;
+    opts->min_tls_version = NULL;
+    opts->allow_ciphers   = NULL;
 }
 
-static void CFNetInit()
+static void CFNetOptionsClear(CFNetOptions *opts)
+{
+    assert(opts != NULL);
+    free(opts->min_tls_version);
+    free(opts->allow_ciphers);
+}
+
+static void CFNetInit(const char *min_tls_version, const char *allow_ciphers)
 {
 #ifdef __MINGW32__
     InitializeWindows();
@@ -219,7 +239,7 @@ static void CFNetInit()
 #endif
     CryptoInitialize();
     LoadSecretKeys(NULL, NULL, NULL, NULL);
-    cfnet_init(NULL, NULL);
+    cfnet_init(min_tls_version, allow_ciphers);
 }
 
 //*******************************************************************
@@ -275,6 +295,7 @@ static int CFNetParse(int argc, char **argv,
                              CF_NET_SHORT_DESCRIPTION,
                              CF_NET_MANPAGE_LONG_DESCRIPTION,
                              OPTIONS, HINTS,
+                             COMMANDS, false,
                              true);
                 FileWriterDetach(out);
                 DoCleanupAndExit(EXIT_SUCCESS);
@@ -312,6 +333,16 @@ static int CFNetParse(int argc, char **argv,
                 LogSetGlobalLevelArgOrExit(optarg);
                 break;
             }
+            case 't':
+            {
+                opts->min_tls_version = xstrdup(optarg);
+                break;
+            }
+            case 'c':
+            {
+                opts->allow_ciphers = xstrdup(optarg);
+                break;
+            }
             default:
             {
                 // printf("Default optarg = '%s', c = '%c' = %i\n",
@@ -385,6 +416,7 @@ static void CFNetSetVerbosity(CFNetOptions *opts)
 
 static int CFNetRun(CFNetOptions *opts, char **args, char *hostnames)
 {
+    assert(opts != NULL);
     assert(args != NULL);
     CFNetSetVerbosity(opts);
 
@@ -415,7 +447,7 @@ static int CFNetRun(CFNetOptions *opts, char **args, char *hostnames)
         return CFNetHelp(args[1]);
     }
 
-    CFNetInit();
+    CFNetInit(opts->min_tls_version, opts->allow_ciphers);
     char *hosts = RequireHostname(hostnames);
     int ret = 0;
     char *hostname = strtok(hosts, ",");
@@ -452,7 +484,7 @@ static AgentConnection *CFNetOpenConnection(const char *server)
     free(buf);
     if (conn == NULL)
     {
-        printf("Failed to connect to '%s' (%d)\n", server, err);
+        printf("Failed to connect to '%s'\n", server);
         return NULL;
     }
     return conn;
@@ -525,7 +557,7 @@ static int CFNetHelp(const char *topic)
     else
     {
         Writer *w = FileWriter(stdout);
-        WriterWriteHelp(w, "cf-net", OPTIONS, HINTS, false, COMMANDS);
+        WriterWriteHelp(w, "cf-net", OPTIONS, HINTS, COMMANDS, false, false);
         FileWriterDetach(w);
         DoCleanupAndExit(EXIT_SUCCESS);
     }
@@ -611,15 +643,14 @@ static int CFNetStat(ARG_UNUSED CFNetOptions *opts, const char *hostname, char *
 {
     assert(opts);
     char *file = args[1];
-    AgentConnection *conn = CFNetOpenConnection(hostname);// FIXME
+    AgentConnection *conn = CFNetOpenConnection(hostname);
     if (conn == NULL)
     {
         return -1;
     }
-    bool encrypt = true;
     struct stat sb;
-    int r = cf_remote_stat(conn, encrypt, file, &sb, "file");
-    if (r != 0)
+    bool ret = ProtocolStat(conn, file, &sb);
+    if (!ret)
     {
         printf("Could not stat: '%s'\n", file);
     }
@@ -650,7 +681,7 @@ typedef struct _GetFileData {
     const char *hostname;
     char remote_file[PATH_MAX];
     char local_file[PATH_MAX];
-    int ret;
+    bool ret;
 } GetFileData;
 
 static void *CFNetGetFile(void *arg)
@@ -659,21 +690,16 @@ static void *CFNetGetFile(void *arg)
     AgentConnection *conn = CFNetOpenConnection(data->hostname);
     if (conn == NULL)
     {
-        data->ret = -1;
+        data->ret = false;
         return NULL;
     }
 
-    struct stat sb;
-    data->ret = cf_remote_stat(conn, true, data->remote_file, &sb, "file");
-    if (data->ret != 0)
+    data->ret = ProtocolStatGet(conn, data->remote_file,
+                                data->local_file, 0644);
+    if (!data->ret)
     {
         printf("Could not stat: '%s'\n", data->remote_file);
     }
-    else
-    {
-        bool ok = CopyRegularFileNet(data->remote_file, data->local_file, sb.st_size, true, conn);
-        data->ret = ok ? 0 : -1;
-    }
     CFNetDisconnect(conn);
     return NULL;
 }
@@ -832,7 +858,7 @@ static int CFNetGet(ARG_UNUSED CFNetOptions *opts, const char *hostname, char **
         }
         else
         {
-            failure = failure && (threads[i]->data->ret != 0);
+            failure = failure && !threads[i]->data->ret;
         }
     }
 
@@ -846,17 +872,12 @@ static int CFNetGet(ARG_UNUSED CFNetOptions *opts, const char *hostname, char **
     return failure ? -1 : 0;
 }
 
-static void PrintDirs(const Item *list)
+static void PrintDirs(const Seq *list)
 {
-    const Item *ip = list;
-
-    while (ip != NULL)
+    for (size_t i = 0; i < SeqLength(list); i++)
     {
-        // TODO: I hate this:
-        struct dirent *de = (struct dirent *) ip->name;
-        printf("%s\n", de->d_name);
-
-        ip = ip->next;
+        char *dir_entry = SeqAt(list, i);
+        printf("%s\n", dir_entry);
     }
 }
 
@@ -884,8 +905,14 @@ static int CFNetOpenDir(ARG_UNUSED CFNetOptions *opts, const char *hostname, cha
 
     const char *remote_path = args[1];
 
-    Item *items = RemoteDirList(remote_path, false, conn);
-    PrintDirs(items);
+    Seq *seq = ProtocolOpenDir(conn, remote_path);
+    if (seq == NULL)
+    {
+        return -1;
+    }
+
+    PrintDirs(seq);
+    SeqDestroy(seq);
     CFNetDisconnect(conn);
     return 0;
 }
