@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -46,7 +46,7 @@
 #include <addr_lib.h>
 #include <loading.h>
 #include <expand.h>                                 /* ProtocolVersionParse */
-#include <files_hashes.h>
+#include <hash.h>
 #include <string_lib.h>
 #include <cleanup.h>
 
@@ -69,8 +69,7 @@ static void ThisAgentInit(void);
 static GenericAgentConfig *CheckOpts(int argc, char **argv);
 
 static void KeepControlPromises(EvalContext *ctx, const Policy *policy);
-static int HailServer(const EvalContext *ctx, const GenericAgentConfig *config,
-                      char *host);
+static bool HailServer(const EvalContext *ctx, const GenericAgentConfig *config, char *host);
 static void SendClassData(AgentConnection *conn);
 static void HailExec(AgentConnection *conn, char *peer);
 static FILE *NewStream(char *name);
@@ -123,7 +122,7 @@ static const char *const HINTS[] =
     "Print the help message",
     "Parallelize connections (50 by default)",
     "Enable debugging output",
-    "Output verbose information about the behaviour of the agent",
+    "Output verbose information about the behaviour of cf-runagent",
     "Specify how detailed logs should be. Possible values: 'error', 'warning', 'notice', 'info', 'verbose', 'debug'",
     "All talk and no action mode - make no changes, only inform of promises not kept",
     "Output the version of the software",
@@ -361,7 +360,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
         case 'h':
         {
             Writer *w = FileWriter(stdout);
-            WriterWriteHelp(w, "cf-runagent", OPTIONS, HINTS, true, NULL);
+            WriterWriteHelp(w, "cf-runagent", OPTIONS, HINTS, NULL, false, true);
             FileWriterDetach(w);
         }
         DoCleanupAndExit(EXIT_SUCCESS);
@@ -373,6 +372,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                          CF_RUNAGENT_SHORT_DESCRIPTION,
                          CF_RUNAGENT_MANPAGE_LONG_DESCRIPTION,
                          OPTIONS, HINTS,
+                         NULL, false,
                          true);
             FileWriterDetach(out);
             DoCleanupAndExit(EXIT_SUCCESS);
@@ -420,7 +420,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
         default:
         {
             Writer *w = FileWriter(stdout);
-            WriterWriteHelp(w, "cf-runagent", OPTIONS, HINTS, true, NULL);
+            WriterWriteHelp(w, "cf-runagent", OPTIONS, HINTS, NULL, false, true);
             FileWriterDetach(w);
         }
         DoCleanupAndExit(EXIT_FAILURE);
@@ -446,8 +446,7 @@ static void ThisAgentInit(void)
 
 /********************************************************************/
 
-static int HailServer(const EvalContext *ctx, const GenericAgentConfig *config,
-                      char *host)
+static bool HailServer(const EvalContext *ctx, const GenericAgentConfig *config, char *host)
 {
     assert(host != NULL);
 
@@ -795,7 +794,6 @@ static void HailExec(AgentConnection *conn, char *peer)
 
 static FILE *NewStream(char *name)
 {
-    FILE *fp;
     char filename[CF_BUFSIZE];
 
     if (OUTPUT_DIRECTORY[0] != '\0')
@@ -808,11 +806,13 @@ static FILE *NewStream(char *name)
                  GetWorkDir(), FILE_SEPARATOR, FILE_SEPARATOR, name);
     }
 
+    FILE *fp;
     if (OUTPUT_TO_FILE)
     {
         printf("Opening file... %s\n", filename);
 
-        if ((fp = fopen(filename, "w")) == NULL)
+        fp = safe_fopen(filename, "w");
+        if (fp == NULL)
         {
             Log(LOG_LEVEL_ERR, "Unable to open file '%s' (fopen: %s)", filename, GetErrorStr());
             fp = stdout;
