@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -35,6 +35,7 @@
 #include <signals.h>
 #include <systype.h>
 #include <mutex.h>
+#include <global_mutex.h>
 #include <locks.h>
 #include <exec_tools.h>
 #include <unix.h>
@@ -69,7 +70,8 @@ static const char *const CF_SERVERD_MANPAGE_LONG_DESCRIPTION =
         "cf-serverd is a socket listening daemon providing two services: it acts as a file server for remote file copying "
         "and it allows an authorized cf-runagent to start a cf-agent run. cf-agent typically connects to a "
         "cf-serverd instance to request updated policy code, but may also request additional files for download. "
-        "cf-serverd employs role based access control (defined in policy code) to authorize requests.";
+        "cf-serverd employs role based access control (defined in policy code) to authorize requests. "
+        "Note: this daemon reloads it's config when the SIGHUP signal is received.";
 
 static const struct option OPTIONS[] =
 {
@@ -117,15 +119,13 @@ static const char *const HINTS[] =
 #ifdef HAVE_AVAHI_COMMON_ADDRESS_H
 static int GenerateAvahiConfig(const char *path)
 {
-    FILE *fout;
-    Writer *writer = NULL;
-    fout = safe_fopen(path, "w+");
+    FILE *fout = safe_fopen(path, "w+");
     if (fout == NULL)
     {
         Log(LOG_LEVEL_ERR, "Unable to open '%s'", path);
         return -1;
     }
-    writer = FileWriter(fout);
+    Writer *writer = FileWriter(fout);
     fprintf(fout, "<?xml version=\"1.0\" standalone='no'?>\n");
     fprintf(fout, "<!DOCTYPE service-group SYSTEM \"avahi-service.dtd\">\n");
     XmlComment(writer, "This file has been automatically generated by cf-serverd.");
@@ -182,8 +182,8 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
                 else
                 {
-                    StringSetJoin(config->heap_soft, defined_classes);
-                    free(defined_classes);
+                    StringSetJoin(config->heap_soft, defined_classes, xstrdup);
+                    StringSetDestroy(defined_classes);
                 }
             }
             break;
@@ -197,8 +197,8 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
                 else
                 {
-                    StringSetJoin(config->heap_negated, negated_classes);
-                    free(negated_classes);
+                    StringSetJoin(config->heap_negated, negated_classes, xstrdup);
+                    StringSetDestroy(negated_classes);
                 }
             }
             break;
@@ -222,10 +222,8 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
 
         case 'L':
         {
-            static char ld_library_path[CF_BUFSIZE]; /* GLOBAL_A */
             Log(LOG_LEVEL_VERBOSE, "Setting LD_LIBRARY_PATH to '%s'", optarg);
-            snprintf(ld_library_path, CF_BUFSIZE - 1, "LD_LIBRARY_PATH=%s", optarg);
-            putenv(ld_library_path);
+            setenv_wrapper("LD_LIBRARY_PATH", optarg, 1);
             break;
         }
 
@@ -240,7 +238,7 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
         case 'h':
             {
                 Writer *w = FileWriter(stdout);
-                WriterWriteHelp(w, "cf-serverd", OPTIONS, HINTS, true, NULL);
+                WriterWriteHelp(w, "cf-serverd", OPTIONS, HINTS, NULL, false, true);
                 FileWriterDetach(w);
             }
             DoCleanupAndExit(EXIT_SUCCESS);
@@ -252,6 +250,7 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
                              CF_SERVERD_SHORT_DESCRIPTION,
                              CF_SERVERD_MANPAGE_LONG_DESCRIPTION,
                              OPTIONS, HINTS,
+                             NULL, false,
                              true);
                 FileWriterDetach(out);
                 DoCleanupAndExit(EXIT_SUCCESS);
@@ -289,7 +288,7 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
         default:
             {
                 Writer *w = FileWriter(stdout);
-                WriterWriteHelp(w, "cf-serverd", OPTIONS, HINTS, true, NULL);
+                WriterWriteHelp(w, "cf-serverd", OPTIONS, HINTS, NULL, false, true);
                 FileWriterDetach(w);
             }
             DoCleanupAndExit(EXIT_FAILURE);
@@ -478,6 +477,7 @@ static void InitSignals()
 
     signal(SIGINT, HandleSignalsForDaemon);
     signal(SIGTERM, HandleSignalsForDaemon);
+    signal(SIGBUS, HandleSignalsForDaemon);
     signal(SIGHUP, HandleSignalsForDaemon);
     signal(SIGPIPE, SIG_IGN);
     signal(SIGUSR1, HandleSignalsForDaemon);
@@ -502,11 +502,7 @@ static CfLock AcquireServerLock(EvalContext *ctx,
     }
     assert(pp);
 
-    TransactionContext tc = {
-        .ifelapsed = 0,
-        .expireafter = 1,
-    };
-    return AcquireLock(ctx, pp->promiser, VUQNAME, CFSTARTTIME, tc, pp, false);
+    return AcquireLock(ctx, pp->promiser, VUQNAME, CFSTARTTIME, 0, 1, pp, false);
 }
 
 /* Final preparations for running as server */
@@ -552,11 +548,9 @@ static int WaitOnThreads()
     int result = 1;
     for (int i = 2; i > 0; i--)
     {
-        if (ThreadLock(cft_server_children))
-        {
-            result = ACTIVE_THREADS;
-            ThreadUnlock(cft_server_children);
-        }
+        ThreadLock(cft_server_children);
+        result = ACTIVE_THREADS;
+        ThreadUnlock(cft_server_children);
 
         if (result == 0)
         {
@@ -609,7 +603,8 @@ static void CollectCallIfDue(EvalContext *ctx)
         {
             ConnectionInfo *info = ConnectionInfoNew();
             assert(info);
-
+            Log(LOG_LEVEL_DEBUG,
+                "Hub has %d seconds to complete report collection (collect_window)", COLLECT_WINDOW);
             ConnectionInfoSetSocket(info, new_client);
             info->is_call_collect = true; /* Mark processed when done. */
             ServerEntryPoint(ctx, PolicyServerGetIP(), info);
@@ -625,21 +620,19 @@ static void CollectCallIfDue(EvalContext *ctx)
 static void PolicyUpdateIfSafe(EvalContext *ctx, Policy **policy,
                                GenericAgentConfig *config)
 {
-    if (ThreadLock(cft_server_children))
+    ThreadLock(cft_server_children);
+    int prior = COLLECT_INTERVAL;
+    if (ACTIVE_THREADS == 0)
     {
-        int prior = COLLECT_INTERVAL;
-        if (ACTIVE_THREADS == 0)
-        {
-            CheckFileChanges(ctx, policy, config);
-        }
-        ThreadUnlock(cft_server_children);
+        CheckFileChanges(ctx, policy, config);
+    }
+    ThreadUnlock(cft_server_children);
 
-        /* Check for change in call-collect interval: */
-        if (prior != COLLECT_INTERVAL)
-        {
-            /* Start, stop or change schedule, as appropriate. */
-            CollectCallStart(COLLECT_INTERVAL);
-        }
+    /* Check for change in call-collect interval: */
+    if (prior != COLLECT_INTERVAL)
+    {
+        /* Start, stop or change schedule, as appropriate. */
+        CollectCallStart(COLLECT_INTERVAL);
     }
 }
 
