@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -35,7 +35,7 @@
 #include <string_lib.h>                               /* ToLowerStrInplace */
 #include <regex.h>                                    /* StringMatchFull */
 #include <lastseen.h>                                 /* LastSaw1 */
-#include <files_hashes.h>                             /* HashString */
+#include <hash.h>                                     /* HashString */
 #include <crypto.h>                                   /* HavePublicKey */
 #include <cf-serverd-enterprise-stubs.h>              /* ReceiveCollectCall */
 #include <tls_generic.h>
@@ -169,9 +169,9 @@ static bool PathMatch(const char *stem, const char *request)
             strncmp(stem, request, stemlen) == 0);
 }
 
-static int AccessControl(EvalContext *ctx, const char *req_path, ServerConnectionState *conn, int encrypt)
+static bool AccessControl(EvalContext *ctx, const char *req_path, ServerConnectionState *conn, int encrypt)
 {
-    int access = false;
+    bool access = false;
     char transrequest[CF_BUFSIZE];
     struct stat statbuf;
     char translated_req_path[CF_BUFSIZE];
@@ -569,7 +569,7 @@ static void SetConnectionData(ServerConnectionState *conn, char *buf)
     SetConnIdentity(conn, username);
 }
 
-static int CheckStoreKey(ServerConnectionState *conn, RSA *key)
+static bool CheckStoreKey(ServerConnectionState *conn, RSA *key)
 {
     RSA *savedkey;
 
@@ -623,7 +623,7 @@ static int CheckStoreKey(ServerConnectionState *conn, RSA *key)
     }
 }
 
-static int AuthenticationDialogue(ServerConnectionState *conn, char *recvbuffer, int recvlen)
+static bool AuthenticationDialogue(ServerConnectionState *conn, char *recvbuffer, int recvlen)
 {
     unsigned char digest[EVP_MAX_MD_SIZE + 1] = { 0 };
 
@@ -1039,7 +1039,7 @@ return true;
 
 
 
-int BusyWithClassicConnection(EvalContext *ctx, ServerConnectionState *conn)
+bool BusyWithClassicConnection(EvalContext *ctx, ServerConnectionState *conn)
 {
     time_t tloc, trem = 0;
     char recvbuffer[CF_BUFSIZE + CF_BUFEXT], check[CF_BUFSIZE];
