@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -34,7 +34,7 @@ static const int CF_NOSIZE = -1;
 #include <crypto.h>                                   /* EncryptString */
 #include <files_names.h>
 #include <files_interfaces.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <file_lib.h>
 #include <eval_context.h>
 #include <dir.h>
@@ -44,6 +44,7 @@ static const int CF_NOSIZE = -1;
 #include <classic.h>                  /* SendSocketStream */
 #include <net.h>                      /* SendTransaction,ReceiveTransaction */
 #include <openssl/err.h>                                   /* ERR_get_error */
+#include <protocol.h>                                  /* ProtocolIsKnown() */
 #include <tls_generic.h>              /* TLSSend */
 #include <rlist.h>
 #include <cf-serverd-enterprise-stubs.h>
@@ -80,7 +81,7 @@ bool IsUserNameValid(const char *username)
     return false;
 }
 
-int AllowedUser(char *user)
+bool AllowedUser(char *user)
 {
     if (IsItemIn(SERVER_ACCESS.allowuserlist, user))
     {
@@ -167,7 +168,7 @@ static void ReplyNothing(ServerConnectionState *conn)
 
 /* Used only in EXEC protocol command, to check if any of the received classes
  * is defined in the server. */
-int MatchClasses(const EvalContext *ctx, ServerConnectionState *conn)
+bool MatchClasses(const EvalContext *ctx, ServerConnectionState *conn)
 {
     char recvbuffer[CF_BUFSIZE];
     Item *classlist = NULL, *ip;
@@ -269,8 +270,10 @@ void Terminate(ConnectionInfo *connection)
     }
 }
 
-static int TransferRights(const ServerConnectionState *conn,
-                          const char *filename, const struct stat *sb)
+static bool TransferRights(
+    const ServerConnectionState *conn,
+    const char *filename,
+    const struct stat *sb)
 {
     Log(LOG_LEVEL_DEBUG, "Checking ownership of file: %s", filename);
 
@@ -420,11 +423,14 @@ void CfGetFile(ServerFileGetState *args)
         Log(LOG_LEVEL_INFO, "REFUSE access to file: %s", filename);
         RefuseAccess(args->conn, args->replyfile);
         snprintf(sendbuffer, CF_BUFSIZE, "%s", CF_FAILEDSTR);
-        if (ConnectionInfoProtocolVersion(conn_info) == CF_PROTOCOL_CLASSIC)
+
+        const ProtocolVersion version = ConnectionInfoProtocolVersion(conn_info);
+        assert(ProtocolIsKnown(version));
+        if (ProtocolIsClassic(version))
         {
             SendSocketStream(ConnectionInfoSocket(conn_info), sendbuffer, args->buf_size);
         }
-        else if (ConnectionInfoProtocolVersion(conn_info) == CF_PROTOCOL_TLS)
+        else if (ProtocolIsTLS(version))
         {
             TLSSend(ConnectionInfoSSL(conn_info), sendbuffer, args->buf_size);
         }
@@ -438,11 +444,14 @@ void CfGetFile(ServerFileGetState *args)
         Log(LOG_LEVEL_ERR, "Open error of file '%s'. (open: %s)",
             filename, GetErrorStr());
         snprintf(sendbuffer, CF_BUFSIZE, "%s", CF_FAILEDSTR);
-        if (ConnectionInfoProtocolVersion(conn_info) == CF_PROTOCOL_CLASSIC)
+
+        const ProtocolVersion version = ConnectionInfoProtocolVersion(conn_info);
+        assert(ProtocolIsKnown(version));
+        if (ProtocolIsClassic(version))
         {
             SendSocketStream(ConnectionInfoSocket(conn_info), sendbuffer, args->buf_size);
         }
-        else if (ConnectionInfoProtocolVersion(conn_info) == CF_PROTOCOL_TLS)
+        else if (ProtocolIsTLS(version))
         {
             TLSSend(ConnectionInfoSSL(conn_info), sendbuffer, args->buf_size);
         }
@@ -492,14 +501,16 @@ void CfGetFile(ServerFileGetState *args)
                 {
                     snprintf(sendbuffer, CF_BUFSIZE, "%s%s: %s", CF_CHANGEDSTR1, CF_CHANGEDSTR2, filename);
 
-                    if (ConnectionInfoProtocolVersion(conn_info) == CF_PROTOCOL_CLASSIC)
+                    const ProtocolVersion version = ConnectionInfoProtocolVersion(conn_info);
+
+                    if (ProtocolIsClassic(version))
                     {
                         if (SendSocketStream(ConnectionInfoSocket(conn_info), sendbuffer, blocksize) == -1)
                         {
                             Log(LOG_LEVEL_VERBOSE, "Send failed in GetFile. (send: %s)", GetErrorStr());
                         }
                     }
-                    else if (ConnectionInfoProtocolVersion(conn_info) == CF_PROTOCOL_TLS)
+                    else if (ProtocolIsTLS(version))
                     {
                         if (TLSSend(ConnectionInfoSSL(conn_info), sendbuffer, blocksize) == -1)
                         {
@@ -525,7 +536,9 @@ void CfGetFile(ServerFileGetState *args)
 
             total += n_read;
 
-            if (ConnectionInfoProtocolVersion(conn_info) == CF_PROTOCOL_CLASSIC)
+            const ProtocolVersion version = ConnectionInfoProtocolVersion(conn_info);
+
+            if (ProtocolIsClassic(version))
             {
                 if (SendSocketStream(ConnectionInfoSocket(conn_info), sendbuffer, sendlen) == -1)
                 {
@@ -533,7 +546,7 @@ void CfGetFile(ServerFileGetState *args)
                     break;
                 }
             }
-            else if (ConnectionInfoProtocolVersion(conn_info) == CF_PROTOCOL_TLS)
+            else if (ProtocolIsTLS(version))
             {
                 if (TLSSend(ConnectionInfoSSL(conn_info), sendbuffer, sendlen) == -1)
                 {
@@ -846,8 +859,10 @@ int StatFile(ServerConnectionState *conn, char *sendbuffer, char *ofilename)
 }
 
 bool CompareLocalHash(const char *filename, const char digest[EVP_MAX_MD_SIZE + 1],
-                      char sendbuffer[CF_BUFSIZE])
+                      char sendbuffer[CFD_FALSE_SIZE])
 {
+    assert(CFD_FALSE_SIZE == (strlen(CFD_FALSE) + 1));
+    assert(strlen(CFD_FALSE) >= strlen(CFD_TRUE));
     char translated_filename[CF_BUFSIZE] = { 0 };
     TranslatePath(translated_filename, filename);
 
@@ -857,14 +872,12 @@ bool CompareLocalHash(const char *filename, const char digest[EVP_MAX_MD_SIZE +
 
     if (HashesMatch(digest, file_digest, CF_DEFAULT_DIGEST))
     {
-        assert(strlen(CFD_FALSE) < CF_BUFSIZE);
         strcpy(sendbuffer, CFD_FALSE);
         Log(LOG_LEVEL_DEBUG, "Hashes matched ok");
         return true;
     }
     else
     {
-        assert(strlen(CFD_TRUE) < CF_BUFSIZE);
         strcpy(sendbuffer, CFD_TRUE);
         Log(LOG_LEVEL_DEBUG, "Hashes didn't match");
         return false;
@@ -902,7 +915,7 @@ void GetServerLiteral(EvalContext *ctx, ServerConnectionState *conn, char *sendb
     }
 }
 
-int GetServerQuery(ServerConnectionState *conn, char *recvbuffer, int encrypt)
+bool GetServerQuery(ServerConnectionState *conn, char *recvbuffer, int encrypt)
 {
     char query[CF_BUFSIZE];
 
@@ -1438,18 +1451,16 @@ void SetConnIdentity(ServerConnectionState *conn, const char *username)
         static pthread_mutex_t pwnam_mtx = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
         struct passwd *pw = NULL;
 
-        if (ThreadLock(&pwnam_mtx))
+        ThreadLock(&pwnam_mtx);
+        /* TODO Redmine#7643: looking up the UID is expensive and should
+         * not be needed, since today's agent machine VS hub most probably
+         * do not share the accounts. */
+        pw = getpwnam(conn->username);
+        if (pw != NULL)
         {
-            /* TODO Redmine#7643: looking up the UID is expensive and should
-             * not be needed, since today's agent machine VS hub most probably
-             * do not share the accounts. */
-            pw = getpwnam(conn->username);
-            if (pw != NULL)
-            {
-                conn->uid = pw->pw_uid;
-            }
-            ThreadUnlock(&pwnam_mtx);
+            conn->uid = pw->pw_uid;
         }
+        ThreadUnlock(&pwnam_mtx);
     }
 
 #endif
