@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -25,6 +25,7 @@
 
 #include <server_tls.h>
 #include <server_common.h>
+#include <protocol.h> // ParseProtocolVersionNetwork()
 
 #include <openssl/err.h>                                   /* ERR_get_error */
 
@@ -46,6 +47,7 @@
 
 static SSL_CTX *SSLSERVERCONTEXT = NULL;
 
+#define MAX_ACCEPT_RETRIES 5
 
 /**
  * @param[in]  priv_key private key to use (or %NULL to use the global PRIVKEY)
@@ -100,27 +102,26 @@ bool ServerTLSInitialize(RSA *priv_key, RSA *pub_key, SSL_CTX **ssl_ctx)
      * CFEngine is not a web server so it does not need to support many
      * ciphers. It only allows a safe but very common subset by default,
      * extensible via "allowciphers" in body server control. By default
-     * it allows:
+     * the server side allows:
      *
      *     AES256-GCM-SHA384: most high-grade RSA-based cipher from TLSv1.2
      *     AES256-SHA: most backwards compatible but high-grade, from SSLv3
+     *     TLS_AES_256_GCM_SHA384: most high-grade RSA-based cipher from TLSv1.3
+     *
+     * Client side is using the OpenSSL's defaults by default.
      */
     const char *cipher_list = SERVER_ACCESS.allowciphers;
     if (cipher_list == NULL)
     {
-        cipher_list ="AES256-GCM-SHA384:AES256-SHA";
+#ifdef SSL_OP_NO_TLSv1_3        /* defined if TLS 1.3 is supported */
+        cipher_list = "AES256-GCM-SHA384:AES256-SHA:TLS_AES_256_GCM_SHA384";
+#else
+        cipher_list = "AES256-GCM-SHA384:AES256-SHA";
+#endif
     }
 
-    Log(LOG_LEVEL_VERBOSE,
-        "Setting cipher list for incoming TLS connections to: %s",
-        cipher_list);
-
-    ret = SSL_CTX_set_cipher_list(*ssl_ctx, cipher_list);
-    if (ret != 1)
+    if (!TLSSetCipherList(*ssl_ctx, cipher_list))
     {
-        Log(LOG_LEVEL_ERR,
-            "No valid ciphers in cipher list: %s",
-            cipher_list);
         goto err;
     }
 
@@ -215,6 +216,8 @@ void ServerTLSDeInitialize(RSA **priv_key, RSA **pub_key, SSL_CTX **ssl_ctx)
  */
 bool ServerTLSPeek(ConnectionInfo *conn_info)
 {
+    assert(conn_info != NULL);
+
     assert(SSLSERVERCONTEXT != NULL);
     assert(PRIVKEY != NULL);
     assert(PUBKEY  != NULL);
@@ -286,14 +289,12 @@ bool ServerIdentificationDialog(ConnectionInfo *conn_info,
 {
     int ret;
     char input[1024] = "";
-    /* The only protocol version we support inside TLS, for now. */
-    const int SERVER_PROTOCOL_VERSION = CF_PROTOCOL_LATEST;
 
     /* Send "CFE_v%d cf-serverd version". */
     char version_string[CF_MAXVARSIZE];
     int len = snprintf(version_string, sizeof(version_string),
                        "CFE_v%d cf-serverd %s\n",
-                       SERVER_PROTOCOL_VERSION, VERSION);
+                       CF_PROTOCOL_LATEST, VERSION);
 
     ret = TLSSend(conn_info->ssl, version_string, len);
     if (ret != len)
@@ -311,9 +312,8 @@ bool ServerIdentificationDialog(ConnectionInfo *conn_info,
         return false;
     }
 
-    int version_received = -1;
-    ret = sscanf(input, "CFE_v%d", &version_received);
-    if (ret != 1)
+    const ProtocolVersion protocol = ParseProtocolVersionNetwork(input);
+    if (ProtocolIsUndefined(protocol))
     {
         Log(LOG_LEVEL_NOTICE,
             "Protocol version negotiation failed! Received: %s",
@@ -321,17 +321,25 @@ bool ServerIdentificationDialog(ConnectionInfo *conn_info,
         return false;
     }
 
-    /* For now we support only one version inside TLS. */
-    /* TODO value should not be hardcoded but compared to enum ProtocolVersion. */
-    if (version_received != SERVER_PROTOCOL_VERSION)
+    /* This is already inside TLS code, so TLS is required at this point*/
+    if (ProtocolIsClassic(protocol))
     {
         Log(LOG_LEVEL_NOTICE,
             "Client advertises disallowed protocol version: %d",
-            version_received);
+            protocol);
         return false;
-        /* TODO send "BAD ..." ? */
     }
 
+    if (ProtocolIsTooNew(protocol))
+    {
+        Log(LOG_LEVEL_NOTICE,
+            "Client attempted a protocol version which is too new for us: %d",
+            protocol);
+        return false;
+    }
+
+    assert(ProtocolIsKnown(protocol));
+
     /* Did we receive 2nd line or do we need to receive again? */
     const char id_line[] = "\nIDENTITY ";
     char *line2 = memmem(input, input_len, id_line, strlen(id_line));
@@ -396,7 +404,8 @@ bool ServerIdentificationDialog(ConnectionInfo *conn_info,
     }
 
     /* Version client and server agreed on. */
-    conn_info->protocol = version_received;
+    assert(ProtocolIsKnown(protocol));
+    conn_info->protocol = protocol;
 
     return true;
 }
@@ -471,7 +480,25 @@ bool BasicServerTLSSessionEstablish(ServerConnectionState *conn, SSL_CTX *ssl_ct
     /* Now we are letting OpenSSL take over the open socket. */
     SSL_set_fd(ssl, ConnectionInfoSocket(conn->conn_info));
 
-    int ret = SSL_accept(ssl);
+    int remaining_tries = MAX_ACCEPT_RETRIES;
+    int ret = -1;
+    bool should_retry = true;
+    while ((ret < 0) && should_retry)
+    {
+        ret = SSL_accept(ssl);
+        if (ret < 0)
+        {
+            int code = TLSLogError(ssl, LOG_LEVEL_VERBOSE, "SSL accept failed", ret);
+            should_retry = ((remaining_tries > 0) &&
+                            ((code == SSL_ERROR_WANT_READ) || (code == SSL_ERROR_WANT_WRITE)));
+
+        }
+        if ((ret < 0) && should_retry)
+        {
+            sleep(1);
+            remaining_tries--;
+        }
+    }
     if (ret <= 0)
     {
         TLSLogError(ssl, LOG_LEVEL_ERR,
@@ -570,8 +597,8 @@ bool ServerTLSSessionEstablish(ServerConnectionState *conn, SSL_CTX *ssl_ctx)
     SetConnIdentity(conn, username);
 
     /* No CAUTH, SAUTH in non-classic protocol. */
-    conn->user_data_set = 1;
-    conn->rsa_auth = 1;
+    conn->user_data_set = true;
+    conn->rsa_auth = true;
 
     LastSaw1(conn->ipaddr, KeyPrintableHash(ConnectionInfoKey(conn->conn_info)),
              LAST_SEEN_ROLE_ACCEPT);
@@ -612,6 +639,8 @@ ProtocolCommandNew GetCommandNew(char *str)
  */
 bool BusyWithNewProtocol(EvalContext *ctx, ServerConnectionState *conn)
 {
+    assert(conn != NULL);
+
     /* The CF_BUFEXT extra space is there to ensure we're not *reading* out of
      * bounds in commands that carry extra binary arguments, like MD5. */
     char recvbuffer[CF_BUFSIZE + CF_BUFEXT] = { 0 };
@@ -624,8 +653,8 @@ bool BusyWithNewProtocol(EvalContext *ctx, ServerConnectionState *conn)
     const int encrypted = 0;
 
     /* Legacy stuff only for old protocol. */
-    assert(conn->rsa_auth == 1);
-    assert(conn->user_data_set == 1);
+    assert(conn->rsa_auth == true);
+    assert(conn->user_data_set == true);
 
     /* Receive up to CF_BUFSIZE - 1 bytes. */
     const int received = ReceiveTransaction(conn->conn_info,
@@ -847,7 +876,7 @@ bool BusyWithNewProtocol(EvalContext *ctx, ServerConnectionState *conn)
             return true;
         }
 
-        if (IsDirReal(filename) == 1)
+        if (IsDirReal(filename))
         {
             PathAppendTrailingSlash(filename, strlen(filename));
         }
@@ -1034,10 +1063,12 @@ bool BusyWithNewProtocol(EvalContext *ctx, ServerConnectionState *conn)
             goto protocol_error;
         }
 
-        if (acl_CheckExact(query_acl, name,
-                           conn->ipaddr, conn->revdns,
-                           KeyPrintableHash(ConnectionInfoKey(conn->conn_info)))
-            == false)
+        const char *hostkey = KeyPrintableHash(
+            ConnectionInfoKey(conn->conn_info));
+        const bool access_to_query = acl_CheckExact(
+            query_acl, name, conn->ipaddr, conn->revdns, hostkey);
+
+        if (!access_to_query)
         {
             Log(LOG_LEVEL_INFO, "access denied to query: %s", query);
             RefuseAccess(conn, recvbuffer);
@@ -1051,6 +1082,32 @@ bool BusyWithNewProtocol(EvalContext *ctx, ServerConnectionState *conn)
 
         break;
     }
+    case PROTOCOL_COMMAND_COOKIE:
+    {
+        if (ConnectionInfoProtocolVersion(conn->conn_info) < CF_PROTOCOL_COOKIE)
+        {
+            goto protocol_error;
+        }
+
+        const char *hostkey = KeyPrintableHash(
+            ConnectionInfoKey(conn->conn_info));
+        const bool access_to_query_delta = acl_CheckExact(
+            query_acl, "delta", conn->ipaddr, conn->revdns, hostkey);
+
+        if (!access_to_query_delta)
+        {
+            Log(LOG_LEVEL_INFO, "access denied to cookie query: %s", recvbuffer);
+            RefuseAccess(conn, recvbuffer);
+            return true;
+        }
+
+        if (ReturnCookies(conn))
+        {
+            return true;
+        }
+
+        break;
+    }
     case PROTOCOL_COMMAND_CALL_ME_BACK:
         /* Server side, handing the collect call off to cf-hub. */
 
