@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -259,7 +259,7 @@ static bool SetMaxOpenFiles(int n)
         Log(LOG_LEVEL_INFO, "Failed setting max open files limit"
             " (setrlimit(NOFILE, %d): %s)", n, GetErrorStr());
         Log(LOG_LEVEL_INFO,
-            "Please ensure that 'nofile' ulimit is at least 2x maxconnections");
+            "Please ensure that 'nofile' ulimit is at least 5x maxconnections");
         return false;
     }
     else
@@ -355,7 +355,7 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
                                                    + EnterpriseGetMaxCfHubProcesses() + 10);
 
                 /* Set RLIMIT_NOFILE to be enough for all threads. */
-                SetMaxOpenFiles(CFD_MAXPROCESSES * 2 + 10);
+                SetMaxOpenFiles(CFD_MAXPROCESSES * 5 + 10);
             }
             else if (IsControlBody(SERVER_CONTROL_CALL_COLLECT_INTERVAL))
             {
@@ -376,7 +376,7 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
                 COLLECT_WINDOW = (int) IntFromString(value);
                 Log(LOG_LEVEL_VERBOSE,
                     "Setting collect_window to %d (seconds)",
-                    COLLECT_INTERVAL);
+                    COLLECT_WINDOW);
             }
             else if (IsControlBody(SERVER_CONTROL_CFRUNCOMMAND))
             {
@@ -473,6 +473,10 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
             {
                 bool ret = SetCfenginePort(value);
                 assert(ret);
+                if (ret == false)
+                {
+                    Log(LOG_LEVEL_VERBOSE, "Could not set port number, continuing (See errors above)");
+                }
             }
             else if (IsControlBody(SERVER_CONTROL_BIND_TO_INTERFACE))
             {
@@ -566,7 +570,7 @@ static const char *const COMMON_TYPESEQUENCE[] =
     NULL
 };
 
-/* Check if promise is NOT belonging to default server types 
+/* Check if promise is NOT belonging to default server types
  * (see SERVER_TYPESEQUENCE)*/
 static bool IsPromiseTypeNotInTypeSequence(const char *promise_type,
                                            const char * const *seq)
@@ -1245,8 +1249,8 @@ static void KeepFileAccessPromise(const EvalContext *ctx, const Promise *pp)
     {
         /* If it's a directory append trailing '/' */
         path_len = ret2;
-        int is_dir = IsDirReal(path);
-        if (is_dir == 1 && path[path_len - 1] != FILE_SEPARATOR)
+        bool is_dir = IsDirReal(path);
+        if (is_dir && path[path_len - 1] != FILE_SEPARATOR)
         {
             if (path_len + 2 > sizeof(path))
             {
