@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -144,7 +144,7 @@ void CFTestD_ConfigDestroy(CFTestD_Config *config)
 void CFTestD_Help()
 {
     Writer *w = FileWriter(stdout);
-    WriterWriteHelp(w, "cf-testd", OPTIONS, HINTS, true, NULL);
+    WriterWriteHelp(w, "cf-testd", OPTIONS, HINTS, NULL, false, true);
     FileWriterDetach(w);
 }
 
@@ -269,8 +269,8 @@ bool CFTestD_TLSSessionEstablish(ServerConnectionState *conn, CFTestD_Config *co
     }
 
     /* No CAUTH, SAUTH in non-classic protocol. */
-    conn->user_data_set = 1;
-    conn->rsa_auth      = 1;
+    conn->user_data_set = true;
+    conn->rsa_auth      = true;
 
     ServerSendWelcome(conn);
     return true;
@@ -371,6 +371,15 @@ static bool CFTestD_BusyLoop(
     // See BusyWithNewProtocol() in server_tls.c for how to add other commands
     switch (GetCommandNew(recvbuffer))
     {
+    case PROTOCOL_COMMAND_COOKIE:
+    {
+        const char response[] = "COOKIES 0 0";
+        const size_t size = sizeof(response) - 1;
+        SendTransaction(conn->conn_info, response, size, CF_DONE);
+
+        return true;
+        break;
+    }
     case PROTOCOL_COMMAND_QUERY:
     {
         char query[256], name[128];
