@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -29,6 +29,7 @@
 #include <classic.h>                  /* RecvSocketStream */
 #include <net.h>                      /* SendTransaction,ReceiveTransaction */
 #include <openssl/err.h>                                   /* ERR_get_error */
+#include <protocol.h>                              /* ProtocolIsUndefined() */
 #include <libcrypto-compat.h>
 #include <tls_client.h>               /* TLSTry */
 #include <tls_generic.h>              /* TLSVerifyPeer */
@@ -38,7 +39,7 @@
 #include <client_protocol.h>
 #include <crypto.h>         /* CryptoInitialize,SavePublicKey,EncryptString */
 #include <logging.h>
-#include <files_hashes.h>                                       /* HashFile */
+#include <hash.h>                                               /* HashFile */
 #include <mutex.h>                                            /* ThreadLock */
 #include <files_lib.h>                               /* FullWrite,safe_open */
 #include <string_lib.h>                           /* MemSpan,MemSpanInverse */
@@ -183,7 +184,7 @@ int TLSConnect(ConnectionInfo *conn_info, bool trust_server,
 }
 
 /**
- * @NOTE if #flags.protocol_version is CF_PROTOCOL_UNDEFINED, then classic
+ * @NOTE if #flags.protocol_version is CF_PROTOCOL_UNDEFINED, then latest
  *       protocol is used by default.
  */
 AgentConnection *ServerConnection(const char *server, const char *port,
@@ -232,14 +233,17 @@ AgentConnection *ServerConnection(const char *server, const char *port,
     assert(sizeof(conn->remoteip) >= sizeof(txtaddr));
     strcpy(conn->remoteip, txtaddr);
 
-    switch (flags.protocol_version)
+    ProtocolVersion protocol_version = flags.protocol_version;
+    if (ProtocolIsUndefined(protocol_version))
     {
-    case CF_PROTOCOL_UNDEFINED:
-    case CF_PROTOCOL_TLS:
+        protocol_version = CF_PROTOCOL_LATEST;
+    }
 
+    if (ProtocolIsTLS(protocol_version))
+    {
         /* Set the version to request during protocol negotiation. After
          * TLSConnect() it will have the version we finally ended up with. */
-        conn->conn_info->protocol = CF_PROTOCOL_LATEST;
+        conn->conn_info->protocol = protocol_version;
 
         ret = TLSConnect(conn->conn_info, flags.trust_server,
                          conn->remoteip, conn->username);
@@ -265,10 +269,9 @@ AgentConnection *ServerConnection(const char *server, const char *port,
             LastSaw1(conn->remoteip, KeyPrintableHash(conn->conn_info->remote_key),
                      LAST_SEEN_ROLE_CONNECT);
         }
-        break;
-
-    case CF_PROTOCOL_CLASSIC:
-
+    }
+    else if (ProtocolIsClassic(protocol_version))
+    {
         conn->conn_info->protocol = CF_PROTOCOL_CLASSIC;
         conn->encryption_type = CfEnterpriseOptions();
 
@@ -290,11 +293,11 @@ AgentConnection *ServerConnection(const char *server, const char *port,
             return NULL;
         }
         conn->conn_info->status = CONNECTIONINFO_STATUS_ESTABLISHED;
-        break;
-
-    default:
+    }
+    else
+    {
         ProgrammingError("ServerConnection: ProtocolVersion %d!",
-                         flags.protocol_version);
+                         protocol_version);
     }
 
     conn->authenticated = true;
@@ -456,7 +459,7 @@ Item *RemoteDirList(const char *dirname, bool encrypt, AgentConnection *conn)
 
 /*********************************************************************/
 
-int CompareHashNet(const char *file1, const char *file2, bool encrypt, AgentConnection *conn)
+bool CompareHashNet(const char *file1, const char *file2, bool encrypt, AgentConnection *conn)
 {
     unsigned char d[EVP_MAX_MD_SIZE + 1];
     char *sp;
@@ -544,9 +547,9 @@ int CompareHashNet(const char *file1, const char *file2, bool encrypt, AgentConn
 /*********************************************************************/
 
 
-int EncryptCopyRegularFileNet(const char *source, const char *dest, off_t size, AgentConnection *conn)
+static bool EncryptCopyRegularFileNet(const char *source, const char *dest, off_t size, AgentConnection *conn)
 {
-    int dd, blocksize = 2048, n_read = 0, plainlen, more = true, finlen, cnt = 0;
+    int blocksize = 2048, n_read = 0, plainlen, more = true, finlen, cnt = 0;
     int tosend, cipherlen = 0;
     char *buf, in[CF_BUFSIZE], out[CF_BUFSIZE], workbuf[CF_BUFSIZE], cfchangedstr[265];
     unsigned char iv[32] =
@@ -562,7 +565,8 @@ int EncryptCopyRegularFileNet(const char *source, const char *dest, off_t size,
 
     unlink(dest);                /* To avoid link attacks */
 
-    if ((dd = safe_open(dest, O_WRONLY | O_CREAT | O_TRUNC | O_EXCL | O_BINARY, 0600)) == -1)
+    int dd = safe_open_create_perms(dest, O_WRONLY | O_CREAT | O_TRUNC | O_EXCL | O_BINARY, CF_PERMS_DEFAULT);
+    if (dd == -1)
     {
         Log(LOG_LEVEL_ERR,
             "Copy from server '%s' to destination '%s' failed (open: %s)",
@@ -720,8 +724,8 @@ static void FlushFileStream(int sd, int toget)
 
 /* TODO finalise socket or TLS session in all cases that this function fails
  * and the transaction protocol is out of sync. */
-int CopyRegularFileNet(const char *source, const char *dest, off_t size,
-                       bool encrypt, AgentConnection *conn)
+bool CopyRegularFileNet(const char *source, const char *dest, off_t size,
+                        bool encrypt, AgentConnection *conn)
 {
     char *buf, workbuf[CF_BUFSIZE], cfchangedstr[265];
     const int buf_size = 2048;
@@ -745,7 +749,7 @@ int CopyRegularFileNet(const char *source, const char *dest, off_t size,
 
     unlink(dest);                /* To avoid link attacks */
 
-    int dd = safe_open(dest, O_WRONLY | O_CREAT | O_TRUNC | O_EXCL | O_BINARY, 0600);
+    int dd = safe_open_create_perms(dest, O_WRONLY | O_CREAT | O_TRUNC | O_EXCL | O_BINARY, CF_PERMS_DEFAULT);
     if (dd == -1)
     {
         Log(LOG_LEVEL_ERR,
@@ -789,15 +793,19 @@ int CopyRegularFileNet(const char *source, const char *dest, off_t size,
 
         /* Stage C1 - receive */
         int n_read;
-        switch(conn->conn_info->protocol)
+
+        const ProtocolVersion version = conn->conn_info->protocol;
+
+        if (ProtocolIsClassic(version))
         {
-        case CF_PROTOCOL_CLASSIC:
             n_read = RecvSocketStream(conn->conn_info->sd, buf, toget);
-            break;
-        case CF_PROTOCOL_TLS:
+        }
+        else if (ProtocolIsTLS(version))
+        {
             n_read = TLSRecv(conn->conn_info->ssl, buf, toget);
-            break;
-        default:
+        }
+        else
+        {
             UnexpectedError("CopyRegularFileNet: ProtocolVersion %d!",
                             conn->conn_info->protocol);
             n_read = -1;
