@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -46,9 +46,9 @@ AgentConnection *ServerConnection(const char *server, const char *port,
                                   ConnectionFlags flags, int *err);
 void DisconnectServer(AgentConnection *conn);
 
-int CompareHashNet(const char *file1, const char *file2, bool encrypt, AgentConnection *conn);
-int CopyRegularFileNet(const char *source, const char *dest, off_t size,
-                       bool encrypt, AgentConnection *conn);
+bool CompareHashNet(const char *file1, const char *file2, bool encrypt, AgentConnection *conn);
+bool CopyRegularFileNet(const char *source, const char *dest, off_t size,
+                        bool encrypt, AgentConnection *conn);
 Item *RemoteDirList(const char *dirname, bool encrypt, AgentConnection *conn);
 
 int TLSConnectCallCollect(ConnectionInfo *conn_info, const char *username);
