@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -41,9 +41,8 @@ extern char VFQNAME[];
 #include <unix.h>                       /* GetCurrentUsername */
 #include <lastseen.h>                   /* LastSaw */
 #include <crypto.h>                     /* PublicKeyFile */
-#include <files_hashes.h>               /* HashString,HashesMatch,HashPubKey*/
+#include <hash.h>                       /* HashString,HashesMatch,HashPubKey*/
 #include <known_dirs.h>
-#include <hash.h>
 #include <connection_info.h>
 #include <tls_generic.h>                                  /* TLSErrorString */
 
@@ -61,8 +60,10 @@ void SetSkipIdentify(bool enabled)
 
 /*********************************************************************/
 
-int IdentifyAgent(ConnectionInfo *conn_info)
+bool IdentifyAgent(ConnectionInfo *conn_info)
 {
+    assert(conn_info != NULL);
+
     char uname[CF_BUFSIZE], sendbuff[CF_BUFSIZE];
     char dnsname[CF_MAXVARSIZE], localip[CF_MAX_IP_LEN];
     int ret;
@@ -206,11 +207,11 @@ static bool SetSessionKey(AgentConnection *conn)
     return true;
 }
 
-int AuthenticateAgent(AgentConnection *conn, bool trust_key)
+bool AuthenticateAgent(AgentConnection *conn, bool trust_key)
 {
     char sendbuffer[CF_EXPANDSIZE], in[CF_BUFSIZE], *out, *decrypted_cchall;
     BIGNUM *nonce_challenge, *bn = NULL;
-    unsigned char digest[EVP_MAX_MD_SIZE];
+    unsigned char digest[EVP_MAX_MD_SIZE + 1];
     int encrypted_len, nonce_len = 0, len, session_size;
     bool need_to_implicitly_trust_server;
     char enterprise_field = 'c';
@@ -537,21 +538,21 @@ int AuthenticateAgent(AgentConnection *conn, bool trust_key)
 
 /*********************************************************************/
 
-int BadProtoReply(char *buf)
+bool BadProtoReply(char *buf)
 {
     return (strncmp(buf, "BAD: ", 5) == 0);
 }
 
 /*********************************************************************/
 
-int OKProtoReply(char *buf)
+bool OKProtoReply(char *buf)
 {
     return (strncmp(buf, "OK:", 3) == 0);
 }
 
 /*********************************************************************/
 
-int FailedProtoReply(char *buf)
+bool FailedProtoReply(char *buf)
 {
     return (strncmp(buf, CF_FAILEDSTR, strlen(CF_FAILEDSTR)) == 0);
 }
