@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -31,6 +31,7 @@
 #include <logging.h>
 #include <misc_lib.h>
 #include <cf3.defs.h>
+#include <protocol.h>
 
 
 /* TODO remove libpromises dependency. */
@@ -88,7 +89,7 @@ int SendTransaction(ConnectionInfo *conn_info,
     LogRaw(LOG_LEVEL_DEBUG, "SendTransaction data: ",
            work + CF_INBAND_OFFSET, len);
 
-    switch(conn_info->protocol)
+    switch (ProtocolClassicOrTLS(conn_info->protocol))
     {
 
     case CF_PROTOCOL_CLASSIC:
@@ -151,7 +152,7 @@ int ReceiveTransaction(ConnectionInfo *conn_info, char *buffer, int *more)
     int ret;
 
     /* Get control channel. */
-    switch(conn_info->protocol)
+    switch (ProtocolClassicOrTLS(conn_info->protocol))
     {
     case CF_PROTOCOL_CLASSIC:
         ret = RecvSocketStream(conn_info->sd, proto, CF_INBAND_OFFSET);
@@ -244,7 +245,7 @@ int ReceiveTransaction(ConnectionInfo *conn_info, char *buffer, int *more)
     }
 
     /* Get data. */
-    switch(conn_info->protocol)
+    switch (ProtocolClassicOrTLS(conn_info->protocol))
     {
     case CF_PROTOCOL_CLASSIC:
         ret = RecvSocketStream(conn_info->sd, buffer, len);
@@ -302,7 +303,7 @@ uint32_t bwlimit_kbytes = 0; /* desired limit, in kB/s */
 
 
 /** Throttle traffic, if next packet happens too soon after the previous one
- * 
+ *
  *  This function is global, across all network operations (and interfaces, perhaps)
  *  @param tosend Length of current packet being sent out (in bytes)
  */
