@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -22,7 +22,7 @@
   included file COSL.txt.
 */
 
-
+#include <cf3.defs.h> // CF_DEFAULT_DIGEST
 #include <cfnet.h>
 
 #include <openssl/err.h>
@@ -31,16 +31,58 @@
 
 #include <logging.h>                                            /* LogLevel */
 #include <misc_lib.h>
+#include <string_lib.h>
 
 /* TODO move crypto.h to libutils */
 #include <crypto.h>                                    /* HavePublicKeyByIP */
-#include <files_hashes.h>                              /* HashPubKey */
+#include <hash.h>                                             /* HashPubKey */
 
 #include <assert.h>
 
+/* known TLS versions */
+enum tls_version {
+    TLS_1_0 = 0,
+    TLS_1_1 = 1,
+    TLS_1_2 = 2,
+    TLS_1_3 = 3,
+};
+#define TLS_LAST TLS_1_3
+
+/* determine the highest TLS version supported by the available/used version of
+ * OpenSSL */
+#if defined(SSL_OP_NO_TLSv1_3)
+#define TLS_HIGHEST_SUPPORTED TLS_1_3
+#elif defined(SSL_OP_NO_TLSv1_2)
+#define TLS_HIGHEST_SUPPORTED TLS_1_2
+#elif defined(SSL_OP_NO_TLSv1_1)
+#define TLS_HIGHEST_SUPPORTED TLS_1_1
+#else
+#define TLS_HIGHEST_SUPPORTED TLS_1_0
+#endif
+
+/* the lowest version of TLS we always require */
+#define TLS_LOWEST_REQUIRED TLS_1_0
+
+/* the lowest version of TLS we recommend (also the default) */
+#define TLS_LOWEST_RECOMMENDED TLS_1_1
+
+#ifndef SSL_OP_NO_TLSv1_3
+#define SSL_OP_NO_TLSv1_3 0     /* no-op when ORed with bit flags */
+#endif
+#ifndef SSL_OP_NO_TLSv1_2
+#define SSL_OP_NO_TLSv1_2 0
+#endif
+#ifndef SSL_OP_NO_TLSv1_1
+#define SSL_OP_NO_TLSv1_1 0
+#endif
+
+static const char *const tls_version_strings[TLS_LAST + 1] = {"1.0", "1.1", "1.2", "1.3"};
+static unsigned int tls_disable_flags[TLS_LAST + 1] = {SSL_OP_NO_TLSv1, SSL_OP_NO_TLSv1_1, SSL_OP_NO_TLSv1_2, SSL_OP_NO_TLSv1_3};
 
 int CONNECTIONINFO_SSL_IDX = -1;
 
+#define MAX_READ_RETRIES 5
+#define MAX_WRITE_RETRIES 5
 
 const char *TLSErrorString(intmax_t errcode)
 {
@@ -637,21 +679,32 @@ int TLSSend(SSL *ssl, const char *buffer, int length)
 
     EnforceBwLimit(length);
 
-    int sent = SSL_write(ssl, buffer, length);
-    if (sent == 0)
+    int sent = -1;
+    bool should_retry = true;
+    int remaining_tries = MAX_WRITE_RETRIES;
+    while ((sent < 0) && should_retry)
     {
-        if ((SSL_get_shutdown(ssl) & SSL_RECEIVED_SHUTDOWN) != 0)
+        sent = SSL_write(ssl, buffer, length);
+        if (sent <= 0)
         {
-            Log(LOG_LEVEL_ERR,
-                "Remote peer terminated TLS session (SSL_write)");
-            return 0;
+            if ((SSL_get_shutdown(ssl) & SSL_RECEIVED_SHUTDOWN) != 0)
+            {
+                Log(LOG_LEVEL_ERR,
+                    "Remote peer terminated TLS session (SSL_write)");
+                return 0;
+            }
+
+            /* else */
+            int code = TLSLogError(ssl, LOG_LEVEL_VERBOSE, "SSL write failed", sent);
+            /* If renegotiation happens, SSL_ERROR_WANT_READ or
+             * SSL_ERROR_WANT_WRITE can be reported. See man:SSL_write(3).*/
+            should_retry = ((remaining_tries > 0) &&
+                            ((code == SSL_ERROR_WANT_READ) || (code == SSL_ERROR_WANT_WRITE)));
         }
-        else
+        if ((sent < 0) && should_retry)
         {
-            TLSLogError(ssl, LOG_LEVEL_ERR,
-                        "Connection unexpectedly closed (SSL_write)",
-                        sent);
-            return 0;
+            sleep(1);
+            remaining_tries--;
         }
     }
     if (sent < 0)
@@ -687,22 +740,42 @@ int TLSRecv(SSL *ssl, char *buffer, int toget)
     assert(toget < CF_BUFSIZE);
     assert_SSLIsBlocking(ssl);
 
-    int received = SSL_read(ssl, buffer, toget);
+    int received = -1;
+    bool should_retry = true;
+    int remaining_tries = MAX_READ_RETRIES;
+    while ((received < 0) && should_retry)
+    {
+        received = SSL_read(ssl, buffer, toget);
+        if (received < 0)
+        {
+            int code = TLSLogError(ssl, LOG_LEVEL_VERBOSE, "SSL read failed", received);
+            /* SSL_read() might get an internal recv() timeout, since we've set
+             * SO_RCVTIMEO. In that case SSL_read() returns SSL_ERROR_WANT_READ.
+             * Also, if renegotiation happens, SSL_ERROR_WANT_READ or
+             * SSL_ERROR_WANT_WRITE can be reported. See man:SSL_read(3).*/
+            should_retry = ((remaining_tries > 0) &&
+                            ((code == SSL_ERROR_WANT_READ) || (code == SSL_ERROR_WANT_WRITE)));
+
+        }
+        if ((received < 0) && should_retry)
+        {
+            sleep(1);
+            remaining_tries--;
+        }
+    }
+
     if (received < 0)
     {
-        int errcode = TLSLogError(ssl, LOG_LEVEL_ERR, "SSL_read", received);
-
-        /* SSL_read() might get an internal recv() timeout, since we've set
-         * SO_RCVTIMEO. In that case, the internal socket returns EAGAIN or
-         * EWOULDBLOCK and SSL_read() returns SSL_ERROR_WANT_READ. */
-        if (errcode == SSL_ERROR_WANT_READ)               /* recv() timeout */
+        int errcode = TLSLogError(ssl, LOG_LEVEL_ERR, "SSL read after retries", received);
+        /* if all the retries didn't help, let's at least make sure proper
+         * cleanup is done */
+        if ((errcode == SSL_ERROR_WANT_READ) || (errcode == SSL_ERROR_WANT_WRITE))
         {
             /* Make sure that the peer will send us no more data. */
             SSL_shutdown(ssl);
             shutdown(SSL_get_fd(ssl), SHUT_RDWR);
 
             /* Empty possible SSL_read() buffers. */
-
             int ret = 1;
             int bytes_still_buffered = SSL_pending(ssl);
             while (bytes_still_buffered > 0 && ret > 0)
@@ -816,65 +889,68 @@ void TLSSetDefaultOptions(SSL_CTX *ssl_ctx, const char *min_version)
       Log(LOG_LEVEL_WARNING, "Please update your CFEngine package or "
           "compile it against your current OpenSSL version.");
     }
-#endif
-
-    const char *compiletime_min_version = "1.2";
-
-#ifndef  SSL_OP_NO_TLSv1_1
-#   define SSL_OP_NO_TLSv1_1 0x0                                /* nop */
-    compiletime_min_version = "1.1";
-
-# ifndef  SSL_OP_NO_TLSv1
-#   define SSL_OP_NO_TLSv1 0x0                                  /* nop */
-    compiletime_min_version = "1.0";
-# endif
-
 #endif
 
     /* In any case use only TLS v1 or later. */
     long options = SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
 
-    if (min_version == NULL
-        || strcmp(min_version, "1")   == 0
-        || strcmp(min_version, "1.0") == 0)
+    assert(TLS_LOWEST_RECOMMENDED >= TLS_LOWEST_REQUIRED);
+    enum tls_version min_tls_version = TLS_LOWEST_RECOMMENDED;
+    if (!NULL_OR_EMPTY(min_version))
     {
-        /* Do nothing, that's our default setting */
-        Log(LOG_LEVEL_VERBOSE,
-            "Setting minimum acceptable TLS version: 1.0");
-    }
-    else if (strcmp(min_version, "1.1") == 0)
-    {
-        if (strcmp(compiletime_min_version, "1.0") == 0)
+        bool found = false;
+        for (enum tls_version v = TLS_1_0; !found && v <= TLS_LAST; v++)
         {
-            Log(LOG_LEVEL_WARNING, "Minimum requested TLS version is %s,"
-                " however because of old OpenSSL version it is set to: %s",
-                min_version, compiletime_min_version);
+            if (StringSafeEqual(min_version, tls_version_strings[v]))
+            {
+                found = true;
+                if (v < TLS_LOWEST_REQUIRED)
+                {
+                    Log(LOG_LEVEL_WARNING, "Minimum requested TLS version is %s,"
+                        " but minimum version required by CFEngine is %s."
+                        " Using the minimum required version.",
+                        min_version, tls_version_strings[TLS_LOWEST_REQUIRED]);
+                    min_tls_version = TLS_LOWEST_REQUIRED;
+                }
+                else if (v < TLS_LOWEST_RECOMMENDED)
+                {
+                    Log(LOG_LEVEL_WARNING, "Minimum requested TLS version is %s,"
+                        " but minimum version recommended by CFEngine is %s.",
+                        min_version, tls_version_strings[TLS_LOWEST_RECOMMENDED]);
+                    min_tls_version = v;
+                }
+                else if (v > TLS_HIGHEST_SUPPORTED)
+                {
+                    Log(LOG_LEVEL_WARNING, "Minimum requested TLS version is %s,"
+                        " but maximum version supported by OpenSSL is %s."
+                        " Using the maximum supported version.",
+                        min_version, tls_version_strings[TLS_HIGHEST_SUPPORTED]);
+                    min_tls_version = TLS_HIGHEST_SUPPORTED;
+                }
+                else
+                {
+                    min_tls_version = v;
+                }
+            }
         }
-        options |= SSL_OP_NO_TLSv1;
-        Log(LOG_LEVEL_VERBOSE,
-            "Setting minimum acceptable TLS version: 1.1");
-
-    }
-    else if (strcmp(min_version, "1.2") == 0)
-    {
-        if (strcmp(compiletime_min_version, "1.0") == 0 ||
-            strcmp(compiletime_min_version, "1.1") == 0)
+        if (!found)
         {
-            Log(LOG_LEVEL_WARNING, "Minimum requested TLS version is %s,"
-                " however because of old OpenSSL version it is set to: %s",
-                min_version, compiletime_min_version);
+            Log(LOG_LEVEL_WARNING,
+                "Unrecognized requested minimum TLS version '%s',"
+                " using the minimum required version %s.",
+                min_version, tls_version_strings[TLS_LOWEST_REQUIRED]);
+            min_tls_version = TLS_LOWEST_REQUIRED;
         }
-        options |= SSL_OP_NO_TLSv1 | SSL_OP_NO_TLSv1_1;
-        Log(LOG_LEVEL_VERBOSE,
-            "Setting minimum acceptable TLS version: 1.2");
-    }
-    else
-    {
-        Log(LOG_LEVEL_WARNING, "Unsupported TLS version '%s' requested,"
-            " minimum acceptable TLS version set to: 1.0",
-            min_version);
     }
 
+    Log(LOG_LEVEL_VERBOSE,
+        "Setting minimum acceptable TLS version: %s", tls_version_strings[min_tls_version]);
+
+    /* disable all the lower versions than the minimum requested/determined */
+    for (enum tls_version v = TLS_1_0; v < min_tls_version; v++)
+    {
+        options |= tls_disable_flags[v];
+    }
 
     /* No session resumption or renegotiation for now. */
     options |= SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION;
@@ -907,3 +983,83 @@ void TLSSetDefaultOptions(SSL_CTX *ssl_ctx, const char *min_version)
      * specific pointer to the callback (so we would have to lock).  */
     SSL_CTX_set_cert_verify_callback(ssl_ctx, TLSVerifyCallback, NULL);
 }
+
+bool TLSSetCipherList(SSL_CTX *ssl_ctx, const char *cipher_list)
+{
+    assert(ssl_ctx);
+
+    if (cipher_list == NULL)
+    {
+        Log(LOG_LEVEL_VERBOSE, "Using the OpenSSL's default cipher list");
+        /* nothing more to do */
+        return true;
+    }
+
+    Log(LOG_LEVEL_VERBOSE, "Setting cipher list for TLS connections to: %s",
+        cipher_list);
+
+    const size_t max_len = strlen(cipher_list) + 1; /* NUL byte */
+    size_t n_specs = StringCountTokens(cipher_list, max_len, ":");
+
+    /* TLS 1.3 defines cipher suites, they start with "TLS_" */
+    char ciphers[max_len];
+    size_t ciphers_len = 0;
+
+    char cipher_suites[max_len];
+    size_t cipher_suites_len = 0;
+
+    for (size_t i = 0; i < n_specs; i++)
+    {
+        StringRef spec_ref = StringGetToken(cipher_list, max_len, i, ":");
+        if (StringStartsWith(spec_ref.data, "TLS_"))
+        {
+            StrCat(cipher_suites, max_len, &cipher_suites_len, spec_ref.data, spec_ref.len + 1);
+        }
+        else
+        {
+            StrCat(ciphers, max_len, &ciphers_len, spec_ref.data, spec_ref.len + 1);
+        }
+    }
+
+    if (ciphers_len != 0)       /* TLS <= 1.2 ciphers */
+    {
+        int ret = SSL_CTX_set_cipher_list(ssl_ctx, ciphers);
+        if (ret != 1)
+        {
+            Log(LOG_LEVEL_ERR, "No valid ciphers in the cipher list: %s", cipher_list);
+            return false;
+        }
+    }
+
+#ifdef HAVE_TLS_1_3
+    if (cipher_suites_len != 0) /* TLS >= 1.3 ciphers */
+    {
+        int ret = SSL_CTX_set_ciphersuites(ssl_ctx, cipher_suites);
+        if (ret != 1)
+        {
+            Log(LOG_LEVEL_ERR, "No valid cipher suites in the list: %s", cipher_list);
+            return false;
+        }
+    }
+    else
+    {
+        /* Allowed ciphers specified, but no TLS 1.3 ciphersuites among them.
+           Let's disable TLS 1.3 because otherwise OpenSSL uses the default
+           ciphersuites for TLS 1.3 and thus effectively extends the specified
+           list of allowed ciphers behind our back. */
+        Log(LOG_LEVEL_WARNING,
+            "Disabling TLS 1.3 because no TLS 1.3 ciphersuites specified in allowed ciphers: '%s'",
+            cipher_list);
+        SSL_CTX_set_options(ssl_ctx, SSL_OP_NO_TLSv1_3);
+    }
+#else
+    if (cipher_suites_len != 0)
+    {
+        Log(LOG_LEVEL_WARNING,
+            "Ignoring requested TLS 1.3 ciphersuites '%s', TLS 1.3 not supported",
+            cipher_suites);
+    }
+#endif
+
+    return true;
+}
