@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -30,6 +30,7 @@
 #include <misc_lib.h>
 #include <communication.h>
 #include <string_lib.h>
+#include <string_sequence.h>                             /* SeqStringFromString */
 #include <regex.h>                                       /* StringMatchFull */
 #include <files_interfaces.h>
 #include <files_names.h>
@@ -79,7 +80,7 @@
 static int aix_get_mac_addr(const char *device_name, uint8_t mac[6]);
 #endif
 
-static void FindV6InterfacesInfo(EvalContext *ctx);
+static void FindV6InterfacesInfo(EvalContext *ctx, Rlist **interfaces, Rlist **hardware, Rlist **ips);
 static bool IgnoreJailInterface(int ifaceidx, struct sockaddr_in *inaddr);
 static bool IgnoreInterface(char *name);
 static void InitIgnoreInterfaces(void);
@@ -133,7 +134,7 @@ static bool IgnoreJailInterface(
 
 /******************************************************************/
 
-static void GetMacAddress(EvalContext *ctx, int fd, struct ifreq *ifr, struct ifreq *ifp, Rlist **interfaces,
+static void GetMacAddress(EvalContext *ctx, ARG_UNUSED int fd, struct ifreq *ifr, struct ifreq *ifp, Rlist **interfaces,
                           Rlist **hardware)
 {
     char name[CF_MAXVARSIZE];
@@ -164,8 +165,14 @@ static void GetMacAddress(EvalContext *ctx, int fd, struct ifreq *ifr, struct if
              (unsigned char) ifr->ifr_hwaddr.sa_data[5]);
 
     EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, name, hw_mac, CF_DATA_TYPE_STRING, "source=agent");
-    RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
-    RlistAppend(interfaces, ifp->ifr_name, RVAL_TYPE_SCALAR);
+    if (!RlistContainsString(*hardware, hw_mac))
+    {
+        RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
+    }
+    if (!RlistContainsString(*interfaces, ifp->ifr_name))
+    {
+        RlistAppend(interfaces, ifp->ifr_name, RVAL_TYPE_SCALAR);
+    }
 
     snprintf(name, sizeof(name), "mac_%s", CanonifyName(hw_mac));
     EvalContextClassPutHard(ctx, name, "inventory,attribute_name=none,source=agent");
@@ -203,7 +210,10 @@ static void GetMacAddress(EvalContext *ctx, int fd, struct ifreq *ifr, struct if
                     (unsigned char) m[5]);
 
                 EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, name, hw_mac, CF_DATA_TYPE_STRING, "source=agent");
-                RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
+                if (!RlistContainsString(*hardware, hw_mac))
+                {
+                    RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
+                }
                 RlistAppend(interfaces, ifa->ifa_name, RVAL_TYPE_SCALAR);
 
                 snprintf(name, sizeof(name), "mac_%s", CanonifyName(hw_mac));
@@ -224,8 +234,15 @@ static void GetMacAddress(EvalContext *ctx, int fd, struct ifreq *ifr, struct if
 	       mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
 
         EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, name, hw_mac, CF_DATA_TYPE_STRING, "source=agent");
-        RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
-        RlistAppend(interfaces, ifp->ifr_name, RVAL_TYPE_SCALAR);
+
+        if (!RlistContainsString(*hardware, hw_mac))
+        {
+            RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
+        }
+        if (!RlistContainsString(*interfaces, ifp->ifr_name))
+        {
+            RlistAppend(interfaces, ifp->ifr_name, RVAL_TYPE_SCALAR);
+        }
 
         snprintf(name, CF_MAXVARSIZE, "mac_%s", CanonifyName(hw_mac));
         EvalContextClassPutHard(ctx, name, "inventory,attribute_name=none,source=agent");
@@ -272,8 +289,16 @@ static void GetMacAddress(EvalContext *ctx, int fd, struct ifreq *ifr, struct if
     EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, name,
                                   hw_mac, CF_DATA_TYPE_STRING,
                                   "source=agent");
-    RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
-    RlistAppend(interfaces, ifp->ifr_name, RVAL_TYPE_SCALAR);
+
+    if (!RlistContainsString(*hardware, hw_mac))
+    {
+        RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
+    }
+
+    if (!RlistContainsString(*interfaces, ifp->ifr_name))
+    {
+        RlistAppend(interfaces, ifp->ifr_name, RVAL_TYPE_SCALAR);
+    }
 
     snprintf(name, sizeof(name), "mac_%s", CanonifyName(hw_mac));
     EvalContextClassPutHard(ctx, name,
@@ -555,23 +580,29 @@ void GetInterfacesInfo(EvalContext *ctx)
 
     close(fd);
 
+    FindV6InterfacesInfo(ctx, &interfaces, &hardware, &ips);
+
     if (interfaces)
     {
+        // Define sys.interfaces:
         EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, "interfaces", interfaces, CF_DATA_TYPE_STRING_LIST,
                                       "inventory,source=agent,attribute_name=Interfaces");
     }
     if (hardware)
     {
+        // Define sys.hardware_addresses:
         EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, "hardware_addresses", hardware, CF_DATA_TYPE_STRING_LIST,
                                       "inventory,source=agent,attribute_name=MAC addresses");
     }
     if (flags)
     {
+        // Define sys.hardware_flags:
         EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, "hardware_flags", flags, CF_DATA_TYPE_STRING_LIST,
                                       "source=agent");
     }
     if (ips)
     {
+        // Define sys.ip_addresses:
         EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, "ip_addresses", ips, CF_DATA_TYPE_STRING_LIST,
                                       "source=agent");
     }
@@ -580,16 +611,16 @@ void GetInterfacesInfo(EvalContext *ctx)
     RlistDestroy(hardware);
     RlistDestroy(flags);
     RlistDestroy(ips);
-
-    FindV6InterfacesInfo(ctx);
 }
 
 /*******************************************************************/
 
-static void FindV6InterfacesInfo(EvalContext *ctx)
+static void FindV6InterfacesInfo(EvalContext *ctx, Rlist **interfaces, Rlist **hardware, Rlist **ips)
 {
+    assert(interfaces != NULL);
+    assert(hardware != NULL);
+
     FILE *pp = NULL;
-    char buffer[CF_BUFSIZE];
 
 /* Whatever the manuals might say, you cannot get IPV6
    interface configuration from the ioctls. This seems
@@ -626,9 +657,12 @@ static void FindV6InterfacesInfo(EvalContext *ctx)
 
 /* Don't know the output format of ifconfig on all these .. hope for the best*/
 
+    char ifconfig_line[CF_BUFSIZE];
+    char current_interface[CF_BUFSIZE];
+    current_interface[0] = '\0';
     for(;;)
     {
-        if (fgets(buffer, sizeof(buffer), pp) == NULL)
+        if (fgets(ifconfig_line, sizeof(ifconfig_line), pp) == NULL)
         {
             if (ferror(pp))
             {
@@ -641,12 +675,86 @@ static void FindV6InterfacesInfo(EvalContext *ctx)
             }
         }
 
-        if (strcasestr(buffer, "inet6"))
+        if (!isspace(ifconfig_line[0])) // Name of interface followed by colon
+        {
+            // Find colon:
+            const char *colon = strchr(ifconfig_line, ':');
+            if (colon != NULL)
+            {
+                // Bytes to "copy" includes colon(src)/NUL-byte(dst):
+                const size_t bytes_to_copy = colon - ifconfig_line + 1;
+                assert(bytes_to_copy <= sizeof(current_interface));
+
+                const size_t src_length = StringCopy(
+                    ifconfig_line, current_interface, bytes_to_copy);
+                // StringCopy effectively returns the length of the src string,
+                // up to (inclusive) a maximum of passed in buffer size
+
+                // We know there was more data, at least a colon:
+                assert(src_length == bytes_to_copy);
+                const size_t dst_length = src_length - 1;
+
+                // We copied everything up to, but not including, the colon:
+                assert(ifconfig_line[dst_length] == ':');
+                assert(current_interface[dst_length] == '\0');
+            }
+        }
+
+
+        const char *const stripped_ifconfig_line =
+            TrimWhitespace(ifconfig_line);
+        if (StringStartsWith(stripped_ifconfig_line, "ether "))
+        {
+            Seq *const ether_line =
+                SeqStringFromString(stripped_ifconfig_line, ' ');
+            const size_t length = SeqLength(ether_line);
+
+            if (length < 2)
+            {
+                Log(LOG_LEVEL_ERR,
+                    "Failed to parse hw_mac address for: %s",
+                    current_interface);
+            }
+            else
+            {
+                const char *const hw_mac = SeqAt(ether_line, 1);
+
+                if (!RlistContainsString(*hardware, hw_mac))
+                {
+                    Log(LOG_LEVEL_VERBOSE,
+                        "Adding MAC address: %s for %s",
+                        hw_mac,
+                        current_interface);
+
+                    RlistAppendString(hardware, hw_mac);
+
+                    char variable_name[CF_MAXVARSIZE];
+
+                    snprintf(
+                        variable_name,
+                        sizeof(variable_name),
+                        "hardware_mac[%s]",
+                        CanonifyName(current_interface));
+
+                    EvalContextVariablePutSpecial(
+                        ctx,
+                        SPECIAL_SCOPE_SYS,
+                        variable_name,
+                        hw_mac,
+                        CF_DATA_TYPE_STRING,
+                        "source=agent,derived-from=ifconfig");
+                }
+            }
+
+            SeqDestroy(ether_line);
+        }
+
+        if (strcasestr(ifconfig_line, "inet6"))
         {
             Item *ip, *list = NULL;
             char *sp;
 
-            list = SplitStringAsItemList(buffer, ' ');
+            list = SplitStringAsItemList(ifconfig_line, ' ');
 
             for (ip = list; ip != NULL; ip = ip->next)
             {
@@ -662,11 +770,36 @@ static void FindV6InterfacesInfo(EvalContext *ctx)
                 {
                     char prefixed_ip[CF_MAX_IP_LEN + sizeof(IPV6_PREFIX)] = {0};
                     Log(LOG_LEVEL_VERBOSE, "Found IPv6 address %s", ip->name);
-                    EvalContextAddIpAddress(ctx, ip->name, NULL); // interface unknown
-                    EvalContextClassPutHard(ctx, ip->name, "inventory,attribute_name=none,source=agent");
 
-                    xsnprintf(prefixed_ip, sizeof(prefixed_ip), IPV6_PREFIX"%s", ip->name);
-                    EvalContextClassPutHard(ctx, prefixed_ip, "inventory,attribute_name=none,source=agent");
+                    if (current_interface[0] != '\0'
+                        && !IgnoreInterface(current_interface))
+                    {
+                        EvalContextAddIpAddress(
+                            ctx, ip->name, current_interface);
+                        EvalContextClassPutHard(
+                            ctx,
+                            ip->name,
+                            "inventory,attribute_name=none,source=agent");
+
+                        xsnprintf(
+                            prefixed_ip,
+                            sizeof(prefixed_ip),
+                            IPV6_PREFIX "%s",
+                            ip->name);
+                        EvalContextClassPutHard(
+                            ctx,
+                            prefixed_ip,
+                            "inventory,attribute_name=none,source=agent");
+
+                        // Add IPv6 address to sys.ip_addresses
+                        RlistAppendString(ips, ip->name);
+
+                        if (!RlistContainsString(
+                                *interfaces, current_interface))
+                        {
+                            RlistAppendString(interfaces, current_interface);
+                        }
+                    }
                 }
             }
 
@@ -841,7 +974,7 @@ static long JsonExtractParsedNumber(JsonElement* element, const char* raw_key, c
 
 /*******************************************************************/
 
-static ProcPostProcessFn NetworkingRoutesPostProcessInfo(
+static void NetworkingRoutesPostProcessInfo(
     ARG_LINUX_ONLY void *passed_ctx, ARG_LINUX_ONLY void *json)
 {
 # if defined (__linux__)
@@ -886,7 +1019,6 @@ static ProcPostProcessFn NetworkingRoutesPostProcessInfo(
         BufferDestroy(formatter);
     }
 # endif
-    return NULL;
 }
 
 static ProcPostProcessFn NetworkingIPv6RoutesPostProcessInfo(
