@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -41,7 +41,7 @@
 
 #ifdef HAVE_LIBACL
 
-int CopyACLs(const char *src, const char *dst)
+bool CopyACLs(const char *src, const char *dst)
 {
     acl_t acls;
     struct stat statbuf;
@@ -105,7 +105,7 @@ int CopyACLs(const char *src, const char *dst)
 
 #elif !defined(__MINGW32__) /* !HAVE_LIBACL */
 
-int CopyACLs(ARG_UNUSED const char *src, ARG_UNUSED const char *dst)
+bool CopyACLs(ARG_UNUSED const char *src, ARG_UNUSED const char *dst)
 {
     return true;
 }
