@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -30,6 +30,7 @@
 #include <logging.h>
 #include <chflags.h>
 #include <audit.h>
+#include <protocol.h> // ParseProtocolVersionPolicy()
 
 #define CF_DEFINECLASSES "classes"
 #define CF_TRANSACTION   "action"
@@ -44,7 +45,7 @@ void ClearFilesAttributes(Attributes *whom)
 
 Attributes GetFilesAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
 // default for file copy
 
@@ -115,7 +116,7 @@ Attributes GetFilesAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetReportsAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.transaction = GetTransactionConstraints(ctx, pp);
     attr.classes = GetClassDefinitionConstraints(ctx, pp);
@@ -128,7 +129,7 @@ Attributes GetReportsAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetEnvironmentsAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.transaction = GetTransactionConstraints(ctx, pp);
     attr.classes = GetClassDefinitionConstraints(ctx, pp);
@@ -141,7 +142,7 @@ Attributes GetEnvironmentsAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetServicesAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.transaction = GetTransactionConstraints(ctx, pp);
     attr.classes = GetClassDefinitionConstraints(ctx, pp);
@@ -155,7 +156,7 @@ Attributes GetServicesAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetPackageAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.transaction = GetTransactionConstraints(ctx, pp);
     attr.classes = GetClassDefinitionConstraints(ctx, pp);
@@ -201,7 +202,7 @@ static User GetUserConstraints(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetUserAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.havebundle = PromiseBundleOrBodyConstraintExists(ctx, "home_bundle", pp);
 
@@ -217,7 +218,7 @@ Attributes GetUserAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetDatabaseAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.transaction = GetTransactionConstraints(ctx, pp);
     attr.classes = GetClassDefinitionConstraints(ctx, pp);
@@ -229,7 +230,7 @@ Attributes GetDatabaseAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetClassContextAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes a = { {0} };
+    Attributes a = ZeroAttributes;
 
     a.transaction = GetTransactionConstraints(ctx, pp);
     a.classes = GetClassDefinitionConstraints(ctx, pp);
@@ -242,7 +243,7 @@ Attributes GetClassContextAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetExecAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.contain = GetExecContainConstraints(ctx, pp);
     attr.havecontain = PromiseGetConstraintAsBoolean(ctx, "contain", pp);
@@ -251,6 +252,16 @@ Attributes GetExecAttributes(const EvalContext *ctx, const Promise *pp)
     attr.arglist = PromiseGetConstraintAsList(ctx, "arglist", pp);
     attr.module = PromiseGetConstraintAsBoolean(ctx, "module", pp);
 
+    // Possible to suppress info level messages per commands promise:
+    if (PromiseBundleOrBodyConstraintExists(ctx, "inform", pp))
+    {
+        attr.inform = PromiseGetConstraintAsBoolean(ctx, "inform", pp);
+    }
+    else
+    {
+        attr.inform = true; // Default to printing the inform messages
+    }
+
 /* Common ("included") */
 
     attr.havetrans = PromiseGetConstraintAsBoolean(ctx, CF_TRANSACTION, pp);
@@ -266,7 +277,7 @@ Attributes GetExecAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetProcessAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.signals = PromiseGetConstraintAsList(ctx, "signals", pp);
     attr.process_stop = PromiseGetConstraintAsRval(pp, "process_stop", RVAL_TYPE_SCALAR);
@@ -292,7 +303,7 @@ Attributes GetProcessAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetStorageAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.mount = GetMountConstraints(ctx, pp);
     attr.volume = GetVolumeConstraints(ctx, pp);
@@ -319,7 +330,7 @@ Attributes GetStorageAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetMethodAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.havebundle = PromiseBundleOrBodyConstraintExists(ctx, "usebundle", pp);
 
@@ -338,7 +349,7 @@ Attributes GetMethodAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetMeasurementAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.measure = GetMeasurementConstraint(ctx, pp);
 
@@ -926,15 +937,10 @@ FileCopy GetCopyConstraints(const EvalContext *ctx, const Promise *pp)
     f.protocol_version = CF_PROTOCOL_UNDEFINED;
     if (protocol_version != NULL)
     {
-        if (strcmp(protocol_version, "1") == 0 ||
-            strcmp(protocol_version, "classic") == 0)
+        ProtocolVersion parsed = ParseProtocolVersionPolicy(protocol_version);
+        if (ProtocolIsKnown(parsed))
         {
-            f.protocol_version = CF_PROTOCOL_CLASSIC;
-        }
-        else if (strcmp(protocol_version, "2") == 0 ||
-                 strcmp(protocol_version, "latest") == 0)
-        {
-            f.protocol_version = CF_PROTOCOL_TLS;
+            f.protocol_version = parsed;
         }
     }
 
@@ -1131,6 +1137,7 @@ Packages GetPackageConstraints(const EvalContext *ctx, const Promise *pp)
             CopyBodyConstraintsToPromise((EvalContext*)ctx, (Promise*)pp, bp);
             has_generic_package_method = true;
         }
+        SeqDestroy(bodies_and_args);
     }
 
 
@@ -1409,7 +1416,7 @@ ProcessCount GetMatchesConstraints(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetInsertionAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.havelocation = PromiseGetConstraintAsBoolean(ctx, "location", pp);
     attr.location = GetLocationAttributes(pp);
@@ -1466,7 +1473,7 @@ EditLocation GetLocationAttributes(const Promise *pp)
 
 Attributes GetDeletionAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.not_matching = PromiseGetConstraintAsBoolean(ctx, "not_matching", pp);
 
@@ -1493,7 +1500,7 @@ Attributes GetDeletionAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetColumnAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.havecolumn = PromiseGetConstraintAsBoolean(ctx, "edit_field", pp);
     attr.column = GetColumnConstraints(ctx, pp);
@@ -1516,7 +1523,7 @@ Attributes GetColumnAttributes(const EvalContext *ctx, const Promise *pp)
 
 Attributes GetReplaceAttributes(const EvalContext *ctx, const Promise *pp)
 {
-    Attributes attr = { {0} };
+    Attributes attr = ZeroAttributes;
 
     attr.havereplace = PromiseGetConstraintAsBoolean(ctx, "replace_patterns", pp);
     attr.replace = GetReplaceConstraints(pp);
