@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -40,6 +40,9 @@
 #include <communication.h>
 #include <client_code.h>
 #include <assert.h>
+#include <crypto.h>
+#include <openssl/rand.h>
+#include <encode.h>
 
 /*
 Bootstrapping is a tricky sequence of fragile events. We need to map shakey/IP
@@ -266,7 +269,7 @@ bool WriteBuiltinFailsafePolicyToPath(const char *filename)
 
     Log(LOG_LEVEL_INFO, "Writing built-in failsafe policy to '%s'", filename);
 
-    FILE *fout = fopen(filename, "w");
+    FILE *fout = safe_fopen(filename, "w");
     if (!fout)
     {
         Log(LOG_LEVEL_ERR, "Unable to write failsafe to '%s' (fopen: %s)", filename, GetErrorStr());
@@ -276,12 +279,6 @@ bool WriteBuiltinFailsafePolicyToPath(const char *filename)
     fputs(bootstrap_content, fout);
     fclose(fout);
 
-    if (chmod(filename, S_IRUSR | S_IWUSR) == -1)
-    {
-        Log(LOG_LEVEL_ERR, "Failed setting permissions on generated failsafe file '%s'", filename);
-        return false;
-    }
-
     return true;
 }
 
@@ -353,3 +350,83 @@ bool MasterfileExists(const char *masterdir)
 
     return true;
 }
+
+static char *BootstrapIDFilename(const char *workdir)
+{
+    assert(workdir != NULL);
+    return StringFormat("%s%cbootstrap_id.dat", workdir, FILE_SEPARATOR);
+}
+
+char *CreateBootstrapIDFile(const char *workdir)
+{
+    assert(workdir != NULL);
+    char *filename = BootstrapIDFilename(workdir);
+
+    FILE *file = safe_fopen_create_perms(filename, "w", CF_PERMS_DEFAULT);
+    if (file == NULL)
+    {
+        Log(LOG_LEVEL_ERR, "Unable to write bootstrap id file '%s' (fopen: %s)", filename, GetErrorStr());
+        free(filename);
+        return NULL;
+    }
+    CryptoInitialize();
+    #define RANDOM_BYTES 240 / 8 // 240 avoids padding (divisible by 6)
+    #define BASE_64_LENGTH_NO_PADDING (4 * (RANDOM_BYTES / 3))
+    unsigned char buf[RANDOM_BYTES];
+    RAND_bytes(buf, RANDOM_BYTES);
+    char *b64_id = StringEncodeBase64(buf, RANDOM_BYTES);
+    fprintf(file, "%s\n", b64_id);
+    fclose(file);
+
+    free(filename);
+    return b64_id;
+}
+
+char *ReadBootstrapIDFile(const char *workdir)
+{
+    assert(workdir != NULL);
+
+    char *const path = BootstrapIDFilename(workdir);
+    Writer *writer = FileRead(path, BASE_64_LENGTH_NO_PADDING + 1, NULL);
+    if (writer == NULL)
+    {
+        // Not having a bootstrap id file is considered normal
+        Log(LOG_LEVEL_DEBUG,
+            "Could not read bootstrap ID from file: '%s'",
+            path);
+        free(path);
+        return NULL;
+    }
+    char *data = StringWriterClose(writer);
+
+    size_t data_length = strlen(data);
+    assert(data_length == BASE_64_LENGTH_NO_PADDING + 1);
+    assert(data[data_length - 1] == '\n');
+    if (data_length != BASE_64_LENGTH_NO_PADDING + 1)
+    {
+        Log(LOG_LEVEL_ERR, "'%s' contains invalid data: '%s'", path, data);
+        free(path);
+        free(data);
+        return NULL;
+    }
+
+    data[data_length - 1] = '\0';
+
+    Log(LOG_LEVEL_VERBOSE,
+        "Successfully read bootstrap ID '%s' from file '%s'",
+        data,
+        path);
+    free(path);
+    return data;
+}
+
+void EvalContextSetBootstrapID(EvalContext *ctx, char *bootstrap_id)
+{
+    EvalContextVariablePutSpecial(
+        ctx,
+        SPECIAL_SCOPE_SYS,
+        "bootstrap_id",
+        bootstrap_id,
+        CF_DATA_TYPE_STRING,
+        "source=bootstrap");
+}
