@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -35,8 +35,8 @@ int NovaWin_chmod(const char *path, mode_t mode);
 /* win_file.c */
 
 int NovaWin_rename(const char *oldpath, const char *newpath);
-int NovaWin_FileExists(const char *fileName);
-int NovaWin_IsDir(char *fileName);
+bool NovaWin_FileExists(const char *fileName);
+bool NovaWin_IsDir(char *fileName);
 int NovaWin_TakeFileOwnership(char *path);
 int NovaWin_SetFileOwnership(char *path, SID *sid);
 off_t NovaWin_GetDiskUsage(char *file, CfSize type);
@@ -70,12 +70,6 @@ int NovaWin_GetSysDir(char *sysDir, int sysDirSz);
 int NovaWin_GetProgDir(char *progDir, int progDirSz);
 int NovaWin_GetEnv(char *varName, char *varContents, int varContentsSz);
 
-const char *GetDefaultWorkDir(void);
-const char *GetDefaultLogDir(void);
-const char *GetDefaultPidDir(void);
-const char *GetDefaultMasterDir(void);
-const char *GetDefaultInputDir(void);
-
 /* win_user.c */
 
 int NovaWin_UserNameToSid(char *userName, SID *sid, DWORD sidSz, int shouldExist);
@@ -89,7 +83,7 @@ FnCallResult FnCallGroupExists(EvalContext *ctx, const Policy *policy, const FnC
 
 /* win_wmi.c */
 
-int NovaWin_PackageListInstalledFromAPI(EvalContext *ctx, PackageItem ** pkgList, Attributes a, Promise *pp);
+int NovaWin_PackageListInstalledFromAPI(EvalContext *ctx, PackageItem ** pkgList, const Attributes *a, Promise *pp);
 
 /* win_execd_pipe.c */
 
