@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -35,7 +35,8 @@
 #include <sequence.h>
 #include <logging.h>
 
-#include <cfnet.h>                       /* ProtocolVersion, CF_BUFSIZE etc */
+#include <definitions.h>                 /* CF_MAXVARSIZE, CF_BUFSIZE etc   */
+#include <cfnet.h>                       /* ProtocolVersion, etc */
 #include <misc_lib.h>                    /* xsnprintf, ProgrammingError etc */
 
 /*******************************************************************/
@@ -46,26 +47,10 @@
 #undef interface
 #endif
 
-
-/*******************************************************************/
-/* Preprocessor tricks                                             */
-/*******************************************************************/
-
-/* Convert integer constant to string */
-#define STRINGIFY__INTERNAL_MACRO(x) #x
-#define TOSTRING(x) STRINGIFY__INTERNAL_MACRO(x)
-
 /*******************************************************************/
 /* Various defines                                                 */
 /*******************************************************************/
 
-#define CF_MAXSIZE 102400000
-#define CF_BILLION 1000000000L
-#define CF_EXPANDSIZE (2*CF_BUFSIZE)
-#define CF_BUFFERMARGIN 128
-#define CF_BLOWFISHSIZE 16
-#define CF_MAXVARSIZE 1024
-#define CF_MAXSIDSIZE 2048      /* Windows only: Max size (bytes) of security identifiers */
 #define CF_MAXFRAGMENT 19       /* abbreviate long promise names to 2*MAXFRAGMENT+3 */
 #define CF_NONCELEN (CF_BUFSIZE/16)
 #define CF_MAXLINKSIZE 256
@@ -109,12 +94,15 @@
 
 #define CF_ENV_FILE      "env_data"
 
+#define CF_DB_REPAIR_TRIGGER "db_repair_required"
+
 #define CF_SAVED ".cfsaved"
 #define CF_EDITED ".cfedited"
 #define CF_NEW ".cfnew"
 #define CFD_TERMINATOR "---cfXen/gine/cfXen/gine---"
 #define CFD_TRUE "CFD_TRUE"
 #define CFD_FALSE "CFD_FALSE"
+#define CFD_FALSE_SIZE sizeof(CFD_FALSE) // size of CFD_FALSE including terminator
 #define CF_ANYCLASS "any"
 #define CF_SMALL_OFFSET 2
 
@@ -131,15 +119,15 @@
 /* Auditing key */
 
 typedef enum
-{
-    PROMISE_RESULT_SKIPPED,
-    PROMISE_RESULT_NOOP = 'n',
-    PROMISE_RESULT_CHANGE = 'c',
-    PROMISE_RESULT_WARN = 'w', // something wrong but nothing done
-    PROMISE_RESULT_FAIL = 'f',
-    PROMISE_RESULT_DENIED = 'd',
-    PROMISE_RESULT_TIMEOUT = 't',
-    PROMISE_RESULT_INTERRUPTED = 'i'
+{                                     // Logging of outcomes in cf-agent.c:
+    PROMISE_RESULT_SKIPPED = 's',     // <Also skipped in logging>
+    PROMISE_RESULT_NOOP = 'n',        // Kept
+    PROMISE_RESULT_CHANGE = 'c',      // Repaired
+    PROMISE_RESULT_WARN = 'w',        // Not kept
+    PROMISE_RESULT_FAIL = 'f',        // Not kept
+    PROMISE_RESULT_DENIED = 'd',      // Not kept
+    PROMISE_RESULT_TIMEOUT = 't',     // Timed out
+    PROMISE_RESULT_INTERRUPTED = 'i'  // Not kept
 } PromiseResult;
 
 /*****************************************************************************/
@@ -157,6 +145,8 @@ typedef enum
 
 #define CF_OBSERVABLES 100
 
+/* Special exit codes */
+#define EC_EVAL_ABORTED 6       /* like SIGABRT, but signal exit codes are 120+SIG */
 
 typedef struct
 {
@@ -262,9 +252,18 @@ typedef struct
 
 /******************************************************************/
 
-// Please ensure the padding in this struct is initialized to 0
-// LockData lock = { 0 }; // Will zero padding as well as members
-// lock.pid = [...]
+/** Data for an individual promise (lock), as it's stored in the lock DB
+ * (/var/cfengine/state/cf_lock.lmdb). This is the value, the key is an MD5
+ * hash of various strings identifying the promise.
+ *
+ * Most C code doesn't use this struct directly, instead AcquireLock() is called
+ * which uses functions in locks.c to read the LockData from LMDB and create a
+ * new CfLock struct.
+ *
+ * Please ensure the padding in this struct is initialized to 0
+ * LockData lock = { 0 }; // Will zero padding as well as members
+ * lock.pid = [...]
+ */
 typedef struct
 {
     pid_t pid;                  // 4 bytes
@@ -1161,6 +1160,32 @@ typedef struct
     char *process_result;
 } ProcessSelect;
 
+#define PROCESS_SELECT_INIT { \
+    .owner = NULL,          \
+    .min_pid = CF_NOINT,    \
+    .max_pid = CF_NOINT,    \
+    .min_ppid = CF_NOINT,   \
+    .max_ppid = CF_NOINT,   \
+    .min_pgid = CF_NOINT,   \
+    .max_pgid = CF_NOINT,   \
+    .min_rsize = CF_NOINT,  \
+    .max_rsize = CF_NOINT,  \
+    .min_vsize = CF_NOINT,  \
+    .max_vsize = CF_NOINT,  \
+    .min_ttime = CF_NOINT,  \
+    .max_ttime = CF_NOINT,  \
+    .min_stime = CF_NOINT,  \
+    .max_stime = CF_NOINT,  \
+    .min_pri = CF_NOINT,    \
+    .max_pri = CF_NOINT,    \
+    .min_thread = CF_NOINT, \
+    .max_thread = CF_NOINT, \
+    .status = NULL,         \
+    .command = NULL,        \
+    .tty = NULL,            \
+    .process_result = NULL, \
+}
+
 /*************************************************************************/
 
 typedef struct
@@ -1337,6 +1362,8 @@ typedef struct
     int updates_ifelapsed;
     int installed_ifelapsed;
     Rlist *options;
+    char *interpreter;
+    char *module_path;
 } PackageModuleBody;
 
 
@@ -1544,6 +1571,7 @@ typedef struct
     char *args;
     Rlist *arglist;
     int module;
+    bool inform;
 
     Rlist *signals;
     char *process_stop;
@@ -1595,6 +1623,88 @@ typedef struct
     Rlist *insert_match;
 } Attributes;
 
+#define ZeroAttributes {\
+    .select = {0},\
+    .perms = {0},\
+    .copy = {0},\
+    .delete = {0},\
+    .rename = {0},\
+    .change = {0},\
+    .link = {0},\
+    .edits = {0},\
+    .packages = {0},\
+    .new_packages = {0},\
+    .context = {0},\
+    .measure = {0},\
+    .acl = {0},\
+    .database = {0},\
+    .service = {0},\
+    .users = {0},\
+    .env = {0},\
+    .transformer = NULL,\
+    .pathtype = NULL,\
+    .file_type = NULL,\
+    .repository = NULL,\
+    .edit_template = NULL,\
+    .edit_template_string = NULL,\
+    .template_method = NULL,\
+    .template_data = NULL,\
+    .touch = 0,\
+    .create = 0,\
+    .move_obstructions = 0,\
+    .inherit = 0,\
+    .recursion = { 0 },\
+    .transaction = { 0 },\
+    .classes = { 0 },\
+    .contain = { 0 },\
+    .args = NULL,\
+    .arglist = NULL,\
+    .module = 0,\
+    .inform = false,\
+    .signals = NULL,\
+    .process_stop = NULL,\
+    .restart_class = NULL,\
+    .process_count = { 0 },\
+    .process_select = { 0 },\
+    .report = { 0 },\
+    .mount = { 0 },\
+    .volume = { 0 },\
+    .havedepthsearch = 0,\
+    .haveselect = 0,\
+    .haverename = 0,\
+    .havedelete = 0,\
+    .haveperms = 0,\
+    .havechange = 0,\
+    .havecopy = 0,\
+    .havelink = 0,\
+    .haveeditline = 0,\
+    .haveeditxml = 0,\
+    .haveedit = 0,\
+    .havecontain = 0,\
+    .haveclasses = 0,\
+    .havetrans = 0,\
+    .haveprocess_count = 0,\
+    .havemount = 0,\
+    .havevolume = 0,\
+    .havebundle = 0,\
+    .havepackages = 0,\
+    .region = { 0 },\
+    .location = { 0 },\
+    .column = { 0 },\
+    .replace = { 0 },\
+    .xml = { 0 },\
+    .haveregion = 0,\
+    .havelocation = 0,\
+    .havecolumn = 0,\
+    .havereplace = 0,\
+    .haveinsertselect = 0,\
+    .havedeleteselect = 0,\
+    .line_select = { 0 },\
+    .sourcetype = NULL,\
+    .expandvars = 0,\
+    .not_matching = 0,\
+    .insert_match = NULL\
+}
 
 /*************************************************************************/
 /* common macros                                                         */
