@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -32,8 +32,7 @@
 #include <cf3.defs.h>
 #include <lastseen.h>
 #include <files_interfaces.h>
-#include <files_hashes.h>
-#include <hashes.h>
+#include <hash.h>
 #include <pipes.h>
 #include <mutex.h>
 #include <known_dirs.h>
@@ -47,10 +46,12 @@
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #endif
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000
 /* The deprecated is the easy way to setup threads for OpenSSL. */
 #ifdef OPENSSL_NO_DEPRECATED
 void CRYPTO_set_id_callback(unsigned long (*func)(void));
 #endif
+#endif
 
 static void RandomSeed(void);
 static void SetupOpenSSLThreadLocks(void);
@@ -176,7 +177,7 @@ bool LoadSecretKeys(const char *const priv_key_path,
         {
             privkeyfile = PrivateKeyFile(GetWorkDir());
         }
-        FILE *fp = fopen(privkeyfile != NULL ? privkeyfile : priv_key_path, "r");
+        FILE *fp = safe_fopen(privkeyfile != NULL ? privkeyfile : priv_key_path, "r");
         if (!fp)
         {
             /* VERBOSE in case it's a custom, local-only installation. */
@@ -197,6 +198,8 @@ bool LoadSecretKeys(const char *const priv_key_path,
         {
             DESTROY_AND_NULL(RSA_free, *priv_key);
         }
+
+        // Can read both encrypted (old) and unencrypted(new) key files:
         *priv_key = PEM_read_RSAPrivateKey(fp, NULL, NULL, (void*) priv_passphrase);
         if (*priv_key == NULL)
         {
@@ -219,7 +222,7 @@ bool LoadSecretKeys(const char *const priv_key_path,
         {
             pubkeyfile = PublicKeyFile(GetWorkDir());
         }
-        FILE *fp = fopen(pubkeyfile != NULL ? pubkeyfile : pub_key_path, "r");
+        FILE *fp = safe_fopen(pubkeyfile != NULL ? pubkeyfile : pub_key_path, "r");
         if (!fp)
         {
             /* VERBOSE in case it's a custom, local-only installation. */
@@ -342,7 +345,6 @@ RSA *HavePublicKey(const char *username, const char *ipaddress, const char *dige
 {
     char keyname[CF_MAXVARSIZE], newname[CF_BUFSIZE], oldname[CF_BUFSIZE];
     struct stat statbuf;
-    FILE *fp;
     RSA *newkey = NULL;
     const char* const workdir = GetWorkDir();
 
@@ -387,7 +389,8 @@ RSA *HavePublicKey(const char *username, const char *ipaddress, const char *dige
         }
     }
 
-    if ((fp = fopen(newname, "r")) == NULL)
+    FILE *fp = safe_fopen(newname, "r");
+    if (fp == NULL)
     {
         Log(LOG_LEVEL_ERR, "Couldn't open public key file '%s' (fopen: %s)",
             newname, GetErrorStr());
@@ -427,7 +430,6 @@ bool SavePublicKey(const char *user, const char *digest, const RSA *key)
 {
     char keyname[CF_MAXVARSIZE], filename[CF_BUFSIZE];
     struct stat statbuf;
-    FILE *fp;
     int ret;
 
     ret = snprintf(keyname, sizeof(keyname), "%s-%s", user, digest);
@@ -457,7 +459,8 @@ bool SavePublicKey(const char *user, const char *digest, const RSA *key)
 
     Log(LOG_LEVEL_VERBOSE, "Saving public key to file '%s'", filename);
 
-    if ((fp = fopen(filename, "w")) == NULL)
+    FILE *fp = safe_fopen_create_perms(filename, "w", CF_PERMS_DEFAULT);
+    if (fp == NULL)
     {
         Log(LOG_LEVEL_ERR, "Unable to write a public key '%s'. (fopen: %s)", filename, GetErrorStr());
         return false;
@@ -769,11 +772,15 @@ static pthread_mutex_t *cf_openssl_locks = NULL;
 
 
 #ifndef __MINGW32__
+#if OPENSSL_VERSION_NUMBER < 0x10100000
 unsigned long ThreadId_callback(void)
 {
     return (unsigned long) pthread_self();
 }
 #endif
+#endif
+
+#if OPENSSL_VERSION_NUMBER < 0x10100000
 
 static void OpenSSLLock_callback(int mode, int index, char *file, int line)
 {
@@ -799,6 +806,8 @@ static void OpenSSLLock_callback(int mode, int index, char *file, int line)
     }
 }
 
+#endif // Callback only for openssl < 1.1.0
+
 static void SetupOpenSSLThreadLocks(void)
 {
     const int num_locks = CRYPTO_num_locks();
@@ -832,6 +841,8 @@ static void SetupOpenSSLThreadLocks(void)
 #ifndef __MINGW32__
     CRYPTO_set_id_callback((unsigned long (*)())ThreadId_callback);
 #endif
+    // This is a no-op macro for OpenSSL >= 1.1.0
+    // The callback function is not used (or defined) then
     CRYPTO_set_locking_callback((void (*)())OpenSSLLock_callback);
 }
 
