@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -22,6 +22,7 @@
   included file COSL.txt.
 */
 
+#include <platform.h>
 #include <file_lib.h>
 
 #include <mutex.h>                                            /* ThreadLock */
@@ -33,10 +34,11 @@
 #include <misc_lib.h>
 #include <known_dirs.h>
 #include <string_lib.h>
+#include <time.h>          /* time() */
 
 
-static int DBPathLock(const char *filename);
-static void DBPathUnLock(int fd);
+static bool DBPathLock(FileLock *lock, const char *filename);
+static void DBPathUnLock(FileLock *lock);
 static void DBPathMoveBroken(const char *filename);
 
 struct DBHandle_
@@ -54,6 +56,15 @@ struct DBHandle_
 
     /* This lock protects initialization of .priv element, and .refcount manipulation */
     pthread_mutex_t lock;
+
+    /* Record when the DB was opened (to check if possible corruptions are
+     * already repaired) */
+    time_t open_tstamp;
+
+    /**
+     * @see FreezeDB()
+     */
+    bool frozen;
 };
 
 struct DBCursor_
@@ -82,6 +93,7 @@ static pthread_once_t db_shutdown_once = PTHREAD_ONCE_INIT; /* GLOBAL_T */
 
 /******************************************************************************/
 
+// Only append to the end, keep in sync with dbid enum in dbm_api.h
 static const char *const DB_PATHS_STATEDIR[] = {
     [dbid_classes] = "cf_classes",
     [dbid_variables] = "cf_variables",
@@ -105,7 +117,8 @@ static const char *const DB_PATHS_STATEDIR[] = {
     [dbid_agent_execution] = "nova_agent_execution",
     [dbid_bundles] = "bundles",
     [dbid_packages_installed] = "packages_installed",
-    [dbid_packages_updates] = "packages_updates"
+    [dbid_packages_updates] = "packages_updates",
+    [dbid_cookies] = "nova_cookies",
 };
 
 /*
@@ -197,10 +210,7 @@ bool IsSubHandle(DBHandle *handle, dbid id, const char *name)
 
 static DBHandle *DBHandleGetSubDB(dbid id, const char *name)
 {
-    if (!ThreadLock(&db_handles_lock))
-    {
-        return NULL;
-    }
+    ThreadLock(&db_handles_lock);
 
     DynamicDBHandles *handles_list = db_dynamic_handles;
 
@@ -240,28 +250,22 @@ static DBHandle *DBHandleGet(int id)
 {
     assert(id >= 0 && id < dbid_max);
 
-    if (ThreadLock(&db_handles_lock))
+    ThreadLock(&db_handles_lock);
+    if (db_handles[id].filename == NULL)
     {
-        if (db_handles[id].filename == NULL)
-        {
-            db_handles[id].filename = DBIdToPath(id);
+        db_handles[id].filename = DBIdToPath(id);
 
-            /* Initialize mutexes as error-checking ones. */
-            pthread_mutexattr_t attr;
-            pthread_mutexattr_init(&attr);
-            pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
-            pthread_mutex_init(&db_handles[id].lock, &attr);
-            pthread_mutexattr_destroy(&attr);
-        }
-
-        ThreadUnlock(&db_handles_lock);
-
-        return &db_handles[id];
-    }
-    else
-    {
-        return NULL;
+        /* Initialize mutexes as error-checking ones. */
+        pthread_mutexattr_t attr;
+        pthread_mutexattr_init(&attr);
+        pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_ERRORCHECK);
+        pthread_mutex_init(&db_handles[id].lock, &attr);
+        pthread_mutexattr_destroy(&attr);
     }
+
+    ThreadUnlock(&db_handles_lock);
+
+    return &db_handles[id];
 }
 
 static inline
@@ -270,6 +274,14 @@ void CloseDBInstance(DBHandle *handle)
     /* Wait until all DB users are served, or a threshold is reached */
     int count = 0;
     ThreadLock(&handle->lock);
+    if (handle->frozen)
+    {
+        /* Just clean some allocated memory, but don't touch the DB itself. */
+        free(handle->filename);
+        free(handle->subname);
+        ThreadUnlock(&handle->lock);
+        return;
+    }
     while (handle->refcount > 0 && count < 1000)
     {
         ThreadUnlock(&handle->lock);
@@ -358,63 +370,66 @@ void DBSetMaximumConcurrentTransactions(int max_txn)
 static inline
 bool OpenDBInstance(DBHandle **dbp, dbid id, DBHandle *handle)
 {
-    if (ThreadLock(&handle->lock))
+    assert(handle != NULL);
+
+    ThreadLock(&handle->lock);
+    if (handle->frozen)
     {
-        if (handle->refcount == 0)
+        Log(LOG_LEVEL_WARNING, "Attempt to open a frozen DB '%s'", handle->filename);
+        ThreadUnlock(&handle->lock);
+        return false;
+    }
+    if (handle->refcount == 0)
+    {
+        FileLock lock = EMPTY_FILE_LOCK;
+        if (DBPathLock(&lock, handle->filename))
         {
-            int lock_fd = DBPathLock(handle->filename);
+            handle->open_tstamp = time(NULL);
+            handle->priv = DBPrivOpenDB(handle->filename, id);
 
-            if(lock_fd != -1)
+            if (handle->priv == DB_PRIV_DATABASE_BROKEN)
             {
+                DBPathMoveBroken(handle->filename);
                 handle->priv = DBPrivOpenDB(handle->filename, id);
-
                 if (handle->priv == DB_PRIV_DATABASE_BROKEN)
                 {
-                    DBPathMoveBroken(handle->filename);
-                    handle->priv = DBPrivOpenDB(handle->filename, id);
-                    if (handle->priv == DB_PRIV_DATABASE_BROKEN)
-                    {
-                        handle->priv = NULL;
-                    }
-                }
-
-                DBPathUnLock(lock_fd);
-            }
-
-            if (handle->priv)
-            {
-                if (!DBMigrate(handle, id))
-                {
-                    DBPrivCloseDB(handle->priv);
                     handle->priv = NULL;
                 }
             }
+
+            DBPathUnLock(&lock);
         }
 
         if (handle->priv)
         {
-            handle->refcount++;
-            *dbp = handle;
-
-            /* Only register shutdown handler if any database was opened
-             * correctly. Otherwise this shutdown caller may be called too early,
-             * and shutdown handler installed by the database library may end up
-             * being called before CloseAllDB function */
-
-            pthread_once(&db_shutdown_once, RegisterShutdownHandler);
-        }
-        else
-        {
-            *dbp = NULL;
+            if (!DBMigrate(handle, id))
+            {
+                DBPrivCloseDB(handle->priv);
+                handle->priv = NULL;
+                handle->open_tstamp = -1;
+            }
         }
+    }
 
-        ThreadUnlock(&handle->lock);
-        return *dbp != NULL;
+    if (handle->priv)
+    {
+        handle->refcount++;
+        *dbp = handle;
+
+        /* Only register shutdown handler if any database was opened
+         * correctly. Otherwise this shutdown caller may be called too early,
+         * and shutdown handler installed by the database library may end up
+         * being called before CloseAllDB function */
+
+        pthread_once(&db_shutdown_once, RegisterShutdownHandler);
     }
     else
     {
-        return false;
+        *dbp = NULL;
     }
+
+    ThreadUnlock(&handle->lock);
+    return *dbp != NULL;
 }
 
 bool OpenSubDB(DBHandle **dbp, dbid id, const char *sub_name)
@@ -429,45 +444,96 @@ bool OpenDB(DBHandle **dbp, dbid id)
     return OpenDBInstance(dbp, id, handle);
 }
 
+/**
+ * @db_file_name Absolute path of the DB file
+ */
+DBHandle *GetDBHandleFromFilename(const char *db_file_name)
+{
+    ThreadLock(&db_handles_lock);
+    for(dbid id=0; id < dbid_max; id++)
+    {
+        if (StringSafeEqual(db_handles[id].filename, db_file_name))
+        {
+            ThreadUnlock(&db_handles_lock);
+            return &(db_handles[id]);
+        }
+    }
+    ThreadUnlock(&db_handles_lock);
+    return NULL;
+}
+
+
+time_t GetDBOpenTimestamp(const DBHandle *handle)
+{
+    assert(handle != NULL);
+    return handle->open_tstamp;
+}
+
 void CloseDB(DBHandle *handle)
 {
+    assert(handle != NULL);
+
     /* Skip in case of nested locking, for example signal handler.
      * DB behaviour becomes erratic otherwise (CFE-1996). */
-    if (ThreadLock(&handle->lock))
+    ThreadLock(&handle->lock);
+    if (handle->frozen)
     {
-        DBPrivCommit(handle->priv);
+        /* Just clean some allocated memory, but don't touch the DB itself. */
+        free(handle->filename);
+        free(handle->subname);
+        ThreadUnlock(&handle->lock);
+        return;
+    }
+    DBPrivCommit(handle->priv);
 
-        if (handle->refcount < 1)
-        {
-            Log(LOG_LEVEL_ERR,
+    if (handle->refcount < 1)
+    {
+        Log(LOG_LEVEL_ERR,
                 "Trying to close database which is not open: %s",
                 handle->filename);
-        }
-        else
-        {
-            handle->refcount--;
-            if (handle->refcount == 0)
-            {
-                DBPrivCloseDB(handle->priv);
-            }
-        }
-
-        ThreadUnlock(&handle->lock);
     }
+    else
+    {
+        handle->refcount--;
+        if (handle->refcount == 0)
+        {
+            DBPrivCloseDB(handle->priv);
+            handle->open_tstamp = -1;
+        }
+    }
+
+    ThreadUnlock(&handle->lock);
 }
 
 bool CleanDB(DBHandle *handle)
 {
-    bool ret = false;
-    if (ThreadLock(&handle->lock))
+    ThreadLock(&handle->lock);
+    if (handle->frozen)
     {
-        ret = DBPrivClean(handle->priv);
-
+        Log(LOG_LEVEL_WARNING, "Attempt to clean a frozen DB '%s'", handle->filename);
         ThreadUnlock(&handle->lock);
+        return false;
     }
+    bool ret = DBPrivClean(handle->priv);
+    ThreadUnlock(&handle->lock);
+
     return ret;
 }
 
+/**
+ * Freezes the DB so that it is never touched by this process again. In
+ * particular, new OpenDB() calls are ignored and CloseAllDBExit() also ignores
+ * the DB.
+ */
+void FreezeDB(DBHandle *handle)
+{
+    /* This is intentionally NOT using the handle->lock to avoid deadlocks.
+     * Nothing ever sets this to 'false' explicitly, that's only done in the
+     * initialization with '{ { 0 } }', so this bit-flip is safe. */
+    Log(LOG_LEVEL_NOTICE, "Freezing the DB '%s'", handle->filename);
+    handle->frozen = true;
+}
+
 /*****************************************************************************/
 
 bool ReadComplexKeyDB(DBHandle *handle, const char *key, int key_size,
@@ -548,7 +614,7 @@ bool DeleteDBCursor(DBCursor *cursor)
     return true;
 }
 
-static int DBPathLock(const char *filename)
+static bool DBPathLock(FileLock *lock, const char *filename)
 {
     char *filename_lock;
     if (xasprintf(&filename_lock, "%s.lock", filename) == -1)
@@ -556,34 +622,21 @@ static int DBPathLock(const char *filename)
         ProgrammingError("Unable to construct lock database filename for file %s", filename);
     }
 
-    int fd = open(filename_lock, O_CREAT | O_RDWR, 0666);
-
-    if(fd == -1)
+    if (ExclusiveFileLockPath(lock, filename_lock, true) != 0)
     {
-        Log(LOG_LEVEL_ERR, "Unable to open database lock file '%s'. (flock: %s)", filename_lock, GetErrorStr());
+        Log(LOG_LEVEL_ERR, "Unable to lock database lock file '%s'.", filename_lock);
         free(filename_lock);
-        return -1;
-    }
-
-    if (ExclusiveLockFile(fd) == -1)
-    {
-        Log(LOG_LEVEL_ERR, "Unable to lock database lock file '%s'. (fcntl(F_SETLK): %s)", filename_lock, GetErrorStr());
-        free(filename_lock);
-        close(fd);
-        return -1;
+        return false;
     }
 
     free(filename_lock);
 
-    return fd;
+    return true;
 }
 
-static void DBPathUnLock(int fd)
+static void DBPathUnLock(FileLock *lock)
 {
-    if(ExclusiveUnlockFile(fd) != 0)
-    {
-        Log(LOG_LEVEL_ERR, "Could not close db lock-file. (close: %s)", GetErrorStr());
-    }
+    ExclusiveFileUnlock(lock, true);
 }
 
 static void DBPathMoveBroken(const char *filename)
@@ -648,3 +701,29 @@ StringMap *LoadDatabaseToStringMap(dbid database_id)
 
     return db_map;
 }
+
+/**
+ * Checks if a DB repair flag file is present and if it is, removes it.
+ *
+ * @return Whether the DB repair flag file was present or not.
+ */
+bool CheckDBRepairFlagFile()
+{
+    /* The DB repair flag file can be created by user or by some process
+     * that hit an error condition potentially caused by local DB corruption
+     * that it was not able to handle properly by repairing the corrupted DB
+     * file(s). For example, if a process is killed by a signal. */
+    char repair_flag_file[PATH_MAX] = { 0 };
+    bool present = false;
+    xsnprintf(repair_flag_file, PATH_MAX, "%s%c%s",
+              GetStateDir(), FILE_SEPARATOR, CF_DB_REPAIR_TRIGGER);
+    /* This is full of race-conditions, but it's just a best-effort
+     * thing. If a force-repair is missed, it will happen next time. If it's
+     * done twice, no big deal. */
+    if (access(repair_flag_file, F_OK) == 0)
+    {
+        present = true;
+        unlink(repair_flag_file);
+    }
+    return present;
+}
