@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -25,8 +25,12 @@
 #ifndef CFENGINE_DBM_API_H
 #define CFENGINE_DBM_API_H
 
+#define EC_CORRUPTION_REPAIRED 120
+#define EC_CORRUPTION_REPAIR_FAILED 121
+
 #include <map.h>
 
+// Only append to the end, keep in sync with DB_PATHS_STATEDIR array
 typedef enum
 {
     dbid_classes,   // Deprecated
@@ -52,6 +56,7 @@ typedef enum
     dbid_bundles,   // Deprecated
     dbid_packages_installed, //new package promise installed packages list
     dbid_packages_updates,   //new package promise list of available updates
+    dbid_cookies, // Enterprise reporting cookies for duplicate host detection
 
     dbid_max
 } dbid;
@@ -69,6 +74,9 @@ bool OpenSubDB(DBHandle **dbp, dbid id, const char *sub_name);
 bool CleanDB(DBHandle *handle);
 void CloseDB(CF_DB *dbp);
 
+DBHandle *GetDBHandleFromFilename(const char *db_file_name);
+time_t GetDBOpenTimestamp(const DBHandle *handle);
+
 bool HasKeyDB(CF_DB *dbp, const char *key, int key_size);
 int ValueSizeDB(CF_DB *dbp, const char *key, int key_size);
 bool ReadComplexKeyDB(CF_DB *dbp, const char *key, int key_size, void *dest, int destSz);
@@ -77,6 +85,7 @@ bool DeleteComplexKeyDB(CF_DB *dbp, const char *key, int size);
 bool ReadDB(CF_DB *dbp, const char *key, void *dest, int destSz);
 bool WriteDB(CF_DB *dbp, const char *key, const void *src, int srcSz);
 bool DeleteDB(CF_DB *dbp, const char *key);
+void FreezeDB(DBHandle *handle);
 
 /*
  * Creating cursor locks the whole database, so keep the amount of work here to
@@ -96,4 +105,6 @@ char *DBIdToPath(dbid id);
 char *DBIdToSubPath(dbid id, const char *subdb_name);
 
 StringMap *LoadDatabaseToStringMap(dbid database_id);
+
+bool CheckDBRepairFlagFile();
 #endif  /* NOT CFENGINE_DBM_API_H */
