@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -112,6 +112,7 @@ static pcre *context_expression_whitespace_rx = NULL;
 
 static bool BundleAborted(const EvalContext *ctx);
 static void SetBundleAborted(EvalContext *ctx);
+static void SetEvalAborted(EvalContext *ctx);
 
 static bool EvalContextStackFrameContainsSoft(const EvalContext *ctx, const char *context);
 static bool EvalContextHeapContainsSoft(const EvalContext *ctx, const char *ns, const char *name);
@@ -130,6 +131,7 @@ struct EvalContext_
 
     int eval_options;
     bool bundle_aborted;
+    bool eval_aborted;
     bool checksum_updates_default;
     Item *ip_addresses;
     bool ignore_locks;
@@ -219,7 +221,7 @@ int PackageManagerSeqCompare(const void *a, const void *b, ARG_UNUSED void *data
 
 void AddPackageModuleToContext(const EvalContext *ctx, PackageModuleBody *pm)
 {
-    /* First check if the body is there added from previous pre-evaluation 
+    /* First check if the body is there added from previous pre-evaluation
      * iteration. If it is there update it as we can have new expanded variables. */
     ssize_t pm_seq_index;
     if ((pm_seq_index = SeqIndexOf(ctx->package_promise_context->package_modules_bodies,
@@ -435,7 +437,8 @@ static void EvalContextStackFrameAddSoft(EvalContext *ctx, const char *context,
 
     if (IsRegexItemIn(ctx, ctx->heap_abort, copy))
     {
-        FatalError(ctx, "cf-agent aborted on defined class '%s'", copy);
+        Log(LOG_LEVEL_NOTICE, "cf-agent aborted on defined class '%s'", copy);
+        SetEvalAborted(ctx);
     }
 
     if (EvalContextStackFrameContainsSoft(ctx, copy))
@@ -470,7 +473,7 @@ static ExpressionValue EvalTokenAsClass(const char *classname, void *param)
             if (EvalContextHeapContainsHard(ctx, ref.name))
             {
                 ClassRefDestroy(ref);
-                return true;
+                return EXPRESSION_VALUE_TRUE;
             }
         }
     }
@@ -479,7 +482,7 @@ static ExpressionValue EvalTokenAsClass(const char *classname, void *param)
         if (EvalContextHeapContainsHard(ctx, ref.name))
         {
             ClassRefDestroy(ref);
-            return true;
+            return EXPRESSION_VALUE_TRUE;
         }
 
         const char *ns = EvalContextCurrentNamespace(ctx);
@@ -499,7 +502,7 @@ static ExpressionValue EvalTokenAsClass(const char *classname, void *param)
                    EvalContextStackFrameContainsSoft(ctx, ref.name));
 
     ClassRefDestroy(ref);
-    return classy; /* ExpressionValue is just an enum extending bool... */
+    return (ExpressionValue) classy; // ExpressionValue extends bool
 }
 
 /**********************************************************************/
@@ -521,13 +524,15 @@ bool ClassCharIsWhitespace(char ch)
     return ch == ' ' || ch == '\t' || ch == '\n' || ch == '\r';
 }
 
-bool IsDefinedClass(const EvalContext *ctx, const char *context)
+ExpressionValue CheckClassExpression(const EvalContext *ctx, const char *context)
 {
+    assert(context != NULL);
     ParseResult res;
 
     if (!context)
     {
-        return true;
+        // TODO: Remove this, seems like a hack
+        return EXPRESSION_VALUE_TRUE;
     }
 
     if (context_expression_whitespace_rx == NULL)
@@ -538,13 +543,13 @@ bool IsDefinedClass(const EvalContext *ctx, const char *context)
     if (context_expression_whitespace_rx == NULL)
     {
         Log(LOG_LEVEL_ERR, "The context expression whitespace regular expression could not be compiled, aborting.");
-        return false;
+        return EXPRESSION_VALUE_ERROR;
     }
 
     if (StringMatchFullWithPrecompiledRegex(context_expression_whitespace_rx, context))
     {
         Log(LOG_LEVEL_INFO, "class names can't be separated by whitespace without an intervening operator in expression '%s'", context);
-        return false;
+        return EXPRESSION_VALUE_ERROR;
     }
 
     Buffer *condensed = BufferNewFrom(context, strlen(context));
@@ -555,7 +560,7 @@ bool IsDefinedClass(const EvalContext *ctx, const char *context)
     if (!res.result)
     {
         Log(LOG_LEVEL_ERR, "Unable to parse class expression '%s'", context);
-        return false;
+        return EXPRESSION_VALUE_ERROR;
     }
     else
     {
@@ -564,9 +569,7 @@ bool IsDefinedClass(const EvalContext *ctx, const char *context)
                                            (void *)ctx); // controlled cast. None of these should modify EvalContext
 
         FreeExpression(res.result);
-
-        /* r is EvalResult which could be ERROR */
-        return r == EXPRESSION_VALUE_TRUE;
+        return r;
     }
 }
 
@@ -575,7 +578,7 @@ bool IsDefinedClass(const EvalContext *ctx, const char *context)
 static ExpressionValue EvalTokenFromList(const char *token, void *param)
 {
     StringSet *set = param;
-    return StringSetContains(set, token);
+    return (ExpressionValue) StringSetContains(set, token); // EV extends bool
 }
 
 /**********************************************************************/
@@ -609,6 +612,12 @@ static bool EvalWithTokenFromList(const char *expr, StringSet *token_set)
 
 bool EvalProcessResult(const char *process_result, StringSet *proc_attr)
 {
+    assert(process_result != NULL);
+    if (StringSafeEqual(process_result, ""))
+    {
+        /* nothing to evaluate */
+        return false;
+    }
     return EvalWithTokenFromList(process_result, proc_attr);
 }
 
@@ -784,8 +793,9 @@ void EvalContextHeapPersistentLoadAll(EvalContext *ctx)
     CloseDB(dbp);
 }
 
-bool Abort(EvalContext *ctx)
+bool BundleAbort(EvalContext *ctx)
 {
+    assert(ctx != NULL);
     if (ctx->bundle_aborted)
     {
         ctx->bundle_aborted = false;
@@ -795,19 +805,33 @@ bool Abort(EvalContext *ctx)
     return false;
 }
 
-bool BundleAborted(const EvalContext* ctx)
+static bool BundleAborted(const EvalContext* ctx)
 {
+    assert(ctx != NULL);
     return ctx->bundle_aborted;
 }
 
-void SetBundleAborted(EvalContext *ctx)
+static void SetBundleAborted(EvalContext *ctx)
 {
+    assert(ctx != NULL);
     ctx->bundle_aborted = true;
 }
 
+static void SetEvalAborted(EvalContext *ctx)
+{
+    assert(ctx != NULL);
+    ctx->eval_aborted = true;
+}
+
+bool EvalAborted(const EvalContext *ctx)
+{
+    assert(ctx != NULL);
+    return ctx->eval_aborted;
+}
 
 void EvalContextHeapAddAbort(EvalContext *ctx, const char *context, const char *activated_on_context)
 {
+    assert(ctx != NULL);
     if (!IsItemIn(ctx->heap_abort, context))
     {
         AppendItem(&ctx->heap_abort, context, activated_on_context);
@@ -817,12 +841,14 @@ void EvalContextHeapAddAbort(EvalContext *ctx, const char *context, const char *
 
     if (aborting_context)
     {
-        FatalError(ctx, "cf-agent aborted on defined class '%s'", aborting_context);
+        Log(LOG_LEVEL_NOTICE, "cf-agent aborted on defined class '%s'", aborting_context);
+        SetEvalAborted(ctx);
     }
 }
 
 void EvalContextHeapAddAbortCurrentBundle(EvalContext *ctx, const char *context, const char *activated_on_context)
 {
+    assert(ctx != NULL);
     if (!IsItemIn(ctx->heap_abort_current_bundle, context))
     {
         AppendItem(&ctx->heap_abort_current_bundle, context, activated_on_context);
@@ -833,6 +859,7 @@ void EvalContextHeapAddAbortCurrentBundle(EvalContext *ctx, const char *context,
 
 bool MissingDependencies(EvalContext *ctx, const Promise *pp)
 {
+    assert(ctx != NULL);
     const Rlist *dependenies = PromiseGetConstraintAsList(ctx, "depends_on", pp);
     if (RlistIsNullList(dependenies))
     {
@@ -1007,11 +1034,13 @@ void EvalContextDestroy(EvalContext *ctx)
         free(ctx->launch_directory);
         free(ctx->entry_point);
 
+        // Freeing logging context doesn't belong here...
         {
             LoggingPrivContext *pctx = LoggingPrivGetContext();
             free(pctx);
             LoggingPrivSetContext(NULL);
         }
+        LoggingFreeCurrentThreadContext();
 
         EvalContextDeleteIpAddresses(ctx);
 
@@ -1445,10 +1474,7 @@ void EvalContextStackPushPromiseFrame(EvalContext *ctx, const Promise *owner)
             {
                 EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "with", RvalScalarValue(final), CF_DATA_TYPE_STRING, "source=promise_iteration/with");
             }
-            else
-            {
-                RvalDestroy(final);
-            }
+            RvalDestroy(final);
         }
     }
 }
@@ -1620,7 +1646,8 @@ static bool EvalContextClassPut(EvalContext *ctx, const char *ns, const char *na
 
         if (IsRegexItemIn(ctx, ctx->heap_abort, context_copy))
         {
-            FatalError(ctx, "cf-agent aborted on defined class '%s'", context_copy);
+            Log(LOG_LEVEL_NOTICE, "cf-agent aborted on defined class '%s'", context_copy);
+            SetEvalAborted(ctx);
         }
     }
 
@@ -1935,6 +1962,36 @@ bool EvalContextVariablePutSpecial(EvalContext *ctx, SpecialScope scope, const c
     }
 }
 
+const void *EvalContextVariableGetSpecial(
+    const EvalContext *const ctx,
+    const SpecialScope scope,
+    const char *const varname,
+    DataType *const type_out)
+{
+    VarRef *const ref = VarRefParseFromScope(
+        varname, SpecialScopeToString(scope));
+    const void *const result = EvalContextVariableGet(ctx, ref, type_out);
+    VarRefDestroy(ref);
+
+    return result;
+}
+
+/**
+ * @note Only use this when you know the variable is a string
+ * @see EvalContextVariableGetSpecial()
+ */
+const char *EvalContextVariableGetSpecialString(
+    const EvalContext *const ctx,
+    const SpecialScope scope,
+    const char *const varname)
+{
+    DataType type_out;
+    const void *const result = EvalContextVariableGetSpecial(
+        ctx, scope, varname, &type_out);
+    assert(type_out == CF_DATA_TYPE_STRING); // Programming error if not string
+    return (type_out == CF_DATA_TYPE_STRING) ? result : NULL;
+}
+
 bool EvalContextVariableRemoveSpecial(const EvalContext *ctx, SpecialScope scope, const char *lval)
 {
     switch (scope)
@@ -2633,7 +2690,7 @@ ENTERPRISE_VOID_FUNC_2ARG_DEFINE_STUB(void, TrackTotalCompliance, ARG_UNUSED Pro
 {
 }
 
-static void SetPromiseOutcomeClasses(EvalContext *ctx, PromiseResult status, DefineClasses dc)
+static void SetPromiseOutcomeClasses(EvalContext *ctx, PromiseResult status, const DefineClasses *dc)
 {
     Rlist *add_classes = NULL;
     Rlist *del_classes = NULL;
@@ -2641,50 +2698,50 @@ static void SetPromiseOutcomeClasses(EvalContext *ctx, PromiseResult status, Def
     switch (status)
     {
     case PROMISE_RESULT_CHANGE:
-        add_classes = dc.change;
-        del_classes = dc.del_change;
+        add_classes = dc->change;
+        del_classes = dc->del_change;
         break;
 
     case PROMISE_RESULT_TIMEOUT:
-        add_classes = dc.timeout;
-        del_classes = dc.del_notkept;
+        add_classes = dc->timeout;
+        del_classes = dc->del_notkept;
         break;
 
     case PROMISE_RESULT_WARN:
     case PROMISE_RESULT_FAIL:
     case PROMISE_RESULT_INTERRUPTED:
-        add_classes = dc.failure;
-        del_classes = dc.del_notkept;
+        add_classes = dc->failure;
+        del_classes = dc->del_notkept;
         break;
 
     case PROMISE_RESULT_DENIED:
-        add_classes = dc.denied;
-        del_classes = dc.del_notkept;
+        add_classes = dc->denied;
+        del_classes = dc->del_notkept;
         break;
 
     case PROMISE_RESULT_NOOP:
-        add_classes = dc.kept;
-        del_classes = dc.del_kept;
+        add_classes = dc->kept;
+        del_classes = dc->del_kept;
         break;
 
     default:
         ProgrammingError("Unexpected status '%c' has been passed to SetPromiseOutcomeClasses", status);
     }
 
-    AddAllClasses(ctx, add_classes, dc.persist, dc.timer, dc.scope);
+    AddAllClasses(ctx, add_classes, dc->persist, dc->timer, dc->scope);
     DeleteAllClasses(ctx, del_classes);
 }
 
-static void SummarizeTransaction(EvalContext *ctx, TransactionContext tc, const char *logname)
+static void SummarizeTransaction(EvalContext *ctx, const TransactionContext *tc, const char *logname)
 {
-    if (logname && (tc.log_string))
+    if (logname && (tc->log_string))
     {
         Buffer *buffer = BufferNew();
-        ExpandScalar(ctx, NULL, NULL, tc.log_string, buffer);
+        ExpandScalar(ctx, NULL, NULL, tc->log_string, buffer);
 
         if (strcmp(logname, "udp_syslog") == 0)
         {
-            RemoteSysLog(tc.log_priority, BufferData(buffer));
+            RemoteSysLog(tc->log_priority, BufferData(buffer));
         }
         else if (strcmp(logname, "stdout") == 0)
         {
@@ -2723,11 +2780,14 @@ static void SummarizeTransaction(EvalContext *ctx, TransactionContext tc, const
         }
 
         BufferDestroy(buffer);
-        tc.log_string = NULL;     /* To avoid repetition */
+        // FIXME: This was overwriting a local copy, with no side effects.
+        // The intention was clearly to skip this function if called
+        // repeatedly. Try to introduce this change:
+        // tc.log_string = NULL;     /* To avoid repetition */
     }
 }
 
-static void DoSummarizeTransaction(EvalContext *ctx, PromiseResult status, const Promise *pp, TransactionContext tc)
+static void DoSummarizeTransaction(EvalContext *ctx, PromiseResult status, const Promise *pp, const TransactionContext *tc)
 {
     if (!IsPromiseValuableForLogging(pp))
     {
@@ -2739,7 +2799,7 @@ static void DoSummarizeTransaction(EvalContext *ctx, PromiseResult status, const
     switch (status)
     {
     case PROMISE_RESULT_CHANGE:
-        log_name = tc.log_repaired;
+        log_name = tc->log_repaired;
         break;
 
     case PROMISE_RESULT_WARN:
@@ -2750,11 +2810,11 @@ static void DoSummarizeTransaction(EvalContext *ctx, PromiseResult status, const
     case PROMISE_RESULT_FAIL:
     case PROMISE_RESULT_DENIED:
     case PROMISE_RESULT_INTERRUPTED:
-        log_name = tc.log_failed;
+        log_name = tc->log_failed;
         break;
 
     case PROMISE_RESULT_NOOP:
-        log_name = tc.log_kept;
+        log_name = tc->log_kept;
         break;
 
     default:
@@ -2785,16 +2845,17 @@ void NotifyDependantPromises(EvalContext *ctx, const Promise *pp, PromiseResult
     }
 }
 
-void ClassAuditLog(EvalContext *ctx, const Promise *pp, Attributes attr, PromiseResult status)
+void ClassAuditLog(EvalContext *ctx, const Promise *pp, const Attributes *attr, PromiseResult status)
 {
+    assert(attr != NULL);
     if (IsPromiseValuableForStatus(pp))
     {
         TrackTotalCompliance(status, pp);
         UpdatePromiseCounters(status);
     }
 
-    SetPromiseOutcomeClasses(ctx, status, attr.classes);
-    DoSummarizeTransaction(ctx, status, pp, attr.transaction);
+    SetPromiseOutcomeClasses(ctx, status, &(attr->classes));
+    DoSummarizeTransaction(ctx, status, pp, &(attr->transaction));
 }
 
 static void LogPromiseContext(const EvalContext *ctx, const Promise *pp)
@@ -2842,7 +2903,7 @@ static void LogPromiseContext(const EvalContext *ctx, const Promise *pp)
     WriterClose(w);
 }
 
-void cfPS(EvalContext *ctx, LogLevel level, PromiseResult status, const Promise *pp, Attributes attr, const char *fmt, ...)
+void cfPS(EvalContext *ctx, LogLevel level, PromiseResult status, const Promise *pp, const Attributes *attr, const char *fmt, ...)
 {
     /*
      * This stub implementation of cfPS delegates to the new logging backend.
@@ -2862,6 +2923,7 @@ void cfPS(EvalContext *ctx, LogLevel level, PromiseResult status, const Promise
     /* FIXME: Ensure that NULL pp is never passed into cfPS */
 
     assert(pp);
+    assert(attr != NULL);
 
     if (level >= LOG_LEVEL_VERBOSE)
     {
@@ -2922,7 +2984,7 @@ void EvalContextSetEvalOption(EvalContext *ctx, EvalContextOption option, bool v
 
 bool EvalContextGetEvalOption(EvalContext *ctx, EvalContextOption option)
 {
-    return !!(ctx->eval_options & option);
+    return ((ctx->eval_options & option) != 0);
 }
 
 void EvalContextSetLaunchDirectory(EvalContext *ctx, const char *path)
