@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -188,13 +188,21 @@ static void MapIteratorsFromRval(EvalContext *ctx,
 }
 
 static PromiseResult ExpandPromiseAndDo(EvalContext *ctx, PromiseIterator *iterctx,
-                                        PromiseActuator *act_on_promise, void *param)
+                                        PromiseActuator *act_on_promise, void *param,
+                                        bool actuate_ifelse)
 {
     PromiseResult result = PROMISE_RESULT_SKIPPED;
 
+    /* In the case of ifelse() we must always include an extra round of "actuation"
+     * in the while loop below. PromiseIteratorNext() will return false in the case
+     * that there are doubly-unresolved Rvals like $($(missing)).
+     * We can't add an empty wheel because that is skipped as well as noted in
+     * libpromises/iteration.c ShouldAddVariableAsIterationWheel(). */
+    bool ifelse_actuated = !actuate_ifelse;
+
     /* TODO this loop could be completely skipped for for non vars/classes if
      *      act_on_promise is CommonEvalPromise(). */
-    while (PromiseIteratorNext(iterctx, ctx))
+    while (PromiseIteratorNext(iterctx, ctx) || !ifelse_actuated)
     {
         /*
          * ACTUAL WORK PART 1: Get a (another) copy of the promise.
@@ -208,6 +216,7 @@ static PromiseResult ExpandPromiseAndDo(EvalContext *ctx, PromiseIterator *iterc
         if (pexp == NULL)                       /* is the promise excluded? */
         {
             result = PromiseResultUpdate(result, PROMISE_RESULT_SKIPPED);
+            ifelse_actuated = true;
             continue;
         }
 
@@ -239,6 +248,7 @@ static PromiseResult ExpandPromiseAndDo(EvalContext *ctx, PromiseIterator *iterc
         /* Why do we push/pop an iteration frame, if all iterated variables
          * are Put() on the previous scope? */
         EvalContextStackPopFrame(ctx);
+        ifelse_actuated = true;
     }
 
     return result;
@@ -272,16 +282,22 @@ PromiseResult ExpandPromise(EvalContext *ctx, const Promise *pp,
         MapIteratorsFromRval(ctx, iterctx, pcopy->promisee);
     }
 
+    bool actuate_ifelse = false;
     for (size_t i = 0; i < SeqLength(pcopy->conlist); i++)
     {
         Constraint *cp = SeqAt(pcopy->conlist, i);
+        if (cp->rval.type == RVAL_TYPE_FNCALL &&
+            strcmp(RvalFnCallValue(cp->rval)->name, "ifelse") == 0)
+        {
+            actuate_ifelse = true;
+        }
         MapIteratorsFromRval(ctx, iterctx, cp->rval);
     }
 
     /* 3. GO! */
     PutHandleVariable(ctx, pcopy);
     PromiseResult result = ExpandPromiseAndDo(ctx, iterctx,
-                                              act_on_promise, param);
+                                              act_on_promise, param, actuate_ifelse);
 
     EvalContextStackPopFrame(ctx);
     PromiseIteratorDestroy(iterctx);
@@ -734,33 +750,6 @@ void BundleResolve(EvalContext *ctx, const Bundle *bundle)
     BundleResolvePromiseType(ctx, bundle, "vars", VerifyVarPromise);
 }
 
-ProtocolVersion ProtocolVersionParse(const char *s)
-{
-    if (s == NULL ||
-        strcmp(s, "0") == 0 ||
-        strcmp(s, "undefined") == 0)
-    {
-        return CF_PROTOCOL_UNDEFINED;
-    }
-    if (strcmp(s, "1") == 0 ||
-        strcmp(s, "classic") == 0)
-    {
-        return CF_PROTOCOL_CLASSIC;
-    }
-    else if (strcmp(s, "2") == 0)
-    {
-        return CF_PROTOCOL_TLS;
-    }
-    else if (strcmp(s, "latest") == 0)
-    {
-        return CF_PROTOCOL_LATEST;
-    }
-    else
-    {
-        return CF_PROTOCOL_UNDEFINED;
-    }
-}
-
 /**
  * Evaluate the relevant control body, and set the
  * relevant fields in #ctx and #config.
@@ -911,7 +900,7 @@ static void ResolveControlBody(EvalContext *ctx, GenericAgentConfig *config,
             config->protocol_version = ProtocolVersionParse(
                 RvalScalarValue(evaluated_rval));
             Log(LOG_LEVEL_VERBOSE, "SET common protocol_version: %s",
-                PROTOCOL_VERSION_STRING[config->protocol_version]);
+                ProtocolVersionString(config->protocol_version));
         }
 
         /* Those are package_inventory and package_module common control body options */
@@ -979,6 +968,14 @@ static void ResolvePackageManagerBody(EvalContext *ctx, const Body *pm_body)
         {
             new_manager->options = RlistCopy(RvalRlistValue(returnval));
         }
+        else if (strcmp(cp->lval, "interpreter") == 0)
+        {
+            new_manager->interpreter = SafeStringDuplicate(RvalScalarValue(returnval));
+        }
+        else if (strcmp(cp->lval, "module_path") == 0)
+        {
+            new_manager->module_path = SafeStringDuplicate(RvalScalarValue(returnval));
+        }
         else
         {
             /* This should be handled by the parser. */
