@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -28,7 +28,7 @@
 #include <promises.h>
 #include <dir.h>
 #include <files_names.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <files_copy.h>
 #include <item_lib.h>
 #include <vars.h>
@@ -38,7 +38,7 @@
 #include <rlist.h>
 #include <stat_cache.h>                                      /* remote_stat */
 
-int cf_lstat(const char *file, struct stat *buf, FileCopy fc, AgentConnection *conn)
+int cf_lstat(const char *file, struct stat *buf, const FileCopy *fc, AgentConnection *conn)
 {
     if (conn == NULL)
     {
@@ -51,7 +51,7 @@ int cf_lstat(const char *file, struct stat *buf, FileCopy fc, AgentConnection *c
     }
     else
     {
-        assert(fc.servers && strcmp(fc.servers->val.item, "localhost"));
-        return cf_remote_stat(conn, fc.encrypt, file, buf, "link");
+        assert(fc->servers && strcmp(fc->servers->val.item, "localhost"));
+        return cf_remote_stat(conn, fc->encrypt, file, buf, "link");
     }
 }
