@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -64,7 +64,7 @@ void PurgeItemList(Item **list, char *name)
 
 bool FileWriteOver(char *filename, char *contents)
 {
-    FILE *fp = safe_fopen(filename, "w");
+    FILE *fp = safe_fopen_create_perms(filename, "w", CF_PERMS_DEFAULT);
 
     if(fp == NULL)
     {
@@ -102,7 +102,7 @@ bool FileWriteOver(char *filename, char *contents)
  *          Windows if the path contains double (back)slashes!
  **/
 
-int MakeParentDirectory2(char *parentandchild, int force, bool enforce_promise)
+bool MakeParentDirectory2(char *parentandchild, int force, bool enforce_promise)
 {
     if(enforce_promise)
     {
@@ -290,7 +290,7 @@ bool MakeParentDirectory(const char *parentandchild, bool force)
             {
                 mask = umask(0);
 
-                if (mkdir(currentpath, DEFAULTMODE) == -1)
+                if (mkdir(currentpath, DEFAULTMODE) == -1 && errno != EEXIST)
                 {
                     Log(LOG_LEVEL_ERR,
                         "Unable to make directory: %s (mkdir: %s)",
@@ -339,7 +339,7 @@ bool MakeParentDirectory(const char *parentandchild, bool force)
     return true;
 }
 
-int LoadFileAsItemList(Item **liststart, const char *file, EditDefaults edits)
+bool LoadFileAsItemList(Item **liststart, const char *file, EditDefaults edits)
 {
     {
         struct stat statbuf;
@@ -353,7 +353,7 @@ int LoadFileAsItemList(Item **liststart, const char *file, EditDefaults edits)
         {
             Log(LOG_LEVEL_INFO, "File '%s' is bigger than the edit limit. max_file_size = %jd > %d bytes", file,
                   (intmax_t) statbuf.st_size, edits.maxfilesize);
-            return (false);
+            return false;
         }
 
         if (!S_ISREG(statbuf.st_mode))
@@ -640,8 +640,6 @@ void RotateFiles(char *name, int number)
 
 void CreateEmptyFile(char *name)
 {
-    int tempfd;
-
     if (unlink(name) == -1)
     {
         if (errno != ENOENT)
@@ -650,7 +648,8 @@ void CreateEmptyFile(char *name)
         }
     }
 
-    if ((tempfd = safe_open(name, O_CREAT | O_EXCL | O_WRONLY, 0600)) < 0)
+    int tempfd = safe_open(name, O_CREAT | O_EXCL | O_WRONLY);
+    if (tempfd < 0)
     {
         Log(LOG_LEVEL_ERR, "Couldn't open a file '%s'. (open: %s)", name, GetErrorStr());
     }
@@ -659,5 +658,3 @@ void CreateEmptyFile(char *name)
 }
 
 #endif
-
-
