@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -31,10 +31,10 @@
 void PurgeItemList(Item **list, char *name);
 bool FileWriteOver(char *filename, char *contents);
 
-int LoadFileAsItemList(Item **liststart, const char *file, EditDefaults edits);
+bool LoadFileAsItemList(Item **liststart, const char *file, EditDefaults edits);
 
 bool MakeParentDirectory(const char *parentandchild, bool force);
-int MakeParentDirectory2(char *parentandchild, int force, bool enforce_promise);
+bool MakeParentDirectory2(char *parentandchild, int force, bool enforce_promise);
 
 void RotateFiles(char *name, int number);
 void CreateEmptyFile(char *name);
