@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -38,7 +38,7 @@
 
 /*********************************************************************/
 
-int IsNewerFileTree(const char *dir, time_t reftime)
+bool IsNewerFileTree(const char *dir, time_t reftime)
 {
     const struct dirent *dirp;
     Dir *dirh;
@@ -124,7 +124,7 @@ int IsNewerFileTree(const char *dir, time_t reftime)
 
 /*********************************************************************/
 
-int IsDir(const char *path)
+bool IsDir(const char *path)
 /*
 Checks if the object pointed to by path exists and is a directory.
 Returns true if so, false otherwise.
@@ -211,7 +211,7 @@ char *JoinPaths(char *path, size_t path_size, const char *leaf_path)
     return path;
 }
 
-int IsAbsPath(const char *path)
+bool IsAbsPath(const char *path)
 {
     if (IsFileSep(*path))
     {
@@ -396,7 +396,7 @@ const char *FirstFileSeparator(const char *str)
 /*********************************************************************/
 
 const char *LastFileSeparator(const char *str)
-  /* Return pointer to last file separator in string, or NULL if 
+  /* Return pointer to last file separator in string, or NULL if
      string does not contains any file separtors */
 {
     const char *sp;
@@ -421,14 +421,14 @@ const char *LastFileSeparator(const char *str)
 
 bool ChopLastNode(char *str)
   /* Chop off trailing node name (possible blank) starting from
-     last character and removing up to the first / encountered 
+     last character and removing up to the first / encountered
      e.g. /a/b/c -> /a/b
      /a/b/ -> /a/b
      Will also collapse redundant/repeating path separators.
   */
 {
     char *sp;
-    int ret;
+    bool ret;
 
     DeleteRedundantSlashes(str);
 
