@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -67,6 +67,11 @@ Rlist *NewExpArgs(EvalContext *ctx, const Policy *policy, const FnCall *fp, cons
     }
 
     const FnCallType *fn = FnCallTypeGet(fp->name);
+    if (fn == NULL)
+    {
+        FatalError(ctx, "Function call '%s' has unknown type", fp->name);
+    }
+    else
     {
         int len = RlistLen(fp->args);
 
@@ -251,6 +256,10 @@ static FnCallResult CallFunction(EvalContext *ctx, const Policy *policy, const F
 {
     const Rlist *rp = fp->args;
     const FnCallType *fncall_type = FnCallTypeGet(fp->name);
+    if (fncall_type == NULL)
+    {
+        FatalError(ctx, "Function call '%s' has unknown type", fp->name);
+    }
 
     int argnum = 0;
     for (argnum = 0; rp != NULL && fncall_type->args[argnum].pattern != NULL; argnum++)
