@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -35,7 +35,7 @@
 #include <files_lib.h>
 #include <files_names.h>
 #include <files_interfaces.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <parser.h>
 #include <dbm_api.h>
 #include <crypto.h>
@@ -49,6 +49,7 @@
 #include <unix.h>
 #include <client_code.h>
 #include <string_lib.h>
+#include <writer.h>
 #include <exec_tools.h>
 #include <list.h>
 #include <misc_lib.h>
@@ -170,17 +171,61 @@ Policy *SelectAndLoadPolicy(GenericAgentConfig *config, EvalContext *ctx, bool v
             GenericAgentConfigSetInputFile(config, GetInputDir(), "failsafe.cf");
             Log(LOG_LEVEL_ERR, "CFEngine failsafe.cf: %s %s", config->input_dir, config->input_file);
             policy = LoadPolicy(ctx, config);
+
+            /* Doing failsafe, set the release_id to "failsafe" and also
+             * overwrite the cfe_release_id file so that sub-agent executed as
+             * part of failsafe can just pick it up and then rewrite it with the
+             * actual value from masterfiles. */
+            free(policy->release_id);
+            policy->release_id = xstrdup("failsafe");
+
+            char filename[PATH_MAX];
+            GetReleaseIdFile(GetInputDir(), filename, sizeof(filename));
+            FILE *release_id_stream = safe_fopen_create_perms(filename, "w",
+                                                              CF_PERMS_DEFAULT);
+            if (release_id_stream == NULL)
+            {
+                Log(LOG_LEVEL_ERR, "Failed to open the release_id file for writing during failsafe");
+            }
+            else
+            {
+                Writer *release_id_writer = FileWriter(release_id_stream);
+                WriterWrite(release_id_writer, "{ releaseId: \"failsafe\" }\n");
+                WriterClose(release_id_writer);
+            }
         }
     }
     return policy;
 }
 
-static bool CheckContextClassmatch(EvalContext *ctx, const char* class_str)
+static bool CheckContextClassmatch(EvalContext *ctx, const char *class_str)
 {
+    if (StringEndsWith(class_str, "::")) // Treat as class expression, not regex
+    {
+        const size_t length = strlen(class_str);
+        if (length <= 2)
+        {
+            assert(length == 2); // True because StringEndsWith
+            Log(LOG_LEVEL_ERR,
+                "Invalid class expression in augments: '%s'",
+                class_str);
+            return false;
+        }
+
+        char *const tmp_class_str = xstrdup(class_str);
+        assert(strlen(tmp_class_str) == length);
+
+        tmp_class_str[length - 2] = '\0'; // 2 = strlen("::")
+        const bool found = IsDefinedClass(ctx, tmp_class_str);
+
+        free(tmp_class_str);
+        return found;
+    }
+
     ClassTableIterator *iter = EvalContextClassTableIteratorNewGlobal(ctx, NULL, true, true);
     StringSet *global_matches = ClassesMatching(ctx, iter, class_str, NULL, true); // returns early
 
-    bool found = (StringSetSize(global_matches) > 0);
+    const bool found = (StringSetSize(global_matches) > 0);
 
     StringSetDestroy(global_matches);
     ClassTableIteratorDestroy(iter);
@@ -380,17 +425,17 @@ static bool LoadAugmentsData(EvalContext *ctx, const char *filename, const JsonE
                 const JsonElement *el;
                 while ((el = JsonIteratorNextValueByType(&iter, JSON_ELEMENT_TYPE_PRIMITIVE, true)) != NULL)
                 {
-                    char *filename = JsonPrimitiveToString(el);
-                    bool further_loaded = LoadAugmentsFiles(ctx, filename);
+                    char *nested_filename = JsonPrimitiveToString(el);
+                    bool further_loaded = LoadAugmentsFiles(ctx, nested_filename);
                     if (further_loaded)
                     {
-                        Log(LOG_LEVEL_VERBOSE, "Completed augmenting from file '%s'", filename);
+                        Log(LOG_LEVEL_VERBOSE, "Completed augmenting from file '%s'", nested_filename);
                     }
                     else
                     {
-                        Log(LOG_LEVEL_ERR, "Could not load requested further augments from file '%s'", filename);
+                        Log(LOG_LEVEL_ERR, "Could not load requested further augments from file '%s'", nested_filename);
                     }
-                    free(filename);
+                    free(nested_filename);
                 }
             }
             else
@@ -505,6 +550,11 @@ void GenericAgentDiscoverContext(EvalContext *ctx, GenericAgentConfig *config)
     {
         EvalContextClassPutHard(ctx, "bootstrap_mode", "report,source=environment");
 
+        if (!config->agent_specific.agent.bootstrap_trigger_policy)
+        {
+            EvalContextClassPutHard(ctx, "skip_policy_on_bootstrap", "report,source=environment");
+        }
+
         if (!RemoveAllExistingPolicyInInputs(GetInputDir()))
         {
             Log(LOG_LEVEL_ERR,
@@ -559,6 +609,12 @@ void GenericAgentDiscoverContext(EvalContext *ctx, GenericAgentConfig *config)
 
         PolicyServerWriteFile(GetWorkDir(), bootstrap_arg);
         EvalContextSetPolicyServer(ctx, bootstrap_arg);
+        char *const bootstrap_id = CreateBootstrapIDFile(GetWorkDir());
+        if (bootstrap_id != NULL)
+        {
+            EvalContextSetBootstrapID(ctx, bootstrap_id);
+            free(bootstrap_id);
+        }
 
         /* FIXME: Why it is called here? Can't we move both invocations to before if? */
         UpdateLastPolicyUpdateTime(ctx);
@@ -571,6 +627,12 @@ void GenericAgentDiscoverContext(EvalContext *ctx, GenericAgentConfig *config)
             Log(LOG_LEVEL_VERBOSE, "This agent is bootstrapped to: %s",
                 existing_policy_server);
             EvalContextSetPolicyServer(ctx, existing_policy_server);
+            char *const bootstrap_id = ReadBootstrapIDFile(GetWorkDir());
+            if (bootstrap_id != NULL)
+            {
+                EvalContextSetBootstrapID(ctx, bootstrap_id);
+                free(bootstrap_id);
+            }
             free(existing_policy_server);
             UpdateLastPolicyUpdateTime(ctx);
             if (GetAmPolicyHub())
@@ -708,7 +770,7 @@ static bool WritePolicyValidatedFile(ARG_UNUSED const GenericAgentConfig *config
         return false;
     }
 
-    int fd = creat(filename, 0600);
+    int fd = creat(filename, CF_PERMS_DEFAULT);
     if (fd == -1)
     {
         Log(LOG_LEVEL_ERR, "While writing policy validated marker file '%s', could not create file (create: %s)", filename, GetErrorStr());
@@ -825,7 +887,7 @@ static bool WriteReleaseIdFile(const char *filename, const char *dirname)
         return false;
     }
 
-    int fd = creat(filename, 0600);
+    int fd = creat(filename, CF_PERMS_DEFAULT);
     if (fd == -1)
     {
         Log(LOG_LEVEL_ERR, "While writing policy release ID file '%s', could not create file (create: %s)", filename, GetErrorStr());
@@ -848,21 +910,21 @@ static bool WriteReleaseIdFile(const char *filename, const char *dirname)
 bool GenericAgentArePromisesValid(const GenericAgentConfig *config)
 {
     char cmd[CF_BUFSIZE];
-    const char* const workdir = GetWorkDir();
+    const char* const bindir = GetBinDir();
 
     Log(LOG_LEVEL_VERBOSE, "Verifying the syntax of the inputs...");
     {
         char cfpromises[CF_MAXVARSIZE];
 
-        snprintf(cfpromises, sizeof(cfpromises), "%s%cbin%ccf-promises%s",
-                 workdir, FILE_SEPARATOR, FILE_SEPARATOR, EXEC_SUFFIX);
+        snprintf(cfpromises, sizeof(cfpromises), "%s%ccf-promises%s",
+                 bindir, FILE_SEPARATOR, EXEC_SUFFIX);
 
         struct stat sb;
         if (stat(cfpromises, &sb) == -1)
         {
             Log(LOG_LEVEL_ERR,
-                "cf-promises%s needs to be installed in %s%cbin for pre-validation of full configuration",
-                EXEC_SUFFIX, workdir, FILE_SEPARATOR);
+                "cf-promises%s needs to be installed in %s for pre-validation of full configuration",
+                EXEC_SUFFIX, bindir);
 
             return false;
         }
@@ -971,6 +1033,7 @@ void GenericAgentInitialize(EvalContext *ctx, GenericAgentConfig *config)
 /* Define trusted directories */
 
     const char *workdir = GetWorkDir();
+    const char *bindir = GetBinDir();
 
     if (!workdir)
     {
@@ -984,7 +1047,7 @@ void GenericAgentInitialize(EvalContext *ctx, GenericAgentConfig *config)
 
     snprintf(vbuff, CF_BUFSIZE, "%s%cupdate.conf", GetInputDir(), FILE_SEPARATOR);
     MakeParentDirectory(vbuff, force);
-    snprintf(vbuff, CF_BUFSIZE, "%s%cbin%ccf-agent", workdir, FILE_SEPARATOR, FILE_SEPARATOR);
+    snprintf(vbuff, CF_BUFSIZE, "%s%ccf-agent", bindir, FILE_SEPARATOR);
     MakeParentDirectory(vbuff, force);
     snprintf(vbuff, CF_BUFSIZE, "%s%coutputs%cspooled_reports", workdir, FILE_SEPARATOR, FILE_SEPARATOR);
     MakeParentDirectory(vbuff, force);
@@ -1153,7 +1216,7 @@ static bool GeneratePolicyReleaseIDFromGit(char *release_id_out,
 
     // Note: Probably we should not be reading all of these filenames directly,
     // and should instead use git plumbing commands to retrieve the data.
-    FILE *git_file = fopen(git_filename, "r");
+    FILE *git_file = safe_fopen(git_filename, "r");
     if (git_file)
     {
         char git_head[128];
@@ -1165,7 +1228,7 @@ static bool GeneratePolicyReleaseIDFromGit(char *release_id_out,
             fclose(git_file);
             snprintf(git_filename, PATH_MAX, "%s/.git/%s",
                      policy_dir, git_head);
-            git_file = fopen(git_filename, "r");
+            git_file = safe_fopen(git_filename, "r");
             Log(LOG_LEVEL_DEBUG, "Found a git HEAD ref");
         }
         else
@@ -1723,13 +1786,12 @@ static void RegisterPidCleanup(void)
 
 void WritePID(char *filename)
 {
-    FILE *fp;
-
     pthread_once(&pid_cleanup_once, RegisterPidCleanup);
 
     snprintf(PIDFILE, CF_BUFSIZE - 1, "%s%c%s", GetPidDir(), FILE_SEPARATOR, filename);
 
-    if ((fp = fopen(PIDFILE, "w")) == NULL)
+    FILE *fp = safe_fopen_create_perms(PIDFILE, "w", CF_PERMS_DEFAULT);
+    if (fp == NULL)
     {
         Log(LOG_LEVEL_INFO, "Could not write to PID file '%s'. (fopen: %s)", filename, GetErrorStr());
         return;
@@ -1871,6 +1933,9 @@ GenericAgentConfig *GenericAgentConfigNewDefault(AgentType agent_type, bool tty_
     /* By default we trust the network when bootstrapping. */
     config->agent_specific.agent.bootstrap_trust_server = true;
 
+    /* By default we run promises.cf as the last step of boostrapping */
+    config->agent_specific.agent.bootstrap_trigger_policy = true;
+
     /* Log classes */
     config->agent_specific.agent.report_class_log = false;
 
@@ -2044,6 +2109,7 @@ void SetupSignalsForAgent(void)
 {
     signal(SIGINT, HandleSignalsForAgent);
     signal(SIGTERM, HandleSignalsForAgent);
+    signal(SIGBUS, HandleSignalsForAgent);
     signal(SIGHUP, SIG_IGN);
     signal(SIGPIPE, SIG_IGN);
     signal(SIGUSR1, HandleSignalsForAgent);
