@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -62,19 +62,19 @@ bool RawSaveItemList(const Item *liststart, const char *filename, NewLineMode ne
 Item *RawLoadItemList(const char *filename);
 Item *SplitStringAsItemList(const char *string, char sep);
 Item *SplitString(const char *string, char sep);
-int DeleteItemGeneral(Item **filestart, const char *string, ItemMatchType type);
-int DeleteItemLiteral(Item **filestart, const char *string);
-int DeleteItemStarting(Item **list, const char *string);
-int DeleteItemNotStarting(Item **list, const char *string);
-int DeleteItemMatching(Item **list, const char *string);
-int DeleteItemNotMatching(Item **list, const char *string);
-int DeleteItemContaining(Item **list, const char *string);
-int DeleteItemNotContaining(Item **list, const char *string);
+bool DeleteItemGeneral(Item **filestart, const char *string, ItemMatchType type);
+bool DeleteItemLiteral(Item **filestart, const char *string);
+bool DeleteItemStarting(Item **list, const char *string);
+bool DeleteItemNotStarting(Item **list, const char *string);
+bool DeleteItemMatching(Item **list, const char *string);
+bool DeleteItemNotMatching(Item **list, const char *string);
+bool DeleteItemContaining(Item **list, const char *string);
+bool DeleteItemNotContaining(Item **list, const char *string);
 int ListLen(const Item *list);
 bool IsItemIn(const Item *list, const char *item);
 bool ListsCompare(const Item *list1, const Item *list2);
 bool ListSubsetOfList(const Item *list1, const Item *list2);
-int IsMatchItemIn(const Item *list, const char *item);
+bool IsMatchItemIn(const Item *list, const char *item);
 Item *ConcatLists(Item *list1, Item *list2);
 void CopyList(Item **dest, const Item *source);
 void IdempItemCount(Item **liststart, const char *itemstring, const char *classes);
