@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -26,7 +26,7 @@
 
 #include <lastseen.h>
 #include <conversion.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <locks.h>
 #include <item_lib.h>
 #include <known_dirs.h>
@@ -681,7 +681,7 @@ int LastSeenHostKeyCount(void)
  * @brief removes all traces of entry 'input' from lastseen DB
  *
  * @param[in] key digest (SHA/MD5 format) or free host name string
- * @param[in] must_be_coherent. false : delete if lastseen is incoherent, 
+ * @param[in] must_be_coherent. false : delete if lastseen is incoherent,
  *                              true :  don't if lastseen is incoherent
  * @param[out] equivalent. If input is a host, return its corresponding
  *                         digest. If input is a digest, return its
