@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -24,10 +24,11 @@
 #include <loading.h>
 
 #include <eval_context.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <file_lib.h>
 #include <files_names.h>
 #include <string_lib.h>
+#include <writer.h>
 #include <parser.h>
 #include <expand.h>
 #include <rlist.h>
@@ -45,7 +46,8 @@
 
 
 static Policy *LoadPolicyFile(EvalContext *ctx, GenericAgentConfig *config, const char *policy_file,
-                              StringSet *parsed_files_and_checksums, StringSet *failed_files);
+                              StringMap *policy_files_hashes, StringSet *parsed_files_checksums,
+                              StringSet *failed_files);
 
 
 
@@ -143,7 +145,9 @@ Policy *Cf3ParseFile(const GenericAgentConfig *config, const char *input_path)
     return policy;
 }
 
-static Policy *LoadPolicyInputFiles(EvalContext *ctx, GenericAgentConfig *config, const Rlist *inputs, StringSet *parsed_files_and_checksums, StringSet *failed_files)
+static Policy *LoadPolicyInputFiles(EvalContext *ctx, GenericAgentConfig *config, const Rlist *inputs,
+                                    StringMap *policy_files_hashes, StringSet *parsed_files_checksums,
+                                    StringSet *failed_files)
 {
     Policy *policy = PolicyNew();
 
@@ -173,11 +177,15 @@ static Policy *LoadPolicyInputFiles(EvalContext *ctx, GenericAgentConfig *config
                 break;
             }
 
-            aux_policy = LoadPolicyFile(ctx, config, GenericAgentResolveInputPath(config, RvalScalarValue(resolved_input)), parsed_files_and_checksums, failed_files);
+            aux_policy = LoadPolicyFile(ctx, config,
+                                        GenericAgentResolveInputPath(config, RvalScalarValue(resolved_input)),
+                                        policy_files_hashes, parsed_files_checksums, failed_files);
             break;
 
         case RVAL_TYPE_LIST:
-            aux_policy = LoadPolicyInputFiles(ctx, config, RvalRlistValue(resolved_input), parsed_files_and_checksums, failed_files);
+            aux_policy = LoadPolicyInputFiles(ctx, config,
+                                              RvalRlistValue(resolved_input),
+                                              policy_files_hashes, parsed_files_checksums, failed_files);
             break;
 
         default:
@@ -303,27 +311,28 @@ static void RenameMainBundle(EvalContext *ctx, Policy *policy)
     free(entry_point);
 }
 
-static Policy *LoadPolicyFile(EvalContext *ctx, GenericAgentConfig *config, const char *policy_file, StringSet *parsed_files_and_checksums, StringSet *failed_files)
+static Policy *LoadPolicyFile(EvalContext *ctx, GenericAgentConfig *config, const char *policy_file,
+                              StringMap *policy_files_hashes, StringSet *parsed_files_checksums,
+                              StringSet *failed_files)
 {
     unsigned char digest[EVP_MAX_MD_SIZE + 1] = { 0 };
     char hashbuffer[CF_HOSTKEY_STRING_SIZE] = { 0 };
-    char hashprintbuffer[CF_BUFSIZE] = { 0 };
 
     HashFile(policy_file, digest, CF_DEFAULT_DIGEST, false);
-    snprintf(hashprintbuffer, CF_BUFSIZE - 1, "{checksum}%s",
-             HashPrintSafe(hashbuffer, sizeof(hashbuffer), digest,
-                           CF_DEFAULT_DIGEST, true));
+    HashPrintSafe(hashbuffer, sizeof(hashbuffer), digest,
+                  CF_DEFAULT_DIGEST, true);
 
-    Log(LOG_LEVEL_DEBUG, "Hashed policy file %s to %s", policy_file, hashprintbuffer);
+    Log(LOG_LEVEL_DEBUG, "Hashed policy file %s to %s", policy_file, hashbuffer);
 
-    if (StringSetContains(parsed_files_and_checksums, policy_file))
+    if (StringMapHasKey(policy_files_hashes, policy_file))
     {
         Log(LOG_LEVEL_VERBOSE, "Skipping loading of duplicate policy file %s", policy_file);
         return NULL;
     }
-    else if (StringSetContains(parsed_files_and_checksums, hashprintbuffer))
+    else if (StringSetContains(parsed_files_checksums, hashbuffer))
     {
-        Log(LOG_LEVEL_VERBOSE, "Skipping loading of duplicate (detected by hash) policy file %s", policy_file);
+        Log(LOG_LEVEL_VERBOSE, "Skipping loading of duplicate (detected by hash) policy file %s",
+            policy_file);
         return NULL;
     }
     else
@@ -332,9 +341,9 @@ static Policy *LoadPolicyFile(EvalContext *ctx, GenericAgentConfig *config, cons
     }
 
     Policy *policy = Cf3ParseFile(config, policy_file);
-    // we keep the checksum and the policy file name to help debugging
-    StringSetAdd(parsed_files_and_checksums, xstrdup(policy_file));
-    StringSetAdd(parsed_files_and_checksums, xstrdup(hashprintbuffer));
+
+    StringMapInsert(policy_files_hashes, xstrdup(policy_file), xstrdup(hashbuffer));
+    StringSetAdd(parsed_files_checksums, xstrdup(hashbuffer));
 
     if (policy)
     {
@@ -376,7 +385,9 @@ static Policy *LoadPolicyFile(EvalContext *ctx, GenericAgentConfig *config, cons
 
         if (cp)
         {
-            Policy *aux_policy = LoadPolicyInputFiles(ctx, config, RvalRlistValue(cp->rval), parsed_files_and_checksums, failed_files);
+            Policy *aux_policy = LoadPolicyInputFiles(ctx, config, RvalRlistValue(cp->rval),
+                                                      policy_files_hashes, parsed_files_checksums,
+                                                      failed_files);
             if (aux_policy)
             {
                 policy = PolicyMerge(policy, aux_policy);
@@ -392,7 +403,9 @@ static Policy *LoadPolicyFile(EvalContext *ctx, GenericAgentConfig *config, cons
 
         if (cp)
         {
-            Policy *aux_policy = LoadPolicyInputFiles(ctx, config, RvalRlistValue(cp->rval), parsed_files_and_checksums, failed_files);
+            Policy *aux_policy = LoadPolicyInputFiles(ctx, config, RvalRlistValue(cp->rval),
+                                                      policy_files_hashes, parsed_files_checksums,
+                                                      failed_files);
             if (aux_policy)
             {
                 policy = PolicyMerge(policy, aux_policy);
@@ -414,7 +427,7 @@ static bool VerifyBundleSequence(EvalContext *ctx, const Policy *policy, const G
     }
 
     const char *name;
-    int ok = true;
+    bool ok = true;
     for (const Rlist *rp = bundlesequence; rp != NULL; rp = rp->next)
     {
         switch (rp->val.type)
@@ -481,13 +494,15 @@ static JsonElement *ReadReleaseIdFileFromInputs()
 
 Policy *LoadPolicy(EvalContext *ctx, GenericAgentConfig *config)
 {
-    StringSet *parsed_files_and_checksums = StringSetNew();
+    StringMap *policy_files_hashes = StringMapNew();
+    StringSet *parsed_files_checksums = StringSetNew();
     StringSet *failed_files = StringSetNew();
 
     Banner("Loading policy");
 
     Policy *policy = LoadPolicyFile(ctx, config, config->input_file,
-                                    parsed_files_and_checksums, failed_files);
+                                    policy_files_hashes, parsed_files_checksums,
+                                    failed_files);
 
     if (StringSetSize(failed_files) > 0)
     {
@@ -495,8 +510,16 @@ Policy *LoadPolicy(EvalContext *ctx, GenericAgentConfig *config)
         DoCleanupAndExit(EXIT_FAILURE);
     }
 
-    StringSetDestroy(parsed_files_and_checksums);
+    StringSetDestroy(parsed_files_checksums);
     StringSetDestroy(failed_files);
+    if (policy != NULL)
+    {
+        policy->policy_files_hashes = policy_files_hashes;
+    }
+    else
+    {
+        StringMapDestroy(policy_files_hashes);
+    }
 
     {
         Seq *errors = SeqNew(100, PolicyErrorDestroy);
@@ -593,15 +616,42 @@ Policy *LoadPolicy(EvalContext *ctx, GenericAgentConfig *config)
         }
     }
 
-    JsonElement *validated_doc = ReadReleaseIdFileFromInputs();
-    if (validated_doc)
+    if (config->agent_type == AGENT_TYPE_AGENT &&
+        config->agent_specific.agent.bootstrap_argument != NULL)
     {
-        const char *release_id = JsonObjectGetAsString(validated_doc, "releaseId");
-        if (release_id)
+        /* Doing bootstrap, set the release_id to "bootstrap" and also write it
+         * into a file so that sub-agent executed as part of bootstrap can just
+         * pick it up and then rewrite it with the actual value from
+         * masterfiles. */
+        policy->release_id = xstrdup("bootstrap");
+
+        char filename[PATH_MAX];
+        GetReleaseIdFile(GetInputDir(), filename, sizeof(filename));
+        FILE *release_id_stream = safe_fopen_create_perms(filename, "w",
+                                                          CF_PERMS_DEFAULT);
+        if (release_id_stream == NULL)
         {
-            policy->release_id = xstrdup(release_id);
+            Log(LOG_LEVEL_ERR, "Failed to open the release_id file for writing during bootstrap");
+        }
+        else
+        {
+            Writer *release_id_writer = FileWriter(release_id_stream);
+            WriterWrite(release_id_writer, "{ releaseId: \"bootstrap\" }\n");
+            WriterClose(release_id_writer);
+        }
+    }
+    else
+    {
+        JsonElement *validated_doc = ReadReleaseIdFileFromInputs();
+        if (validated_doc)
+        {
+            const char *release_id = JsonObjectGetAsString(validated_doc, "releaseId");
+            if (release_id)
+            {
+                policy->release_id = xstrdup(release_id);
+            }
+            JsonDestroy(validated_doc);
         }
-        JsonDestroy(validated_doc);
     }
 
     return policy;
