@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -23,13 +23,14 @@
 */
 
 #include <locks.h>
+#include <global_mutex.h>
 #include <mutex.h>
 #include <string_lib.h>
 #include <files_interfaces.h>
 #include <files_lib.h>
 #include <cleanup.h>
 #include <policy.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <rb-tree.h>
 #include <files_names.h>
 #include <rlist.h>
@@ -42,6 +43,11 @@
 #include <openssl/evp.h>
 #include <libcrypto-compat.h>
 
+#ifdef LMDB
+// Be careful if you want to change this,
+// it must match mdb_env_get_maxkeysize(env)
+#define LMDB_MAX_KEY_SIZE 511
+#endif
 
 #define CFLOGSIZE 1048576       /* Size of lock-log before rotation */
 #define CF_LOCKHORIZON ((time_t)(SECONDS_PER_WEEK * 4))
@@ -85,6 +91,66 @@ static CfLockStack *PopLock()
     return lock;
 }
 
+static void CopyLockDatabaseAtomically(const char *from, const char *to,
+                                       const char *from_pretty_name,
+                                       const char *to_pretty_name);
+
+static void RestoreLockDatabase(void);
+
+static void VerifyThatDatabaseIsNotCorrupt_once(void)
+{
+    int uptime = GetUptimeSeconds(time(NULL));
+    if (uptime <= 0)
+    {
+        Log(LOG_LEVEL_VERBOSE,
+            "Not able to determine uptime when verifying lock database. "
+            "Will assume the database is in order.");
+        return;
+    }
+
+    char *db_path = DBIdToPath(dbid_locks);
+    struct stat statbuf;
+    if (stat(db_path, &statbuf) == 0)
+    {
+        if (statbuf.st_mtime < time(NULL) - uptime)
+        {
+            // We have rebooted since the database was last updated.
+            // Restore it from our backup.
+            RestoreLockDatabase();
+        }
+    }
+
+    free(db_path);
+}
+
+static void VerifyThatDatabaseIsNotCorrupt(void)
+{
+    static pthread_once_t uptime_verified = PTHREAD_ONCE_INIT;
+    pthread_once(&uptime_verified, &VerifyThatDatabaseIsNotCorrupt_once);
+}
+
+CF_DB *OpenLock()
+{
+    CF_DB *dbp;
+
+    VerifyThatDatabaseIsNotCorrupt();
+
+    if (!OpenDB(&dbp, dbid_locks))
+    {
+        return NULL;
+    }
+
+    return dbp;
+}
+
+void CloseLock(CF_DB *dbp)
+{
+    if (dbp)
+    {
+        CloseDB(dbp);
+    }
+}
+
 static pthread_once_t lock_cleanup_once = PTHREAD_ONCE_INIT; /* GLOBAL_X */
 
 #ifdef LMDB
@@ -116,29 +182,11 @@ static inline void log_lock(const char *op,
     }
 }
 
-static void GenerateMd5Hash(const char *istring, char *ohash)
+static void HashLockKeyIfNecessary(const char *const istring, char *const ohash)
 {
-    if (!strcmp(istring, "CF_CRITICAL_SECTION"))
-    {
-        strcpy(ohash, istring);
-        return;
-    }
-
-    unsigned char digest[EVP_MAX_MD_SIZE + 1];
-    HashString(istring, strlen(istring), digest, HASH_METHOD_MD5);
-
-    const char lookup[]="0123456789abcdef";
-    for (int i=0; i<16; i++)
-    {
-        ohash[i*2]   = lookup[digest[i] >> 4];
-        ohash[i*2+1] = lookup[digest[i] & 0xf];
-    }
-    ohash[16*2] = '\0';
-
-    if (!strncmp(istring, "lock.track_license_bundle.track_license", 39))
-    {
-        ohash[0] = 'X';
-    }
+    assert(strlen("CF_CRITICAL_SECTION") < LMDB_MAX_KEY_SIZE);
+    assert(strlen("lock.track_license_bundle.track_license") < LMDB_MAX_KEY_SIZE);
+    StringCopyTruncateAndHashIfNecessary(istring, ohash, LMDB_MAX_KEY_SIZE);
 }
 #endif
 
@@ -147,16 +195,9 @@ static bool WriteLockData(CF_DB *dbp, const char *lock_id, LockData *lock_data)
     bool ret;
 
 #ifdef LMDB
-    unsigned char digest2[EVP_MAX_MD_SIZE*2 + 1];
+    unsigned char digest2[LMDB_MAX_KEY_SIZE];
 
-    if (!strcmp(lock_id, "CF_CRITICAL_SECTION"))
-    {
-        strcpy(digest2, lock_id);
-    }
-    else
-    {
-        GenerateMd5Hash(lock_id, digest2);
-    }
+    HashLockKeyIfNecessary(lock_id, digest2);
 
     LOG_LOCK_ENTRY(lock_id, digest2, lock_data);
     ret = WriteDB(dbp, digest2, lock_data, sizeof(LockData));
@@ -178,7 +219,25 @@ static bool WriteLockDataCurrent(CF_DB *dbp, const char *lock_id)
     return WriteLockData(dbp, lock_id, &lock_data);
 }
 
-time_t FindLockTime(const char *name)
+static int WriteLock(const char *name)
+{
+    CF_DB *dbp = OpenLock();
+
+    if (dbp == NULL)
+    {
+        return -1;
+    }
+
+    ThreadLock(cft_lock);
+    WriteLockDataCurrent(dbp, name);
+
+    CloseLock(dbp);
+    ThreadUnlock(cft_lock);
+
+    return 0;
+}
+
+static time_t FindLockTime(const char *name)
 {
     bool ret;
     CF_DB *dbp = OpenLock();
@@ -191,8 +250,8 @@ time_t FindLockTime(const char *name)
     entry.process_start_time = PROCESS_START_TIME_UNKNOWN;
 
 #ifdef LMDB
-    unsigned char ohash[EVP_MAX_MD_SIZE*2 + 1];
-    GenerateMd5Hash(name, ohash);
+    unsigned char ohash[LMDB_MAX_KEY_SIZE];
+    HashLockKeyIfNecessary(name, ohash);
 
     LOG_LOCK_ENTRY(name, ohash, &entry);
     ret = ReadDB(dbp, ohash, &entry, sizeof(entry));
@@ -290,16 +349,9 @@ static int RemoveLock(const char *name)
 
     ThreadLock(cft_lock);
 #ifdef LMDB
-    unsigned char digest2[EVP_MAX_MD_SIZE*2 + 1];
+    unsigned char digest2[LMDB_MAX_KEY_SIZE];
 
-    if (!strcmp(name, "CF_CRITICAL_SECTION"))
-    {
-        strcpy(digest2, name);
-    }
-    else
-    {
-        GenerateMd5Hash(name, digest2);
-    }
+    HashLockKeyIfNecessary(name, digest2);
 
     LOG_LOCK_ENTRY(name, digest2, NULL);
     DeleteDB(dbp, digest2);
@@ -447,8 +499,8 @@ static bool KillLockHolder(const char *lock)
     lock_data.process_start_time = PROCESS_START_TIME_UNKNOWN;
 
 #ifdef LMDB
-    unsigned char ohash[EVP_MAX_MD_SIZE*2 + 1];
-    GenerateMd5Hash(lock, ohash);
+    unsigned char ohash[LMDB_MAX_KEY_SIZE];
+    HashLockKeyIfNecessary(lock, ohash);
 
     LOG_LOCK_ENTRY(lock, ohash, &lock_data);
     ret = ReadDB(dbp, ohash, &lock_data, sizeof(lock_data));
@@ -485,13 +537,37 @@ static bool KillLockHolder(const char *lock)
         else
         {
             Log(LOG_LEVEL_ERR,
-                "Failed to kill process with PID %jd",
-                (intmax_t) lock_data.pid);
+                "Failed to kill process with PID: %jd (kill: %s)",
+                (intmax_t) lock_data.pid, GetErrorStr());
             return false;
         }
     }
 }
 
+static void RvalDigestUpdate(EVP_MD_CTX *context, Rlist *rp)
+{
+    assert(context != NULL);
+    assert(rp != NULL);
+
+    switch (rp->val.type)
+    {
+    case RVAL_TYPE_SCALAR:
+        EVP_DigestUpdate(context, RlistScalarValue(rp),
+                         strlen(RlistScalarValue(rp)));
+        break;
+
+    case RVAL_TYPE_FNCALL:
+        // TODO: This should be recursive and not just hash the function name
+        EVP_DigestUpdate(context, RlistFnCallValue(rp)->name,
+                         strlen(RlistFnCallValue(rp)->name));
+        break;
+
+    default:
+        ProgrammingError("Unhandled case in switch");
+        break;
+    }
+}
+
 void PromiseRuntimeHash(const Promise *pp, const char *salt,
                         unsigned char digest[EVP_MAX_MD_SIZE + 1],
                         HashMethod type)
@@ -503,7 +579,7 @@ void PromiseRuntimeHash(const Promise *pp, const char *salt,
     Rlist *rp;
     FnCall *fp;
 
-    char *noRvalHash[] = { "mtime", "atime", "ctime", NULL };
+    char *noRvalHash[] = { "mtime", "atime", "ctime", "stime_range", "ttime_range", NULL };
     int doHash;
 
     md = HashDigestFromId(type);
@@ -593,8 +669,7 @@ void PromiseRuntimeHash(const Promise *pp, const char *salt,
             case RVAL_TYPE_LIST:
                 for (rp = cp->rval.item; rp != NULL; rp = rp->next)
                 {
-                    EVP_DigestUpdate(context, RlistScalarValue(rp),
-                                     strlen(RlistScalarValue(rp)));
+                    RvalDigestUpdate(context, rp);
                 }
                 break;
 
@@ -608,22 +683,7 @@ void PromiseRuntimeHash(const Promise *pp, const char *salt,
 
                 for (rp = fp->args; rp != NULL; rp = rp->next)
                 {
-                    switch (rp->val.type)
-                    {
-                    case RVAL_TYPE_SCALAR:
-                        EVP_DigestUpdate(context, RlistScalarValue(rp),
-                                         strlen(RlistScalarValue(rp)));
-                        break;
-
-                    case RVAL_TYPE_FNCALL:
-                        EVP_DigestUpdate(context, RlistFnCallValue(rp)->name,
-                                         strlen(RlistFnCallValue(rp)->name));
-                        break;
-
-                    default:
-                        ProgrammingError("Unhandled case in switch");
-                        break;
-                    }
+                    RvalDigestUpdate(context, rp);
                 }
                 break;
 
@@ -658,7 +718,7 @@ static CfLock CfLockNull(void)
 }
 
 CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host,
-                   time_t now, TransactionContext tc, const Promise *pp,
+                   time_t now, int ifelapsed, int expireafter, const Promise *pp,
                    bool ignoreProcesses)
 {
     if (now == 0)
@@ -703,7 +763,7 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host,
 
     Log(LOG_LEVEL_DEBUG,
         "AcquireLock(%s,%s), ExpireAfter = %d, IfElapsed = %d",
-        cc_operator, cc_operand, tc.expireafter, tc.ifelapsed);
+        cc_operator, cc_operand, expireafter, ifelapsed);
 
     int sum = 0;
     for (int i = 0; cc_operator[i] != '\0'; i++)
@@ -738,7 +798,7 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host,
 
     // For promises/locks with ifelapsed == 0, skip all detection logic of
     // previously acquired locks, whether in this agent or a parallel one.
-    if (tc.ifelapsed != 0)
+    if (ifelapsed != 0)
     {
         if (elapsedtime < 0)
         {
@@ -750,11 +810,11 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host,
             return CfLockNull();
         }
 
-        if (elapsedtime < tc.ifelapsed)
+        if (elapsedtime < ifelapsed)
         {
             Log(LOG_LEVEL_VERBOSE,
                 "Nothing promised here [%.40s] (%jd/%u minutes elapsed)",
-                cflast, (intmax_t) elapsedtime, tc.ifelapsed);
+                cflast, (intmax_t) elapsedtime, ifelapsed);
             ReleaseCriticalSection(CF_CRITIAL_SECTION);
             return CfLockNull();
         }
@@ -768,11 +828,11 @@ CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host,
 
         if (lastcompleted != 0)
         {
-            if (elapsedtime >= tc.expireafter)
+            if (elapsedtime >= expireafter)
             {
                 Log(LOG_LEVEL_INFO,
                     "Lock expired after %jd/%u minutes: %s",
-                    (intmax_t) elapsedtime, tc.expireafter, cflock);
+                    (intmax_t) elapsedtime, expireafter, cflock);
 
                 if (KillLockHolder(cflock))
                 {
@@ -935,6 +995,23 @@ void GetLockName(char *lockname, const char *locktype,
     }
 }
 
+static void RestoreLockDatabase(void)
+{
+    // We don't do any locking here (since we can't trust the database), but
+    // this should be right after bootup, so we should be the only one.
+    // Worst case someone else will just copy the same file to the same
+    // location.
+    char *db_path = DBIdToPath(dbid_locks);
+    char *db_path_backup;
+    xasprintf(&db_path_backup, "%s.backup", db_path);
+
+    CopyLockDatabaseAtomically(db_path_backup, db_path, "lock database backup",
+                               "lock database");
+
+    free(db_path);
+    free(db_path_backup);
+}
+
 static void CopyLockDatabaseAtomically(const char *from, const char *to,
                                        const char *from_pretty_name,
                                        const char *to_pretty_name)
@@ -1012,23 +1089,6 @@ void BackupLockDatabase(void)
     ReleaseCriticalSection(CF_CRITIAL_SECTION);
 }
 
-static void RestoreLockDatabase(void)
-{
-    // We don't do any locking here (since we can't trust the database), but
-    // this should be right after bootup, so we should be the only one.
-    // Worst case someone else will just copy the same file to the same
-    // location.
-    char *db_path = DBIdToPath(dbid_locks);
-    char *db_path_backup;
-    xasprintf(&db_path_backup, "%s.backup", db_path);
-
-    CopyLockDatabaseAtomically(db_path_backup, db_path, "lock database backup",
-                               "lock database");
-
-    free(db_path);
-    free(db_path_backup);
-}
-
 void PurgeLocks(void)
 {
     CF_DBC *dbcp;
@@ -1072,17 +1132,11 @@ void PurgeLocks(void)
     {
 #ifdef LMDB
         LOG_LOCK_OP("<unknown>", key, entry);
-
-        if (key[0] == 'X')
-        {
-            continue;
-        }
-#else
+#endif
         if (STARTSWITH(key, "last.internal_bundle.track_license.handle"))
         {
             continue;
         }
-#endif
 
         if (now - entry->time > (time_t) CF_LOCKHORIZON)
         {
@@ -1100,75 +1154,3 @@ void PurgeLocks(void)
     WriteDB(dbp, "lock_horizon", &lock_horizon, sizeof(lock_horizon));
     CloseLock(dbp);
 }
-
-int WriteLock(const char *name)
-{
-    CF_DB *dbp = OpenLock();
-
-    if (dbp == NULL)
-    {
-        return -1;
-    }
-
-    ThreadLock(cft_lock);
-    WriteLockDataCurrent(dbp, name);
-
-    CloseLock(dbp);
-    ThreadUnlock(cft_lock);
-
-    return 0;
-}
-
-static void VerifyThatDatabaseIsNotCorrupt_once(void)
-{
-    int uptime = GetUptimeSeconds(time(NULL));
-    if (uptime <= 0)
-    {
-        Log(LOG_LEVEL_VERBOSE,
-            "Not able to determine uptime when verifying lock database. "
-            "Will assume the database is in order.");
-        return;
-    }
-
-    char *db_path = DBIdToPath(dbid_locks);
-    struct stat statbuf;
-    if (stat(db_path, &statbuf) == 0)
-    {
-        if (statbuf.st_mtime < time(NULL) - uptime)
-        {
-            // We have rebooted since the database was last updated.
-            // Restore it from our backup.
-            RestoreLockDatabase();
-        }
-    }
-
-    free(db_path);
-}
-
-static void VerifyThatDatabaseIsNotCorrupt(void)
-{
-    static pthread_once_t uptime_verified = PTHREAD_ONCE_INIT;
-    pthread_once(&uptime_verified, &VerifyThatDatabaseIsNotCorrupt_once);
-}
-
-CF_DB *OpenLock()
-{
-    CF_DB *dbp;
-
-    VerifyThatDatabaseIsNotCorrupt();
-
-    if (!OpenDB(&dbp, dbid_locks))
-    {
-        return NULL;
-    }
-
-    return dbp;
-}
-
-void CloseLock(CF_DB *dbp)
-{
-    if (dbp)
-    {
-        CloseDB(dbp);
-    }
-}
