@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -29,7 +29,7 @@
 
 
 CfLock AcquireLock(EvalContext *ctx, const char *operand, const char *host,
-                   time_t now, TransactionContext tc, const Promise *pp,
+                   time_t now, int ifelapsed, int expireafter, const Promise *pp,
                    bool ignoreProcesses);
 void YieldCurrentLock(CfLock lock);
 void YieldCurrentLockAndRemoveFromCache(EvalContext *ctx, CfLock lock,
@@ -39,13 +39,9 @@ void GetLockName(char *lockname, const char *locktype,
 void PurgeLocks(void);
 void BackupLockDatabase(void);
 
-
-/* TODO remove from header file, these are currently only used internally! */
-int WriteLock(const char *lock);
-CF_DB *OpenLock(void);
+// Used in enterprise/nova code:
+CF_DB *OpenLock();
 void CloseLock(CF_DB *dbp);
-time_t FindLockTime(const char *name);
-
 
 /* These are only used in ENT! TODO fix and remove from header file. */
 void WaitForCriticalSection(const char *section_id);
