@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -25,7 +25,7 @@
 #include <platform.h>
 
 #include <cf3.defs.h>
-#include <bool.h>
+#include <stdbool.h>
 #include <logic_expressions.h>
 #include <misc_lib.h>
 
@@ -214,11 +214,11 @@ ExpressionValue EvalExpression(const Expression *expr,
 
         if (expr->op == LOGICAL_OP_OR)
         {
-            return lhs || rhs;
+            return ((ExpressionValue) (lhs || rhs));
         }
         else
         {
-            return lhs && rhs;
+            return ((ExpressionValue) (lhs && rhs));
         }
     }
 
