@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -135,36 +135,41 @@ MonitoringSlot *Nova_MakeSlot(const char *name, const char *description,
 
 void Nova_LoadSlots(void)
 {
-    FILE *f;
     char filename[CF_BUFSIZE];
     int i;
 
     snprintf(filename, CF_BUFSIZE - 1, "%s%cts_key", GetStateDir(), FILE_SEPARATOR);
 
-    struct stat sb;
-
-    if (stat(filename, &sb) != 0)
+    FILE *f = safe_fopen(filename, "r");
+    if (f == NULL)
     {
         return;
     }
 
-    if(sb.st_mtime <= slots_load_time)
+    int fd = fileno(f);
+    struct stat sb;
+    if ((fstat(fd, &sb) != 0) ||
+        (sb.st_mtime <= slots_load_time))
     {
+        fclose(f);
         return;
     }
 
     slots_load_time = sb.st_mtime;
 
-    if ((f = fopen(filename, "r")) == NULL)
-    {
-        return;
-    }
-
     for (i = 0; i < CF_OBSERVABLES; ++i)
     {
         if (i < ob_spare)
         {
-            fscanf(f, "%*[^\n]\n");
+            int r;
+            do
+            {
+                r = fgetc(f);
+            } while (r != (int)'\n' && r != EOF);
+            if (r == EOF)
+            {
+                break;
+            }
         }
         else
         {
@@ -179,7 +184,7 @@ void Nova_LoadSlots(void)
             if (fgets(line, CF_MAXVARSIZE, f) == NULL)
             {
                 Log(LOG_LEVEL_ERR, "Error trying to read ts_key from file '%s'. (fgets: %s)", filename, GetErrorStr());
-                continue;
+                break;
             }
 
             int fields = sscanf(line, "%*d,%1023[^,],%1023[^,],%1023[^,],%lf,%lf,%d",
