@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -84,29 +84,6 @@ endnetgrent(void)
 
 #endif
 
-#ifndef HAVE_SETEUID
-
-# if !defined __STDC__ || !__STDC__
-/* This is a separate conditional since some stdc systems
-   reject `defined (const)'.  */
-
-#  ifndef const
-#   define const
-#  endif
-# endif
-
-int seteuid(uid_t uid)
-{
-# ifdef HAVE_SETREUID
-    return setreuid(-1, uid);
-# else
-    Log(LOG_LEVEL_VERBOSE, "(This system does not have setreuid (patches.c)");
-    return -1;
-# endif
-}
-
-#endif
-
 /***********************************************************/
 
 #ifndef HAVE_SETEGID
@@ -125,7 +102,7 @@ int setegid(gid_t gid)
 
 /*******************************************************************/
 
-int IsPrivileged()
+bool IsPrivileged()
 {
 #ifdef _WIN32
     return true;
@@ -202,7 +179,7 @@ static char *cf_format_strtimestamp(struct tm *tm, char *buf)
 
 /*******************************************************************/
 
-int LinkOrCopy(const char *from, const char *to, int sym)
+bool LinkOrCopy(const char *from, const char *to, int sym)
 /**
  *  Creates symlink to file on platforms supporting it, copies on
  *  others.
@@ -237,30 +214,3 @@ int LinkOrCopy(const char *from, const char *to, int sym)
 
     return true;
 }
-
-#if !defined(__MINGW32__)
-
-int ExclusiveLockFile(int fd)
-{
-    struct flock lock = {
-        .l_type = F_WRLCK,
-        .l_whence = SEEK_SET,
-    };
-
-    while (fcntl(fd, F_SETLKW, &lock) == -1)
-    {
-        if (errno != EINTR)
-        {
-            return -1;
-        }
-    }
-
-    return 0;
-}
-
-int ExclusiveUnlockFile(int fd)
-{
-    return close(fd);
-}
-
-#endif
