@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -34,11 +34,10 @@
 #include <fncall.h>
 #include <rlist.h>
 #include <set.h>
-#include <hashes.h>
 #include <eval_context.h>
 #include <promises.h>
 #include <item_lib.h>
-#include <files_hashes.h>
+#include <hash.h>
 #include <files_names.h>
 #include <audit.h>
 #include <logging.h>
@@ -105,6 +104,7 @@ Policy *PolicyNew(void)
     policy->release_id = NULL;
     policy->bundles = SeqNew(100, BundleDestroy);
     policy->bodies = SeqNew(100, BodyDestroy);
+    policy->policy_files_hashes = NULL;
 
     return policy;
 }
@@ -125,6 +125,10 @@ void PolicyDestroy(Policy *policy)
         SeqDestroy(policy->bundles);
         SeqDestroy(policy->bodies);
         free(policy->release_id);
+        if (policy->policy_files_hashes != NULL)
+        {
+            StringMapDestroy(policy->policy_files_hashes);
+        }
 
         free(policy);
     }
@@ -260,6 +264,21 @@ StringSet *PolicySourceFiles(const Policy *policy)
 
 /*************************************************************************/
 
+/**
+ * Get hash digest of the given policy file.
+ *
+ * @param policy Policy that is supposed to contain (have loaded) the file
+ * @param policy_file_path Absolute path of the policy file to get the digest for
+ * @return Hash digest of the given policy file or %NULL if unknown
+ * @note   The returned hash digest is owned by the policy, **do not free it**.
+ */
+const char *PolicyGetPolicyFileHash(const Policy *policy, const char *policy_file_path)
+{
+    return StringMapGet(policy->policy_files_hashes, policy_file_path);
+}
+
+/*************************************************************************/
+
 static const char *StripNamespace(const char *full_symbol)
 {
     char *sep = strchr(full_symbol, CF_NS);
@@ -389,6 +408,39 @@ Policy *PolicyMerge(Policy *a, Policy *b)
         bdp->parent_policy = result;
     }
 
+    StringMap *extra_hashes = NULL;
+    if (a->policy_files_hashes != NULL)
+    {
+        result->policy_files_hashes = a->policy_files_hashes;
+        a->policy_files_hashes = NULL;
+        extra_hashes = b->policy_files_hashes;
+        b->policy_files_hashes = NULL;
+    }
+    else if (b->policy_files_hashes != NULL)
+    {
+        result->policy_files_hashes = b->policy_files_hashes;
+        b->policy_files_hashes = NULL;
+    }
+    else
+    {
+        result->policy_files_hashes = NULL;
+    }
+
+    if (extra_hashes != NULL)
+    {
+        MapIterator it = MapIteratorInit((Map*) extra_hashes);
+        MapKeyValue *item;
+        while ((item = MapIteratorNext(&it)) != NULL)
+        {
+            /* Move data and duplicate just the keys (which are always owned by
+               the map). */
+            StringMapInsert(result->policy_files_hashes,
+                            xstrdup((char*) item->key), (char*) item->value);
+        }
+        /* Destroy only the map and the keys, data was moved. */
+        StringMapSoftDestroy(extra_hashes);
+    }
+
     /* Should result take over a release_id ? */
     free(a->release_id);
     free(b->release_id);
@@ -1139,8 +1191,8 @@ static SourceOffset PolicyElementSourceOffset(PolicyElementType type, const void
 
     case POLICY_ELEMENT_TYPE_PROMISE_TYPE:
     {
-        const PromiseType *type = (const PromiseType *)element;
-        return type->offset;
+        const PromiseType *promise_type = (const PromiseType *)element;
+        return promise_type->offset;
     }
 
     case POLICY_ELEMENT_TYPE_PROMISE:
@@ -1186,8 +1238,8 @@ static const char *PolicyElementSourceFile(PolicyElementType type, const void *e
 
     case POLICY_ELEMENT_TYPE_PROMISE_TYPE:
     {
-        const PromiseType *type = (const PromiseType *)element;
-        return PolicyElementSourceFile(POLICY_ELEMENT_TYPE_BUNDLE, type->parent_bundle);
+        const PromiseType *promise_type = (const PromiseType *)element;
+        return PolicyElementSourceFile(POLICY_ELEMENT_TYPE_BUNDLE, promise_type->parent_bundle);
     }
 
     case POLICY_ELEMENT_TYPE_PROMISE:
@@ -1310,11 +1362,11 @@ Body *PolicyAppendBody(Policy *policy, const char *ns, const char *name, const c
     // TODO: move to standard callback
     if (strcmp("service_method", body->name) == 0)
     {
-        Rlist *args = NULL;
-        RlistAppendRval(&args, RvalNew("$(this.promiser)", RVAL_TYPE_SCALAR));
-        RlistAppendRval(&args, RvalNew("$(this.service_policy)", RVAL_TYPE_SCALAR));
+        Rlist *bundle_args = NULL;
+        RlistAppendRval(&bundle_args, RvalNew("$(this.promiser)", RVAL_TYPE_SCALAR));
+        RlistAppendRval(&bundle_args, RvalNew("$(this.service_policy)", RVAL_TYPE_SCALAR));
 
-        FnCall *service_bundle = FnCallNew("standard_services", args);
+        FnCall *service_bundle = FnCallNew("standard_services", bundle_args);
         BodyAppendConstraint(body, "service_bundle", (Rval) { service_bundle, RVAL_TYPE_FNCALL }, "any", false);
     }
 
@@ -1590,6 +1642,11 @@ static Buffer *EscapeQuotes(const char *raw, Buffer *out)
     return out;
 }
 
+/**
+ * Converts the given attribute rval to a JSON object.
+ *
+ * @return A JsonElement of type JSON_ELEMENT_TYPE_CONTAINER
+ */
 static JsonElement *AttributeValueToJson(Rval rval, bool symbolic_reference)
 {
     switch (rval.type)
@@ -1757,7 +1814,15 @@ static JsonElement *BundleContextsToJson(const Seq *promises)
                 JsonObjectAppendInteger(json_attribute, "line", cp->offset.line);
 
                 JsonObjectAppendString(json_attribute, "lval", cp->lval);
-                JsonObjectAppendObject(json_attribute, "rval", AttributeValueToJson(cp->rval, cp->references_body));
+                JsonElement *json_rval = AttributeValueToJson(cp->rval, cp->references_body);
+                if (JsonGetContainerType(json_rval) == JSON_CONTAINER_TYPE_ARRAY)
+                {
+                    JsonObjectAppendArray(json_attribute, "rval", json_rval);
+                }
+                else
+                {
+                    JsonObjectAppendObject(json_attribute, "rval", json_rval);
+                }
                 JsonArrayAppendObject(json_promise_attributes, json_attribute);
             }
 
@@ -2111,6 +2176,7 @@ static Rval RvalFromJson(JsonElement *json_rval)
         {
             Rval list_value = RvalFromJson(JsonArrayGetAsObject(json_list, i));
             RlistAppend(&rlist, list_value.item, list_value.type);
+            RvalDestroy(list_value);
         }
 
         return ((Rval) { rlist, RVAL_TYPE_LIST });
@@ -2127,6 +2193,7 @@ static Rval RvalFromJson(JsonElement *json_rval)
             Rval arg = RvalFromJson(json_arg);
 
             RlistAppend(&args, arg.item, arg.type);
+            RvalDestroy(arg);
         }
 
         FnCall *fn = FnCallNew(name, args);
@@ -3105,10 +3172,9 @@ static SyntaxTypeMatch ConstraintCheckType(const Constraint *cp)
 
                         for (size_t l = 0; bs[l].lval != NULL; l++)
                         {
-                            if (bs[l].dtype == CF_DATA_TYPE_BUNDLE)
-                            {
-                            }
-                            else if (bs[l].dtype == CF_DATA_TYPE_BODY)
+                            // No validation for CF_DATA_TYPE_BUNDLE here
+                            // see: PolicyCheckUndefinedBundles() etc.
+                            if (bs[l].dtype == CF_DATA_TYPE_BODY)
                             {
                                 const ConstraintSyntax *bs2 = bs[l].range.body_type_syntax->constraints;
 
