@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -34,6 +34,7 @@
 #include <string_lib.h>                                         /* Chop */
 #include <regex.h> /* CompileRegex,StringMatchWithPrecompiledRegex,StringMatchFull */
 #include <item_lib.h>
+#include <file_lib.h>   // SetUmask(), RestoreUmask()
 #include <pipes.h>
 #include <files_interfaces.h>
 #include <rlist.h>
@@ -115,7 +116,7 @@ static const char *UCB_STYLE_PS_ARGS = "axwww";
 static const PsColumnAlgorithm UCB_STYLE_PS_COLUMN_ALGORITHM = PCA_ZombieSkipEmptyColumns;
 #endif
 
-static int SelectProcRangeMatch(char *name1, char *name2, int min, int max, char **names, char **line);
+static bool SelectProcRangeMatch(char *name1, char *name2, int min, int max, char **names, char **line);
 static bool SelectProcRegexMatch(const char *name1, const char *name2, const char *regex, bool anchored, char **colNames, char **line);
 static bool SplitProcLine(const char *proc,
                           time_t pstime,
@@ -124,8 +125,8 @@ static bool SplitProcLine(const char *proc,
                           int *end,
                           PsColumnAlgorithm pca,
                           char **line);
-static int SelectProcTimeCounterRangeMatch(char *name1, char *name2, time_t min, time_t max, char **names, char **line);
-static int SelectProcTimeAbsRangeMatch(char *name1, char *name2, time_t min, time_t max, char **names, char **line);
+static bool SelectProcTimeCounterRangeMatch(char *name1, char *name2, time_t min, time_t max, char **names, char **line);
+static bool SelectProcTimeAbsRangeMatch(char *name1, char *name2, time_t min, time_t max, char **names, char **line);
 static int GetProcColumnIndex(const char *name1, const char *name2, char **names);
 static void GetProcessColumnNames(const char *proc, char **names, int *start, int *end);
 static int ExtractPid(char *psentry, char **names, int *end);
@@ -139,7 +140,7 @@ static bool SelectProcess(const char *procentry,
                           int *start,
                           int *end,
                           const char *process_regex,
-                          ProcessSelect a,
+                          const ProcessSelect *a,
                           bool attrselect)
 {
     bool result = true;
@@ -147,6 +148,7 @@ static bool SelectProcess(const char *procentry,
     Rlist *rp;
 
     assert(process_regex);
+    assert(a != NULL);
 
     StringSet *process_select_attributes = StringSetNew();
 
@@ -179,77 +181,83 @@ static bool SelectProcess(const char *procentry,
         goto cleanup;
     }
 
-    for (rp = a.owner; rp != NULL; rp = rp->next)
+    for (rp = a->owner; rp != NULL; rp = rp->next)
     {
-        if (SelectProcRegexMatch("USER", "UID", RlistScalarValue(rp), true, names, column))
+        if (rp->val.type == RVAL_TYPE_FNCALL)
+        {
+            Log(LOG_LEVEL_VERBOSE,
+                "Function call '%s' in process_select body was not resolved, skipping",
+                RlistFnCallValue(rp)->name);
+        }
+        else if (SelectProcRegexMatch("USER", "UID", RlistScalarValue(rp), true, names, column))
         {
             StringSetAdd(process_select_attributes, xstrdup("process_owner"));
             break;
         }
     }
 
-    if (SelectProcRangeMatch("PID", "PID", a.min_pid, a.max_pid, names, column))
+    if (SelectProcRangeMatch("PID", "PID", a->min_pid, a->max_pid, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("pid"));
     }
 
-    if (SelectProcRangeMatch("PPID", "PPID", a.min_ppid, a.max_ppid, names, column))
+    if (SelectProcRangeMatch("PPID", "PPID", a->min_ppid, a->max_ppid, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("ppid"));
     }
 
-    if (SelectProcRangeMatch("PGID", "PGID", a.min_pgid, a.max_pgid, names, column))
+    if (SelectProcRangeMatch("PGID", "PGID", a->min_pgid, a->max_pgid, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("pgid"));
     }
 
-    if (SelectProcRangeMatch("VSZ", "SZ", a.min_vsize, a.max_vsize, names, column))
+    if (SelectProcRangeMatch("VSZ", "SZ", a->min_vsize, a->max_vsize, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("vsize"));
     }
 
-    if (SelectProcRangeMatch("RSS", "RSS", a.min_rsize, a.max_rsize, names, column))
+    if (SelectProcRangeMatch("RSS", "RSS", a->min_rsize, a->max_rsize, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("rsize"));
     }
 
-    if (SelectProcTimeCounterRangeMatch("TIME", "TIME", a.min_ttime, a.max_ttime, names, column))
+    if (SelectProcTimeCounterRangeMatch("TIME", "TIME", a->min_ttime, a->max_ttime, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("ttime"));
     }
 
     if (SelectProcTimeAbsRangeMatch
-        ("STIME", "START", a.min_stime, a.max_stime, names, column))
+        ("STIME", "START", a->min_stime, a->max_stime, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("stime"));
     }
 
-    if (SelectProcRangeMatch("NI", "PRI", a.min_pri, a.max_pri, names, column))
+    if (SelectProcRangeMatch("NI", "PRI", a->min_pri, a->max_pri, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("priority"));
     }
 
-    if (SelectProcRangeMatch("NLWP", "NLWP", a.min_thread, a.max_thread, names, column))
+    if (SelectProcRangeMatch("NLWP", "NLWP", a->min_thread, a->max_thread, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("threads"));
     }
 
-    if (SelectProcRegexMatch("S", "STAT", a.status, true, names, column))
+    if (SelectProcRegexMatch("S", "STAT", a->status, true, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("status"));
     }
 
-    if (SelectProcRegexMatch("CMD", "COMMAND", a.command, true, names, column))
+    if (SelectProcRegexMatch("CMD", "COMMAND", a->command, true, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("command"));
     }
 
-    if (SelectProcRegexMatch("TTY", "TTY", a.tty, true, names, column))
+    if (SelectProcRegexMatch("TTY", "TTY", a->tty, true, names, column))
     {
         StringSetAdd(process_select_attributes, xstrdup("tty"));
     }
 
-    if (!a.process_result)
+    if (!a->process_result)
     {
         if (StringSetSize(process_select_attributes) == 0)
         {
@@ -274,7 +282,7 @@ static bool SelectProcess(const char *procentry,
     }
     else
     {
-        result = EvalProcessResult(a.process_result, process_select_attributes);
+        result = EvalProcessResult(a->process_result, process_select_attributes);
     }
 
 cleanup:
@@ -288,8 +296,9 @@ cleanup:
     return result;
 }
 
-Item *SelectProcesses(const char *process_name, ProcessSelect a, bool attrselect)
+Item *SelectProcesses(const char *process_name, const ProcessSelect *a, bool attrselect)
 {
+    assert(a != NULL);
     const Item *processes = PROCESSTABLE;
     Item *result = NULL;
 
@@ -339,7 +348,7 @@ Item *SelectProcesses(const char *process_name, ProcessSelect a, bool attrselect
     return result;
 }
 
-static int SelectProcRangeMatch(char *name1, char *name2, int min, int max, char **names, char **line)
+static bool SelectProcRangeMatch(char *name1, char *name2, int min, int max, char **names, char **line)
 {
     int i;
     long value;
@@ -419,7 +428,7 @@ static long TimeCounter2Int(const char *s)
     return ((days * 24 + hours) * 60 + minutes) * 60 + seconds;
 }
 
-static int SelectProcTimeCounterRangeMatch(char *name1, char *name2, time_t min, time_t max, char **names, char **line)
+static bool SelectProcTimeCounterRangeMatch(char *name1, char *name2, time_t min, time_t max, char **names, char **line)
 {
     if ((min == CF_NOINT) || (max == CF_NOINT))
     {
@@ -434,8 +443,8 @@ static int SelectProcTimeCounterRangeMatch(char *name1, char *name2, time_t min,
         if (value == CF_NOINT)
         {
             Log(LOG_LEVEL_INFO,
-                "Failed to extract a valid integer from %c => '%s' in process list",
-                name1[i], line[i]);
+                "Failed to extract a valid integer from %s => '%s' in process list",
+                names[i], line[i]);
             return false;
         }
 
@@ -528,7 +537,7 @@ static time_t TimeAbs2Int(const char *s)
     return mktime(&tm);
 }
 
-static int SelectProcTimeAbsRangeMatch(char *name1, char *name2, time_t min, time_t max, char **names, char **line)
+static bool SelectProcTimeAbsRangeMatch(char *name1, char *name2, time_t min, time_t max, char **names, char **line)
 {
     int i;
     time_t value;
@@ -873,7 +882,7 @@ Solaris 9:
         {
             // Last field, slurp up the rest, but discard trailing whitespace.
             last = linelen;
-            while (isspace(line[last - 1]))
+            while (last > pos && isspace(line[last - 1]))
             {
                 last--;
             }
@@ -1291,7 +1300,7 @@ static void CheckPsLineLimitations(void)
     char *buf = NULL;
     size_t bufsize = 0;
 
-    ps_fd = fopen("/etc/default/ps", "r");
+    ps_fd = safe_fopen("/etc/default/ps", "r");
     if (!ps_fd)
     {
         Log(LOG_LEVEL_VERBOSE, "Could not open '/etc/default/ps' "
@@ -1543,7 +1552,7 @@ static inline void ApplyPlatformExtraTable(ARG_UNUSED char **names, ARG_UNUSED c
 #endif
 
 #ifndef _WIN32
-int LoadProcessTable()
+bool LoadProcessTable()
 {
     FILE *prp;
     char pscomm[CF_MAXLINKSIZE];
@@ -1650,6 +1659,7 @@ int LoadProcessTable()
     const char* const statedir = GetStateDir();
 
     snprintf(vbuff, CF_MAXVARSIZE, "%s%ccf_procs", statedir, FILE_SEPARATOR);
+
     RawSaveItemList(PROCESSTABLE, vbuff, NewLineMode_Unix);
 
 # ifdef HAVE_GETZONEID
@@ -1689,6 +1699,9 @@ int LoadProcessTable()
         PrependItem(&rootprocs, otherprocs->name, NULL);
     }
 
+    // TODO: Change safe_fopen() to default to 0600, then remove this.
+    const mode_t old_umask = SetUmask(0077);
+
     snprintf(vbuff, CF_MAXVARSIZE, "%s%ccf_rootprocs", statedir, FILE_SEPARATOR);
     RawSaveItemList(rootprocs, vbuff, NewLineMode_Unix);
     DeleteItemList(rootprocs);
@@ -1697,6 +1710,8 @@ int LoadProcessTable()
     RawSaveItemList(otherprocs, vbuff, NewLineMode_Unix);
     DeleteItemList(otherprocs);
 
+    RestoreUmask(old_umask);
+
     free(vbuff);
     return true;
 }
