@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -377,15 +377,18 @@ Promise *DeRefCopyPromise(EvalContext *ctx, const Promise *pp)
                 }
 
                 if (callee == NULL &&
+                    cp->rval.type != RVAL_TYPE_FNCALL &&
                     strcmp("ifvarclass", cp->lval) != 0 &&
                     strcmp("if",         cp->lval) != 0)
                 {
+                    char *rval_string = RvalToString(cp->rval);
                     Log(LOG_LEVEL_ERR,
                         "Apparent bundle '%s' was undeclared, but "
                         "used in a promise near line %zu of %s "
                         "(possible unquoted literal value)",
-                        RvalScalarValue(cp->rval), pp->offset.line,
+                        rval_string, pp->offset.line,
                         PromiseGetBundle(pp)->source_path);
+                    free(rval_string);
                 }
 
                 Log(LOG_LEVEL_DEBUG,
@@ -488,7 +491,7 @@ static bool EvaluateConstraintIteration(EvalContext *ctx, const Constraint *cp,
   @brief Helper function to determine whether the Rval of ifvarclass/if/unless is defined.
   If the Rval is a function, call that function.
 */
-static bool IsVarClassDefined(const EvalContext *ctx, const Constraint *cp, Promise *pcopy)
+static ExpressionValue CheckVarClassExpression(const EvalContext *ctx, const Constraint *cp, Promise *pcopy)
 {
     assert(ctx);
     assert(cp);
@@ -503,7 +506,7 @@ static bool IsVarClassDefined(const EvalContext *ctx, const Constraint *cp, Prom
     Rval final;
     if (!EvaluateConstraintIteration((EvalContext*)ctx, cp, &final))
     {
-        return false;
+        return EXPRESSION_VALUE_ERROR;
     }
 
     char *classes = NULL;
@@ -522,18 +525,18 @@ static bool IsVarClassDefined(const EvalContext *ctx, const Constraint *cp, Prom
         break;
     }
 
-    if (!classes)
+    if (classes == NULL)
     {
-        return false;
+        return EXPRESSION_VALUE_ERROR;
     }
     // sanity check for unexpanded variables
     if (strchr(classes, '$') || strchr(classes, '@'))
     {
         Log(LOG_LEVEL_DEBUG, "Class expression did not evaluate");
-        return false;
+        return EXPRESSION_VALUE_ERROR;
     }
 
-    return IsDefinedClass(ctx, classes);
+    return CheckClassExpression(ctx, classes);
 }
 
 /* Expands "$(this.promiser)" comment if present. Writes the result to pp. */
@@ -562,9 +565,11 @@ static void DereferenceAndPutComment(Promise* pp, const char *comment)
 
 Promise *ExpandDeRefPromise(EvalContext *ctx, const Promise *pp, bool *excluded)
 {
-    assert(pp->promiser);
-    assert(pp->classes);
-    assert(excluded);
+    assert(pp != NULL);
+    assert(pp->parent_promise_type != NULL);
+    assert(pp->promiser != NULL);
+    assert(pp->classes != NULL);
+    assert(excluded != NULL);
 
     *excluded = false;
 
@@ -620,24 +625,40 @@ Promise *ExpandDeRefPromise(EvalContext *ctx, const Promise *pp, bool *excluded)
 
     /* Look for 'if'/'ifvarclass' exclusion. */
     {
+        /* We need to make sure to check both 'if' and 'ifvarclass' constraints. */
+        bool checked_if = false;
         const Constraint *ifvarclass = PromiseGetConstraint(pp, "ifvarclass");
         if (!ifvarclass)
         {
             ifvarclass = PromiseGetConstraint(pp, "if");
+            checked_if = true;
         }
 
-        if (ifvarclass && !IsVarClassDefined(ctx, ifvarclass, pcopy))
+        // if - Skip if false or error:
+        while (ifvarclass != NULL)
         {
-            if (LogGetGlobalLevel() >= LOG_LEVEL_VERBOSE)
+            if (CheckVarClassExpression(ctx, ifvarclass, pcopy) != EXPRESSION_VALUE_TRUE)
             {
-                char *ifvarclass_string =  RvalToString(ifvarclass->rval);
-                Log(LOG_LEVEL_VERBOSE, "Skipping promise '%s'"
-                    " because constraint '%s => %s' is not met",
-                    pp->promiser, ifvarclass->lval, ifvarclass_string);
-                free(ifvarclass_string);
+                if (LogGetGlobalLevel() >= LOG_LEVEL_VERBOSE)
+                {
+                    char *ifvarclass_string =  RvalToString(ifvarclass->rval);
+                    Log(LOG_LEVEL_VERBOSE, "Skipping promise '%s'"
+                        " because constraint '%s => %s' is not met",
+                        pp->promiser, ifvarclass->lval, ifvarclass_string);
+                    free(ifvarclass_string);
+                }
+                *excluded = true;
+                return pcopy;
+            }
+            if (!checked_if)
+            {
+                ifvarclass = PromiseGetConstraint(pp, "if");
+                checked_if = true;
+            }
+            else
+            {
+                ifvarclass = NULL;
             }
-            *excluded = true;
-            return pcopy;
         }
     }
 
@@ -645,18 +666,44 @@ Promise *ExpandDeRefPromise(EvalContext *ctx, const Promise *pp, bool *excluded)
     {
         const Constraint *unless = PromiseGetConstraint(pp, "unless");
 
-        if (unless && IsVarClassDefined(ctx, unless, pcopy))
+        // unless - Skip if true or error:
+        if (unless != NULL)
         {
-            if (LogGetGlobalLevel() >= LOG_LEVEL_VERBOSE)
+            // If the rval is a function, CheckVarClassExpression will call it
+            // It will evaluate the class expression as well:
+            const ExpressionValue value = CheckVarClassExpression(ctx, unless, pcopy);
+            // If it returns EXPRESSION_VALUE_ERROR, it most likely means there
+            // are unexpanded variables in the rval (possibly in function calls)
+
+            if ((EvalContextGetPass(ctx) == CF_DONEPASSES-1)
+                && (value == EXPRESSION_VALUE_ERROR))
             {
                 char *unless_string =  RvalToString(unless->rval);
-                Log(LOG_LEVEL_VERBOSE, "Skipping promise '%s'"
-                    " because constraint '%s => %s' is not met",
-                    pp->promiser, unless->lval, unless_string);
+                // The rval is most likely a string or a function call,
+                // with unexpanded variables, for example:
+                // unless => "$(no_such_var)"
+                // We default to NOT skipping (since if would skip).
+                Log(LOG_LEVEL_VERBOSE,
+                    "Not skipping %s promise '%s' with constraint '%s => %s' in last evaluation pass (since if would skip)",
+                    pp->parent_promise_type->name,
+                    pp->promiser,
+                    unless->lval,
+                    unless_string);
                 free(unless_string);
             }
-            *excluded = true;
-            return pcopy;
+            else if (value != EXPRESSION_VALUE_FALSE)
+            {
+                if (LogGetGlobalLevel() >= LOG_LEVEL_VERBOSE)
+                {
+                    char *unless_string =  RvalToString(unless->rval);
+                    Log(LOG_LEVEL_VERBOSE,
+                        "Skipping promise '%s' because constraint '%s => %s' is not met",
+                        pp->promiser, unless->lval, unless_string);
+                    free(unless_string);
+                }
+                *excluded = true;
+                return pcopy;
+            }
         }
     }
 
