@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -287,6 +287,27 @@ bool RlistIsInListOfRegex(const Rlist *list, const char *str)
     return false;
 }
 
+bool RlistContainsString(const Rlist *list, const char *string)
+{
+    assert(string != NULL);
+
+    if (list == NULL)
+    {
+        // Empty Rlist is represented as a NULL pointer
+        return false;
+    }
+
+    for (const Rlist *rp = list; rp != NULL; rp = rp->next)
+    {
+        if (rp->val.type == RVAL_TYPE_SCALAR &&
+            StringSafeEqual(RlistScalarValue(rp), string))
+        {
+            return true;
+        }
+    }
+    return false;
+}
+
 /*******************************************************************/
 
 static Rval RvalCopyScalar(Rval rval)
@@ -349,7 +370,7 @@ Rval RvalNewRewriter(const void *item, RvalType type, JsonElement *map)
             char *buffer_to = xmalloc(max_size);
 
             Buffer *format = BufferNew();
-            strncpy(buffer_from, item, max_size);
+            StringCopy(item, buffer_from, max_size);
 
             for (int iteration = 0; iteration < 10; iteration++)
             {
@@ -397,7 +418,7 @@ Rval RvalNewRewriter(const void *item, RvalType type, JsonElement *map)
                             // Reset location to immediately after the replacement.
                             c = var_start + repl_len - 1;
                             var_start = -1;
-                            strcpy(buffer_from, buffer_to);
+                            StringCopy(buffer_to, buffer_from, max_size);
                             closing_brace = 0;
                             replacement_made = true;
                         }
@@ -526,6 +547,16 @@ Rlist *RlistAppendScalar(Rlist **start, const char *scalar)
     return RlistAppendRval(start, RvalCopyScalar((Rval) { (char *)scalar, RVAL_TYPE_SCALAR }));
 }
 
+Rlist *RlistAppendString(Rlist **start, const char *string)
+{
+    Rlist *l = RlistAppendScalar(start, string);
+
+    assert(RlistContainsString(l, string));
+
+    return l;
+}
+
+// NOTE: Copies item, does NOT take ownership
 Rlist *RlistAppend(Rlist **start, const void *item, RvalType type)
 {
     return RlistAppendAllTypes(start, item, type, false);
@@ -1194,9 +1225,17 @@ Rlist *RlistFromRegexSplitNoOverflow(const char *string, const char *regex, int
     while (count < max - 1 &&
            StringMatchWithPrecompiledRegex(pattern, sp, &start, &end))
     {
-        assert(start < CF_MAXVARSIZE);
-        memcpy(node, sp, start);
-        node[start] = '\0';
+        size_t len = start;
+        if (len >= CF_MAXVARSIZE)
+        {
+            len = CF_MAXVARSIZE - 1;
+            Log(LOG_LEVEL_WARNING,
+                "Segment in string_split() is %d bytes and will be truncated to %zu bytes",
+                start,
+                len);
+        }
+        memcpy(node, sp, len);
+        node[len] = '\0';
         RlistAppendScalar(&liststart, node);
         count++;
 
