@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -53,6 +53,9 @@ static SyntaxTypeMatch CheckFnCallType(const char *s, DataType dtype);
 
 static const PromiseTypeSyntax *PromiseTypeSyntaxGetStrict(const char *bundle_type, const char *promise_type)
 {
+    assert(bundle_type != NULL);
+    assert(promise_type != NULL);
+
     for (int module_index = 0; module_index < CF3_MODULES; module_index++)
     {
         for (int promise_type_index = 0; CF_ALL_PROMISE_TYPES[module_index][promise_type_index].promise_type; promise_type_index++)
@@ -111,6 +114,9 @@ const ConstraintSyntax *BodySyntaxGetConstraintSyntax(const ConstraintSyntax *bo
 
 const ConstraintSyntax *PromiseTypeSyntaxGetConstraintSyntax(const PromiseTypeSyntax *promise_type_syntax, const char *lval)
 {
+    assert(promise_type_syntax != NULL);
+    assert(lval != NULL);
+
     for (int i = 0; promise_type_syntax->constraints[i].lval; i++)
     {
         if (strcmp(promise_type_syntax->constraints[i].lval, lval) == 0)
@@ -175,13 +181,21 @@ const BodySyntax *BodySyntaxGet(const char *body_type)
 
 const char *SyntaxStatusToString(SyntaxStatus status)
 {
-    static const char *const status_strings[] =
+    assert( status == SYNTAX_STATUS_DEPRECATED ||
+            status == SYNTAX_STATUS_NORMAL ||
+            status == SYNTAX_STATUS_REMOVED );
+    switch (status)
     {
-        [SYNTAX_STATUS_DEPRECATED] = "deprecated",
-        [SYNTAX_STATUS_NORMAL] = "normal",
-        [SYNTAX_STATUS_REMOVED] = "removed"
-    };
-    return status_strings[status];
+        case SYNTAX_STATUS_DEPRECATED:
+            return "deprecated";
+        case SYNTAX_STATUS_NORMAL:
+            return "normal";
+        case SYNTAX_STATUS_REMOVED:
+            return "removed";
+        default:
+            break;
+    }
+    return NULL;
 }
 
 /****************************************************************************/
@@ -810,28 +824,41 @@ static SyntaxTypeMatch CheckParseOpts(const char *s, const char *range)
 
 /****************************************************************************/
 
-int CheckParseVariableName(const char *name)
+bool CheckParseVariableName(const char *const name)
 {
-    const char *reserved[] = { "promiser", "handle", "promise_filename", "promise_dirname", "promise_linenumber", "this", NULL };
-    char scopeid[CF_MAXVARSIZE], vlval[CF_MAXVARSIZE];
-    int count = 0, level = 0;
+    assert(name != NULL);
+
+    const char *const reserved[] = {
+        "promiser",
+        "handle",
+        "promise_filename",
+        "promise_dirname",
+        "promise_linenumber",
+        "this",
+        NULL
+    };
 
     if (IsStrIn(name, reserved))
     {
         return false;
     }
 
-    scopeid[0] = '\0';
+    int count = 0, level = 0;
 
-    if (strchr(name, '.'))
+    const char *const first_dot = strchr(name, '.');
+
+    if (first_dot != NULL)
     {
         for (const char *sp = name; *sp != '\0'; sp++)
         {
             switch (*sp)
             {
             case '.':
-                if (++count > 1 && level != 1)
+                count++;
+                if (count > 1 && level != 1)
                 {
+                    // Adding a second dot is not allowed,
+                    // except inside 1 level of square brackets
                     return false;
                 }
                 break;
@@ -853,14 +880,12 @@ int CheckParseVariableName(const char *name)
                 yyerror("Too many levels of [] reserved for array use");
                 return false;
             }
-
         }
 
         if (count == 1)
         {
-            sscanf(name, "%[^.].%s", scopeid, vlval);
-
-            if (strlen(scopeid) == 0 || strlen(vlval) == 0)
+            // Check that there is something before and after first dot:
+            if (name[0] == '.' || first_dot[1] == '\0')
             {
                 return false;
             }
