@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -28,14 +28,13 @@
 #include <expand.h>
 #include <scope.h>
 #include <matching.h>
-#include <hashes.h>
 #include <unix.h>
 #include <misc_lib.h>
 #include <rlist.h>
 #include <policy.h>
 #include <eval_context.h>
 
-static int IsCf3Scalar(char *str);
+static bool IsCf3Scalar(char *str);
 
 /*******************************************************************/
 
@@ -195,12 +194,12 @@ bool IsCf3VarString(const char *str)
         return false;
     }
 
-    return vars;
+    return (vars != 0);
 }
 
 /*********************************************************************/
 
-static int IsCf3Scalar(char *str)
+static bool IsCf3Scalar(char *str)
 {
     char *sp;
     char left = 'x', right = 'x';
@@ -274,7 +273,7 @@ static int IsCf3Scalar(char *str)
         return false;
     }
 
-    return vars;
+    return (vars != 0);
 }
 
 /* Extract everything up to the dollar sign. */
