@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -56,7 +56,7 @@ PromiseResult VerifyReportPromise(EvalContext *ctx, const Promise *pp)
 
     // We let AcquireLock worry about making a unique name
     snprintf(unique_name, CF_EXPANDSIZE - 1, "%s", pp->promiser);
-    thislock = AcquireLock(ctx, unique_name, VUQNAME, CFSTARTTIME, a.transaction, pp, false);
+    thislock = AcquireLock(ctx, unique_name, VUQNAME, CFSTARTTIME, a.transaction.ifelapsed, a.transaction.expireafter, pp, false);
 
     // Handle return values before locks, as we always do this
 
@@ -92,7 +92,7 @@ PromiseResult VerifyReportPromise(EvalContext *ctx, const Promise *pp)
 
     if (a.transaction.action == cfa_warn)
     {
-        cfPS(ctx, LOG_LEVEL_WARNING, PROMISE_RESULT_WARN, pp, a, "Need to repair reports promise: %s", pp->promiser);
+        cfPS(ctx, LOG_LEVEL_WARNING, PROMISE_RESULT_WARN, pp, &a, "Need to repair reports promise: %s", pp->promiser);
         YieldCurrentLock(thislock);
         return PROMISE_RESULT_WARN;
     }
@@ -117,7 +117,7 @@ PromiseResult VerifyReportPromise(EvalContext *ctx, const Promise *pp)
 
     YieldCurrentLock(thislock);
 
-    ClassAuditLog(ctx, pp, a, result);
+    ClassAuditLog(ctx, pp, &a, result);
     return result;
 }
 
@@ -135,7 +135,7 @@ static void ReportToLog(const char *message)
 
 static void ReportToFile(const char *logfile, const char *message)
 {
-    FILE *fp = safe_fopen(logfile, "a");
+    FILE *fp = safe_fopen_create_perms(logfile, "a", CF_PERMS_DEFAULT);
     if (!fp)
     {
         Log(LOG_LEVEL_ERR, "Could not open log file '%s', message '%s'. (fopen: %s)", logfile, message, GetErrorStr());
