@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2019 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -78,6 +78,10 @@ static bool FsSupportsSparseFiles(const char *filename)
         " 'disk usage' than their true size, and this is verified by du");
     return false;
 #endif
+#ifdef __APPLE__
+    Log(LOG_LEVEL_NOTICE, "OS X detected, skipping sparseness tests!");
+    return false;
+#endif
 
     int fd = open(filename, O_CREAT | O_WRONLY | O_BINARY, 0700);
     assert_int_not_equal(fd, -1);
@@ -272,9 +276,9 @@ static bool FileIsSparse(const char *filename)
 
     Log(LOG_LEVEL_DEBUG,
         " st_size=%ju ST_NBYTES=%ju ST_NBLOCKS=%ju ST_BLKSIZE=%ju DEV_BSIZE=%ju",
-        (uint64_t) statbuf.st_size, (uint64_t) ST_NBYTES(statbuf),
-        (uint64_t) ST_NBLOCKS(statbuf), (uint64_t) ST_BLKSIZE(statbuf),
-        (uint64_t) DEV_BSIZE);
+        (uintmax_t) statbuf.st_size, (uintmax_t) ST_NBYTES(statbuf),
+        (uintmax_t) ST_NBLOCKS(statbuf), (uintmax_t) ST_BLKSIZE(statbuf),
+        (uintmax_t) DEV_BSIZE);
 
     if (statbuf.st_size <= ST_NBYTES(statbuf))
     {
