@@ -53,6 +53,9 @@ static void TestSysVar(EvalContext *ctx, const char *lval, const char *expected)
     VarRef *ref = VarRefParseFromScope(lval, "sys");
     assert_string_equal(expected, EvalContextVariableGet(ctx, ref, NULL));
     VarRefDestroy(ref);
+
+    assert_string_equal(expected, EvalContextVariableGetSpecial(ctx, SPECIAL_SCOPE_SYS, lval, NULL));
+    assert_string_equal(expected, EvalContextVariableGetSpecialString(ctx, SPECIAL_SCOPE_SYS, lval));
 }
 
 static void test_set_names(void)
